<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Account_Model_Customer_Address extends Ecart_Db_Table
{
    protected $_name = 'account_customer_address';
        
    public function countByCustomer($customerId)
    {
        return  $this->count("customer_id = " . $customerId, 'COUNT(id)');
    }

    /**
     *
     * @param int $customerId
     * @return int
     */
    public function getDefaultBillingByCustomerId($customerId)
    {
       $query = "SELECT ab.id " 
              . "FROM " . self::$_db_prefix . "account_customer c " 
              . "INNER JOIN {$this->_name} ab ON ab.id = c.default_billing_address_id " 
              . "WHERE c.id = ?";
       $res = $this->getAdapter()->fetchRow($query, $customerId);
       return $res['id'];
    }

    /**
     *
     * @param int $customerId
     * @return int
     */
    public function getDefaultShippingByCustomerId($customerId)
    {
        $query = "SELECT ab.id " 
               . "FROM " . self::$_db_prefix . "account_customer c " 
               . "INNER JOIN {$this->_name} ab ON ab.id = c.default_shipping_address_id " 
               . "WHERE c.id = ?";
        $res = $this->getAdapter()->fetchRow($query, $customerId);
        return $res['id'];
    }

    /**
     *
     * @param int $id
     * @return array
     */
    public function getById($id)
    {
        $query = 'SELECT ca.*, c.name as country, z.name as zone ' 
               . 'FROM ' . $this->_name . ' ca '
               . 'LEFT JOIN ' . self::$_db_prefix . 'location_country c ON c.id = ca.country_id '
               . 'LEFT JOIN ' . self::$_db_prefix . 'location_zone z ON z.id = ca.zone_id '
               . 'WHERE ca.id = ?';
        return $this->getAdapter()->fetchRow($query, $id);
    }
    
    /**
     *
     * @param int $customerId
     * @return mixed
     */
    public function getDefaultBillingId($customerId = null)
    {
        if (null === $customerId && !$customerId = Ecart::getCustomerId()) {
            return false;
        }
        
        $customer = Ecart::single('account/customer')->find($customerId);
        if (!count($customer)) {
            return false;
        }
        return $customer->current()->default_billing_address_id;
    }

    /**
     *
     * @param int $customerId
     * @return mixed
     */
    public function getDefaultDeliveryId($customerId = null)
    {
        if (null === $customerId 
            && !$customerId = Ecart::getCustomerId()) {

            return false;
        }
                    
        $customers = Ecart::single('account/customer')->find($customerId);
        if (!count($customers)
            && ! $customers instanceof Ecart_Db_Table_Rowset) {

            return false;
        }
        return $customers->current()->default_shipping_address_id;
    }

    /**
     *
     * @param int $customerId
     * @return array
     */
    public function getListByCustomer($customerId)
    {
        $select = $this->getAdapter()->select();
        $select->from(array('a' => $this->_name))
           ->joinLeft(array('c' => self::$_db_prefix . 'location_country'), 
               'c.id = a.country_id', 
               array('country' => 'name')
           )
           ->joinLeft(array('z' => self::$_db_prefix . 'location_zone'), 
               'z.id = a.zone_id', 
               array('zone' => 'name')
           );
        $select->where('a.customer_id = ' . intval($customerId));
        return $select->query()->fetchAll();
    }
    
    /**
     * Return full address with country & zone data
     *
     * @param int $id
     * @return array
     */
    public function get($id)
    {
        $rows = $this->find($id);
        if (!count($rows)) {
            return false;
        }
        
        $address = $rows->current()->toArray();
        
        if ($address['zone_id']) {
            $address['zone'] = Ecart::single('location/zone')
                ->find($address['zone_id'])->current()->toArray();
        }
        if ($address['country_id']) {
            $address['country'] = Ecart::single('location/country')
                ->find($address['country_id'])->current()->toArray();
        }
        return $address;
    }

    /**
     *
     * @param int $customerId
     * @return array
     */
    public function getCustomListByCustomer($customerId = null) 
    {
        if (null === $customerId) {
            $customerId = Ecart::getCustomerId();
        }
        $alist = $this->fetchAll(
            $this->getAdapter()->quoteInto('customer_id = ?', $customerId)
        )->toArray();
        $j = 0;
        for ($i = 0, $n = count($alist); $i < $n; $i++) {
            if ($alist[$i]['id'] == $this->getDefaultBillingId($customerId) ||
                $alist[$i]['id'] == $this->getDefaultDeliveryId($customerId))
            {
                list($alist[$i], $alist[$j]) = array($alist[$j], $alist[$i]);
                ++$j;
            }
        }
        return $alist;
    }
    
    /**
     * Update or insert address row.
     * 
     * @param array $data
     *  array (
     *      'id' => int optional
     *      'customer_id' => int,
     *      'primary_billing' => bool optional
     *      'primary_shipping' => bool optional
     *      ..
     *      'account_customer_address rows' => optional
     *      ..
     *  )
     * @return int
     */
    public function save($data)
    {
        if (!isset($data['id']) || !$row = $this->find($data['id'])->current()) {
            if (isset($data['id'])) {
                unset($data['id']);
            }
            $row = $this->createRow();
        }
        $row->setFromArray($data);
        $addressId = $row->save();
        
        $customer = Ecart::single('account/customer')
            ->find($data['customer_id'])->current();
        
        if ($this->countByCustomer($data['customer_id']) < 2) {
            $customer->default_billing_address_id  = $addressId;
            $customer->default_shipping_address_id = $addressId;
        } else {
            if (isset($data['primary_billing']) && $data['primary_billing']) {
                $customer->default_billing_address_id  = $addressId;
            }
            if (isset($data['primary_shipping']) && $data['primary_shipping']) {
                $customer->default_shipping_address_id = $addressId;
            }
        }
        $customer->save();
        return $addressId;
    }
}