<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Account_Model_Customer_ForgotPassword extends Ecart_Db_Table
{
    protected $_name = 'account_customer_forgotpassword';

    /**
     *
     * @param array $data
     * @return mixed
     */
    public function save(array $data)
    {
        if (empty($data['created_at'])) {
            $data['created_at'] = Ecart_Date::now()->toZendDbExpr();
        }
        if (count($this->find($data['customer_id'])))
            return $this->update(
                $data, 
                $this->getAdapter()->quoteInto('customer_id = ?', $data['customer_id'])
            );
        return $this->insert($data);
    }

    /**
     *
     * @param string $hash
     * @param string $email
     * @return bool
     */
    public function isValid($hash, $email = null) 
    {
         $select = $this->getAdapter()->select();
         $date = Ecart_Date::now()->addDay(-1)->toSQLString();
         $select->from(array('cfp' => $this->_name), 'COUNT(*)')
                ->joinLeft( array('c' => self::$_db_prefix . 'account_customer'),
                    'c.id = cfp.customer_id', array())
                ->where("cfp.hash = '{$hash}'")
                ->where('c.site_id = ' . Ecart::getSiteId())
                ->where("cfp.created_at > {$date}");
         if ($email)
            $select->where("c.email = '{$email}'");
         $count = $this->getAdapter()->fetchOne($select->__toString());
         return $count ? true : false;
    }

    /**
     *
     * @param string $hash
     * @return string
     */
    public function getEmailByHash($hash)
    {
        $select = $this->getAdapter()->select();
        $select->from(array('c' => self::$_db_prefix . 'account_customer'), 'email')
               ->joinLeft(array('cfp' => $this->_name),
                   'c.id = cfp.customer_id', array())
               ->where("cfp.hash = '{$hash}'");
        return $this->getAdapter()->fetchOne($select->__toString());
    }
}