<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Account_Model_Customer_Group extends Ecart_Db_Table
{
    protected $_name = 'account_customer_group';
    protected $_primary = 'id';

    /**
     *
     * @param array $params
     * @return array
     */
    public function getList($params = array())
    {
        $select = $this->getAdapter()->select();
        
        $select->from($this->_name);
        
        if (isset($params['sort'])&& (isset($params['dir']))) {
            $select->order($params['sort'] . ' ' . $params['dir']);
        } 
        
        if (isset($params['where'])) {
          $select->where($params['where']);
        }
        
        return $this->getAdapter()->fetchAll($select->__toString());
    }

    /**
     * @return int
     */
    public function getCount()
    {
        return $this->getAdapter()->fetchOne(
            "SELECT COUNT(*) FROM " . $this->_name
        );
    }

    /**
     *
     * @param array $data
     * @return bool
     */
    public function save($data)
    {
        $existsCustomerGroups = $this->getAdapter()->fetchCol(
            "SELECT id FROM {$this->_name}"
        );

        $return = true;
        foreach ($data as $groupId => $values) {
            $row = array(
                'name' => $values['name'],
                'description' => $values['description']
            );
            
            if (in_array($groupId, $existsCustomerGroups)) {
                $return = $return && $this->update(
                    $row,
                    $this->getAdapter()->quoteInto(
                        "id = ? AND name <> 'Guest'", $groupId
                    )
                );
            } else {
                $return = $return && $this->insert($row);
            }
        }
        
        Ecart::message()->addSuccess(
            Ecart::translate('account')->__(
                'Group was saved successfully'
            )
        );
        return $return;
    }
}