<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Account_TagController extends Ecart_Account_Controller_Account
{
    /**
     *
     * @var Ecart_Tag_Model_Customer
     */
    private $_table;
     
    /**
     * Initialize
     * @see app/code/Ecart/Account/Controller/Ecart_Account_Controller_Account#init()
     */
    public function init()
    {
        parent::init();
        if (!Ecart::single('core/module')->isInstalled('Ecart_Tag')) {
            $this->_redirect('/account');
        }
        $this->view->crumbs()->add(
            Ecart::translate('tag')->__('Tags'), '/account/tag'
        );
        $this->_table = Ecart::single('tag/customer');
    }
    
    /**
     * Render all account tags 
     * @return void
     */
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('account')->__('My Tags');

        $this->view->tags = $this->_table->getMyWithWeight();
        $this->view->tags()->enableWeight();
        $this->render();
    }
    
    /**
     * Customer add new tag on product 
     * @return void
     */
    public function addAction()
    {
        Ecart::single('tag/customer')->save(
            $this->_getParam('tags'), $this->_getParam('productId')
        );
        $this->_redirect($_SERVER['HTTP_REFERER']);
    }

    /**
     * customer remove self tag 
     * @return void 
     */
    public function removeAction()
    {
        $integer = new Zend_Filter_Int();
        
        $tagId = $integer->filter($this->_getParam('tagId'));
        
        $this->_table->find($tagId)->current()->delete();
        
        $this->_redirect('/account/tag');
    }
    
    /**
     * Remove tag item for current user
     * @return void
     */
    public function removeItemAction()
    {
        $tableTagProduct = Ecart::single('tag/product');
        
        $integer = new Zend_Filter_Int();
        $id = $integer->filter($this->_getParam('itemId'));
        $tableTagProduct->deleteMy($id);
        
        $tagId = $integer->filter($this->_getParam('tagId'));
        $weightTag = $tableTagProduct->weightTag($tagId);
        if (!$weightTag) {
            $this->_table->find($tagId)->current()->delete();
            $this->_redirect('/account/tag');
        }
        $this->_redirect('/tag/index/show-products/tagId/' . $this->_getParam('tagId'));
    }
}