<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 * @abstract
 */
abstract class Ecart_Admin_Controller_Back extends Ecart_Controller_Action
{
    /**
     * Acl
     * @var Ecart_Acl
     */
    public $acl;

    /**
     *
     * @var bool
     */
    protected $_disableAuth = false;

    /**
     *
     * @var bool
     */
    protected $_disableAcl  = false;
    
    public function init()
    {
        parent::init();
        
        $this->view->adminUrl = '/' . trim(
            Ecart::config()->main->store->adminRoute, '/ '
        );

        Zend_Auth::getInstance()->setStorage(
            new Zend_Auth_Storage_Session('admin')
        );
        
        $this->acl = Ecart::single('admin/acl');
        if (!empty(Ecart::session()->roleId)) {
            $this->acl->loadRules(Ecart::session()->roleId);
        }
    }
    
    public function preDispatch()
    {
        $url = Ecart::getCurrentUrl();
        if (Ecart::config()->main->store->useBackendSsl &&
            substr($url, 0, strlen('https://')) != 'https://') {

            parent::_redirect('https://' . substr($url, strlen('http://')));
            die();
        }
        $this->auth();
        $this->checkPermission();
    }

    public function auth()
    {
        if (!$this->_disableAuth) {
            if (!Zend_Auth::getInstance()->hasIdentity()) {
                if ($this->getRequest()->isXmlHttpRequest()) {
                    Ecart::message()->addError(
                        Ecart::translate('admin')->__(
                            'Your session has been expired. Please relogin'
                        )
                    );

                    $this->_helper->json->sendFailure();

                    die();
                    // Zend_Controller_Action_Helper_Json if $suppressExit = true;
                }
                $this->_forward('index', 'auth', 'Ecart_Admin');
            } elseif (!Ecart::single('admin/user')->find(
                    Zend_Auth::getInstance()->getIdentity())->current()) {
                $this->view->action('logout', 'auth', 'Ecart_Admin');
            }
        }
    }
    
    public function checkPermission()
    {
        if ($this->_disableAcl) {
            return true;
        }
        
        $request = $this->getRequest();
        
        $action = $request->getActionName();
        //$controller = str_replace('_', '/', $request->getControllerName());
        $controller = $request->getControllerName();
        $role = Ecart::session()->roleId;
        $resourceIds = explode('/', "admin/$controller/$action");
        // admin is the name of parent resource_id

        while (count($resourceIds)) {
            $resourceId = implode('/', $resourceIds);
            if ($this->acl->has($resourceId)) {
                if ($this->acl->isAllowed($role, $resourceId)) {
                    return true;
                } else {
                    break;
                }
            }
            array_pop($resourceIds);
        }
        
        if ($request->isXmlHttpRequest()) {
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    'You have no permission for this operation'
                )
            );

            $this->_helper->json->sendFailure();
            
            die(); // Zend_Controller_Action_Helper_Json if $suppressExit = true;
        }
        
        $this->_forward('access-denied', 'informer', 'Ecart_Admin');
        return false;
    }
    
    /**
     * Set "actionName to aclResource" assignment
     * 
     * Assignment used for auto-checkPermission in preDispatch method
     * 
     * Use this method in init() method
     * 
     * Example:
     * setActionToAclAssignment(array(
     *  'index' => 'admin/site/view',
     *  'edit'  => 'admin/site/edit'
     * ))
     * 
     * @param array
     */
    public function setActionToAclAssignment($assignment)
    {
        $this->_aclAssignment = $assignment;
    }
    
    /**
     * Redirect to another URL. Adds adminRoute by default to given $url parameter
     *
     * @param string $url
     * @param bool $addAdmin
     * @param array $options Options to be used when redirecting
     * @return void
     */
     //@todo */*/* === referer , */*/otherAction 
    protected function _redirect($url, array $options = array(), $addAdmin = true)
    {
        if ((isset($_SERVER['HTTP_REFERER']) 
             && $url == $_SERVER['HTTP_REFERER']) || !$addAdmin) {

            parent::_redirect($url, $options);
        }
        
        parent::_redirect($this->view->adminUrl . '/' . ltrim($url, '/ '), $options);
    }
}