<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Model_Acl_Role extends Ecart_Db_Table
{
    protected $_name = 'admin_acl_role';
    protected $_rolesTree = null;
        
    private function _initTree()
    {
        if (null !== $this->_rolesTree)
            return;
        $result = $this->getAdapter()->query(
            "SELECT ar.*, arp.role_id as child_id FROM {$this->_name} ar "
          . "LEFT JOIN " . self::$_db_prefix . "admin_acl_role_parent arp ON arp.role_parent_id = ar.id"
        );
        
        $rolesTree = array();

        while (($row = $result->fetch())) {
            if (!isset($rolesTree[$row['id']]))
                $rolesTree[$row['id']] = $row;
            if ($row['child_id']) {
                $rolesTree[$row['id']]['childs'][] = $row['child_id'];
            }
        }
        
        foreach ($rolesTree as $id => $item) {
            if (!isset($item['childs']))
                continue;
            foreach ($item['childs'] as  $child) {
                if (!isset($rolesTree[$child]['parents']))
                    $rolesTree[$child]['parents'] = array();
                $rolesTree[$child]['parents'][] = $id;
            }
        }
        
        $this->_rolesTree = $rolesTree;
    }
    
    public function getRolesTree()
    {
        $this->_initTree();
        return $this->_rolesTree;
    }
    
    public function getAllChilds($id)
    {
        $this->_initTree();
        if (isset($this->_rolesTree[$id]['childs']) && sizeof($this->_rolesTree[$id]['childs'])) {
            $childs = $this->_rolesTree[$id]['childs'];
            $subchilds = array();
            foreach ($childs as $childId) {
                $subchilds = array_merge($subchilds, $this->getAllChilds($childId));
            }
            return array_merge($childs, $subchilds);
        }
        
        return array();
    }
    
    public function getAllParents($id)
    {
        $this->_initTree();
        if (isset($this->_rolesTree[$id]['parents']) && sizeof($this->_rolesTree[$id]['parents'])) {
            $parents = $this->_rolesTree[$id]['parents'];
            $subparents = array();
            foreach ($parents as $parentId) {
                $subparents = array_merge($subparents, $this->getAllParents($parentId));
            }
            return array_merge($parents, $subparents);
        }
        
        return array();
    }
    
    public function getPossibleParents($id)
    {
        $childRoleIds = $this->getAllChilds($id, $this->getRolesTree());
        $possibleParrentRoles = $this->getRolesTree();
        unset($possibleParrentRoles[$id]);
        foreach ($childRoleIds as $id)
            unset($possibleParrentRoles[$id]);
        return $possibleParrentRoles;
    }
    
    public function getParents($id)
    {
        return $this->getAdapter()->fetchCol(
            "SELECT role_parent_id FROM " . self::$_db_prefix . "admin_acl_role_parent WHERE role_id = ?", $id
        );
    }
    
    public function getName($id)
    {
        $this->_initTree();
        return $this->_rolesTree[$id]['role_name'];
    }
    
    public function getRules($roleId)
    {
        return $this->getAdapter()->fetchAll(
            "SELECT resource_id, permission FROM " . self::$_db_prefix . "admin_acl_rule WHERE role_id = ?", $roleId
        );
    }
    
    /**
     * Save parent roles
     * 
     * @param $roleId integer
     * @param $parents array
     */
    public function saveParents($roleId, $parents)
    {
        $db = $this->getAdapter();
        $db->delete(self::$_db_prefix . "admin_acl_role_parent", array(
            $db->quoteInto('role_id = ?', $roleId)
        ));
        foreach ($parents as $parentId) {
            $db->insert(self::$_db_prefix . "admin_acl_role_parent", array(
                'role_id' => $roleId, 
                'role_parent_id' => $parentId
            ));
        }
    }
    
    public function saveRules($roleId, $rules)
    {
        $db = $this->getAdapter();
        /* remove old rules */
        $db->delete(self::$_db_prefix . 'admin_acl_rule', array(
            $db->quoteInto('role_id = ?', $roleId)
        ));

        /* insert allows */
        if (isset($rules['allow'])) {
            foreach ($rules['allow'] as $resourceId) {
                $db->insert(self::$_db_prefix .'admin_acl_rule', array(
                    'role_id' => $roleId,
                    'resource_id' => $resourceId,
                    'permission' => 'allow'
                ));
            }
        }

        /* insert deny */
        if (isset($rules['deny'])) {
            foreach ($rules['deny'] as $resourceId) {
                if ($row = Ecart::single('admin/acl_rule')->find($roleId, $resourceId)->current()) {
                    $row->delete();
                }
                $db->insert(self::$_db_prefix . 'admin_acl_rule', array(
                    'role_id' => $roleId,
                    'resource_id' => $resourceId,
                    'permission' => 'deny'
                ));
            }
        }
    }
}