<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 *
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Ecart_Admin_Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Model_Import_Creloaded extends Ecart_Admin_Model_Import_Abstract
{
    const LANGUAGES = 'languages';
    const CATEGORIES = 'categories';
    const CATEGORIES_DESCRIPTION = 'categories_description';
    const MANUFACTURERS = 'manufacturers';
    const PRODUCTS = 'products';
    const PRODUCTS_TO_CATEGORIES = 'products_to_categories';
    const PRODUCTS_DESCRIPTION = 'products_description';
    const PRODUCTS_ATTRIBUTES = 'products_attributes';
    const PRODUCTS_ATTRIBUTES_DOWNLOAD = 'products_attributes_download';
    const PRODUCTS_EXTRA_FIELDS = 'products_extra_fields';
    const PRODUCTS_NOTIFICATION = 'products_notifications';
    const PRODUCTS_OPTIONS = 'products_options';
    const PRODUCTS_OPTIONS_TEXT = 'products_options_text';
    const PRODUCTS_OPTIONS_VALUES = 'products_options_values';
    const PRODUCTS_OPTIONS_VALUES_TO_PRODUCT_OPTIONS = 'products_options_values_to_products_options';
    const PRODUCTS_TO_PRODUCTS_EXTRA_FIELDS = 'products_to_products_extra_fields';
    const PRODUCTS_XSELL = 'products_xsell';
    const CUSTOMERS = 'customers';
    const CUSTOMERS_BASKET = 'customers_basket';
    const CUSTOMERS_BASKET_ATTRIBUTES = 'customers_basket_attributes';
    const CUSTOMERS_COMMENTS = 'customers_comments';
    const CUSTOMERS_INFO = 'customers_info';
    const CUSTOMERS_WISHLIST = 'customers_wishlist';
    const CUSTOMERS_WISHLIST_ATTRIBUTES = 'customers_wishlist_attributes';
    const ADDRESS_BOOK = 'address_book';
    const ORDERS_STATUS = 'orders_status';
    const ORDERS = 'orders';
    const ORDERS_STATUS_HISTORY = 'orders_status_history';
    const ORDERS_PRODUCTS = 'orders_products';
    const ORDERS_PRODUCTS_ATTRIBUTES = 'orders_products_attributes';
    const ORDERS_TOTAL = 'orders_total';
    
    private $_priority_queue = array(
        'Categories'      => 'category',
        'Manufacturers'   => 'manufacturer',
        //'Tax rates'       => 'taxrate',
        'Product attributes' => 'productAttribute',
        'Product extra fields' => 'productExtraField',
        'Products'        => 'product',
        'Customers'       => 'customer',
        'Orders'          => 'order'
    );
    
    /* oscommerce_type_id => ecart_type_id 
     *
     * @var const array
     */
    private $_option_types = array(
        0 => 0,
        1 => 1,
        2 => 2,
        3 => 3,
        4 => 4
    );
    
    private $_sku = null;

    /**
     *
     * @param array $data
     * @return <type>
     */
    public static function getInstance($data) {
        parent::$__CLASS__ = __CLASS__;
        return parent::getInstance($data);
    }

    public function dispose()
    {
        $this->clearSession();
        parent::dispose();
    }
    
    private function clearSession()
    {
        unset($_SESSION['processed_count']);
        unset($_SESSION['imported_count']);
        unset($_SESSION['category_relations_array']);
        unset($_SESSION['manufacturer_relations_array']);
        unset($_SESSION['option_relations_array']);
        unset($_SESSION['option_value_relations_array']);
        unset($_SESSION['extrafield_relations_array']);
        unset($_SESSION['product_relations_array']);
        unset($_SESSION['customer_relations_array']);
        unset($_SESSION['status_relations_array']);
        unset($_SESSION['message_stack']);
    }
    
    public function getQueue()
    {
        return $this->_priority_queue;
    }
    
    /**
     * @param string $group
     * @param array $language. 
     *      Example: array([1] => [3]) 
     *      /language with id = 3 from oscommerce 
     *      will be written to language 1 in ecart
     * @param integer $site
     * @param integer $primary_language
     * @return array([completed_group] => boolean, [imported] => integer)
     */
    public function import($group, $language, $site, $primary_language)
    {
        $this->_language = $language;
        $this->_site = $site;
        $this->_primary_language = $primary_language;
        $this->_root = parent::$_ecart_adapter->fetchOne(
            "SELECT id 
             FROM " . parent::$_db_prefix . parent::CATEGORY . "
             WHERE lvl = 0
             AND site_id = $this->_site"
        );
        
        if (!isset($_SESSION['processed_count']) || $_SESSION['processed_count'] == 0) {
            $_SESSION['processed_count'] = 0;
            $_SESSION['imported_count'] = 0;
            if (!isset($_SESSION['message_stack']))
                $_SESSION['message_stack'] = array();
            foreach ($_SESSION['message_stack'] as &$stack) {
                $stack = array();
            }
            if ($group == 'category')
                $_SESSION['category_relations_array'] = array(0 => 0);
            else if ($group == 'manufacturer')
                $_SESSION['manufacturer_relations_array'] = array();
            else if ($group == 'productAttribute') {
                $_SESSION['option_relations_array'] = array();
                $_SESSION['option_value_relations_array'] = array();
            } else if ($group == 'productExtraField')
                $_SESSION['extrafield_relations_array'] = array();
            else if ($group == 'product')
                $_SESSION['product_relations_array'] = array();
            else if ($group == 'customer')
                $_SESSION['customer_relations_array'] = array();
            else if ($group == 'order')
                $_SESSION['status_relations_array'] = array();
        }

        $result = $this->importGroup(ucfirst($group));
        if ($result['completed_group']) {
            unset($_SESSION['processed_count']);
            unset($_SESSION['imported_count']);
        }
        
        return $result;
    }
    
    public function getLanguages()
    {
        $query = "SELECT l.*
                  FROM " . parent::$_table_prefix . self::LANGUAGES . " AS l";
        
        return parent::$_adapter->fetchAll($query);
    }
    
    public function importGroup($group)
    {
        $time_start_script = time();
        
        $time_export = 0;
        $completed_group = false;
        
        while (true) {
            $time_start_item = time();

            $entry = call_user_func(array($this, 'get'.$group));
            
            if (!$entry) {
               $completed_group = true;
               break;
            }
            
            call_user_func(array($this, 'add'.$group), $entry);
            
            $time_end_item = time();
            $time_sript = $time_end_item - $time_start_script;
            $time_export_item = max($time_end_item - $time_start_item, $time_export);
            if (($time_sript + $time_export_item) > 20) {
                break;
            }
        }
        
        return array(
            'completed_group' => $completed_group,
            'group' => $group,
            'processed' => $_SESSION['processed_count'],
            'imported' => $_SESSION['imported_count'],
            'messages' => $_SESSION['message_stack']
        );
    }
    
    private function addCategory($entry)
    {
        $date = Ecart_Date::now()->toSQLString();//date("Y-m-d H:i:s");
        $image = "";
        
        if ($entry['category']['categories_image'] != '')
            $image = parent::_copyFile(
                parent::$_image_path . '/import/' . $entry['category']['categories_image'],
                parent::$_image_path . '/category'
            );
        
        $data = array(
            'cat_status' => 'enabled',
            'created_on' => $date,
            'image' => $image
        );
        
        if ($entry['category']['parent_id'] == 0) {
            $insertedId = Ecart::single('catalog/category')->insertItem($data, $this->_root);
        } else {
            $parentId = $_SESSION['category_relations_array'][$entry['category']['parent_id']];
            $insertedId = Ecart::single('catalog/category')->insertItem($data, $parentId);
        }
        
        if (isset($entry['description']) && isset($entry['description'][$this->_primary_language])) {

            $key_word = $this->_prepareString($entry['description'][$this->_primary_language]['categories_name']);
            
            $duplicate = parent::$_ecart_adapter->fetchRow(
                "SELECT hu.* 
                 FROM " .  parent::$_db_prefix . parent::HUMAN_URL . " AS hu 
                 WHERE hu.key_word = '$key_word'
                 AND hu.site_id = $this->_site"
            );
            
            if ($duplicate)
                $key_word .= $insertedId;
            
            $hurl = array (
                'key_word' => $key_word,
                'site_id' => $this->_site,
                'key_type' => 'c',
                'key_id' => $insertedId
            );
            
            parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::HUMAN_URL, $hurl);
            
            $_SESSION['message_stack']['successfully imported'][] = $entry['description'][$this->_primary_language]['categories_name'];
        }
        
        $_SESSION['imported_count']++;
        $_SESSION['processed_count']++;
        $_SESSION['category_relations_array'][$entry['category']['categories_id']] = $insertedId;
        
        foreach (array_keys($this->_language) as $ecart_language) {
            if (isset($entry['description']) && isset($entry['description'][$this->_language[$ecart_language]])) {
                parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::CATEGORY_DESCRIPTION, array(
                    'category_id' => $insertedId,
                    'language_id' => $ecart_language,
                    'name' => $entry['description'][$this->_language[$ecart_language]]['categories_name'],
                    'description' => $entry['description'][$this->_language[$ecart_language]]['categories_description']
                ));
            }
        }
    }
    
    private function addManufacturer($entry)
    {
        $image = "";
        
        if ($entry['manufacturer']['manufacturers_image'] != '')
            $image = parent::_copyFile(
                parent::$_image_path . '/import/' . $entry['manufacturer']['manufacturers_image'],
                parent::$_image_path . '/manufacturer'
            );
        
        $data = array(
            'name' => $entry['manufacturer']['manufacturers_name'],
            'image' => $image
        );
        
        $duplicate = parent::$_ecart_adapter->fetchOne(
            "SELECT m.id 
            FROM " . parent::$_db_prefix . parent::MANUFACTURER . " AS m 
            WHERE m.name = '$data[name]'"
        );
            
        if (!$duplicate) {
            parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::MANUFACTURER, $data);
            $inserted_id = parent::$_ecart_adapter->lastInsertId(parent::$_db_prefix . parent::MANUFACTURER);
            
            //human url
            $key_word = $data['name'];
            
            $duplicate = parent::$_ecart_adapter->fetchRow(
                "SELECT hu.* 
                 FROM " . parent::$_db_prefix . parent::HUMAN_URL ." AS hu 
                 WHERE hu.key_word = '$key_word'
                 AND hu.site_id = $this->_site"
            );
            
            if ($duplicate)
                $key_word .= $inserted_id;
            
            $hurl = array (
                'key_word' => $key_word,
                'site_id' => $this->_site,
                'key_type' => 'm',
                'key_id' => $inserted_id
            );
            
            parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::HUMAN_URL, $hurl);
            
            $_SESSION['imported_count']++;
            $_SESSION['message_stack']['successfully imported'][] = $data['name'];
            
        } else {
            $inserted_id = $duplicate;
            $_SESSION['message_stack']['skipped (duplicate entry)'][] = $data['name'];
        }   
        
        $_SESSION['processed_count']++;
        $_SESSION['manufacturer_relations_array'][$entry['manufacturer']['manufacturers_id']] = $inserted_id;
    }
    
    private function addProduct($entry)
    {
        if (isset($_SESSION['manufacturer_relations_array']) && $entry['product']['manufacturers_id'] != 0) {
            $manufacturer = isset($_SESSION['manufacturer_relations_array'][$entry['product']['manufacturers_id']]) ? 
                $_SESSION['manufacturer_relations_array'][$entry['product']['manufacturers_id']] : 0;
        } else {
            $manufacturer = new Zend_Db_Expr('NULL');
        }
            
        if (isset($_SESSION['taxclass_relations_array'])) {
            $tax_class = new Zend_Db_Expr('NULL'); //@todo import tax rates
        } else { 
            $tax_class = new Zend_Db_Expr('NULL');
        }
        
        $date = Ecart_Date::now()->toPhpFormat()->toString("Y-m-d H:i:s");
        
        $images = array();
        $images = array_filter(array_keys($entry['product']), array($this, "isImage"));
        
        $image_path = array();
        foreach ($images as $column_name) {
            if ($entry['product'][$column_name] != '') {
                $image_path[$column_name] = parent::_copyFile(
                    parent::$_image_path . '/import/' . $entry['product'][$column_name],
                    parent::$_image_path . '/product'
                );
            }
        }
        
        //get overall viewed count
        $viewed = 0;
        foreach ($entry['description'] as $lang_id => $description) {
            $viewed += intval($description['products_viewed']);
        }
        
        //getting sku number
        if (!is_numeric($this->_sku)) {
            $sku_query = "SELECT max(p.id)
                FROM " . parent::$_db_prefix . parent::PRODUCT . " AS p";
            
            $this->_sku = parent::$_ecart_adapter->fetchOne($sku_query);
        }
        
        $product = array(
            'manufacturer_id' => $manufacturer,
            'quantity' => $entry['product']['products_quantity'],
            'sku' => 'CRE Loaloded Store - ' . ++$this->_sku,
            'image_base' => isset($image_path['products_image_med']) ? $image_path['products_image_med'] : '',
            'image_listing' => isset($image_path['products_image']) ? $image_path['products_image'] : '',
            'image_thumbnail' => isset($image_path['products_image']) ? $image_path['products_image'] : '',
            'price' => $entry['product']['products_price'],
            'date_available' => $date,
            'weight' => 'products_weight',
            'is_active' => 1,
            'ordered' => $entry['product']['products_ordered'],
            'created_on' => $date,
            'modified_on' => $date,
            'tax_class_id' => $tax_class,
            'viewed' => $viewed
        );
        
        parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::PRODUCT, $product);
        $product_id = parent::$_ecart_adapter->lastInsertId(parent::$_db_prefix . parent::PRODUCT);
        
        $product_stock = array(
            'product_id' => $product_id,
            'in_stock' => $entry['product']['products_status'],
            'manage' => 1,
            'min_qty' => 1,
            'min_qty_allowed' => 1,
            'max_qty_allowed' => 0,
            'decimal' => 0,
            'notify_qty' => 0,
            'backorder' => 0
        );
        parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::PRODUCT_STOCK, $product_stock);
        
        foreach (array_keys($this->_language) as $ecart_language) {
            if (!isset($entry['description']) || !isset($entry['description'][$this->_language[$ecart_language]]))
                continue;
                
            $head_title = is_null($entry['description'][$this->_language[$ecart_language]]['products_head_title_tag']) ? 
                '' : $entry['description'][$this->_language[$ecart_language]]['products_head_title_tag'];
            $head_description = is_null($entry['description'][$this->_language[$ecart_language]]['products_head_desc_tag']) ? 
                '' : $entry['description'][$this->_language[$ecart_language]]['products_head_desc_tag'];
            $head_keywords = is_null($entry['description'][$this->_language[$ecart_language]]['products_head_keywords_tag']) ? 
                '' : $entry['description'][$this->_language[$ecart_language]]['products_head_keywords_tag'];
            $description = is_null($entry['description'][$this->_language[$ecart_language]]['products_description']) ?
                '' : $entry['description'][$this->_language[$ecart_language]]['products_description'];
                
            //product description
            $description = array(
                'product_id' => $product_id,
                'language_id' => $ecart_language,
                'name' => $entry['description'][$this->_language[$ecart_language]]['products_name'],
                'description' => $description,
                'viewed' => $entry['description'][$this->_language[$ecart_language]]['products_viewed'],
                'image_seo_name' => $this->_prepareString($entry['description'][$this->_language[$ecart_language]]['products_name']),
                'meta_title' => $head_title,
                'meta_description' => $head_description,
                'meta_keyword' => $head_keywords,
                'short_description' => $description
            );
            
            parent::$_ecart_adapter->insert(
                parent::$_db_prefix . parent::PRODUCT_DESCRIPTION, $description
            );
        }
        
        //human url
        $key_word = "";
        if (isset($entry['description'][$this->_primary_language]['products_url']) && $entry['description'][$this->_primary_language]['products_url'] != '') {
            $key_word = $this->_prepareString($entry['description'][$this->_primary_language]['products_url']);
        } else {
            $key_word = $this->_prepareString($entry['description'][$this->_primary_language]['products_name']);
        }
        
        if ($key_word != "") {
            $duplicate = parent::$_ecart_adapter->fetchRow(
                "SELECT hu.* 
                 FROM " . parent::$_db_prefix . parent::HUMAN_URL . " AS hu 
                 WHERE hu.key_word = '$key_word'
                 AND hu.site_id = $this->_site"
            );
            
            if ($duplicate) {
                $key_word .= $product_id;
            }
            
            $hurl = array (
                'key_word' => $key_word,
                'site_id' => $this->_site,
                'key_type' => 'p',
                'key_id' => $product_id
            );
        } else {
            $hurl = array (
                'key_word' => $product['sku'],
                'site_id' => $this->_site,
                'key_type' => 'p',
                'key_id' => $product_id
            );
        }
        
        parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::HUMAN_URL, $hurl);
        
        //additional images
        foreach ($image_path as $path) {
            parent::$_ecart_adapter->insert(
                parent::$_db_prefix . parent::PRODUCT_GALLERY, array(
                    'product_id' => $product_id,
                    'image' => $path
                )
            );
        }
        
        //product_to_category relations
        if (isset($_SESSION['category_relations_array'])) {
            foreach ($entry['categories'] as $category_id) {
                if (isset($_SESSION['category_relations_array'][$category_id]))
                    parent::$_ecart_adapter->insert(
                        parent::$_db_prefix . parent::PRODUCT_TO_CATEGORIES, array(
                            'category_id' =>  $_SESSION['category_relations_array'][$category_id],
                            'product_id' => $product_id
                        )
                    );
            }
        }
        
        //product attributes (modifiers)
        if (isset($_SESSION['option_relations_array']) && isset($_SESSION['option_value_relations_array'])) {
            foreach ($entry['attributes'] as $attribute) {
                
                if (!isset($_SESSION['option_relations_array'][$attribute['options_id']]))
                    continue;
                
                $product_option_value_id = isset($_SESSION['option_value_relations_array'][$attribute['options_values_id']]) ?
                    $_SESSION['option_value_relations_array'][$attribute['options_values_id']] : new Zend_Db_Expr('NULL'); 
                
                parent::$_ecart_adapter->insert(
                    parent::$_db_prefix . parent::PRODUCT_ATTRIBUTE, array(
                        'product_variation_id' => '0',
                        'product_id' => $product_id,
                        'product_option_id' => $_SESSION['option_relations_array'][$attribute['options_id']],
                        'product_option_value_id' => $product_option_value_id,
                        'price' => $attribute['price_prefix'] . $attribute['options_values_price'],
                        'price_type' => 'by',
                        'weight' => '0',
                        'weight_type' => 'to',
                        'modifier' => '1'
                    )
                );
            }
        }
        
        //product attributes (properties)
        if (isset($_SESSION['extrafield_relations_array'])) {
            foreach ($entry['extra_fields'] as $field) {
                
                if (!isset($_SESSION['extrafield_relations_array'][$field['products_extra_fields_id']])) {
                    continue;
                }
                
                parent::$_ecart_adapter->insert(
                    parent::$_db_prefix . parent::PRODUCT_ATTRIBUTE, array(
                        'product_variation_id' => '0',
                        'product_id' => $product_id,
                        'product_option_id' => $_SESSION['extrafield_relations_array'][$field['products_extra_fields_id']],
                        'product_option_value_id' => new Zend_Db_Expr('NULL'),
                        'price' => '0',
                        'price_type' => 'by',
                        'weight' => '0',
                        'weight_type' => 'to',
                        'modifier' => '0'
                    )
                );
                $attribute_id = parent::$_ecart_adapter->lastInsertId(parent::PRODUCT_ATTRIBUTE);
                
                foreach (array_keys($this->_language) as $ecart_language) {
                    parent::$_ecart_adapter->insert(
                        parent::PRODUCT_ATTRIBUTE_VALUE, array(
                            'product_attribute_id' => $attribute_id,
                            'language_id' => $ecart_language,
                            'attribute_value' => $field['products_extra_fields_value']
                        )
                    );
                }
            }
        }
        
        $_SESSION['imported_count']++;
        $_SESSION['processed_count']++;
        $_SESSION['product_relations_array'][$entry['product']['products_id']] = $product_id;
        $_SESSION['message_stack']['successfully imported'][] = $key_word;
    }
    
    private function addCustomer($entry)
    {
        $date = Ecart_Date::now()->toPhpFormat()->toString("Y-m-d H:i:s");
        
        $duplicate = parent::$_ecart_adapter->fetchOne(
            "SELECT c.id 
            FROM " . parent::$_db_prefix . parent::CUSTOMER . " AS c
            WHERE c.email = ?", 
            $entry['customer']['customers_email_address']
        );
        
        if (!$duplicate) {
            $customer = array(
                'email' => $entry['customer']['customers_email_address'],
                'password' => $entry['customer']['customers_password'],
                'site_id' => $this->_site,
                'is_active' => 1,
                'default_shipping_address_id' => '',
                'default_billing_address_id' => '',
                'created_at' => $date,
                'modified_at' => $date,
                'group_id' => 1
            );
            parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::CUSTOMER, $customer);
            $customer_id = parent::$_ecart_adapter->lastInsertId(parent::$_db_prefix . parent::CUSTOMER);
            
            $_SESSION['imported_count']++;
            $_SESSION['customer_relations_array'][$entry['customer']['customers_id']] = $customer_id;
            $_SESSION['message_stack']['successfully imported'][] = $entry['customer']['customers_email_address'];
            
            $address_relations_array = array();
            
            foreach ($entry['addresses'] as $address) {
                $data = array(
                    'customer_id' => $customer_id,
                    'gender' => $address['entry_gender'],
                    'company' => $address['entry_company'],
                    'phone' => $address['entry_telephone'],
                    'fax' => $address['entry_fax'],
                    'firstname' => $address['entry_firstname'],
                    'lastname' => $address['entry_lastname'],
                    'street_address' => $address['entry_street_address'],
                    'suburb' => $address['entry_suburb'],
                    'postcode' => $address['entry_postcode'],
                    'city' => $address['entry_city'],
                    'state'=> $address['entry_state'],
                    'country_id' => $address['entry_country_id'],
                    'zone_id' => $address['entry_zone_id']
                );
                parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::CUSTOMER_ADDRESS, $data);
                $address_id = parent::$_ecart_adapter->lastInsertId(parent::$_db_prefix . parent::CUSTOMER_ADDRESS);
                
                $address_relations_array[$address['address_book_id']] = $address_id;
            }
            
            parent::$_ecart_adapter->update(parent::$_db_prefix . parent::CUSTOMER, array(
                'default_shipping_address_id' => $address_relations_array[$entry['customer']['customers_default_address_id']],
                'default_billing_address_id'  => $address_relations_array[$entry['customer']['customers_default_address_id']]
            ));
            
        } else {
            if (isset($_SESSION['customer_relations_array'][$entry['customer']['customers_id']]))
                unset($_SESSION['customer_relations_array'][$entry['customer']['customers_id']]);
            $_SESSION['message_stack']['skipped (duplicate entry)'][] = $entry['customer']['customers_email_address'];
        }
        
        $_SESSION['processed_count']++;
    }
    
    private function addProductAttribute($entry)
    {
        $valueset_id = 0;
        
        $_SESSION['processed_count']++;
        
        if (!isset($entry['option_text'][$this->_primary_language]['products_options_name']) || 
            $entry['option_text'][$this->_primary_language]['products_options_name'] == '') {
            $_SESSION['message_stack']['skipped (attribute code is undefined)'][] = 'CreLoaded option id: ' . $entry['option']['products_options_id'];
            return;
        }
        
        // create valueset from option_values
        if (count($entry['option_values'])) {
            
            $duplicate_valueset = parent::$_ecart_adapter->fetchOne(
                "SELECT pov.id
                FROM " . parent::$_db_prefix . parent::PRODUCT_OPTION_VALUESET . " AS pov
                WHERE pov.name = ? ", $entry['option_text'][$this->_primary_language]['products_options_name']
            );
            
            if (!$duplicate_valueset) {
                parent::$_ecart_adapter->insert(
                    parent::$_db_prefix . parent::PRODUCT_OPTION_VALUESET, array(
                        'name' => $entry['option_text'][$this->_primary_language]['products_options_name']
                    )
                );
                $valueset_id = parent::$_ecart_adapter->lastInsertId(parent::$_db_prefix . parent::PRODUCT_OPTION_VALUESET);
            } else {
                $valueset_id = $duplicate_valueset;
            }
            
            foreach (array_keys($this->_language) as $ecart_language) {
                $i = 0;
                foreach ($entry['option_values'][$this->_language[$ecart_language]] as $value) {
                    
                    $duplicate_option_value = parent::$_ecart_adapter->fetchOne(
                        "SELECT povt.product_option_value_id
                        FROM " . parent::$_db_prefix . parent::PRODUCT_OPTION_VALUE_TEXT . " AS povt
                        WHERE povt.name = ?",
                        $entry['option_values'][$this->_primary_language][$i++]['products_options_values_name']
                    );
                    
                    if (!$duplicate_option_value) {
                        //inserting product_option_value
                        parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::PRODUCT_OPTION_VALUE, array(
                            'sort_order' => '0',
                            'valueset_id' => $valueset_id
                        ));
                        $option_value_id = parent::$_ecart_adapter->lastInsertId(parent::$_db_prefix . parent::PRODUCT_OPTION_VALUE);
                    }  else {
                        $option_value_id = $duplicate_option_value;
                    }
                    
                    $name = $value['products_options_values_name'];
                    
                    $duplicate_option_value_text = parent::$_ecart_adapter->fetchOne(
                        "SELECT povt.product_option_value_id
                        FROM " . parent::$_db_prefix . parent::PRODUCT_OPTION_VALUE_TEXT . " AS povt
                        WHERE povt.language_id = $ecart_language
                        AND povt.product_option_value_id = $option_value_id"
                    );
                    
                    if (!$duplicate_option_value_text) {
                        //inserting product_option_value_text
                        parent::$_ecart_adapter->insert(
                            parent::$_db_prefix . parent::PRODUCT_OPTION_VALUE_TEXT, array(
                                'product_option_value_id' => $option_value_id,
                                'language_id' => $ecart_language,
                                'name' => isset($name) ? $name : ''
                            )
                        );
                    }
                    
                    $_SESSION['option_value_relations_array'][$value['products_options_values_id']] = $option_value_id;
                }
            }
        }
        
        $duplicate_option = parent::$_ecart_adapter->fetchRow(
            "SELECT po.id, po.valueset_id
            FROM " . parent::$_db_prefix . parent::PRODUCT_OPTION . " AS po
            WHERE po.code = ?", 
            $this->_prepareString($entry['option_text'][$this->_primary_language]['products_options_name'], '_')
        );
        
        //if same option exist and its valueset == our valueset => using it, else create new;
        if (!$duplicate_option || ($duplicate_option['valueset_id'] != $valueset_id)) {
            
            $code = $this->_prepareString($entry['option_text'][$this->_primary_language]['products_options_name'], '_');
            
            if ($duplicate_option)
                $code .= $valueset_id;
                
            //insert product_option
            parent::$_ecart_adapter->insert(
                parent::$_db_prefix . parent::PRODUCT_OPTION, array(
                    'code' => $code,
                    'option_type' => $this->_option_types[$entry['option']['options_type']],
                    'sort_order'  => $entry['option']['products_options_sort_order'],
                    'searchable'  => 1,
                    'comparable'  => 1,
                    'languagable' => 1,
                    'filterable'  => 1,
                    'valueset_id' => $valueset_id
                )
            );
            $option_id = parent::$_ecart_adapter->lastInsertId(parent::$_db_prefix . parent::PRODUCT_OPTION);
            
            //insert product_option_text
            foreach(array_keys($this->_language) as $ecart_language) {
                parent::$_ecart_adapter->insert(
                    parent::$_db_prefix . parent::PRODUCT_OPTION_TEXT, array(
                        'product_option_id' => $option_id,
                        'language_id' => $ecart_language,
                        'name' => isset($entry['option_text'][$this->_language[$ecart_language]]) ? $entry['option_text'][$this->_language[$ecart_language]]['products_options_name'] : '',
                        'instruct' => isset($entry['option_text'][$this->_language[$ecart_language]]) ? $entry['option_text'][$this->_language[$ecart_language]]['products_options_instruct'] : ''
                    )
                );
            }
            
            $_SESSION['imported_count']++;
            $_SESSION['message_stack']['successfully imported'][] = $entry['option_text'][$this->_primary_language]['products_options_name'];
            
        } else {
            $option_id = $duplicate_option['id'];
            $_SESSION['message_stack']['skipped (duplicate entry)'][] = $entry['option_text'][$this->_primary_language]['products_options_name'];
        } 
        
        $_SESSION['option_relations_array'][$entry['option']['products_options_id']] = $option_id;
    }
    
    private function addProductExtraField($entry)
    {
        $code = $this->_prepareString($entry['extra_field']['products_extra_fields_name'], '_');
        
        $duplicate_option = parent::$_ecart_adapter->fetchOne(
            "SELECT po.id
            FROM " . parent::$_db_prefix . parent::PRODUCT_OPTION . " AS po
            WHERE po.code = ? ", $code
        );
        
        if ($duplicate_option) {
            $option_id = $duplicate_option;
            $_SESSION['message_stack']['skipped (duplicate entry)'][] = $entry['extra_field']['products_extra_fields_name'];
        } else {
            parent::$_ecart_adapter->insert(
                parent::$_db_prefix . parent::PRODUCT_OPTION, array(
                    'code' => $code,
                    'option_type' => 4,
                    'sort_order'  => 5,
                    'searchable'  => 1,
                    'comparable'  => 1,
                    'languagable' => 0,
                    'filterable'  => 0,
                    'valueset_id' => 0
                )
            );
            $option_id = parent::$_ecart_adapter->lastInsertId(parent::$_db_prefix . parent::PRODUCT_OPTION);
            
            //insert product_option_text
            foreach(array_keys($this->_language) as $ecart_language) {
                parent::$_ecart_adapter->insert(
                    parent::$_db_prefix . parent::PRODUCT_OPTION_TEXT, array(
                        'product_option_id' => $option_id,
                        'language_id' => $ecart_language,
                        'name' => $entry['extra_field']['products_extra_fields_name'],
                        'instruct' => ''
                    )
                );
            }
            
            $_SESSION['imported_count']++;
            $_SESSION['message_stack']['successfully imported'][] = $entry['extra_field']['products_extra_fields_name'];
        }
        
        $_SESSION['extrafield_relations_array'][$entry['extra_field']['products_extra_fields_id']] = $option_id;
        $_SESSION['processed_count']++;
    }
    
    private function addOrder($entry)
    {
        $_SESSION['processed_count']++;

        if (!isset($_SESSION['customer_relations_array'][$entry['order']['customers_id']])) {
            $_SESSION['message_stack']['skipped (customer not found)'][] = $entry['order']['customers_name'] . ' ' . $entry['order']['customers_email_address'];
            return;
        }
        
        /**
         * array(
         *  [0] => array(
         *    [lang_id] => array(values)
         *  ), 
         *  [1] => ...
         * )
         */
        foreach ($entry['statuses'] as $id => $status) {
            if (!isset($status[$this->_primary_language]) || $status[$this->_primary_language] == '') {
                $current_status = current($status);
                $_SESSION['message_stack']['skipped (status code is undefined)'][] = 'CreLoaded status id: ' . $id;
                continue;
            }
            
            if (!isset($_SESSION['status_relations_array'][$id])) {
                $duplicate = parent::$_ecart_adapter->fetchOne(
                    "SELECT os.id
                    FROM " . parent::$_db_prefix . parent::ORDER_STATUS . " AS os
                    WHERE name = ?",
                    $status[$this->_primary_language]
                );
    
                if (!$duplicate) { //insert new status
                    $status_id = Ecart::single('sales/order_status')->insert(array(
                        'name' => $status[$this->_primary_language],
                        'system' => false
                    ));;
                    foreach(array_keys($this->_language) as $ecart_language) {
                        Ecart::single('sales/order_status_text')->insert(array(
                            'status_id' => $status_id,
                            'language_id' => $ecart_language,
                            'status_name' => $status[$this->_language[$ecart_language]]
                        ));
                    }
                } else { //use existing status
                    $status_id = $duplicate;
                    $_SESSION['message_stack']['skipped statuses (duplicate entry)'][] = $status[$this->_primary_language];
                }
                
                $_SESSION['status_relations_array'][$id] = $status_id;
            }
        }
        
        $total = 0;
        $tax = new Zend_Db_Expr('NULL');
        //get order total
        foreach ($entry['total'] as $key => $line) {
            if ($line['title'] == 'Total:')
                $total = $line['value'];
            if ($line['title'] == 'Tax:')
                $tax = $line['value'];
        }
        
        $order = array(
            'customer_id' => $_SESSION['customer_relations_array'][$entry['order']['customers_id']],
            'customer_email' => $entry['order']['customers_email_address'],
            'delivery_firstname' => $entry['order']['delivery_name'],
            'delivery_lastname' => $entry['order']['delivery_name'],
            'delivery_phone' => '',
            'delivery_fax' => '',
            'delivery_company' => $entry['order']['delivery_company'],
            'delivery_street_address' => $entry['order']['delivery_street_address'],
            'delivery_suburb' => $entry['order']['delivery_suburb'],
            'delivery_city' => $entry['order']['delivery_city'],
            'delivery_postcode' => $entry['order']['delivery_postcode'],
            'delivery_state' => $entry['order']['delivery_state'],
            'delivery_country' => $entry['order']['delivery_country'],
            'delivery_address_format_id' => $entry['order']['delivery_address_format_id'],
            'billing_firstname' => $entry['order']['billing_name'],
            'billing_lastname' => $entry['order']['billing_name'],
            'billing_phone' => '',
            'billing_fax' => '',
            'billing_company' => $entry['order']['billing_company'],
            'billing_street_address' => $entry['order']['billing_street_address'],
            'billing_suburb' => $entry['order']['billing_suburb'],
            'billing_city' => $entry['order']['billing_city'],
            'billing_postcode' => $entry['order']['billing_postcode'],
            'billing_state' => $entry['order']['billing_state'],
            'billing_country' => $entry['order']['billing_country'],
            'billing_address_format_id' => $entry['order']['billing_address_format_id'],
            'payment_method' => $entry['order']['payment_method'],
            'payment_method_code' => $this->_prepareString($entry['order']['payment_method'], '', true),
            'shipping_method' => new Zend_Db_Expr('NULL'),
            'shipping_method_code' => new Zend_Db_Expr('NULL'),
            'coupon_code' => new Zend_Db_Expr('NULL'),
            //'cc_type' => $entry['order']['cc_type'],
            //'cc_owner' => $entry['order']['cc_owner'],
            //'cc_number' => $entry['order']['cc_number'],
            //'cc_expires' => $entry['order']['cc_expires'],
            //'cc_cvv' => $entry['order']['cc_ccv'],
            'date_modified_on' => new Zend_Db_Expr('NULL'),
            'date_purchased_on' => $entry['order']['date_purchased'],
            'date_finished_on' => new Zend_Db_Expr('NULL'),
            'order_status_id' => $_SESSION['status_relations_array'][$entry['order']['orders_status']],
            'currency' => $entry['order']['currency'],
            'currency_rate' => $entry['order']['currency_value'],
            'order_total' => $total,
            'txn_id' => 0,
            'ip_address' => $entry['order']['ipaddy'],
            'site_id' => $this->_site
        );
        
        $order_id = Ecart::single('sales/order')->insert($order);
        
        foreach ($entry['total'] as $line) {
            if ($line['title'] == 'Total:')
                continue;
            $order_total = array(
                'order_id' => $order_id,
                'title' => substr($line['title'], 0, -1),
                'value' => $line['value']
            );
            Ecart::single('sales/order_total')->insert($order_total);
        }
        
        foreach ($entry['status_history'] as $date => $status) {
            $history_data = array(
                'order_id' => $order_id,
                'order_status_id' => $_SESSION['status_relations_array'][$status['orders_status_id']],
                'created_on' => $date,
                'notified' => $status['customer_notified'],
                'comments' => $status['comments']
            );
            Ecart::single('sales/order_status_history')->insert($history_data);
        }
        
        foreach ($entry['products'] as $product) {
            
            //import product if not imported yet
            if (!isset($_SESSION['product_relations_array'][$product['product']['products_id']])) {  
                $entry = $this->getProduct($product['product']['products_id']);
                $this->addProduct($entry);
            }
            
            $product_id = $_SESSION['product_relations_array'][$product['product']['products_id']];
            
            //get product sku
            $sku = Ecart::single('catalog/product')->getSkuById($product_id);
                
            //inserting product
            $product_data = array(
                'order_id' => $order_id,
                'product_id' => $product_id,
                'product_variation_id' => new Zend_Db_Expr('NULL'),
                'product_sku' => $sku,
                'product_name' => $product['product']['products_name'],
                'product_price' => $product['product']['products_price'],
                'final_price' => $product['product']['final_price'],
                'product_tax' => $product['product']['products_tax'],
                'product_quantity' => $product['product']['products_quantity'],
                'backorder' => 0
            );

            $order_product_id = Ecart::single('sales/order_product')->insert($product_data);
            
            if (!count($product['attributes']))
                continue;
                
            //inserting attributes
            foreach ($product['attributes'] as $attribute) {
                $attribute_data = array(
                    'order_product_id' => $order_product_id,
                    'product_option' => $attribute['products_options'],
                    'product_option_value' => $attribute['products_options_values']
                );
                
                parent::$_ecart_adapter->insert(parent::$_db_prefix . parent::ORDER_PRODUCT_ATTRIBUTE, $attribute_data);
            }
        }
        
        $_SESSION['imported_count']++;
    }
    
    /**
     * Get category row, according to $_SESSION['category_relations_array']
     *
     * @return array(category, description) if found, false - if not
     */
    private function getCategory()
    {
        $category = "SELECT c.*
            FROM " . parent::$_table_prefix . self::CATEGORIES . " AS c
            WHERE c.parent_id IN (" . implode(',', array_keys($_SESSION['category_relations_array'])) . ")
            AND   c.categories_id NOT IN (" . implode(',', array_keys($_SESSION['category_relations_array'])) . ")
            ORDER BY c.parent_id ASC, c.categories_id ASC
            LIMIT 1";
                  
        $category = parent::$_adapter->fetchRow($category);
        
        if (!$category) 
            return false;
            
        $description = "SELECT cd.language_id, cd.*
            FROM " . parent::$_table_prefix . self::CATEGORIES_DESCRIPTION . " AS cd
            WHERE cd.categories_id = $category[categories_id]
            AND cd.language_id IN (" . implode(',', array_values($this->_language)) . ")";
    
        $description = parent::$_adapter->fetchAssoc($description);
        
        return array(
            'category' => $category,
            'description' => $description
        );
    }
    
    /**
     * Get manufacturer row, according to $_SESSION[processed_count]
     *
     * @return array(manufacturer)
     */
    private function getManufacturer()
    {
        $manufacturer = "SELECT m.*
                  FROM " . parent::$_table_prefix . self::MANUFACTURERS . " AS m
                  ORDER BY m.manufacturers_id ASC
                  LIMIT $_SESSION[processed_count], 1";
                  
        $manufacturer = parent::$_adapter->fetchRow($manufacturer);
        
        if (!$manufacturer)
            return false;
            
        return array(
            'manufacturer' => $manufacturer
        );
    }
    
    /**
     * Get product row, according to $_SESSION[processed_count]
     *
     * @return array(product, attributes, description) if found, otherwise - false
     */
    private function getProduct($product_id = null)
    {
        if (null === $product_id) {
            $product = "SELECT p.*
                FROM " . parent::$_table_prefix . self::PRODUCTS . " AS p
                ORDER BY p.products_id ASC
                LIMIT $_SESSION[processed_count], 1";
        } else {
            $product = "SELECT p.*
                FROM " . parent::$_table_prefix . self::PRODUCTS . " AS p
                WHERE p.products_id = $product_id";
        }
            
        $product = parent::$_adapter->fetchRow($product);
        
        if (!$product) 
            return false;
            
        $description = "SELECT pd.language_id, pd.*
            FROM " . parent::$_table_prefix . self::PRODUCTS_DESCRIPTION . " AS pd
            WHERE pd.products_id = $product[products_id]
            AND pd.language_id IN (" . implode(',', array_values($this->_language)) . ")";
    
        $description = parent::$_adapter->fetchAssoc($description);
        
        $attributes = "SELECT pa.*
            FROM " . parent::$_table_prefix . self::PRODUCTS_ATTRIBUTES . " AS pa
            WHERE pa.products_id = $product[products_id]";
            
        $attributes = parent::$_adapter->fetchAll($attributes);
        
        $categories = "SELECT ptc.categories_id
            FROM " . parent::$_table_prefix . self::PRODUCTS_TO_CATEGORIES . " AS ptc
            WHERE ptc.products_id = $product[products_id]";
            
        $categories = parent::$_adapter->fetchCol($categories);
        
        $extra_field = "SELECT ptpef.*
            FROM " . parent::$_table_prefix . self::PRODUCTS_TO_PRODUCTS_EXTRA_FIELDS . " AS ptpef 
            WHERE ptpef.products_id = $product[products_id]";
            
        $extra_field = parent::$_adapter->fetchAll($extra_field);
            
        return array(
            'product' => $product,
            'description' => $description,
            'attributes' => $attributes,
            'categories' => $categories,
            'extra_fields' => $extra_field
        );
    }
    
    private function getProductAttribute()
    {
        $option = "SELECT po.*
            FROM " . parent::$_table_prefix . self::PRODUCTS_OPTIONS . " AS po
            LIMIT $_SESSION[processed_count], 1";
            
        $option = parent::$_adapter->fetchRow($option);
            
        if (!$option)
            return false;
            
        $option_text = "SELECT pot.language_id, pot.*
            FROM " . parent::$_table_prefix . self::PRODUCTS_OPTIONS_TEXT . " AS pot
            WHERE pot.products_options_text_id = $option[products_options_id]
            AND pot.language_id IN (" . implode(',', array_values($this->_language)) . ")";
            
        $option_text = parent::$_adapter->fetchAssoc($option_text);
            
        $option_value = "SELECT pov.language_id, pov.*
            FROM " . parent::$_table_prefix . self::PRODUCTS_OPTIONS_VALUES . " AS pov
            INNER JOIN " . parent::$_table_prefix . self::PRODUCTS_OPTIONS_VALUES_TO_PRODUCT_OPTIONS . " 
                AS povtpo ON povtpo.products_options_values_id = pov.products_options_values_id
            WHERE povtpo.products_options_id = $option[products_options_id]
            AND pov.language_id IN (" . implode(',', array_values($this->_language)) . ")";

        $option_value = parent::$_adapter->fetchAll($option_value);
        
        $option_values = array();
        
        foreach ($option_value as $value) {
            $option_values[$value['language_id']][] = $value;
        }
        
        return array(
            'option' => $option,
            'option_text' => $option_text,
            'option_values' => $option_values
        );
    }
    
    private function getProductExtraField()
    {
        $extra_field = "SELECT pef.*
            FROM " . parent::$_table_prefix . self::PRODUCTS_EXTRA_FIELDS . " AS pef
            LIMIT $_SESSION[processed_count], 1";
            
        $extra_field = parent::$_adapter->fetchRow($extra_field);
        
        if (!$extra_field)
            return false;
            
        return array(
            'extra_field' => $extra_field
        );
    }
    
    private function getCustomer()
    {
        $customer = "SELECT c.* 
            FROM " . parent::$_table_prefix . self::CUSTOMERS . " AS c
            LIMIT $_SESSION[processed_count], 1";
            
        $customer = parent::$_adapter->fetchRow($customer);
        
        if (!$customer)
            return false;
            
        $addresses = "SELECT ab.address_book_id, ab.*
            FROM " . parent::$_table_prefix . self::ADDRESS_BOOK . " AS ab
            WHERE ab.customers_id = $customer[customers_id]";
            
        $addresses = parent::$_adapter->fetchAssoc($addresses);
               
        return array(
            'customer' => $customer,
            'addresses' => $addresses
        );
    }
    
    private function getOrder()
    {
        $order = "SELECT o.* 
            FROM " . parent::$_table_prefix . self::ORDERS . " AS o
            LIMIT $_SESSION[processed_count], 1";
            
        $order = parent::$_adapter->fetchRow($order);
        
        if (!$order)
            return false;
            
        $status_history = "SELECT osh.date_added, osh.*
            FROM " . parent::$_table_prefix . self::ORDERS_STATUS_HISTORY . " AS osh
            WHERE osh.orders_id = $order[orders_id]
            ORDER BY osh.date_added DESC";
            
        $status_history = parent::$_adapter->fetchAssoc($status_history);
        
        $status_ids = array();
        
        foreach ($status_history as $status) {
            $status_ids[] = $status['orders_status_id'];
        }
        
        $status = "SELECT os.*
            FROM " . parent::$_table_prefix . self::ORDERS_STATUS . " AS os
            WHERE os.orders_status_id IN (" . implode(',', $status_ids) . ')';
            
        $status = parent::$_adapter->fetchAll($status);
        
        $statuses = array();
        
        foreach ($status as $row) {
            $statuses[$row['orders_status_id']][$row['language_id']] = $row['orders_status_name'];
        }
            
        $products = "SELECT op.*
            FROM " . parent::$_table_prefix . self::ORDERS_PRODUCTS . " AS op
            WHERE op.orders_id = $order[orders_id]";

        $products = parent::$_adapter->fetchAll($products);
        
        $result = array();
        
        foreach($products as $key => $values) {
            
            $attributes = "SELECT opa.*
                FROM " . parent::$_table_prefix . self::ORDERS_PRODUCTS_ATTRIBUTES . " AS opa
                WHERE opa.orders_products_id = $values[orders_products_id]";
                
            $attributes = parent::$_adapter->fetchAll($attributes);
            
            $result[$values['orders_products_id']]['product'] = $values;
            $result[$values['orders_products_id']]['attributes'] = $attributes;
            
        }
        
        $total = "SELECT ot.*
            FROM " . parent::$_table_prefix . self::ORDERS_TOTAL . " AS ot
            WHERE ot.orders_id = $order[orders_id]";
            
        $total = parent::$_adapter->fetchAll($total);
        
        /*error_log(Zend_Debug::dump(array(
            'order' => $order,
            'total' => $total,
            'statuses' => $statuses,
            'status_history' => $status_history,
            'products' => $result
        )));*/
            
        return array(
            'order' => $order,
            'total' => $total,
            'statuses' => $statuses,
            'status_history' => $status_history,
            'products' => $result
        );
    }

    private function isImage($var)
    {
        return false === strpos('products_image', $var) ? false : true;
    }
}