<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Catalog_CategoryController extends Ecart_Admin_Controller_Back
{
    public function init()
    {
        parent::init();
        $this->_helper->layout->disableLayout();
    }
    
    public function getItemsAction()
    {
        $node = $this->_getParam('node', 0);
        
        $items = Ecart::single('catalog/category')->getChildItems($node);
        $i = 0;
        
        $result = array();
        if ($node) { // load child nodes of $node
            $productModel = Ecart::single('catalog/product');
            foreach ($items as $item) {
                $count = $productModel->countByCategoryId($item['id']);
                $result[$i] = array(
                    'text' => $item['name'] . ' ['.$count.']',
                    'id' => $item['id'],
                    'name' => $item['name'],
                    'cls' => $item['cat_status'] == 'enabled' ?
                        '' : 'disabledNode',
                    'leaf' => false,
                    'site_id' => $item['site_id']
                );
                if ($item['rgt'] - $item['lft'] == 1) {
                    $result[$i]['children'] = array();
                    $result[$i]['expanded'] = true;
                }
                $i++;
            }
        } else { // Load root nodes(site names)
            foreach ($items as $item) {
                $result[$i] = array(
                    'id'      => $item['id'],
                    'text'    => $item['name'],
                    'site_node' => true,
                    'leaf'    => false,
                    'site_id' => $item['site_id']
                );
                $i++;
            }
        }
        $this->_helper->json->sendJson($result, false, false);
    }
    
    public function getDataAction()
    {
        $catId = $this->_getParam('catId');
        $cat = Ecart::single('catalog/category')->getInfoWithKeyWord($catId);
            
        $catDesc = array();
        
        $rows = Ecart::single('catalog/category_description')
            ->fetchAll("category_id = {$catId}")->toArray();
        
        $result = array();
        $result['id'] = $cat['id'];
        $result['catKeyWord'] = $cat['key_word'];
        $result['catStatus'] = $cat['cat_status'];
        
        foreach ($rows as $row) {
            foreach ($row as $label => $value) {
                $result[$label . '_' . $row['language_id']] = $value;
            }
        }
        
        $this->_helper->json->sendJson(array($result), false, false);
    }

    /**
     * Save product category
     */
    public function saveAction()
    {
        $success = true;

        $categoryModel = Ecart::single('catalog/category');
        /* @var $categoryModel Ecart_Catalog_Model_Category */

        $parentId = $this->_getParam('pId', 0);
        $categoryId = $this->_getParam('catId', 0);
        $catKeyWord = $this->_getParam('catKeyWord');
        $siteId = $this->_getParam('siteId');
        
        /* if human url already exist */
        if (!empty($catKeyWord) && 
            Ecart::single('catalog/hurl')
                ->hasDuplicate($catKeyWord, $siteId, $categoryId)) {

            Ecart::message()->addError(
                Ecart::translate('catalog')->__(
                    'Duplicate entry (url)'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        
        /* Save category */
        $data = array(
            'cat_status' => $this->_getParam('catStatus', 'enabled')
        );
                
        if ($categoryId) {
            $data['modified_on'] = Ecart_Date::now()->toZendDbExpr();
            $categoryModel->update($data, array(
                $this->db->quoteInto('id = ?', $categoryId)
            ));
            Ecart::dispatch('catalog_category_update_success', array(
                'category_id' => $categoryId,
                'data' => $data
            ));
        } else {
            $data['created_on'] = Ecart_Date::now()->toZendDbExpr();
            $categoryId = $categoryModel->insertItem($data, $parentId);
            Ecart::dispatch('catalog_category_add_success', array(
                'category_id' => $categoryId,
                'data' => $data
            ));
        }
        
        if (!$categoryId) {
            Ecart::message()->addError('Unable to save category');
            return $this->_helper->json->sendFailure();
        }
        /* Save category description */
        $categoryName            = $this->_getParam('catName');
        $categoryDescription     = $this->_getParam('catDesc');
        $metaTitle         = $this->_getParam('meta_title');
        $metaDescription   = $this->_getParam('meta_description');
        $metaKeyword       = $this->_getParam('meta_keyword');
        
        foreach (array_keys(Ecart_Collect_Language::collect()) as $languageId) {
            if (!isset($categoryName[$languageId])) {
                continue;
            }
            $row = Ecart::single('catalog/category_description')->save(array(
                'category_id' => $categoryId,
                'language_id' => $languageId,
                'name' => $categoryName[$languageId],
                'description' => $categoryDescription[$languageId],
                'meta_title' => $metaTitle[$languageId],
                'meta_description' => $metaDescription[$languageId],
                'meta_keyword' => $metaKeyword[$languageId]
            ));
            if (!$row) {
                return $this->_helper->json->sendFailure();
            }
        }
        
        /* Save Human Url */
        
        Ecart::single('catalog/hurl')->delete(array(
            $this->db->quoteInto('key_id = ?', $categoryId),
            $this->db->quoteInto('key_type = ?', 'c')
        ));

        
        if (!empty($catKeyWord)) {
            $hurlId =  Ecart::single('catalog/hurl')->insert(array(
                'key_word' => $catKeyWord,
                'key_type' => 'c',
                'site_id' => $siteId,
                'key_id'   => $categoryId
            ));
            if (!$hurlId) {
                return $this->_helper->json->sendFailure();
            }
        }
        Ecart::message()->addSuccess(
            Ecart::translate('catalog')->__(
                'Category successfully saved'
            )
        );
        $this->_helper->json->sendSuccess(array(
            'id' => $categoryId
        ));
    }
    
    public function deleteAction()
    {
        $categoryId = $this->_getParam('catId');
        Ecart::single('catalog/category')->deleteItem($categoryId);
        Ecart::dispatch('catalog_category_remove_success', array(
            'category_id' => $categoryId
        ));
        $this->_helper->json->sendSuccess(array(
            'status' => 'ok'
        ));
    }
    
    public function moveAction()
    {
        $moveType = $this->_getParam('moveType');
        $newParentId = $this->_getParam('newParentId');
        $categoryId = $this->_getParam('catId');
        
        $nsTreeTable = new Ecart_NSTree_Table();
        
        switch ($moveType) {
            case 'moveTo':
                $success = $nsTreeTable
                    ->replaceNode($categoryId, $newParentId);
                break;
            case 'moveBefore':
                $success = $nsTreeTable
                    ->replaceBefore($categoryId, $newParentId);
                break;
        }
        
        $this->_helper->json->sendJson(array(
            'success' => $success
        ));
    }
}