<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Catalog_ImageController extends Ecart_Admin_Controller_Back
{    
    private function _saveImage($src, $fileName)
    {
        $fileName = str_replace(' ', '_', $fileName);
        $imagesPath = $savePath = Ecart::config()->system->path
            . '/media/product';
        if (!is_readable($savePath)) {
            mkdir($savePath, 0777);
        } elseif(!is_writable($savePath)) {
            chmod($savePath, 0777);
        }
        for ($i = 0; $i < 2; $i++) {
            if (!preg_match('/[a-zA-Z0-9]{1}/', $fileName[$i])) {
                break;
            }
            
            $savePath .= '/' . strtolower($fileName[$i]);
            if (!file_exists($savePath)) {
                @mkdir($savePath);
            }
        }
        $savePath .= '/' . $fileName;
        @move_uploaded_file($src, $savePath);
        return substr($savePath, strlen($imagesPath));
    }
    
    public function saveImageAction()
    {
        $this->_helper->layout->disableLayout();
        $saved = array();
        foreach (array_keys($_FILES['images']['name']) as $file) {
            if (!is_uploaded_file($_FILES['images']['tmp_name'][$file])) {
                continue;
            }
            $name = preg_replace('/[^a-zA-Z0-9\.]/', '_', $_FILES['images']['name'][$file]);
            // saving file
            $saved[$file] = $this->_saveImage(
                $_FILES['images']['tmp_name'][$file], $name
            );
        }
        $this->getResponse()->appendBody(Zend_Json_Encoder::encode($saved));
    }
    
    private function _getPath($path)
    {
        $root = Ecart::config()->system->path . '/media/product';
        
        $path = trim($path, '/\\');
        $path = Ecart::config()->system->path . '/media/' . $path;
        
        $path = str_replace('\\', '/', realpath($path));
        if (file_exists($path) && strpos($path, $root) === 0) {
            return $path;
        }
        return false;
    }
    
    private function _getIcon($path)
    {
        if (is_dir($path)) {
            return 'folder';
        } else {
            if (false !== strrpos($path, '.')) {
                return 'file-' . substr($path, strrpos($path, '.') + 1);
            }
        }
        return 'file';
    }
    
    private function _get()
    {
        $path = $this->_getPath($this->_getParam('path'));
        if (!$path) {
            return array();
        }
        
        $items = array();
        $d = dir($path);
        while (false !== ($item = $d->read())) {
            if ($item == '.' || $item == '..') {
                continue;
            }
            
            $fpath = $d->path . '/' . $item;
            
            $item = array(
                'text' => $item,
                'iconCls' => $this->_getIcon($fpath),
                'disabled' => false,
                'leaf' => is_file($fpath) ? true : false
            );
            if (is_file($fpath)) {
                $item['qtip'] = 'Size: ' . filesize($fpath);
            }
            $items[] = $item;
        }
        $d->close();
        return $items;
    }
    
    private function _rename()
    {
        $newName = $this->_getParam('newname');
        $oldName = $this->_getParam('oldname');
    }
    
    public function treePanelAction()
    {
        $cmd = $this->_getParam('cmd');
        $method = '_' . $cmd;
        
        if (!method_exists($this, $method)) {
            return $this->_helper->json->sendFailure(array(
                'error'   => Ecart::translate('catalog')->__(
                    'Method %s not exist', $method
                )
            ));
        }
        $response = $this->$method();
        if (!is_string($response)) {
            return $this->_helper->json->sendJson($response, false, false);
        }
        return $this->_helper->json->sendFailure(array('error'   => $response));

    }
}