<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Catalog_ProductAttributesController extends Ecart_Admin_Controller_Back
{
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('catalog')->__(
            'Product Attributes'
        );
        $this->view->optionItems = Ecart::single('catalog/product_option')
            ->getList(array('languageId' => $this->_langId));
        $this->view->languages = Ecart_Collect_Language::collect();
        $this->view->optionTypes = Ecart_Catalog_Model_Product_Option::getTypes();
        $this->render();
    }
    
    public function listAction()
    {
        $dbField = new Ecart_Filter_DbField();
        
        $count  = Ecart::single('catalog/product_option')->count();
        
        $dataset = Ecart::single('catalog/product_option')->getList(array(
            'languageId' => $this->_langId,
            'sort'       => $dbField->filter($this->_getParam('sort', 'option_id')),
            'dir'        => $dbField->filter($this->_getParam('dir', 'ASC')),
            'start'      => (int) $this->_getParam('start', 0),
            'limit'      => (int) $this->_getParam('limit', 20)
        ));
        
        return $this->_helper->json->sendSuccess(array(
            'data' => $dataset,
            'count' => $count
        ));
    }
    
    public function saveAction()
    {
        $data = array(
            'id'            => (int) $this->_getParam('optId', 0),
            'code'          => $this->_getParam('optionCode'),
            'option_type'   => $this->_getParam('optionType'),
            'sort_order'    => $this->_getParam('optionSortOrder'),
            'searchable'    => $this->_getParam('searchable', 0),
            'comparable'    => $this->_getParam('comparable', 0),
            'languagable'   => $this->_getParam('languagable', 0),
            'valueset_id'   => $this->_getParam('optionValueSet', 0),
            'filterable'    => $this->_getParam('filterable', 1), //@todo this args never send
            'names'         => $this->_getParam('optionName'),
            'instructs'     => $this->_getParam('optionInstruct')
        );
        
        return $this->_helper->json->sendJson(array(
            'success' => Ecart::single('catalog/product_option')->save($data)
        ));
    }
    
    public function editAction()
    {
        $this->_helper->layout->disableLayout();
        
        $valueSet = Ecart::single('catalog/product_option_ValueSet');
        
        $vset = array('0' => 'None');
        
        foreach ($valueSet->fetchAll() as $set) {
            $vset[$set->id] = $set->name;
        }
            
        $this->view->assign(array(
            'languages' => Ecart_Collect_Language::collect(),
            'optionTypes' => Ecart_Catalog_Model_Product_Option::getTypes(),
            'valueSets' => $vset
        ));
        
        $optionId = $this->_getParam('optId', 0);
        if ($optionId) {
            $row = Ecart::single('catalog/product_option')
                ->find($optionId)->current();
            $this->view->option = $row->toArray();
            $textRows = $row->findDependentRowset('Ecart_Catalog_Model_Product_Option_Text');
            $optionText = array();
            foreach ($textRows as $row) {
                $optionText[$row->language_id] = $row->toArray();
            }
            $this->view->optionText = $optionText;
        }
        
        $this->render('form-option');
    }
    
    public function deleteAction()
    {
        Ecart::single('catalog/product_option')->delete(
            $this->db->quoteInto('id IN(?)', 
            Zend_Json_Decoder::decode($this->_getParam('data'))
        ));
        Ecart::message()->addSuccess(
            Ecart::translate('catalog')->__(
                'Option was deleted successfully'
            )
        );
        return $this->_helper->json->sendSuccess();
    }
    
}