<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Cms_BlockController extends Ecart_Admin_Controller_Back 
{
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('cms')->__('Static Blocks');
        
        $this->render();
    }
    
    public function getBlocksAction()
    {
        $this->_helper->layout->disableLayout();
        
        $this->_helper->json->sendSuccess(array(
            'blocks' => Ecart::single('admin/cms_block')->getBlocks()
        ));
    }
    
    public function getBlockDataAction()
    {
        $this->_helper->layout->disableLayout();
        
        $currentBlock = Ecart::single('admin/cms_block')
            ->find($this->_getParam('blockId'))->current();
        
        $this->_helper->json->sendSuccess(array(
            'block' => array($currentBlock->getContent())
        ));
    }
    
    public function saveBlockAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = $this->_getAllParams();
        
        $tableBlock = Ecart::single('admin/cms_block');
        
        if ($data['blockId'] != 'new') {
            $currentBlock = $tableBlock->find($this->_getParam('blockId'))
                ->current();
            $currentBlock->name = $data['name'];
            $currentBlock->is_active = $data['is_active'];
            $currentBlock->content = $data['content'];
            $currentBlock->save();
        } else {
            $tableBlock->insert(array(
                'name' => $data['name'],
                'is_active' => $data['is_active'] != '' ? $data['is_active'] : 1,
                'content' => $data['content'] 
            ));
        }
        $this->_helper->json->sendSuccess();
    }
    
    public function quickSaveBlockAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = Zend_Json_Decoder::decode($this->_getParam('data'));
        
        $tableBlock = Ecart::single('admin/cms_block');
        
        foreach ($data as $blockId => $values) {
            $currentBlock = $tableBlock->find($blockId)->current();
            $currentBlock->name = $values['name'];
            $currentBlock->is_active = $values['is_active'] ? 1 : 0;
            $currentBlock->save();
        }
        $this->_helper->json->sendSuccess();
    }
    
    public function deleteBlockAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = Zend_Json_Decoder::decode($this->_getParam('data'));
        
        $this->_helper->json->sendJson(array('success' =>
            Ecart::single('admin/cms_block')->delete(
                $this->db->quoteInto('id IN(?)', $data)
            )
        ));
    }
}