<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Cms_IndexController extends Ecart_Admin_Controller_Back
{
    
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('cms')->__(
            'Categories/Pages'
        );
        
        $this->view->languages = Ecart_Collect_Language::collect();
        
        $tablePage = Ecart::single('admin/cms_page');
        $this->view->selectLayout = $tablePage->getLayout();
        
        $sites = Ecart_Collect_Site::collect();
        $result = "";
        
	    function getChilds($node, $root) {
	        $result = "";
	        $tableCmsCategory = Ecart::single('admin/cms_category');
	            
	        //if !$root -> load subcategories, else -> load root categories of site
	        if ($root) {
	            $cats = $tableCmsCategory->getRootCategory($node);
	        } else {
	            $cats = $tableCmsCategory->getChildCategory($node);
	        }
	            
	        //get categories
	        if (sizeof($cats)) {
	            $result .= "menu: [";
		        foreach ($cats as $cat) {
		            $result .= "{
		                text: '$cat[name]',
		                " . getChilds($cat['id'], false) . "
		                handler: function() {
	                        batch('category', $cat[id]);
		                }
		            },\n";
		        }
		        $result = substr($result, 0, -2);
		        $result .= "],";
		        return $result;
	        } else {
	        	return false;
            }
	    }
	        
        //autogenerated nodes
        foreach ($sites as $siteId => $siteName) {
            $result .= "{
                    " . getChilds($siteId, true). "
                    text: '$siteName'
            },\n";
        }
        
        $this->view->categoryMenu = substr($result, 0, -2);
        
        $this->render();
    }
    
    public function getSiteTreeAction()
    {
        $this->_helper->layout->disableLayout();
        
        $sites = Ecart_Collect_Site::collect();
        $result = array();
        
        function getChilds($node, $root) {
            $result = array();
            $tableCmsCategory = Ecart::single('admin/cms_category');
            
            //if !$root -> load subcategories, else -> load root categories of site
            if ($root) {
                $cats = $tableCmsCategory->getRootCategory($node);
            } else {
                $cats = $tableCmsCategory->getChildCategory($node);
            }
            
            //get categories
            foreach ($cats as $cat) {
                $result[] = array(
                    'leaf' => false,
                    'id' => $cat['id'],
                    'siteId' => 'null',
                    'text' => $cat['name'],
                    'iconCls' => 'icon-folder',
                    'expanded' => true,
                    'cls' => $cat['is_active'] ? '' : 'disabledNode',
                    'children' => getChilds($cat['id'], false)
                );
            }
            return $result;
        }
        
        //autogenerated nodes
        foreach ($sites as $siteId => $siteName) {
            $result[] = array(
                    'leaf' => false,
                    'id' => "_" . $siteId,
                    'siteId' => $siteId,
                    'text' => $siteName,
                    'checked' => 'undefined',
                    'iconCls' => 'icon-folder',
                    'cls' => 'root-node',
                    'expanded' => true,
                    'children' => getChilds($siteId, true)
            );
        }
        $this->_helper->json->sendJson($result, false, false);
    }
    
    public function saveCategoryAction()
    {
        $this->_helper->layout->disableLayout();
        $data = $this->_getAllParams();

        $languages = Ecart_Collect_Language::collect();
        $tableCategoryContent = Ecart::single('cms/category_content');
        $tableCategory = Ecart::single('admin/cms_category');
        
        /* save category */
        $rowCategory = array(
            'name' => $data['name'],
            'is_active' => $data['is_active'] != '' ? $data['is_active'] : 1
        );
        
        if ($data['catId'] == 'new') {
            if ($data['parentId']) {
                $rowCategory['parent_id'] = $data['parentId'];
                $rowCategory['site_id']   = $tableCategory
                    ->find($data['parentId'])->current()->site_id;
            } else {
                $rowCategory['site_id']   = $data['siteId'];
            }
            //$rowCategory['parent_id'] = $data['parentId'] ? $data['parentId'] : new Zend_Db_Expr('NULL');
            //$rowCategory['site_id']   = $data['siteId'] ? $data['siteId'] : new Zend_Db_Expr('NULL');
            $catId = $tableCategory->insert($rowCategory);
        } else {
            $catId = $data['catId'];
            $currentCategory = $tableCategory->find($data['catId'])->current();
            $currentCategory->setFromArray($rowCategory);
            $currentCategory->save();
        }
        
        /* save category content */
        foreach (array_keys($languages) as $langId) {
            $rowContent = array(
                'title' => $data['title_'.$langId],
                'description' => $data['description_'.$langId],
                'link' => $data['link_'.$langId] == '' ?
                    new Zend_Db_Expr('NULL') : $data['link_'.$langId],
                'meta_title' => $data['meta_title_'.$langId],
                'meta_keyword' => $data['meta_keyword_'.$langId],
                'meta_description' => $data['meta_description_'.$langId]
            );
            
            if ($row = $tableCategoryContent->find($catId, $langId)->current()) {
                $row->setFromArray($rowContent);
            } else {
                $row = $tableCategoryContent->createRow($rowContent);
                $row->cms_category_id = $catId;
                $row->language_id = $langId;
            }
            $row->save();
        }
        Ecart::message()->addSuccess(
            Ecart::translate('cms')->__(
                'Category was saved successfully'
        ));
        $this->_helper->json->sendSuccess(array(
            'data' => array('catId' => $catId)
        ));
    }
    
    public function moveCategoryAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = $this->_getAllParams();
        
        $tableCategory = Ecart::single('admin/cms_category');
        $success = true;
        if ($data['parentId']) {
            $success = $tableCategory->update(array(
	            'parent_id' => $data['parentId'] ?
                        $data['parentId'] : new Zend_Db_Expr('NULL'),
	            'site_id'   => $tableCategory->find($data['parentId'])
                        ->current()->site_id
	        ), 'id = ' . intval($data['catId']));
        } else {
            $success = $tableCategory->update(array(
                'parent_id' => new Zend_Db_Expr('NULL'),
                'site_id'   => $data['siteId']
            ), 'id = ' . intval($data['catId']));    
        }
        
        $this->_helper->json->sendJson(array(
            'success' => $success
        ));
    }
    
    public function getCategoryAction()
    {
        $this->_helper->layout->disableLayout();
        
        $catId = $this->_getParam('catId');
        
        $currentCategory = Ecart::single('admin/cms_category')
            ->find($catId)->current();
        $content = $currentCategory->getContent();
        
        $category = array();
        
        foreach($content as $langTab) {

            $category['title_'.$langTab['language_id']] = $langTab['title'];
            $category['link_'.$langTab['language_id']] = $langTab['link'];
            $category['description_'.$langTab['language_id']]
                = $langTab['description'];
            $category['meta_title_'.$langTab['language_id']]
                = $langTab['meta_title'];
            $category['meta_keyword_'.$langTab['language_id']]
                = $langTab['meta_keyword'];
            $category['meta_description_'.$langTab['language_id']]
                = $langTab['meta_description'];
        }
        
        $category['name'] = $currentCategory->name;
        $category['is_active'] = $currentCategory->is_active;
        
        $this->_helper->json->sendSuccess(array(
            'category' => array($category)
        ));
    }
        
    public function deleteCategoryAction()
    {
        $this->_helper->layout->disableLayout();
        
        $categoryId = $this->_getParam('id');
        
        $success = Ecart::single('admin/cms_category')->delete(
            $this->db->quoteInto('id IN(?)', $categoryId)
        );
        if ($success) {
            Ecart::message()->addSuccess(
                Ecart::translate('cms')->__(
                    'Category was deleted successfully'
            ));
        }
        $this->_helper->json->sendJson(array(
            'success' => $success
        ));
    }
    
    public function getPagesAction()
    {
        $this->_helper->layout->disableLayout();
        
        $filterTree = $this->_getParam('filterTree');
        $filterSearch = $this->_getParam('filterSearch');
        
        $pagingParams = array(
            'start' => (int) $this->_getParam('start'),
            'limit' => (int) $this->_getParam('limit'),
            'sort'  => $this->_getParam('sort'),
            'dir'   => $this->_getParam('dir')
        );
        
        $tablePage = Ecart::single('admin/cms_page');
        
        $pages = $tablePage->getPages(
            $filterTree, $filterSearch, $this->_langId, $pagingParams
        );
        $totalCount = $tablePage->getCount(
            $filterTree, $filterSearch, $this->_langId
        );

        $this->_helper->json->sendSuccess(array(
            'totalCount' => $totalCount,
            'pages'      => $pages
        ));
    }
    
    public function getPageDataAction()
    {
        $this->_helper->layout->disableLayout();
        
        $pageId = $this->_getParam('pageId');
        
        $currentPage = Ecart::single('admin/cms_page')
            ->find($pageId)->current();
        $pageContent = $currentPage->getContent();
        $parentCategory = $currentPage->getCategory();
        
        $page = array();
        $category = array();

        foreach($parentCategory as $currentCategory) {
            $category[] = $currentCategory['cms_category_id'];
        }
        
        $page['is_active'] = $currentPage->is_active;
        $page['layout'] = $currentPage->layout;
        $page['comment'] = $currentPage->comment;
        $page['access'] = $currentPage->access;
        $page['name'] = $currentPage->name;
        
        //$page['category'] = $currentPage->category;
        
        foreach($pageContent as $langTab) {
            $page['title_' . $langTab['language_id']]            = $langTab['title'];
            $page['link_' . $langTab['language_id']]             = $langTab['link'];
            $page['content_' . $langTab['language_id']]          = $langTab['content'];
            $page['meta_keyword_' . $langTab['language_id']]     = $langTab['meta_keyword'];
            $page['meta_title_' . $langTab['language_id']]       = $langTab['meta_title'];
            $page['meta_description_' . $langTab['language_id']] = $langTab['meta_description'];
        }
        
        $this->_helper->json->sendSuccess(array(
            'page' => array($page),
            'category' => $category
        ));
    }
    
    public function savePageAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = $this->_getAllParams();
        
        $languages = Ecart_Collect_Language::collect();
        $tablePage = Ecart::single('admin/cms_page');
        $tablePageContent = Ecart::single('cms/page_content');
        
        /* save page */
        $rowPage = array(
            'is_active' => $data['is_active'] != '' ? $data['is_active'] : 1,
            'layout' => $data['layout'],
            'comment' => $data['comment'] != '' ? $data['comment'] : 1,
            'name' => $data['name']
        );
        
        if ($data['pageId'] == 'new') {
            $pageId = $tablePage->insert($rowPage);
        } else {
            $pageId = $data['pageId'];
            $currentPage = $tablePage->find($pageId)->current();
            $currentPage->setFromArray($rowPage);
            $currentPage->save();
        }
        
        /*
         * Save page_to_category links
         */
        $where = "cms_page_id = $pageId";
        Ecart::single('cms/page_category')->delete($where);
                
        foreach (Zend_Json::decode($data['category']) as $catId) {
        	if (false !== strpos($catId, '_')) continue;
        	
        	Ecart::single('cms/page_category')->insert(array(
                'cms_category_id' => $catId,
                'cms_page_id' => $pageId
            ));
        }
        
        /* save page content */
        foreach (array_keys($languages) as $langId) {
            $rowContent = array(
                'link' => $data['link_' . $langId] == '' ?
                    new Zend_Db_Expr('NULL') : $data['link_' . $langId],
                'title' => $data['title_' . $langId] == '' ?
                    new Zend_Db_Expr('NULL') : $data['title_' . $langId],
                'content' => $data['content_' . $langId],
                'meta_keyword' => $data['meta_keyword_' . $langId],
                'meta_description' => $data['meta_description_' . $langId],
                'meta_title' => $data['meta_title_' . $langId]
            );
            
            if ($row = $tablePageContent->find($pageId, $langId)->current()) {
            	$row->setFromArray($rowContent);
            } else {
                $row = $tablePageContent->createRow($rowContent);
                $row->cms_page_id = $pageId;
                $row->language_id = $langId;
            }
            $row->save();
        }
        if ($data['pageId'] == 'new') {
            Ecart::dispatch('cms_page_add_success', array(
                'page_id' => $pageId
            ));
        } else {
            Ecart::dispatch('cms_page_update_success', array(
                'page_id' => $pageId
            ));
        }
        Ecart::message()->addSuccess(
            Ecart::translate('cms')->__(
                'Page was saved successfully'
        ));
        $this->_helper->json->sendSuccess();
        
    }
    
    public function quickSavePageAction()
    {
        $this->_helper->layout->disableLayout();
        
        $pages = Zend_Json_Decoder::decode($this->_getParam('data'));
                
        $tablePage = Ecart::single('admin/cms_page');
        $tablePageContent = Ecart::single('cms/page_content');
        
        foreach ($pages as $pageId => $values) {
            $tablePage->update(array(
                'name' => $values['name'],
                'is_active' => (int) $values['is_active'],
                'comment' => (int) $values['comment'],
                'layout' => $values['layout'],
                'show_in_box' => $values['show_in_box']
            ), 'id = ' . intval($pageId));
            
            if ($row = $tablePageContent
                    ->find($pageId, $this->_langId)->current()) {

                $row->link = $values['link'] == '' ? 
                    new Zend_Db_Expr('NULL') : $values['link'];

            } else {
                $row = $tablePageContent->createRow($rowContent);
                $row->cms_page_id = $pageId;
                $row->language_id = $langId;
            }
            $row->save();
        }
        Ecart::message()->addSuccess(
            Ecart::translate('cms')->__(
                'Page was saved successfully'
        ));
        $this->_helper->json->sendSuccess();
    }
    
    public function deletePageAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = Zend_Json::decode($this->_getParam('data'));
        //$catId = $this->_getParam('catId');
        $params = $this->_getAllParams('catId', 'siteId');
        
        if ($params['catId'] == 'all' || $params['catId'] == 'lost' ) {
            
	        $where = $this->db->quoteInto('id IN (?)', $data);
	        Ecart::single('admin/cms_page')->delete($where);
	        
        } else if ($params['siteId'] != 'null') {
            
            $where = $this->db->quoteInto('cms_page_id IN (?)', $data);
            
            Ecart::single('cms/page_category')
                ->deleteBySiteId($where, $params['siteId']);
        } else {
            $where = $this->db->quoteInto(
                'cms_category_id = ?', $params['catId']
            );
            $where .= $this->db->quoteInto('AND cms_page_id IN (?)', $data);
            
            Ecart::single('cms/page_category')->delete($where);
        }
        Ecart::message()->addSuccess(
            Ecart::translate('cms')->__(
                'Page was deleted successfully'
        ));
        $this->_helper->json->sendSuccess();
    }
    
    public function copyPageAction()
    {
        $this->_helper->layout->disableLayout();
        
        $pages = Zend_Json::decode($this->_getParam('pages'));
        $categoryId = $this->_getParam('catId');
        
        
        foreach ($pages as $page) {
            Ecart::single('cms/page_category')->delete(
                "cms_page_id = '$page' AND cms_category_id = '$categoryId'"
            );
        }
        
        foreach ($pages as $page) {
            Ecart::single('cms/page_category')->insert(array(
                'cms_page_id' => $page,
                'cms_category_id' => $categoryId
            ));
        }
        
        $this->_helper->json->sendSuccess();
    }
    
}