<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_CsvController extends Ecart_Admin_Controller_Back
{
    private $_valuesCache = array();
    private $_optionsCache = array();
    private $_supportedTypes = array(
        array('Products', 'products')/*,
        array('Customers', 'customers')*/
    );
    
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('admin')->__(
            "Csv Import/Export"
        );
        $this->render();
    }
    
    public function getListAction()
    {
        $this->_helper->json->sendSuccess(array(
            'data' => Ecart::single('admin/csv_profile')->getList()
        ));
    }
    
    public function getSupportedTypesAction()
    {
        $this->_helper->json->sendJson($this->_supportedTypes, false, false);
    }
    
    public function runAction()
    {
        $data = $this->_getParam('general');
        $filters = $this->_getParam('filter');
        
        if (!method_exists($this, '_' . $data['direction'] . ucfirst($data['type']))) {
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    'Requested method not found'
                )
            );
            $this->_helper->json->sendFailure();
        }
        $response = call_user_func(
            array($this, '_' . $data['direction'] . ucfirst($data['type'])),
            $data, $filters
        );

        $this->_helper->json->sendSuccess(array(
            'messages' => $response
        ));
    }
    
    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = $this->_getParam('general');
        $filters = $this->_getParam('filter');
        $data['site'] = trim($filters['site'], ',');
        
        $this->_helper->json->sendJson(array(
            'success' => Ecart::single('admin/csv_profile')
                ->save($data, $filters)
        ));
    }
    
    public function deleteAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = Zend_Json_Decoder::decode($this->_getParam('data'));
        
        $this->_helper->json->sendJson(array(
            'success' => Ecart::single('admin/csv_profile')->deleteByIds($data)
        ));
    }
    
    private function _getCsvProductTitles($type = null)
    {
        $productTitles = array(
            'sku', 'manufacturer', 'hurl', 'path', 'quantity', 'image_base', 'image_listing', 'image_thumbnail',
            'price', 'weight', 'date_available', 'in_stock', 'is_active', 'ordered',
            'created_on', 'modified_on', 'tax_class');
        $descriptionTitles = array(
            'name', 'description', 'short_description', 'viewed', 'image_seo_name',
            'meta_title', 'meta_description', 'meta_keyword');
            
        switch ($type) {
            case 'product':
                return $productTitles; break;
            case 'description':
                return $descriptionTitles; break;
            default:
                return array_merge(
                    $productTitles, array('product_gallery', 'product_featured'),
                    $descriptionTitles, array('variations'));
                break;
        }
    }
    
    private function _getModifierValue($value, $type)
    {
        if (floatval($value) == 0)
            return '';
        switch ($type) {
            case 'by': 
                return ($value > 0 ? '+' : '-') . $value;
            case 'percent':
                return $value . '%';
            case 'to':
                return $value;
            default:
                return '';
        }
    }
    
    private function _valueToModifier($value)
    {
        $type = 'to';
        $value = trim($value);
        if (false !== strpos($value, '%')) {
            $type = 'percent';
        } elseif (substr($value, 0, 1) == '+' || substr($value, 0, 1) == '-') {
            $type = 'by';
        }
        $value = floatval(trim($value, '%+'));
        return array($value, $type);
    }

    /**
     * 
     * @return value text, this function use cache array for fast execution
     * @param integer $valueId
     * @param enteger $languageId
     */
    private function _getValueText($valueId, $languageId)
    {
        if (!isset($this->_valuesCache[$valueId])) {
            $text = Ecart::single('catalog/product_option_value_text')->find(
                $valueId,
                $languageId
            );

            $this->_valuesCache[$valueId] = $text->valid() ?
                $text->current()->name : '';
        }
        return $this->_valuesCache[$valueId];
    }
    
    private function _exportProducts($preferences, $filters)
    {
        $path = Ecart::config()->system->path . '/' . $preferences['file_path'];
        
        if (!is_dir($path)) {
            mkdir($path, 0777, true);
        }
        
        if (@!$fp = fopen($path . '/' . $preferences['file_name'], 'w')) {
            return array(
                'error' => Ecart::translate('core')->__(
                    'Cannot open file with write permissions'
                )
            );
        }
        
        @chmod($path . '/' . $preferences['file_name'], 0777);
        
        $titles = $this->_getCsvProductTitles();
        $emptyRow = array_fill_keys($titles, '');
        
        // Fill descriptions with empty arrays
        $from = array_search('name', $titles);
        $to = array_search('meta_keywords', $titles);
        $data = array_fill_keys(
            array_slice($titles, $from, $to - $from + 1), array()
        );
        $emptyRow = array_merge($emptyRow, $data);
        
        // Searching product options
        $attrModel = Ecart::single('catalog/product_attribute');
        $optionTypes = Ecart_Catalog_Model_Product_Option::getTypes();
        $options = array();
        $values = array();
        foreach ($attrModel->getUsedOptions() as $option) {
            $options[$option->id] = $option;
            
            $title = $option->code . ',' . $optionTypes[$option->option_type];
            if ($valueset = $option->getValueset()) {
                $title .= ',' . $valueset->name;
            }
            $titles[] = $title;
            
            $emptyRow[$option->code] = '';
        }
        
        // Write titles
        fputcsv($fp, $titles, ',', "'");
        unset($titles);
        
        // Load lanuages
        $filter_languages = explode(',', trim($filters['language_ids'], ' ,'));
        $languageIds = array_keys(Ecart_Collect_Language::collect());
        
        $languageId = Ecart::getLanguageId();
        $langIdToCode = array();
        foreach (Ecart::single('locale/language')->fetchAll() as $lang) {
            $langIdToCode[$lang->id] = $lang->code;
        }
        
        $manufacturersSource = Ecart::single('catalog/product_manufacturer')
            ->getListBackend();
        
        $manufacturers = array();
        foreach ($manufacturersSource as $manufacturer) {
            $manufacturers[$manufacturer['id']] = $manufacturer['url'];
        }
        
        // Init start values
        $end = false;
        $lastExportedId = 0;
        $step = 30;
        
        $mProduct = Ecart::single('catalog/product');
        // Export products
        while (!$end) {
            $products = $mProduct->getProductSet($lastExportedId, $step, $filters);
            
            if (count($products) < $step) {
                $end = true;
            }
            foreach ($products as $product) {
                
                $lastExportedId = $product->id;
                
                $data = array_intersect_key($product->toArray(), $emptyRow);
                // fill manufacture name
                if (!empty($product->manufacturer_id) && 
                    isset($manufacturers[$product->manufacturer_id])) 
                {
                    $data['manufacturer'] = $manufacturers[$product->manufacturer_id];
                }
                
                // put data to row
                $row = array_merge($emptyRow, $data);
                // product paths
                $path = '';
                $paths = $product->getParentItems(null, true);
                foreach ($paths as $pathItem) {
                    foreach ($pathItem as $category)
                        $path .= '/'.$category['key_word'];
                    $path .= ",\n";
                }
                $row['path'] = substr($path, 0, -2);
                
                // human url
                $row['hurl'] = Ecart::single('catalog/hurl')->getProductUrl($product->id);
                
                // stock status
                if ($stock = Ecart::single('catalog/product_stock')
                        ->find($lastExportedId)->current()) 
                {
                    $row['in_stock'] = $stock->in_stock;
                }
                
                // Gallery
                $gallery = '';
                foreach ($product->getGallery() as $item) {
                    $gallery .= $item->image . ",\n";
                }
                $row['product_gallery'] = substr($gallery, 0, -2);
                
                // Attributes
                $attributes = array();
                foreach ($product->getAttributes() as $attribute) {
                    // get option from cache
                    $option = $options[$attribute->product_option_id];
                    // get value from cache
                    if ($attribute->product_option_value_id) {
                        $value = $this->_getValueText($attribute->product_option_value_id, $languageId);
                    } else {
                        $value = '';
                    }
                    
                    if ($attribute->isProperty()) {
                        // inputable attribute
                        if ($option->isInputable()) {
                            $attrValues = $attribute->getAttributeValues();
                            if (!$attrValues->valid()) {
                                continue;
                            }
                            if (!$option->languagable) {
                                $row[$option->code] = $attrValues->current()->attribute_value;
                            } else {
                                foreach ($attrValues as $attrValue) {
                                    $aLangId = $attrValue->language_id;
                                    if (!in_array($aLangId, $languageIds))
                                        continue;
                                    $row[$option->code][$aLangId] = $attrValue->attribute_value;
                                }
                            }
                        } else { // attribute with value from valueset
                            $row[$option->code] = $value;
                        }
                    } elseif ($attribute->isModifier()) {
                        // Modifier
                        if (!is_array($row[$option->code]))
                            $row[$option->code] = array();
                        $row[$option->code][] = array(
                            'value'  => $value,
                            'price'  => $this->_getModifierValue($attribute->price, $attribute->price_type),
                            'weight' => $this->_getModifierValue($attribute->weight, $attribute->weight_type)
                        );
                    } elseif ($attribute->isVariation()) {
                        // Variation
                        if (!is_array($row['variations'])) {
                            $row['variations'] = array();
                        }
                        if (!isset($row['variations'][$attribute->product_variation_id])) {
                            $variation = $attribute->getVariation();
                            $row['variations'][$attribute->product_variation_id] = array(
                                'sku'    => $variation->sku,
                                'price'  => $this->_getModifierValue($variation->price, $variation->price_type),
                                'weight' => $this->_getModifierValue($variation->weight, $variation->weight_type),
                                'quantity' => $variation->quantity
                            );
                        }
                        $row['variations'][$attribute->product_variation_id][$option->code] = $value;
                    }
                }
                if (is_array($row['variations']))
                    $row['variations'] = array_values($row['variations']);
                
                // Description
                $descriptions = array();
                $descriptionRows = $product->findDependentRowset(
                    'Ecart_Catalog_Model_Product_Description',
                    'Product',
                    $mProduct->select()->where('language_id IN(?)', $languageIds)
                );
                
                foreach ($descriptionRows as $description) {
                    if (!in_array($description->language_id, $filter_languages))
                        continue;
                        
                    $langCode = $langIdToCode[$description->language_id];
                    foreach ($description->toArray() as $key => $dValue) {
                        if (isset($row[$key])) {
                            if (!is_array($row[$key]))
                                $row[$key] = array();
                            $row[$key][] = array($langCode => $dValue);
                        }
                    }
                }
                
                // Encode attributes
                foreach ($row as &$item) {
                    if (is_array($item))
                        $item = str_replace('"},{"', "\"},\n {\"", Zend_Json_Encoder::encode($item));
                }
                
                fputcsv($fp, $row, ',', "'");
            }
        }
        return array(
            'success' => 'Products was exported successfully'
        );
    }
    
    private function _removeDir($path) {
        if (is_dir($path)) {
            $path = rtrim($path, '/');
            $dir = dir($path);
            while (false !== ($file = $dir->read())) {
                if ($file != '.' && $file != '..') {
                    (!is_link("$path/$file") && is_dir("$path/$file")) ? 
                        RemoveDir("$path/$file") : unlink("$path/$file");
                }
            }
            $dir->close();
            rmdir($path);
            return true;
        }
        return false;
    }
    
    private function _getCreateValue($valueText, $valuesetId, $createOption = false)
    {
        $key = $valuesetId . '_' . $valueText;
        if (!isset($this->_valuesCache[$key])) {
            $value = Ecart::single('catalog/product_option_value')->getByText($valueText, $valuesetId);
            if (!$value && $createOption) {
                // creating value
                $value = Ecart::single(('catalog/product_option_value'))->createRow(array(
                    'valueset_id' => $valuesetId
                ));
                $value->save();
                
                // caching language_ids
                if (!Zend_Registry::isRegistered('language_ids')) {
                    Zend_Registry::set('language_ids', array_keys(Ecart_Collect_Language::collect()));
                }
                // insert the same text for all languages
                foreach (Zend_Registry::get('language_ids') as $langId) {
                    Ecart::single('catalog/product_option_value_text')->insert(array(
                        'product_option_value_id' => $value->id,
                        'language_id' => $langId,
                        'name' => $valueText
                    ));
                }
            }
            $this->_valuesCache[$key] = $value;
        }
        return $this->_valuesCache[$key];
    }
    
    /**
     * Return option object(Row) for the gived optionName
     * It can create option if it does not exist.
     * 
     * @return Ecart_Catalog_Model_Product_Option_Row
     */
    private function _getCreateOption($optionName, $createNotExists = false)
    {
        // optionName: "option_name,option_type<,valueset_name>"
        $optionName = explode(',', $optionName);
        
        if (!isset($this->_optionsCache[$optionName[0]])) {
            $mOption = Ecart::single('catalog/product_option'); 
            $option = $mOption->fetchRow($mOption->select()->where('code = ?', $optionName[0]));
            if (!$option && $createNotExists && isset($optionName[1])) {
                // create option
                $option = $mOption->createRow();
                
                $types = Ecart_Catalog_Model_Product_Option::getTypes();
                $option->option_type = array_search($optionName[1], $types);
                $option->code = $optionName[0];
                
                if (!$option->isInputable() && isset($optionName[2])) {
                    $valueset  = Ecart::single('catalog/product_option_valueSet')->getCreate($optionName[2]);
                    $option->valueset_id = $valueset->id;
                }
                $option->save();
                
                // caching language_ids
                if (!Zend_Registry::isRegistered('language_ids')) {
                    Zend_Registry::set('language_ids', array_keys(Ecart_Collect_Language::collect()));
                }
                // insert the same text for all languages
                foreach (Zend_Registry::get('language_ids') as $langId) {
                    Ecart::single('catalog/product_option_text')->insert(array(
                        'product_option_id' => $option->id,
                        'language_id' => $langId,
                        'name' => $option->code,
                        'instruct' => ''
                    ));
                }
            }
            $this->_optionsCache[$optionName[0]] = $option;
        }
        return $this->_optionsCache[$optionName[0]];
    }
    
    private function _optionNameToType($optionName)
    {
        list(,$type) = explode(',', $optionName);
        return $type;
    }
    
    private function _arraySearchByKeyValue(&$array, $key, $value)
    {
        foreach ($array as $index => $item) {
            if (isset($item[$key]) && $item[$key] == $value)
                return $index;
        }
        return false;
    }

    private function _importProducts($preferences, $filters)
    {
        $this->_helper->layout->disableLayout();
        $createManufacturer = true;
        $createOptions      = true;
        
        $path = Ecart::config()->system->path . '/' . $preferences['file_path'];
        
        if (@!$fp = fopen($path.'/'.$preferences['file_name'], 'r')) {
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    'Cannot open file with read permissions'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        $titles = fgetcsv($fp, 2048, ',', "'");
        $rowSize = count($titles);
        $optionsIndexStartFrom = array_search('variations', $titles) + 1;
        $optionTypes = Ecart_Catalog_Model_Product_Option::getTypes();
        
        $filter_sites = explode(',', $filters['site']);
        
        $mManufacturer = Ecart::single('catalog/product_manufacturer');
        $manufacturers = $mManufacturer->fetchPairs('name', 'id');
        
        $langCodeToId = Ecart::single('locale/language')->fetchPairs('code', 'id');
        $languages = array_keys(Ecart_Collect_Language::collect());
        
        $mProduct = Ecart::single('catalog/product');
        $select = $mProduct->select();
        $log['skipped'] = array();
        $log['imported'] = array();
        $log['skipped']['count'] = 0;
        $log['imported']['count'] = 0;
        $log['skipped']['duplicate sku'] = array();
        
        while (!feof($fp)) {
            $data = fgetcsv($fp, 2048, ',', "'");
            
            if (!is_array($data)) {
                continue;
            }
            $data = array_pad($data, $rowSize, '');
            $data = array_combine($titles, $data);
            
            if (empty($data['sku'])) {
                $log['skipped']['count']++;
                continue;
            }
            
            $duplicate = $mProduct->fetchRow(
                $select->reset('where')->where('sku = ?', $data['sku'])
            );
            
            if ($duplicate) {
                $log['skipped']['duplicate sku'][] = $duplicate->sku;
                $log['skipped']['count']++;
                continue; 
            }
            
            $log['imported']['count']++;
                
            if (!empty($data['manufacturer'])
                && !isset($manufacturers[$data['manufacturer']])
                && $createManufacturer) {

                $manufacturerId = $mManufacturer->insert(array(
                    'name' => $data['manufacturer']
                ));
                if ($manufacturerId) {
                    $manufacturers[$data['manufacturer']] = $manufacturerId;
                    foreach ($filter_sites as $siteId) {
                        Ecart::single('catalog/hurl')->insert(array(
                            'key_word' => str_replace(
                                array('\\', '/', ' '),
                                '-',
                                strtolower($data['manufacturer'])
                            ),
                            'site_id' => $siteId,
                            'key_type' => 'm',
                            'key_id' => $manufacturerId
                        ));
                    }
                }
            }
            $product = $mProduct->createRow();
            $product->setFromArray($data);
            $product->viewed = array_sum(Zend_Json_Decoder::decode($data['viewed']));
            $product->tax_class_id = 1;
            
            if (!empty($data['manufacturer']) 
                && isset($manufacturers[$data['manufacturer']])) {
                
                $product->manufacturer_id = $manufacturers[$data['manufacturer']];
            }
            $productId = $product->save();
            
            // human url
            foreach ($filter_sites as $siteId) {
                Ecart::single('catalog/hurl')->insert(array(
                    'key_word' => str_replace(array('\\', '/', ' '), '-', $data['hurl']),
                    'site_id' => $siteId,
                    'key_type' => 'p',
                    'key_id' => $productId
                ));
            }
            
            // create categories if not exist
            if (!empty($data['path'])) {
                $paths = explode(',', trim($data['path'], ', '));
                $categoryModel = Ecart::single('catalog/category');
                
                $catData = array(
                    'cat_status' => 'enabled',
                    'modified_on' => Ecart_Date::now()->toZendDbExpr(),
                    'created_on' => Ecart_Date::now()->toZendDbExpr()
                );
                
                foreach ($paths as $path) {
                    $cleanPath = trim($path, "/\n ");
                    $path = explode('/', $cleanPath);
                    foreach ($filter_sites as $siteId) {
                        $i = 0;
                        $categoryId = 0;
                        $root_id = $categoryModel->fetchOne('id', 'site_id = ? AND lvl = 0', $siteId);
                        foreach ($path as $catUrl) {
                            if (!$categoryId = $categoryModel->getIdByUrl($catUrl, $siteId)) {
                                if ($i == 0) {
                                    $categoryId = $categoryModel->insertItem($catData, $root_id);
                                } else {
                                    $categoryId = $categoryModel->insertItem(
                                        $catData,
                                        $categoryModel->getIdByUrl($path[$i-1], $siteId)
                                    );
                                }
                                // description
                                foreach ($languages as $langId) {
                                    Ecart::single('catalog/category_description')->save(array(
                                        'category_id' => $categoryId,
                                        'language_id' => $langId,
                                        'name' => $catUrl,
                                        'description' => '',
                                        'meta_title' => $catUrl,
                                        'meta_description' => '',
                                        'meta_keyword' => $catUrl
                                    ));
                                }
                                
                                // human url
                                Ecart::single('catalog/hurl')->insert(array(
                                    'key_word' => $catUrl,
                                    'site_id' => $siteId,
                                    'key_type' => 'c',
                                    'key_id' => $categoryId
                                ));
                            }
                            $i++;
                        }
                        
                        // product to category linking
                        Ecart::single('catalog/product_category')->insert(array(
                            'category_id' => $categoryId,
                            'product_id' => $productId
                        ));
                    }
                }
            }
            
            // Gallery
            if (!empty($data['product_gallery'])) {
                $images = explode(',', $data['product_gallery']);
                foreach ($images as &$item)
                    $item = trim($item);
                // insert new images to gallery
                $product->addToGallery($images);
            }
            
            // Description
            $descriptions = array();
            foreach ($this->_getCsvProductTitles('description') as $field) {
                $fieldValues = Zend_Json_Decoder::decode($data[$field]);
                foreach ($fieldValues as $value) {
                    $langCode = key($value);
                    $value = current($value);
                    if (!isset($langCodeToId[$langCode]))
                        continue;
                    $descriptions[$langCodeToId[$langCode]][$field] = $value;
                }
            }
            
            foreach ($languages as $langId) {
                if (!isset($descriptions[$langId]))
                    continue;
                $description = Ecart::single('catalog/product_description')->createRow();
                $description->setFromArray($descriptions[$langId]);
                $description->product_id = $productId;
                $description->language_id = $langId;
                $description->save();
            }
            
            // stock
            $stock = Ecart::single('catalog/product_stock')->createRow();
            $stock->setFromArray(array(
                'product_id' => $productId,
                'in_stock' => $data['in_stock']
            ));
            $stock->save();
            
            // Attributes
            
            // Variations
            if (!empty($data['variations'])) {
                $variations = Zend_Json_Decoder::decode($data['variations']);

                foreach ($variations as &$variation) {
                    $pVariation = Ecart::single('catalog/product_variation')->createRow();
                    $pVariation->product_id = $productId;
                    $pVariation->sku = $variation['sku'];
                    list($pVariation->price, 
                         $pVariation->price_type) = $this->_valueToModifier($variation['price']);
                    list($pVariation->weight, 
                         $pVariation->weight_type) = $this->_valueToModifier($variation['weight']);
                    $variation['id'] = $pVariation->save();
                    
                    // insert new attributes
                    foreach (array_slice($variation, 3) as $optionName => $valueText) {
                        $option = $this->_getCreateOption($optionName, $createOptions);
                        
                        // if option does not exists or valueset is empty then skip this attribute
                        // because variation can contain only selectable options 
                        if (!$option || !$option->valueset_id)
                            continue;
                        // the same, values
                        $value  = $this->_getCreateValue($valueText, $option->valueset_id, $createOptions);
                        if (!$value)
                            continue;

                        Ecart::single('catalog/product_attribute')->insert(array(
                            'product_id'              => $productId,
                            'product_variation_id'    => $variation['id'],
                            'product_option_id'       => $option->id,
                            'product_option_value_id' => $value->id
                        ));
                    }
                }
            }
            
            $optionValues = array_slice($data, $optionsIndexStartFrom);
            $i = 0; // used as key for creating new attributes-modifiers
            while (list($optionName, $optionValue) = each($optionValues)) {
                if (is_array($optionValue)) {
                    $modifier = $optionValue;
                    $optionValue = $modifier['value'];
                    $optionName = $modifier['optionName'];
                } else {
                    $modifier = false;
                }
                
                $optionValue = trim($optionValue);
                $option = $this->_getCreateOption($optionName, $createOptions);
                if (!$option || $optionTypes[$option->option_type] != $this->_optionNameToType($optionName) || 
                    empty($optionValue))
                {
                    // if option not exists or type is wrong then skip this option
                    continue;
                }
                
                /* check is this modifier json data */
                if (false !== strpos($optionValue, '[{"') && false !== strpos($optionValue, '"}]')) {
                    $modifiers = Zend_Json_Decoder::decode($optionValue);
                    foreach ($modifiers as $modifier) {
                        $modifier['optionName'] = $optionName;
                        if (empty($modifier['value']))
                            $modifier['value'] = current(explode(',', $optionName));
                        $optionValues['modifier_' . $i++] = $modifier;
                    }
                    continue;
                }
                
                if (!$option->isInputable() && $option->valueset_id) {
                    $value = $this->_getCreateValue($optionValue, $option->valueset_id, $createOptions);
                    if (!$value)
                        continue;
                    $valueId = $value->id;
                } else {
                    $valueId = new Zend_Db_Expr('NULL');
                }
                
                $attribute = Ecart::single('catalog/product_attribute')->createRow();
                $attribute->product_id = $productId;
                $attribute->product_option_id = $option->id;
                
                $attribute->product_option_value_id = $valueId;
                if ($modifier) {
                    list($attribute->price, 
                         $attribute->price_type) = $this->_valueToModifier($modifier['price']);
                    list($attribute->weight, 
                         $attribute->weight_type) = $this->_valueToModifier($modifier['weight']);
                    $attribute->modifier = 1;
                }
                    
                $attribute->save();
            }
        }
        return $log;
    }
    
}