<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Customer_GroupController extends Ecart_Admin_Controller_Back
{
    /**
     *
     * @var Ecart_Account_Model_Customer_Group
     */
    private $_table;
    
    public function init()
    {
        parent::init();
        $this->_table =  Ecart::single('account/customer_group');
    }
    
    public  function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('account')->__(
            'Manage Customer Groups'
        );
        $this->render();
    }
    
    public function listAction()
    {
        $alpha = new Ecart_Filter_DbField();
        
        $params = array();
        $params['sort']     = $alpha->filter($this->_getParam('sort', 'id'));
        $params['dir']     = $alpha->filter($this->_getParam('dir', 'DESC'));
        $params['where'] = 'id > 0';
        
        $dataset = $this->_table->getList($params);
        
        return $this->_helper->json->sendSuccess(array(
            'data' => $dataset
        ));
    }
    
    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = Zend_Json::decode($this->_getParam('data'));
        
        if (!sizeof($data)) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'No data to save'
                )
            );
            return $this->_helper->json->sendFailure();
        }
            
        $this->_table->save($data);
            
        return $this->_helper->json->sendSuccess();
    }
    
    public function deleteAction()
    {
        $this->_helper->layout->disableLayout();
        
        $customerGroupIds = Zend_Json::decode($this->_getParam('data'));

        $isValid = true;
        $guestGroupId = Ecart::single('account/customer_group')
            ->getIdByName('Guest');

        if (in_array($guestGroupId, $customerGroupIds)) {
            $isValid = false;
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    "Your can't delete default Guest group id: %s ",
                    $guestGroupId
                )
            );
        }

        if (true === in_array(
                Ecart::config()->main->store->defaultCustomerGroup,
                $customerGroupIds
            )) {

            $isValid = false;
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    "Your can't delete default customer group id: %s ", $id
                )
            );
        }

        if (!sizeof($customerGroupIds)) {
            $isValid = false;
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    'No data to delete'
                )
            );
        }
        
        if ($isValid) {
            $this->_table->delete(
                $this->db->quoteInto('id IN(?)', $customerGroupIds)
            );
            Ecart::message()->addSuccess(
                Ecart::translate('admin')->__(
                    'Group was deleted successfully'
                )
            );
        }
        $this->_helper->json->sendJson(array(
            'success' => $isValid
        ));
    }
    
    public function getGroupsAction()
    {
        $this->_helper->layout->disableLayout();
        
        $this->_helper->json->sendSuccess(array(
           'data' => $this->_table->fetchAll()->toArray()
        ));
    }
}