<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Customer_IndexController extends Ecart_Admin_Controller_Back
{
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('account')->__(
            'Manage Customers'
        );

        $this->view->userForm = array();
        $this->view->valueSet = array();

        $this->view->languageId = $this->_langId;
        $this->view->languages = Ecart_Collect_Language::collect();

        $value = Ecart::single('account/Customer_ValueSet_Value');

        $fields = Ecart::single('account/customer_field');
        $fieldGroups = Ecart::single('account/Customer_FieldGroup');
        $fieldGroups = $fieldGroups->getGroups($this->_langId);

        foreach ($fieldGroups as $fieldGroup){
            //getting all fields
            $this->view->userForm[$fieldGroup['id']]['title'] = $fieldGroup['title'];
            $this->view->userForm[$fieldGroup['id']]['is_active'] =
                $fieldGroup['is_active'];
            $this->view->userForm[$fieldGroup['id']]['fields'] =
                $fields->getFieldsByGroup($fieldGroup['id']);
            //getting only used valuesets
            foreach ($this->view->userForm[$fieldGroup['id']]['fields'] as $fd){
                if (!isset($fd['customer_valueset_id'])) continue;
                $this->view->valueSet[$fd['customer_valueset_id']]['values'] =
                    $value->getValues($fd['customer_valueset_id']);
            }

        }

        $this->render();
    }

    public function listAction()
    {
        $alpha = new Zend_Filter_Alpha();
        $data = Ecart::single('account/customer')->getList(array(
            'start' => (int) $this->_getParam('start', 0),
            'limit' => (int) $this->_getParam('limit', 25),
            'sort'  => $alpha->filter($this->_getParam('sort', 'id')),
            'dir'   => $alpha->filter($this->_getParam('dir', 'ASC')),
            'getSitesName' => true
        ));
        foreach ($data['accounts'] as &$item) {
            $item['password'] = '';
        }
        return $this->_helper->json->sendSuccess(array(
            'data' => $data['accounts'],
            'count' => $data['count']
        ));
    }

    public function editDetailsAction()
    {
        $this->_helper->layout->disableLayout();

        $form = Ecart::single('account/form_customer');
        $form->setAction($this->view->url(array('action' => 'save')));
        $customerId = $this->_getParam('customerId', 0);

        $customer = Ecart::single('account/customer')
            ->find($customerId)->current();

        $data = array(
            'email' => $customer->email,
            'is_active' => $customer->is_active,
            'site_id' => $customer->site_id,
            'firstname' => $customer->firstname,
            'lastname' => $customer->lastname
        );
        $rows = $customer->findDependentRowset(
            'Ecart_Account_Model_Customer_Detail'
        )->toArray();

        foreach ($rows as $row) {
            $data['field_' . $row['customer_field_id']] = empty($row['data']) ?
            $row['customer_valueset_value_id'] : $row['data'];
        }

        $form->setDefaults($data);
        $this->getResponse()->appendBody($form->render($this->view));
    }

    public function loadFieldsDataAction()
    {
        $this->layout->disableLayout();
        $customerId = $this->_getParam('cId', 0);

        $customerTable = Ecart::single('account/customer');
        
        $customer = $customerTable->find($customerId)->current();
        $info = $customer->toArray();
        $info['password'] = '';
        
        $details = array();
        foreach ($customer->getDetails() as $item) {
            if (isset($details['field_' . $item['customer_field_id']])) {
                $details['field_' . $item['customer_field_id']] .= ','
                    . $item['customer_valueset_value_id'];
            } else
            $details['field_' . $item['customer_field_id']] = isset($item['data']) ?
            $item['data'] :$item['customer_valueset_value_id'];
        }
         
        return $this->_helper->json->sendSuccess(array(
            'fields' => array($info + $details)
        ));
    }

    public function quickSaveAction()
    {
        $this->_helper->layout->disableLayout();
        $data = Zend_Json::decode($this->_getParam('data'));
        $customerTable = Ecart::single('account/customer');
         
        foreach (array_keys($data) as $id ) {
            $customer = $customerTable->find($id)->current();
            $customer->email = $data[$id]['email'];
            $customer->is_active = $data[$id]['is_active'];
            $customer->site_id = $data[$id]['site_id'];
            $customer->group_id = $data[$id]['group_id'];
            $customer->save();
        }
        return $this->_helper->json->sendSuccess();
    }
    
    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        $data = $this->_getAllParams();

        $customerTable = Ecart::single('account/customer');
        $customerId = $data['id'];
        $form = Ecart::single('account/form_customer');

        if ($customerId) {
            $customer = $customerTable->find($customerId)->current();
            $customer->email = $data['email'];
            if (!empty($data['password'])) {
                $customer->password = md5($data['password']);
            }
            $customer->firstname = $data['firstname'];
            $customer->lastname = $data['lastname'];
            $customer->is_active = isset($data['is_active']) ?
                $data['is_active'] : 1 ;
            $customer->site_id = $data['site_id'];
            $customer->save();
        } else {
            $customerId = Ecart::single('account/customer')->insert(array(
                'email'    => $data['email'],
                'password' => md5($data['password']),
                'firstname' => $data['firstname'],
                'lastname' => $data['lastname'],
                'site_id'  => $data['site_id'],
                'is_active' => $data['is_active']
            ));
        }

        $details = Ecart::single('account/customer_detail');
        // clear old data
        $details->delete($this->db->quoteInto('customer_id = ?', $customerId));
        // insert new data
        foreach ($data as $id => $value) {
            if (false === strpos($id, 'field_') || empty($value)) {
                continue;
            }
            $parts = explode('_', $id);
            $id = (int) array_pop($parts);
            $row = array(
                'customer_id' => $customerId,
                'customer_field_id' => $id
            );

            if (false !== strpos($value, ',')
                && $form->getElement("field_$id") instanceof Zend_Form_Element_Multi)
            {
                $value = explode(',', $value);
            }

            if (is_string($value)) {
                if ($form->getElement("field_$id") instanceof Zend_Form_Element_Multi) {
                    $row['customer_valueset_value_id'] = $value;
                } else {
                    $row['data'] = $value;
                }
                $details->insert($row);
            } elseif (is_array($value)) {
                foreach ($value as $v) {
                    $row['customer_valueset_value_id'] = $v;
                    $details->insert($row);
                }
            }
        }

        return $this->_helper->json->sendSuccess();
    }

    public function deleteAction()
    {
        $this->_helper->layout->disableLayout();
        $data = Zend_Json_Decoder::decode($this->_getParam('data'));
        if (!sizeof($data)) {
            return;
        }
        Ecart::single('account/customer')->delete(
            $this->db->quoteInto('id IN(?)', $data)
        );
        return $this->_helper->json->sendSuccess();
    }

    public function editAction()
    {
        $customerId = $this->_getParam('customerId');
        $customer = Ecart::single('account/customer')
            ->find($customerId)->current();
        //title
        $this->view->pageTitle = Ecart::translate('account')->__('Edit Customer')
            . ' :' . Ecart::single('account/customer')->getEmail($customerId);

        //view personal information
        if (empty($customer) 
            && !$customer instanceof Ecart_Account_Model_Customer_Row) {

            return;
        }
        
        $this->view->customer = $customer->toArray();
        $this->view->status = $customer->is_active ? 'Enable': 'Disable';
        
        $this->view->siteName = Ecart_Collect_Site::getName($customer->site_id);
        $this->view->siteId =  $customer->site_id;
        $this->view->groupName = Ecart_Collect_GroupName::getName(
            $customer->group_id
        );
        
        //Account Information edit
        $this->view->fieldGroups = array();
        $this->view->languageId = $this->_langId;
        $this->view->languages = Ecart_Collect_Language::collect();
        $fieldGroup = Ecart::single('account/Customer_FieldGroup');
        $this->view->fieldGroups = $fieldGroup->getGroups($this->_langId);
        
        $data = array(
            'email' => $customer->email,
            'is_active' => $customer->is_active,
            'site_id' => $customer->site_id,
            'firstname' => $customer->firstname,
            'lastname' => $customer->lastname
        );
        $rows = $customer->findDependentRowset(
            'Ecart_Account_Model_Customer_Detail'
        )->toArray();

        foreach ($rows as $row) {
            if (!isset($data['field_' . $row['customer_field_id']])) {
                $data['field_' . $row['customer_field_id']] = empty($row['data']) ?
                $row['customer_valueset_value_id'] : $row['data'];
            } else {
                if (!is_array($data['field_' . $row['customer_field_id']]))
                    $data['field_' . $row['customer_field_id']] =
                        array($data['field_' . $row['customer_field_id']]);
                $data['field_' . $row['customer_field_id']][] =
                    $row['customer_valueset_value_id'];
            }
        }
        
        $form = Ecart::single('account/form_customer');
        $form->setDefaults($data);
        $this->view->form = $form;

        // addresses editing
        $this->view->defaultBillingAddresId = $customer->default_billing_address_id;
        $this->view->defaultShippingAddresId = $customer->default_shipping_address_id;
        $addresses = Ecart::single('account/customer_address');
        $alist = $addresses->fetchAll(
        $this->db->quoteInto('customer_id = ?', $customerId)
        )->toArray();
        $j = 0;
        for ($i = 0, $n = count($alist); $i < $n; $i++) {
            if ($alist[$i]['id'] == $customer->default_billing_address_id ||
            $alist[$i]['id'] == $customer->default_shipping_address_id)
            {
                list($alist[$i], $alist[$j]) = array($alist[$j], $alist[$i]);
                ++$j;
            }
        }

        $this->view->countries = Ecart_Collect_Country::collect();
        $this->view->zones = Ecart_Collect_Zone::collect();
        $this->view->address = $alist;
        $params = array(
            'getProductNames' => true,
            'customerId' => $customerId,
            'languageId' => $this->_langId
        );
        //orders
        $orderStatuses = array();
        $orderStatuses = Ecart_Collect_OrderStatusText::collect();

        $orders = Ecart::single('sales/order')->fetchAll(
            $this->db->quoteInto('customer_id = ?', $customerId)
        )->toArray();

        $sites = Ecart_Collect_Site::collect();
        foreach ($orders as &$order) {
          $order['site_id'] = $sites[$order['site_id']];

          $order['order_status_id'] =  isset($orderStatuses[$order['order_status_id']]) ?
            $orderStatuses[$order['order_status_id']] : 'add order status translation';
        }
        $this->view->orders = $orders;
        //shopping cart 
        $cart = Ecart::single('checkout/cart')->getCustomerCart($customerId);
        if ($cart) {
            $products = $cart->getProducts();
            foreach ($products as &$product) {
                $this->view->attributes = $product['attributes'];
                $product['attributes'] =
                    $this->view->render('customer-index/sc-attributes.phtml');
            }
            $this->view->products = $products;
        } else {
            $this->view->products = array();
        }
        
        //wishlist 
        $this->view->wishlist = Ecart::single('account/wishlist')
            ->getList($params);
        
        //review
        $reviewData = Ecart::single('community/review')->getBackList(
            array('cr.customer_id = ' . $customerId), 'cr.date_created', 'DESC'
        );
        $this->view->reviews = $reviewData['reviews'];
        //tags
        $this->view->tags = Ecart::single('tag/customer')
            ->getMyWithWeight($params);
        
        $this->render();
    }

    public function saveAllAction()
    {
        $this->_helper->layout->disableLayout();
        $data = $this->_getAllParams();
        /*
         * Save AddressBook
         */
        //@todo validate all address
        $modelAddress = Ecart::single('account/customer_address');
        //insert new and update
        $addresses = $data['address'];
        $customerId = $data['customerId'];
        if (count($addresses) && isset($data['customerId'])) {
            foreach ($addresses as $id => $address) {
                $info = array(
                    'customer_id'    => $customerId,
                    'gender'         => isset($address['gender']) ?
                        $address['gender']:'',
                    'company'        => $address['company'],
                    'firstname'      => $address['firstname'],
                    'lastname'       => $address['lastname'],
                    'street_address' => $address['street_address'],
                    'postcode'       => $address['postcode'],
                    'city'           => $address['city'],
                    'state'          => isset($address['state']) ?
                        $address['state']: '',
                    'country_id'     => $address['country_id'],
                    'zone_id'        => isset($address['zone_id']) ?
                        intval($address['zone_id']) : 0
                );
                if ($id < 0){
                    $newAddressId = $modelAddress->insert($info);
                    if ($data['defaultBillingAddressId'] == $id)
                    $data['defaultBillingAddressId'] = $newAddressId;
                    if ($data['defaultShippingAddressId'] == $id)
                    $data['defaultShippingAddressId'] = $newAddressId;
                }
                else
                $modelAddress->update($info, array('id = ' . (int) $id,
                            'customer_id = ' . (int) $customerId));
            }
        }
        // Delete Addresses
        $deleteIds = Zend_Json::decode($data['deleteAddressesId']);
        if (count($deleteIds))
        $modelAddress->delete($this->db->quoteInto('id IN(?)', $deleteIds));
            
        // change default shipping and billing address
        $modelCustomer = Ecart::single('account/customer');
        
        if (isset($data['defaultBillingAddressId'])) {
                
            $modelCustomer->update(array(
                'default_billing_address_id' => $data['defaultBillingAddressId']
            ), 'id = ' .$customerId);
        }

        if (isset($data['defaultShippingAddressId'])) {
            $modelCustomer->update(array(
                'default_shipping_address_id' => $data['defaultShippingAddressId']
            ), 'id = ' . $customerId);
        }
        //account information 
        $form = Ecart::single('account/form_customer');
        if ($customerId) {
            $customer = $modelCustomer->find($customerId)->current();
            $customer->email = $data['email'];
            if (!empty($data['password'])) {
                $customer->password = md5($data['password']);
            }
            $customer->is_active = isset($data['is_active']) ?
                $data['is_active'] : 1 ;
            $customer->site_id = $data['site_id'];
            $customer->firstname = $data['firstname'];
            $customer->lastname = $data['lastname'];
            $customer->save();
        } else {
            $customerId = Ecart::single('account/customer')->insert(array(
                'email'    => $data['email'],
                'password' => md5($data['password']),
                'site_id'  => $data['site_id'],
                'is_active' => $data['is_active'],
                'firstname' => $data['firstname'],
                'lastname' => $data['lastname']
            ));
        }
        $details = Ecart::single('account/customer_detail');
        // clear old data
        $details->delete(
            $this->db->quoteInto('customer_id = ?', $customerId)
        );
        // insert new data 
       
        foreach ($data as $id => $value) {
            if (false === strpos($id, 'field_') || empty($value))
                continue;
            $array = explode('_', $id);
            $id = (int) array_pop($array);
            $row = array(
                'customer_id' => $customerId,
                'customer_field_id' => $id
            );
            
            if (is_string($value)) {
                if ($form->getElement("field_$id") instanceof Zend_Form_Element_Multi) {
                    $row['customer_valueset_value_id'] = $value;
                } else {
                    $row['data'] = $value;
                }
                $details->insert($row);
            } elseif (is_array($value)) {
                foreach ($value as $v) {
                    $row['customer_valueset_value_id'] = $v;
                    $details->insert($row);
                }
            }
        }
        return $this->_helper->json->sendSuccess();
    }
}