<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Discount_IndexController extends Ecart_Admin_Controller_Back
{
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('discount')->__(
            'Discounts'
        );
        $this->render();
    }

    public function listAction()
    {
        $alpha = new Ecart_Filter_DbField();

        $params = array(
            'start' => (int) $this->_getParam('start', 0),
            'limit' => (int) $this->_getParam('limit', 25),
            'sort' => $alpha->filter($this->_getParam('sort', 'id')),
            'dir' => $alpha->filter($this->_getParam('dir', 'ASC'))
        );
        $displayMode = $this->_getParam('displayMode', 'without-special');
        switch ($displayMode) {
            case 'only-special':
                $params['special'] = 1;
                break;
            case 'without-special':
                $params['special'] = 0;
                break;
            default:
                break;
        }

        $this->_helper->json->sendSuccess(array(
            'data' => Ecart::single('discount/discount')->getList($params),
            'count' => Ecart::single('discount/discount')->getCount($params)
        ));
    }

    private function _initForm()
    {
        $this->view->sites = Ecart_Collect_Site::collect();
        $this->view->customerGroups = Ecart_Collect_CustomerGroup::collect();
        $this->view->manufactures = Ecart_Collect_Manufacturer::collect();
        $this->view->categoryTrees = Ecart::single('catalog/category')
            ->getFlatTree($this->_langId);
        $this->view->attributes = Ecart::single('catalog/product_option')
            ->getValueSets($this->_langId);
    }

    public function editAction()
    {
        $this->view->pageTitle = Ecart::translate('discount')->__(
            'Edit Discount'
        );
        $this->_initForm();

        $discount = Ecart::single('discount/discount')
            ->find($this->_getParam('id', 0))
            ->current();
            
        if (!$discount instanceof Ecart_Db_Table_Row) {
            $this->_redirect('/discount_index/create');
        }
        $this->view->discount = $discount->getCustomInfo();
        $this->render('form-discount');
    }

    public function createAction()
    {
        $this->view->pageTitle = Ecart::translate('discount')->__(
            'Add New Discount'
        );
        $this->_initForm();
        $this->render('form-discount');
    }

    public function saveAction()
    {
        $this->layout->disableLayout();
        $params = $this->_getAllParams();
        $discountId = Ecart::single('discount/discount')->save($params);

        Ecart::message()->addSuccess(
            Ecart::translate('discount')->__(
                "Discount '%s' successefull saved", $params['discountName']
            )
        );
        $this->_helper->json->sendSuccess(array(
            'id' => $discountId
        ));
    }

    public function deleteAction()
    {
        $this->layout->disableLayout();
        $discountIds = Zend_Json::decode($this->_getParam('data'));
        if (!sizeof($discountIds)) {
            return;
        }

        $this->_helper->json->sendJson(array(
            'success' => (bool) Ecart::single('discount/discount')->delete(
                $this->db->quoteInto('id IN(?)', $discountIds)
            )
        ));
    }
}