<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Gbase_IndexController extends Ecart_Admin_Controller_Back 
{
    /**
     *
     * @var Zend_Gdata_Gbase
     */
    private $_service;
    
    private function _getProductAttributes(&$product, $language)
    {
        $optionIds = array();
        $optionValueIds = array();
        $attributes = array();
        foreach ($product->findDependentRowset(
                'Ecart_Catalog_Model_Product_Attribute', 'Product') as $attrRow) {

            /* collect attribute value */
            $option = $attrRow->findParentRow(
                'Ecart_Catalog_Model_Product_Option', 'Option'
            );
            if ($option->isInputable() && !$attrRow->isModifier()) {
                foreach ($attrRow->findDependentRowset(
                        'Ecart_Catalog_Model_Product_Attribute_Value') as $value) {

                    $attributes[$attrRow->id]['values'] = $value->attribute_value;
                }
            }
            /* if attr hasn't defined value */
            if (!$attrRow->product_option_value_id 
                && !isset($attributes[$attrRow->id]['values']))
            {
                continue;
            }
            /* collect options to $attributes */
            $optionIds[$attrRow->product_option_id] = $attrRow->product_option_id;
            $optionValueIds[$attrRow->product_option_value_id] = $attrRow->product_option_value_id;

            $attributes[$attrRow->id]['id'] = $attrRow->id;
            $attributes[$attrRow->id]['product_option_id'] = $attrRow->product_option_id;
            $attributes[$attrRow->id]['product_option_value_id'] = $attrRow->product_option_value_id;
        }
        /* collect & fill labels */
        // option_text
        if (sizeof($optionIds)) {
            $where = array(
                $this->db->quoteInto('product_option_id IN(?)', $optionIds),
                $this->db->quoteInto('language_id = ?', $language)
            );
            $optionText = Ecart::single('catalog/product_option_text')
                ->fetchAssoc(null, $where);
        }
        // option_values
        if (sizeof($optionValueIds)) {
            $tableOptionValueText = Ecart::single('catalog/product_option_value_text');
            $where = array(
                $this->db->quoteInto(
                    'product_option_value_id IN(?)', $optionValueIds
                ),
                $this->db->quoteInto('language_id = ?', $language)
            );
            $optionValueText = $tableOptionValueText->fetchAssoc(null, $where);
        }
        foreach ($attributes as $attrId => &$attrRow) {
            $attrRow['optionName'] = $optionText[$attrRow['product_option_id']]['name'];
            if ($attrRow['product_option_value_id']) {
                $attrRow['valueName'] = 
                    $optionValueText[$attrRow['product_option_value_id']]['name'];
            }
        }
        return $attributes;
    }
    
    private function _getProductId($categoryId, $recursive)
    {
        return Ecart::single('catalog/product')->getNextImportProductId(
            $categoryId,
            Ecart::session()->last_exported,
            $recursive
        );
    }
    
    private function _validateLocale($country, $language, $currency)
    {
        switch ($country){
            case 'US':
                    if ($language != 'en' || $currency != 'usd')
                return false;
            case 'DE':
                    if ($language != 'de' || $currency != 'eur')
                return false;
            case 'GB':
                    if ($language != 'en' || $currency != 'gbp')
                return false;
        }
    }
    
    /**
     * Retrieve site url including baseurl
     *
     * @param int $siteId
     * @return string|bool
     */
    private function _getSiteUrl($siteId)
    {
        if ($site = Ecart::single('core/site')->find($siteId)->current()) {
            return $site->base;
        }
        return false;
    }
    
    private function _getAuthSubClient()
    { 
        if (!isset($_SESSION['sessionToken']) && !isset($_GET['token'])) {
                $nextUrl = Ecart::getCurrentUrl();
                $scope = 'http://www.google.com/base/feeds/items';
                $secure = false;
                $session = true;

                $authSubUrl = Zend_Gdata_AuthSub::getAuthSubTokenUri(
                    $nextUrl, $scope, $secure, $session
                );

                header("HTTP/1.0 307 Temporary redirect");
                header("Location: " . $authSubUrl);

                exit();
        }

        if (!isset($_SESSION['sessionToken']) && isset($_GET['token'])) {
                $_SESSION['sessionToken'] =
                    Zend_Gdata_AuthSub::getAuthSubSessionToken($_GET['token']);
        }

        $client = Zend_Gdata_AuthSub::getHttpClient($_SESSION['sessionToken']);

        return $client;
    }

    private function _getService()
    {
        $authType = Ecart::config()->gbase->auth->connection;
        try {
            if ($authType == 'ClientLogin') {
                $this->_service = Zend_Gdata_Gbase::AUTH_SERVICE_NAME;
                $user = Ecart::config()->gbase->auth->login;
                $password = Ecart::config()->gbase->auth->password;
                $client = Zend_Gdata_ClientLogin::getHttpClient(
                    $user, $password, $this->_service
                );
            } else {
                $client = $this->_getAuthSubClient();
            }
            $this->_service = new Zend_Gdata_Gbase($client);
        } catch (Exception $e) {
            Ecart::message()->addError($this->view->escape($e->getMessage()));
            try {
                $log = new Zend_Log(new Zend_Log_Writer_Stream(
                    Ecart::config()->system->path . '/var/logs/gbase.log'
                ));
                $log->err($e->getMessage());
            } catch (Exception $e) { 
                Ecart::message()->addError($e->getMessage());
            }
        }
    }

    private function _insert($productId, &$params)
    {
        $dryRun = Ecart::config()->gbase->main->dryRun;

        $tableLanguage = Ecart::single('locale/language');
        $currentLanguage = $tableLanguage->find($params['language'])->current();
        
        /* Get local product data */
        $tableProduct = Ecart::single('catalog/product');
        $tableManufacture = Ecart::single('catalog/product_manufacturer');
        
        $currencyModel = Ecart::single('locale/currency');
        $zendCurrency = $currencyModel->getCurrencyObject($params['currency']);
        $rate = $currencyModel->getData($params['currency'], 'rate');
        $currencyOptions = array(
            'position' => Ecart_Currency::RIGHT,
            'display'  => Ecart_Currency::USE_SHORTNAME
        );
        
        $product = $tableProduct->find($productId)->current();
        $productUrl = $product->getHumanUrl();
        $productDescription = $tableProduct->getProductInfo(
            $productId, $params['language']
        );
        if ($product->manufacturer_id != '0') {
            $productManufacture = $tableManufacture->find(
                $product->manufacturer_id
            )->current();
        }
            
        $productCategories = $tableProduct->getProductCategories(
            $productId, $params['language']
        );
            
        $entry = $this->_service->newItemEntry();
        
        /* Atom namespace */
        $entry->title = $this->_service->newTitle(
            trim($productDescription['name'])
        );
        $content = $productDescription['description'] == '' ? 
            Ecart::translate('catalog')->__(
                'No description available'
            ) : trim($productDescription['description']);
        $entry->content = $this->_service->newContent($content);
        $entry->content->type = 'text';
        $entry->itemType = Ecart::config()->gbase->main->itemType;
        
        if (Ecart::config()->gbase->main->link == 'Website' 
            && $siteUrl = $this->_getSiteUrl($params['site'])) {

            $link = new Zend_Gdata_App_Extension_Link();
            $link->setHref(
                preg_replace('/\s+/', '%20', trim(
                    $siteUrl .
                    '/' . $this->view->catalogUrl . '/' .
                    $productUrl
                ))
            );
            $link->setRel('alternate');
            $link->setType('text/html');
            $link->setTitle(trim($productDescription['name']));
            $linkArray[] = $link;
            $entry->setLink($linkArray);
        } else {
            $entry->addGbaseAttribute(
                'payment_accepted', 'Google Checkout', 'text'
            );
        }
            
        /* G namespace */
        //$price = $zendCurrency->toCurrency($product->price * $rate, $currencyOptions);
        $price = $product->price * $rate;
        $entry->addGbaseAttribute('price', $price, 'floatUnit');
        $entry->addGbaseAttribute('condition', 'new', 'text');
        $entry->addGbaseAttribute(
            'id', $product->sku . '_' . $params['country'], 'text'
        );
        $entry->addGbaseAttribute('quantity', (int)$product->quantity, 'int');
        $entry->addGbaseAttribute(
            'weight', "$product->weight lbs", 'numberUnit'
        ); //@TODO get weight unit
        if ($productManufacture) {
            $entry->addGbaseAttribute(
                'brand', $productManufacture->name, 'text'
            );
        }
        $entry->addGbaseAttribute(
            'target_country', $params['country'], 'text'
        );
        $entry->addGbaseAttribute(
            'item_language', $currentLanguage->code, 'text'
        );
        if ($siteUrl) {
            $entry->addGbaseAttribute(
                'image_link', 
                preg_replace('/\s+/', '%20', trim(
                    $siteUrl . 
                    '/media' . '/product' .
                    $product->image_base
                )),
                'url'
            );
        }
        foreach ($productCategories as $category) {
            $entry->addGbaseAttribute(
                'product_type', $category['category_name'], 'text'
            );
        }
        foreach (Ecart::config()->gbase->main->payment as $payment) {
            if ($payment != '')
                $entry->addGbaseAttribute('payment', $payment, 'text');
        }
        if (Ecart::config()->gbase->main->notes != '') {
            $entry->addGbaseAttribute(
                'payment_notes', Ecart::config()->gbase->main->notes, 'text'
            );
        }
        if (Ecart::config()->gbase->main->application != '') {
            $entry->addGbaseAttribute(
                'application', Ecart::config()->gbase->main->application, 'text'
            );
        }
            
        $attributes = $this->_getProductAttributes(
            $product, $params['language']
        );
            
        foreach ($attributes as $attr_id => $attribute) {
            $attrName = $attribute['optionName'];
            $attrValue = isset($attribute['valueName']) ?
                $attribute['valueName'] : $attribute['values'];
            $entry->addGbaseAttribute(
                preg_replace('/\s+/', '_', $attrName), $attrValue, 'text'
            );
        }
            
        /* Private attributes */
        $entry->addGbaseAttribute('site', $params['site'], 'int');
        
        $array = $entry->getExtensionElements();

        $privateAttr = array();
        $privateAttr[0]['name'] = 'access';
        $privateAttr[0]['value'] = 'private';
        $privateAttr[1]['name'] = 'type';
        $privateAttr[1]['value'] = 'int';
        
        $localId = new Zend_Gdata_App_Extension_Element(
            'local_id', 'g', 'http://base.google.com/ns/1.0', $product->id
        );
        $localId->setExtensionAttributes($privateAttr);

        $array[count($array)] = $localId;
        
        $entry->setExtensionElements($array);
        
        Ecart::session()->last_exported = $productId;
        Ecart::session()->processed_count++;
        
        try {
            $this->_service->insertGbaseItem($entry, $dryRun);
            Ecart::session()->imported_count++;
        } catch (Exception $e) {
            Ecart::message()->addError($this->view->escape($e->getMessage()));
            try {
                $log = new Zend_Log(new Zend_Log_Writer_Stream(
                    Ecart::config()->system->path . '/var/logs/gbase.log'
                ));
                $log->err($e->getMessage());
            } catch (Exception $e) { 
                Ecart::message()->addError($e->getMessage());
            }
        }
    }
    
    private function _update($productId, &$params)
    {
        $dryRun = Ecart::config()->gbase->main->dryRun;

        $tableLanguage = Ecart::single('locale/language');
        $currentLanguage = $tableLanguage->find($params['language'])->current();
        
        /* Get local product data */
        $tableProduct = Ecart::single('catalog/product');
        $tableManufacture = Ecart::single('catalog/product_manufacturer');
        
        $currencyModel = Ecart::single('locale/currency');
        $zendCurrency = $currencyModel->getCurrencyObject($params['currency']);
        $rate = $currencyModel->getData($params['currency'], 'rate');
        $currencyOptions = array(
            'position' => Ecart_Currency::RIGHT,
            'display'  => Ecart_Currency::USE_SHORTNAME
        );
        
        $product = $tableProduct->find($productId)->current();
        $productUrl = $product->getHumanUrl();
        $productDescription = $tableProduct->getProductInfo(
            $productId, $params['language']
        );
        if ($product->manufacturer_id != '0') {
            $productManufacture = $tableManufacture
                ->find($product->manufacturer_id)->current();
        }
            
        $productCategories = $tableProduct->getProductCategories(
            $productId, $params['language']
        );
            
        $entry = $this->_service->getGbaseItemEntry($params['id']);
        
        //error_log(Zend_Debug::dump($entry));
        //exit();
        
        /* Atom namespace */
        $entry->title = $this->_service->newTitle(
            trim($productDescription['name'])
        );
        $content = $productDescription['description'] == '' ? 
            Ecart::translate('catalog')->__(
                'No description available'
            ) : trim($productDescription['description']);
        $entry->content = $this->_service->newContent($content);
        $entry->content->type = 'text';
        $entry->itemType = Ecart::config()->gbase->main->itemType;
        
        /*$editLink = new Zend_Gdata_App_Extension_Link();
        $editLink->setHref($params['id']);
        $editLink->setRel('edit');
        $linkArray[] = $editLink;*/
        
        if (Ecart::config()->gbase->main->link == 'Website' 
            && $siteUrl = $this->_getSiteUrl($params['site'])) {

            $link = new Zend_Gdata_App_Extension_Link();
            $link->setHref(
                preg_replace('/\s+/', '%20', trim(
                    $siteUrl . 
                    '/' . $this->view->catalogUrl . '/' .
                    $productUrl
                ))
            );
            $link->setRel('alternate');
            $link->setType('text/html');
            $link->setTitle(trim($productDescription['name']));
            $linkArray[] = $link;
        } else {
            $entry->addGbaseAttribute(
                'payment_accepted', 'Google Checkout', 'text'
            );
        }
        
        $entry->setLink($linkArray);    
        /*
         * G namespace
         */
        //$price = $zendCurrency->toCurrency($product->price * $rate, $currencyOptions);
        $price = $product->price * $rate;
        $entry->addGbaseAttribute('price', $price, 'floatUnit');
        $entry->addGbaseAttribute('condition', 'new', 'text');
        $entry->addGbaseAttribute(
            'id', $product->sku . '_' . $params['country'], 'text'
        );
        $entry->addGbaseAttribute('quantity', $product->quantity, 'int');
        $entry->addGbaseAttribute(
            'weight', "$product->weight lbs", 'numberUnit'
        ); //@TODO get weight unit
        if ($productManufacture) {
            $entry->addGbaseAttribute(
                'brand', $productManufacture->name, 'text'
            );
        }
        $entry->addGbaseAttribute(
            'item_language', $params['language_code'], 'text'
        );
        $entry->addGbaseAttribute(
            'image_link', 
            preg_replace('/\s+/', '%20', trim(
                $siteUrl . 
                '/media' . '/product' .
                $product->image_base
            )), 
            'url'
        );
        foreach ($productCategories as $category) {
            $entry->addGbaseAttribute(
                'product_type', $category['category_name'], 'text'
            );
        }
        foreach (Ecart::config()->gbase->main->payment as $payment) {
            if ($payment != '')
                $entry->addGbaseAttribute('payment', $payment, 'text');
        }
        if (Ecart::config()->gbase->main->notes != '') {
            $entry->addGbaseAttribute(
                'payment_notes', Ecart::config()->gbase->main->notes, 'text'
            );
        }
        if (Ecart::config()->gbase->main->application != '') {
            $entry->addGbaseAttribute(
                'application', Ecart::config()->gbase->main->application, 'text'
            );
        }
            
        $attributes = $this->_getProductAttributes(
            $product, $params['language']
        );
            
        foreach ($attributes as $attrId => $attribute) {
            $attrName = $attribute['optionName'];
            $attrValue = isset($attribute['valueName']) ?
                $attribute['valueName'] : $attribute['values'];
            $entry->addGbaseAttribute(
                preg_replace('/\s+/', '_', $attrName), $attrValue, 'text'
            );
        }
            
        /* Private attributes */
        $entry->addGbaseAttribute('site', $params['site'], 'int');
        
        $array = $entry->getExtensionElements();

        $privateAttr = array();
        $privateAttr[0]['name'] = 'access';
        $privateAttr[0]['value'] = 'private';
        $privateAttr[1]['name'] = 'type';
        $privateAttr[1]['value'] = 'int';
        
        $localId = new Zend_Gdata_App_Extension_Element(
            'local_id', 'g', 'http://base.google.com/ns/1.0', $product->id
        );
        $localId->setExtensionAttributes($privateAttr);
        
        $array[count($array)] = $localId;
        
        $entry->setExtensionElements($array);
        
        //error_log(Zend_Debug::dump($entry->saveXML(), 0, false));
        
        Ecart::session()->last_exported = $productId;
        Ecart::session()->processed_count++;
        
        try {
            //$entry->save($dryRun);
            $this->_service->updateGbaseItem($entry, $dryRun);
            Ecart::session()->imported_count++;
        } catch (Exception $e) {
            Ecart::message()->addError($this->view->escape($e->getMessage()));
            try {
                $log = new Zend_Log(new Zend_Log_Writer_Stream(
                    Ecart::config()->system->path . '/var/logs/gbase.log'
                ));
                $log->err($e->getMessage());
            } catch (Exception $e) { 
                Ecart::message()->addError($e->getMessage());
            }
        }
    }
    
    private function _setControl($id, &$control)
    {
        $entry = $this->_service->getEntry($id);
        $entry->setControl($control);
        
        Ecart::session()->processed_count++;
            
        try {
            $entry->save();
            Ecart::session()->imported_count++;
        } catch (Exception $e) {
            Ecart::message()->addError($this->view->escape($e->getMessage()));
            try {
                $log = new Zend_Log(new Zend_Log_Writer_Stream(
                    Ecart::config()->system->path . '/var/logs/gbase.log'
                ));
                $log->err($e->getMessage());
            } catch (Exception $e) {
                Ecart::message()->addError($e->getMessage());
            }
        }
    }
    
    private function _delete($id)
    {
        $entry = $this->_service->getEntry($id);
        
        Ecart::session()->processed_count++;

        try {
            $entry->delete();
            Ecart::session()->imported_count++;
        } catch (Exception $e) {
            Ecart::message()->addError($this->view->escape($e->getMessage()));
            try {
                $log = new Zend_Log(
                    new Zend_Log_Writer_Stream(
                        Ecart::config()->system->path . '/var/logs/gbase.log'
                    )
                );
                $log->err($e->getMessage());
            } catch (Exception $e) { 
                Ecart::message()->addError($e->getMessage());
            }
        }
    }
    
    public function init()
    {
        parent::init();
        $this->_getService();
    }

    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('admin')->__('Google Base');

        $this->view->gCountries = array(
            'US' => 'United States',
            'DE' => 'Germany',
            'GB' => 'United Kingdom'
        );

        /* Get available currencies from store */
        $currencyModel = Ecart::single('locale/currency');
        $currencies = $currencyModel->getList();
        $result = array();
        foreach ($currencies as $currency) {
            $result[$currency['code']] = $currency['title'];
        }

        $this->view->currencies = $result;

        $this->render();
    }
    
    public function getGbaseDataAction()
    {
        $this->_helper->layout->disableLayout();
        
        $filters = $this->_getAllParams();
        
        /*
         * Items count query
         */
        $count_query = $this->_service->newItemQuery();
        $count_query->setContent('stats');
        
        $bq = '';
        if (isset($filters['country']) && $filters['country'] != '')
            $bq .= "[target country: $filters[country]]";
        if (isset($filters['site_id']) && $filters['site_id'] != '')
            $bq .= "[site: $filters[site_id]]";
        if (isset($filters['product_type']) && $filters['product_type'] != '')
            $bq .= "[product_type: $filters[product_type]]";
        if ($bq != '')
            $count_query->setBq($bq);
            
        $count_feed = $this->_service->getGbaseItemFeed($count_query);
        $total_count = $count_feed->getTotalResults();
        
        /*
         * Items content query
         */
        $query = $this->_service->newItemQuery();
        $query->setContent('attributes,meta');
        
        if ($bq != '')
            $query->setBq($bq);
        
        $query->setMaxResults(isset($filters['limit']) ?
            $filters['limit'] : 25);
        $query->setStartIndex(isset($filters['start']) ?
            $filters['start'] + 1 : 1);
        
        $orderBy = '';
        switch ($filters['sort']) {
            case 'price':
                switch ($filters['country']) {
                    case 'US':
                        $orderBy = 'price(float usd)';
                        break;
                    case 'DE':
                        $orderBy = 'price(float eur)';
                        break;
                    case 'GB':
                        $orderBy = 'price(float gbp)';
                        break;
                    default:
                        $orderBy = '';
                        break;
                }
                break;
            case 'modification_time':
                $orderBy = "$filters[sort]";
                break;
            default:
                $orderBy = "$filters[sort]($filters[sortType])";
                break;
        }
        if ($orderBy != '') {
            $query->setOrderBy($orderBy);
            $query->setSortOrder($filters['dir'] == 'ASC' ?
                'ascending' : 'descending');
        }
        
        try {
            $feed = $this->_service->getGbaseItemFeed($query);
        } catch (Exception $e) {
            Ecart::message()->addError($this->view->escape($e->getMessage()));
            try {
                $log = new Zend_Log(
                    new Zend_Log_Writer_Stream(
                        Ecart::config()->system->path . '/var/logs/gbase.log'
                    )
                 );
                $log->err($e->getMessage());
            } catch (Exception $e) {
                Ecart::message()->addError($e->getMessage());
            }
            exit();
        }
        $result = array();
        $i = 0;
        foreach ($feed->entries as $entry) {
          
            $result[$i]['title'] = $entry->title->getText();
            $result[$i]['id'] = $entry->id->getText();
            $result[$i]['modification_time'] = substr(
                $entry->updated->getText(), 0, 10
            );
            
            /*
             * Base attributes
             */
            $price = $entry->getGbaseAttribute('price');
            $quantity = $entry->getGbaseAttribute('quantity');
            $expiration_date = $entry->getGbaseAttribute('expiration_date');
            $language = $entry->getGbaseAttribute('item_language');
            $result[$i]['price'] = $price[0]->text;
            $result[$i]['currency'] = substr($price[0]->text, -3);
            $result[$i]['quantity'] = $quantity[0]->text;
            $result[$i]['expiration_date'] = substr(
                $expiration_date[0]->text, 0, 10
            );
            $result[$i]['language'] = $language[0]->text;
            
            /*
             * Private attributes
             */
            $localId = $entry->getGbaseAttribute('local_id');
            $site = $entry->getGbaseAttribute('site');
            $result[$i]['local_id'] = $localId[0]->text;
            $result[$i]['site_id'] = $site[0]->text;
            
            /*
             * Extension Attributes (getting clicks, imressions, page_views)
             */
            $extensionElements = $entry->getExtensionElements();
            
            foreach ($extensionElements as $extElement) {
                $innerElements = $extElement->getExtensionElements();
                foreach ($innerElements as $innerElement)  {
                    $elName = $innerElement->rootElement;
                    $attributes = $innerElement->getExtensionAttributes();
                    foreach ($attributes as $aName => $aValue) {
                        $result[$i][$elName] = $aValue['value'];
                    }
                }
            }
          
            /*
             * Control attributes (draft, dissaproved, published)
             */
            $control = $entry->getControl();
            $result[$i]['status'] = 'published';
            if (isset($control)) {
                $draftElement = $control->draft;
                $elName = $draftElement->rootElement;
                $result[$i]['status'] = $elName;
              
                $extensionElements = $control->getExtensionElements();
                foreach ($extensionElements as $extElement) {
                    $elName = $extElement->rootElement;
                    $result[$i]['status'] = $elName;
                }
            }  
          
            $i++;
        }
        
        $this->_helper->json->sendJson(array(
            'total_count' => $total_count->getText(),
            'feed' => $result
        ));
    }
    
    public function setStatusAction()
    {
        $timeStartScript = time();

        $this->_helper->layout->disableLayout();
        
        $disabled = $this->_getParam('draft');
        $clearSession = $this->_getParam('clearSession');
        $items = Zend_Json::decode($this->_getParam('items'));
        
        $draft   = new Zend_Gdata_App_Extension_Draft($disabled);
        $control = new Zend_Gdata_App_Extension_Control($draft);
        
        if ($clearSession) {
            unset(Ecart::session()->last_exported);
            Ecart::session()->processed_count = 0;
            Ecart::session()->imported_count = 0;
        }

        $timeExport = 0;
        $last = false;
    
        while(true){
            $timeStartItem = time();
            
            if (Ecart::session()->processed_count == count($items)) {
               $last = true; 
               break;
            }
            
            $this->_setControl(
                $items[Ecart::session()->processed_count], $control
            );
            
            $timeEndItem = time();
            $timeSript = $timeEndItem - $timeStartScript;
            $timeExportItem = max($timeEndItem - $timeStartItem, $timeExport);
            if (($timeSript + $timeExportItem) > 20) {
                break;
            }
        }
        
        $messages = array();
        if ($last) {

            Ecart::message()->addSuccess(
               Ecart::translate('admin')->__(
                   'Status was updated successfully %d item(s) was updated. %d item(s) was skipped',
                    Ecart::session()->imported_count,
                    count($items) - Ecart::session()->imported_count
                )
            );
        }
        
        $this->_helper->json->sendSuccess(array(
            'finalize' => $last,
            'processed' => Ecart::session()->imported_count,
            'count' => count($items)
        ));
    }
    
    public function deleteAction()
    {
        $timeStartScript = time();

        $this->_helper->layout->disableLayout();
        
        $items = Zend_Json::decode($this->_getParam('items'));
        $clearSession = $this->_getParam('clearSession');

        if ($clearSession) {
            unset(Ecart::session()->last_exported);
            Ecart::session()->processed_count = 0;
            Ecart::session()->imported_count = 0;
        }

        $timeExport = 0;
        $last = false;
    
        while(true){
            $timeStartItem = time();
            
            if (Ecart::session()->processed_count == count($items)) {
               $last = true; 
               break;
            }
            
            $this->_delete($items[Ecart::session()->processed_count]);
            
            $timeEndItem = time();
            $timeSript = $timeEndItem - $timeStartScript;
            $timeExportItem = max($timeEndItem - $timeStartItem, $timeExport);
            if (($timeSript + $timeExportItem) > 20) {
                break;
            }
        }
        
        $messages = array();
        if ($last) {
            Ecart::message()->addSuccess(
                Ecart::translate('admin')->__(
                    'Data was deleted successfully. %d item(s) was deleted. %d item(s) was skipped',
                    Ecart::session()->imported_count,
                    count($items) - Ecart::session()->imported_count
                )
            );
        }
        
        $this->_helper->json->sendSuccess(array(
            'finalize' => $last,
            'processed' => Ecart::session()->imported_count,
            'count' => count($items)
        ));
    }
        
    public function updateAction()
    {
        $time_start_script = time();

        $this->_helper->layout->disableLayout();

        $items = Zend_Json::decode($this->_getParam('items'));
        $params = $this->_getAllParams();

        $tableLanguage = Ecart::single('locale/language');
        
        if ($params['clearSession']) {
            unset(Ecart::session()->last_exported);
            Ecart::session()->processed_count = 0;
            Ecart::session()->imported_count = 0;
        }
        
        $time_export = 0;
        $last = false;
        
        while (true) {
            $time_start_item = time();
            
            if (Ecart::session()->processed_count == count($items)) {
               $last = true; 
               break;
            }
            
            $params['id'] = $items[Ecart::session()->processed_count]['id'];
            $params['site'] = $items[Ecart::session()->processed_count]['site'];
            $params['currency'] = $items[Ecart::session()->processed_count]['currency'];
            $params['language'] = $tableLanguage
                ->getIdByCode($items[Ecart::session()->processed_count]['language']);
            $params['language_code'] = $items[Ecart::session()->processed_count]['language'];
            
            $this->_update(
                $items[Ecart::session()->processed_count]['local_id'], $params
            );
            
            $time_end_item = time();
            $time_sript = $time_end_item - $time_start_script;
            $time_export_item = max(
                $time_end_item - $time_start_item, $time_export
            );
            if (($time_sript + $time_export_item) > 20) {
                break;
            }
        }
        
        $messages = array();
        if ($last) {
            Ecart::message()->addSuccess(
                Ecart::translate('admin')->__(
                    'Data was updated successfully. %d item(s) was updated. %d item(s) was skipped',
                    Ecart::session()->imported_count,
                    count($items) - Ecart::session()->imported_count
                )
            );
        }
        
        $this->_helper->json->sendSuccess(array(
            'finalize' => $last,
            'processed' => Ecart::session()->imported_count,
            'count' => count($items)
        ));
    }
        
    public function exportAction()
    {
        $time_start_script = time();

        $this->_helper->layout->disableLayout();
        
        $items = Zend_Json::decode($this->_getParam('items'));
        $params = $this->_getAllParams();
        
        if ($params['clearSession']) {
            unset(Ecart::session()->last_exported);
            Ecart::session()->processed_count = 0;
            Ecart::session()->imported_count = 0;
        }
        
        $time_export = 0;
        $last = false;
        
        while (true) {
            $time_start_item = time();

            if (Ecart::session()->processed_count == count($items)) {
               $last = true; 
               break;
            }

            $this->_insert($items[Ecart::session()->processed_count], $params);
            
            $time_end_item = time();
            $time_sript = $time_end_item - $time_start_script;
            $time_export_item = max(
                $time_end_item - $time_start_item, $time_export
            );
            if (($time_sript + $time_export_item) > 20) {
                break;
            }
        }
        
        $messages = array();
        if ($last) {
            Ecart::message()->addSuccess(
                Ecart::translate('admin')->__(
                    'Export process has been completed. %d item(s) was exported. %d item(s) was skipped',
                    Ecart::session()->imported_count,
                    count($items) - Ecart::session()->imported_count
                )
            );
        }
        
        $this->_helper->json->sendSuccess(array(
            'finalize' => $last,
            'processed' => Ecart::session()->imported_count,
            'count' => count($items)
        ));
    }
    
    public function exportBranchAction()
    {
        $time_start_script = time();

        $this->_helper->layout->disableLayout();

        $params = $this->_getAllParams();

        if ($params['clearSession']) {
            unset(Ecart::session()->last_exported);
            Ecart::session()->processed_count = 0;
            Ecart::session()->imported_count = 0;
        }

        $productModel  = Ecart::single('catalog/product');
        $categoryModel = Ecart::single('catalog/category');
        $category = $categoryModel->find($params['catId'])->current();
        
        if ($category->lvl == 0) {
            $count = $productModel->countBySite($category->site_id);
        } elseif ($params['recursive']) {
            $count = $productModel->countByCategoryId(
                $category->id, 'recursive'
            );
        } else {
            $count = $productModel->countByCategoryId($category->id);
        }
        
        $time_export = 0;
        $last = false;
        
        while (true) {
            $time_start_item = time();
            $product_id = $this->_getProductId(
                $params['catId'], $params['recursive']
            );
            if (!$product_id) {
               $last = true;
               break;
            }
            $this->_insert($product_id, $params);
            $time_end_item = time();
            $time_sript = $time_end_item - $time_start_script;
            
            $time_export_item = max(
                $time_end_item - $time_start_item, $time_export
            );
                
            if (($time_sript + $time_export_item) > 20) {
                break;
            }
        }
        
        $messages = array();
        if ($last) {
            Ecart::message()->addSuccess(
                Ecart::translate('admin')->__(
                    'Export process has been completed. %d item(s) was exported. %d item(s) was skipped',
                    Ecart::session()->imported_count,
                    $count - Ecart::session()->imported_count
                )
            );
        }
        
        $this->_helper->json->sendSuccess(array(
            'finalize' => $last,
            'processed' => Ecart::session()->imported_count,
            'count' => $count
        ));
    }
    
    public function revokeTokenAction()
    {
        $this->_helper->layout->disableLayout();

        Zend_Gdata_AuthSub::AuthSubRevokeToken($_SESSION['sessionToken']);

        unset($_SESSION['sessionToken']);
    }
}