<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_IndexController extends Ecart_Admin_Controller_Back 
{
    const NULLDASHBOARDVALUE = 0/*'null'*/; 
    
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('admin')->__('Home');
        $siteId = $this->_getParam('siteId', 0);
        $this->view->siteId = $siteId;
        $date = Ecart_Date::now()->addDay(-1)->toString('YYYY-MM-dd');
        
        $currency = Ecart::single('locale/currency')->getCurrencyObject(
            Ecart::config()->main->store->currency
        );
        
        $where = array(
            'date' => $this->db->quoteInto('date_purchased_on > ? ', $date)
        );
        $where['siteId'] = '';
        if ($siteId) {
            $where['siteId'] = $this->db->quoteInto('site_id = ?', $siteId);
        } 
        $order = Ecart::single('sales/order');
        
        $this->view->orderTotal = $currency->toCurrency(
            $order->getTotal($where['siteId'])
        );
        
        $this->view->orderTotalToDay = $currency->toCurrency(
            $order->getTotal($where)
        );
        //$this->db->quoteInto(
        $this->view->orderCountToDay = $order->count(
            $where, 'COUNT(DISTINCT customer_email)'
        );
        $this->view->orderAverage = $currency->toCurrency(
            $order->count($where['siteId'])
            ? $order->getTotal($where['siteId'])/$order->count($where['siteId']) : 0);
        
        $where['customer'] = 'customer_id IS NOT NULL';
        $this->view->orderCustomerToDay = $order->count($where);
        
        $this->view->customerCount = Ecart::single('account/customer')
            ->count($where['siteId']);
        unset($where['customer']);
        $where['date'] =  $this->db->quoteInto('last_visit_at > ? ', $date);
        $this->view->visitorToDay = Ecart::single('log/visitor')->count($where);
        $where['date'] =  $this->db->quoteInto('visit_at > ? ', $date);
        $this->view->pageviewsToDay = Ecart::single('log/url')->count($where);
        $where['date'] =  $this->db->quoteInto('created_at > ? ', $date);
        $this->view->mailCountToDay =  Ecart::single('contacts/message')
            ->count($where);
        
        $this->view->sites = Ecart_Collect_Site::collect();
        $this->view->date = $date;
        $this->render();
    }
    
    protected function _dashBoardPeriod($period, $periodIndex, $type, $siteId)
    {
        function days_in_month($month, $year)
        {
            return $month == 2 ?
                ($year % 4 ?
                    28 : ($year % 100 ?
                        29 : ($year % 400 ?
                            28 : 29
                        )
                    )
                ) : (($month - 1) % 7 % 2 ? 30 : 31);
        } 
        $currentDate = array_combine(
            explode('-', 'Y-m-d-H-w-n'), 
            explode(
                '-', 
                Ecart_Date::now()->toPhpFormat()->toString('Y-m-d-H-w-n')
            )
        );
        $xTooltip = '';
        switch ($period){
            case "hour":
                $xLabel = array();
                $data = array_fill(1, 61, self::NULLDASHBOARDVALUE);
                for ($i = 0; $i < 61; $i++ ) {
                    if ($i % 2)
                        $xLabel[] =  '';
                    else 
                        $xLabel[] =  $i;
                }
                $startDate = mktime(
                    $currentDate['H'] - $periodIndex, 
                    59, 59, 
                    $currentDate['m'], 
                    $currentDate['d'],
                    $currentDate['Y']
                );
                $endDate = mktime(
                    $currentDate['H'] - $periodIndex + 1, 
                    59, 59, 
                    $currentDate['m'], 
                    $currentDate['d'],
                    $currentDate['Y']
                );
                $period = 16; //period is minute
                $xTooltip = Ecart_Date::timestamp($startDate)
                    ->toPhpFormat()->toString("F d H");
                $title = Ecart_Date::timestamp($endDate)
                    ->toPhpFormat()->toString("dS F Y H:00");
                break;
            case "week":
                $xLabel = array(
                    'Monday', 'Tuesday','Wednesday', 'Thursday',
                    'Friday', 'Saturday', 'Sunday'
                );
                $weekStart = $currentDate['d'] - $currentDate['w'];
                
                for ($i = 1; $i <= 7; $i++ ) {
                    $data[intval(date('d', mktime(
                        23, 59, 59, 
                        $currentDate['m'], 
                        $weekStart - ($periodIndex  - 1) * 7 + $i,
                        $currentDate['Y']
                    )))] = self::NULLDASHBOARDVALUE;
                }
                
                $startDate = mktime(
                    23, 59, 59, 
                    $currentDate['m'], 
                    $weekStart - ($periodIndex  - 1) * 7,
                    $currentDate['Y']
                );
                $endDate = mktime(
                    23, 59, 59, 
                    $currentDate['m'], 
                    $weekStart - ($periodIndex - 2) * 7,
                    $currentDate['Y']
                );
                $period = 10; //day
                $title = 'Week ' . Ecart_Date::timestamp($endDate)
                    ->toPhpFormat()->toString("W, Y");
                break;
            case "month":
                $xLabel = array();
                $data = array_fill(
                    1,
                    days_in_month($currentDate['n'], $currentDate['Y']),
                    self::NULLDASHBOARDVALUE
                );
                for ($i = 1;
                    $i <= days_in_month($currentDate['n'], $currentDate['Y']);
                    $i++ ) {

                    $xLabel[] = $i;
                }
                $startDate = mktime(
                    23, 59, 59, 
                    $currentDate['m'] - $periodIndex, 
                    days_in_month(
                        $currentDate['m'] - $periodIndex, $currentDate['Y']
                    ),
                    $currentDate['Y']
                );
                $endDate = mktime(
                    23, 59, 59, 
                    $currentDate['m'] - $periodIndex + 1, 
                    days_in_month(
                        $currentDate['m'] - $periodIndex + 1, $currentDate['Y']
                    ),
                    $currentDate['Y']
                );
                $period = 10; //day
                $xTooltip = Ecart_Date::timestamp(mktime(
                    23, 59, 59, 
                    $currentDate['m'] - $periodIndex + 1, 
                    1,
                    $currentDate['Y']
                ))->toPhpFormat()->toString('F');
                $title = Ecart_Date::timestamp($endDate)
                    ->toPhpFormat()->toString("F Y");
                break;
            case "year":
                $xLabel = array(
                    'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep',
                    'Oct', 'Nov', 'Des'
                );
                $data = array_fill(1, 12, self::NULLDASHBOARDVALUE);
                $startDate = mktime(
                    23, 59, 59, 12, 31, 
                    $currentDate['Y'] - $periodIndex  
                );
                $endDate = mktime(
                    23, 59, 59, 12, 31, 
                    $currentDate['Y'] - $periodIndex + 1  
                );
                
                $period = 7;
                $xTooltip = Ecart_Date::timestamp($endDate)
                    ->toPhpFormat()->toString("Y");;
                $title = $xTooltip;
                break;
            case "alltime":
                if ($siteId) {
                     $where = $this->db->quoteInto('site_id = ?', $siteId);
                }
                switch ($type) {
                    case "visit":
                        $startDate = Ecart::single('log/visitor')
                            ->getFirstDate($siteId);
                        $endDate = Ecart::single('log/visitor')
                            ->getLastDate($siteId);
                        break;
                    case "customer":
                        $startDate = Ecart::single('account/customer')
                            ->getFirstDate($siteId);
                        $endDate = Ecart::single('account/customer')
                            ->getLastDate($siteId);
                        break;
                    default:
                        $startDate = Ecart::single('sales/order')
                            ->getFirstDate($siteId);
                        $endDate = Ecart::single('sales/order')
                            ->getLastDate($siteId);
                }
                if (date('m', strtotime($startDate)) == date('m', strtotime($endDate)) 
                    && date('Y', strtotime($startDate)) == date('Y', strtotime($endDate))
                    ) 
                {
                    $period = 10;
                    $xTooltip = date('F', strtotime($endDate));
                } else {
                    $period = 7;
                }
                $startDate = mktime(
                    23, 59, 59, 
                    date('m', strtotime($startDate)) - 1 , 
                    days_in_month(
                        date('m', strtotime($startDate)) - 1,
                        date('Y', strtotime($startDate))
                    ),
                    date('Y', strtotime($startDate))
                );
                $endDate = mktime(
                    23, 59, 59, 
                    date('m', strtotime($endDate)) , 
                    days_in_month(
                        date('m', strtotime($endDate)),
                        date('Y', strtotime($endDate))
                    ),
                    date('Y', strtotime($endDate))
                );
                //$period = 10;
                $curDate = $startDate;
                while ($curDate + 24*60*60 <= $endDate ) {
                    
                    if ( $period == 10 ) {
                        $curDate += 24*60*60;
                        $xLabel[] = date('d', $curDate);
                        $data[date('Y-m-d', $curDate)] = self::NULLDASHBOARDVALUE;
                         
                    } else {
                        $curDate +=  days_in_month(
                            date('m', strtotime($curDate)) - 1, 
                            date('Y', strtotime($curDate))
                        )*24*60*60;
                        $xLabel[] = date('Y-m', $curDate);
                        $data[date('Y-m', $curDate)] = self::NULLDASHBOARDVALUE;
                       
                    }
                }
                $title = 'All time';
                break;
            default:
                $xLabel = array();
                $data = array_fill(1, 24, self::NULLDASHBOARDVALUE);
                for ($i = 1; $i <= 24; $i++ ){
                    $xLabel[] = $i;
                }
                $startDate = mktime(
                    23, 59, 59, 
                    $currentDate['m'], 
                    $currentDate['d'] - $periodIndex,
                    $currentDate['Y']
                );
                $endDate = mktime(
                    23, 59, 59, 
                    $currentDate['m'], 
                    $currentDate['d'] - $periodIndex + 1,
                    $currentDate['Y']
                );
                $period = 13;
                $xTooltip = date('F d',mktime(
                    23, 59, 59, 
                    $currentDate['m'], 
                    $currentDate['d'] - $periodIndex + 1,
                    $currentDate['Y']
                )) . ' :';
                $title = date("dS \o\f F Y", $endDate);
                
        }
        return array (
            'startDate' => $startDate,
            'endDate'   => $endDate,
            'title'     => $title,
            'period'    => $period,  
            'tooltip'   => $xTooltip,
            'label'     => $xLabel,
            'data'      => $data
            
        );
    }
    
    public function dashBoardChartAction()
    {
        $this->_helper->layout->disableLayout();
        $type = $this->_getParam('type', 'order');
        $period = $this->_getParam('period', 'day');
        $periodIndex = $this->_getParam('periodIndex', 1);
        $siteId = $this->_getParam('siteId', 0);
        if ($siteId) {
            $where['siteId'] = $this->db->quoteInto('site_id = ?', $siteId);
        }
        
        $dashBoardPeriod = $this->_dashBoardPeriod(
            $period, $periodIndex, $type, $siteId
        );
        $startDate = date('Y-m-d H:i:s', $dashBoardPeriod['startDate']);
        $endDate   = date('Y-m-d H:i:s', $dashBoardPeriod['endDate']);
        $period    = $dashBoardPeriod['period'];
        $data      = $dashBoardPeriod['data'];
        
        $yTooltip = '';
        switch ($type) {
            case "amount": 
                $where['datagt'] = "date_purchased_on > '{$startDate}'";
                $where['datalt'] = "date_purchased_on < '{$endDate}'";
                $realData = Ecart::single('sales/order')
                    ->getAmountsList($period, $where);
                $yTooltip = 'Amounting';
                break;
            case "visit": 
                $where['datagt'] = "last_visit_at > '{$startDate}'";
                $where['datalt'] = "last_visit_at < '{$endDate}'";
                $realData = Ecart::single('log/visitor')
                    ->getCountList($period, $where);
                $yTooltip = 'Visitors';
                break;
            case "customer": 
                $where['datagt'] = "created_at > '{$startDate}'";
                $where['datalt'] = "created_at < '{$endDate}'";
                $realData = Ecart::single('account/customer')
                    ->getCountList($period, $where);
                $yTooltip = 'New customers';
                break;
            case "rate":
                $where['datagt'] = "last_visit_at > '{$startDate}'";
                $where['datalt'] = "last_visit_at < '{$endDate}'"; 
                $realDataVisitor = Ecart::single('log/visitor')
                    ->getCountList($period, $where);
                $where['datagt'] = "date_purchased_on > '{$startDate}'";
                $where['datalt'] = "date_purchased_on < '{$endDate}'";
                $realDataOrder = Ecart::single('sales/order')
                    ->getCountList($period, $where, true);
                $realData = array();
                $days = array_unique(array_merge(
                    array_keys($realDataOrder), 
                    array_keys($realDataVisitor)
                ));
                foreach ($days as $day) {
                    if (empty($realDataVisitor[$day])){
                        $realDataVisitor[$day] = 1;
                    }
                    $realData[$day] = ($realDataOrder[$day] * 100) / $realDataVisitor[$day];
                    if ($realData[$day] > 100) 
                        $realData[$day] = 100; 
                }
                
                $yTooltip = 'Conversion rate';
                break;
            default:
                $where['datagt'] = "date_purchased_on > '{$startDate}'";
                $where['datalt'] = "date_purchased_on < '{$endDate}'";
                $realData = Ecart::single('sales/order')
                    ->getCountList($period, $where);
                $yTooltip = 'Orders';
        }
        //end types switch
        if ($this->_getParam('period', 'day') == 'alltime') {
            foreach ($realData as $key => $value) {
                $data[$key] = $value;
            }
        } else {
            foreach ($realData as $key => $value) {
                $data[intval(substr($key, -2))] = $value;
            }
        }
        $min = 0; $max = 20;
        if (!empty($realData)) {
            $max = max($realData);
            $min = min($realData);
        }
        if ($max === $min) {
            $min = 0;
        }
        if ($type == 'rate'){
            $min = 0;
            $max = 100;
        }
        
         
        $this->view->chart()->setData(
            $data, 
            $dashBoardPeriod['label'],//array_map(call_user_func(array(&$t, '_')), $x_label),
            $dashBoardPeriod['title'], 
            $max, $min, 
            $dashBoardPeriod['tooltip'], 
            $yTooltip
        );
        $this->getResponse()->appendBody($this->view->chart()->getData());
    }
    
    /**
     * @deprecated
     * @return unknown_type
     */
    public function changeSiteAction()
    {
        $siteId = $this->_getParam('siteId');
        $ctrlName = $this->_getParam('ctrlName');
        $this->_redirect("$ctrlName/index/siteId/$siteId");
    }
    
    public function ajaxContentAction()
    {
        $this->layout->disableLayout();
        $this->render('test-content');
    }
    
    public function infoAction()
    {
        $this->_helper->layout->disableLayout();
        phpinfo();
    }
}