<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Locale_CurrencyController extends Ecart_Admin_Controller_Back
{
    /**
     *
     * @var Ecart_Locale_Model_Currency
     */
    private $_currency;
    
    public function init()
    {
        parent::init();
        $this->_currency = Ecart::single('locale/currency');
    }
    
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('locale')->__('Currency List');
        $this->render();
    }
    
    public function listAction()
    {
        $alpha = new Zend_Filter_Alpha();
        
        $count = Ecart::single('locale/currency')->count();
        
        $dataset = Ecart::single('locale/currency')->getList();
        
        return $this->_helper->json->sendSuccess(array(
            'data' => $dataset,
            'count' => $count
        ));
    }
    
    public function batchSaveAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = Zend_Json::decode($this->_getParam('data'));
        
        if (!sizeof($data)) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'No data to save'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        $this->_currency->save($data);

        Ecart::message()->addSuccess(
            Ecart::translate('locale')->__(
                'Currency was saved successfully'
            )
        );
        return $this->_helper->json->sendSuccess();
    }
    
    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = $this->_getParam('currency');
        
        if (!sizeof($data)) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'No data to save'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        $dataToSave[$data['id']] = $data;
        
        $this->_currency->save($dataToSave);

        Ecart::message()->addSuccess(
            Ecart::translate('locale')->__(
                'Currency was saved successfully'
            )
        );
        return $this->_helper->json->sendSuccess();
    }
    
    public function deleteAction()
    {
        $this->layout->disableLayout();
        
        $data = Zend_Json::decode($this->_getParam('data'));
        
        if (!sizeof($data)) {
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    'No data to delete'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        $this->_currency->delete($this->db->quoteInto('id IN(?)', $data));

        Ecart::message()->addSuccess(
            Ecart::translate('locale')->__(
                'Currency was deleted successfully'
            )
        );
        return $this->_helper->json->sendSuccess();
    }
}