<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Locale_LanguageController extends Ecart_Admin_Controller_Back
{
    /**
     * Language model
     *
     * @var Ecart_Locale_Model_Language
     */
    protected $_table;

    public function init()
    {
        parent::init();
        $this->_table = Ecart::single('locale/language');
    }

    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('locale')->__('Languages');
        $this->render();
    }

    public function listAction()
    {
        $this->_helper->layout->disableLayout();

        $count  = $this->_table->count();
        
        $dbField = new Ecart_Filter_DbField();
        
        $order = $dbField->filter($this->_getParam('sort', 'id')) . ' '
               . $dbField->filter($this->_getParam('dir', 'ASC'));
        $data = $this->_table->fetchAll(
            null,
            $order,
            intval($this->_getParam('limit', 20)),
            intval($this->_getParam('start', 0))
        )->toArray();
        
        return $this->_helper->json->sendSuccess(array(
            'data' => array_values($data),
            'count'   => $count
        ));
    }

    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        
        $language = $this->_getParam('language');
        $locale = $this->_getParam('locale_code');
        $id = $this->_getParam('id');
        
        if (!strstr($locale, '_')) {
            Ecart::message()->addError(
                Ecart::translate('locale')->__(
                    'Locale format is unsupported'
            ));
            return $this->_helper->json->sendFailure();
        }
        $code = current(explode('_', $locale));
        
        if (is_numeric($id)) {
            $row = $this->_table->find($id)->current();
        } else {
            $row = $this->_table->createRow();
        }
        
        $row->setFromArray(array(
            'code' => $code,
            'language' => $language,
            'locale' => $locale
        ));
        
        $row->save();

        Ecart::message()->addSuccess(
            Ecart::translate('locale')->__(
                'Language was saved successfully'
        ));
        return $this->_helper->json->sendSuccess();
    }
    
    public function deleteAction()
    {
        $this->getHelper('layout')->disableLayout();
        $ids = Zend_Json_Decoder::decode($this->_getParam('data'));
        
        if (!count($ids)) {
            Ecart::message()->addError(
                Ecart::translate('locale')->__(
                    'No language found to delete'
            ));
            return $this->_helper->json->sendFailure();
        }
        $this->_table->delete($this->db->quoteInto('id IN(?)', $ids));

        Ecart::message()->addSuccess(
            Ecart::translate('locale')->__(
                'Language was deleted successfully'
        ));
        return $this->_helper->json->sendSuccess();
    }
    
    /**
     * Change the locale
     * 
     */
    public function changeAction()
    {
        $locale = $this->_getParam('new_locale');
        
        if ($locale) {
            Ecart_Locale::setLocale($locale);
        }
        
        $this->_redirect($_SERVER['HTTP_REFERER']);
    }
}