<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Location_ZoneDefinitionController extends Ecart_Admin_Controller_Back
{
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('location')->__(
            'Zones Definitions'
        );

        $countries = Ecart::single('location/country')->fetchPairs(
          'id', 'name', null, 'name'
        );
        $countries = array('241' => 'All') + $countries;
        $zones = Ecart::single('location/zone')->fetchAll()->toArray();
        $countryZones = array();
        foreach ($zones as $zone) {
                $countryZones[$zone['country_id']][] = $zone;
        }
        $this->view->countries = $countries;
        $this->view->countryZones = $countryZones;
        $this->render();
    }
	
    public function listGzonesAction()
    {
        $dbField = new Ecart_Filter_DbField();
        
        $table = Ecart::single('location/geozone');
        $count  = $table->count();
        $order = $dbField->filter($this->_getParam('sort', 'id')) . ' '
               . $dbField->filter($this->_getParam('dir', 'ASC'));
        $data = $table->fetchAll(
            null,
            $order,
            intval($this->_getParam('limit', 20)),
            intval($this->_getParam('start', 0))
        )->toArray();
        return $this->_helper->json->sendSuccess(array(
            'data' => array_values($data),
            'count'   => $count
        ));
    }
    
    public function saveGzoneAction()
    {
        $this->getHelper('layout')->disableLayout();
        $data = Zend_Json::decode($this->_getParam('data'));
        
        if (!sizeof($data)) {
            return;
        }
        // Saving exists values
        Ecart::single('location/geozone')->save($data);
    }
    
    public function deleteGzoneAction()
    {
        $this->getHelper('layout')->disableLayout();
        $ids = Zend_Json_Decoder::decode($this->_getParam('data'));
        if (!count($ids)) {
            return;
        }
        Ecart::single('location/geozone')
            ->delete($this->db->quoteInto('id IN(?)', $ids));
        $this->_helper->json->sendSuccess();
    }
    
    public function listAssignsAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = Ecart::single('location/geozone_zone')
            ->getListByGeozone($this->_getParam('gzoneId'));
        
        $this->_helper->json->sendSuccess(array(
            'data' => $data
        ));
    }
    
    public function getAssignAction()
    {
        $this->_helper->layout->disableLayout();
        
        $this->_helper->json->sendJson(
            Ecart::single('location/geozone_zone')
                ->find($this->_getParam('assignId', 0))->current()->toArray(),
            false, false
        );
    }
    
    public function saveAssignAction()
    {
        $this->_helper->layout->disableLayout();
        $data = $this->_getAllParams();
        $table = Ecart::single('location/geozone_zone');
        if ($data['assignId']) {
        	$table->update(
                array(
                    'country_id' => $data['country'],
                    'zone_id' => $data['zone']
                ),
                'id = ' . intval($data['assignId'])
        	);
        } else {
        	$table->insert(
                array(
                    'geozone_id' => $data['gzoneId'],
                    'country_id' => $data['country'],
                    'zone_id' => $data['zone']
                )
            );
        }
        
        $this->_helper->json->sendSuccess();
    }

    public function deleteAssignsAction()
    {
        $this->_helper->layout->disableLayout();
        $data = Zend_Json_Decoder::decode($this->_getParam('data'));
        if (!sizeof($data)) {
            return;
        }
        Ecart::single('location/geozone_zone')->delete(
            $this->db->quoteInto('id IN(?)', $data)
        );

        $this->_helper->json->sendSuccess();
    }
}