<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Sitemap_IndexController extends Ecart_Admin_Controller_Back
{
    /**
     *
     * @var Ecart_Sitemap_Model_File
     */
    private $_table;
    
    public function init()
    {
        parent::init();
        $this->_table = Ecart::single('sitemap/file');
    }
    
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('sitemap')->__('Sitemap');
        $this->view->sitesList = Ecart::single('core/site')
            ->fetchAll()->toArray();
        $this->view->sites = Ecart_Collect_Site::collect();
        $this->view->engines = array_values(
            Ecart::single('sitemap/file_engine')->getEngines()
        );
        $this->render();
    }
    
    private function _generateXmlSitemap($config)
    {
        /*
         * Get categories
         */
        $this->view->categories = current(Ecart::single('catalog/category')
            ->getFlatTree($config['lang_id'], $config['site_id'], true)
        );
        $conf = Ecart::config()->sitemap;

        $changefreq['categories'] = $conf->categories->frequency ;
        $priority['categories'] = $conf->categories->priority ;

        /*
         * Get products
         */
        $this->view->products = Ecart::single('catalog/product')
            ->getAllActiveProducts($config['lang_id'], array($config['site_id']));

        $changefreq['products'] = $conf->products->frequency;
        $priority['products'] = $conf->products->priority;
        
        /*
         * Get cms pages
         */
        $tableCmsCategory = Ecart::single('admin/cms_category');
        $categoryes = $tableCmsCategory->getActiveCategory();
        
        $categoryIds = array ();
        foreach ($categoryes as $category) {
             $categoryIds[] = $category['id'];
        }
        $pages = array();
        if ($conf->cms->showPages) {
            $pages = Ecart::single('admin/cms_page')
                ->getPageListByActiveCategory($categoryIds, $config['lang_id']);
        }
        $this->view->pages = $pages;
        $this->view->pagesCats = $categoryes;
        $changefreq['pages'] = $conf->cms->frequency;
        $priority['pages'] = $conf->cms->priority;
       
        $this->view->serverName = Ecart::single('core/site')
            ->find($config['site_id'])->current()->base;
        
        $this->view->changefreq = $changefreq;
        $this->view->priority =  $priority; 
        
        $script = $this->getViewScript('xml', false);
        $xml = $this->view->render($script);
        
        $dir = Ecart::config()->system->path . '/' ; 
        if (!is_dir($dir) && !mkdir($dir)) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'Dir %s not exist', $dir
                )
            );
            return false;
        }
        $filename = $dir . $config['filename'] . '.xml';
        if (@file_put_contents($filename, $xml)) {

            return true;
        }
        Ecart::message()->addError(
            Ecart::translate('core')->__(
                'Error write file : %s dir chmod 755 need', $filename
            )
        );

        return false;
    }
      
    private function _pingEngine($engineUrl, $xmlUrl) 
    {
        $url = $engineUrl . htmlentities($xmlUrl, ENT_QUOTES, "UTF-8");
        if (!$curl = curl_init()) {
            return false;
        }
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_HEADER, 0);
        @curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        $response = curl_exec($curl);
        if(!$response) {
            return false;
        } else {
            $info = curl_getinfo($curl);
            curl_close($curl);
            if (empty($info['http_code'])) 
                return false;
            elseif ($info['http_code'] != 200) 
                return false;
            if ($info['size_download'] == 0) 
                return false;
            if (isset($info['download_content_length']) 
                && $info['download_content_length'] > 0
                && $info['download_content_length'] != $info['size_download'])
            { 
                return false;
            }
        }
        $info['html_page'] = $response;
        
        return $info;
    }
    
    public function saveAction()
    {
        $this->layout->disableLayout();
        $data = $this->_getAllParams();
        if (!sizeof($data)) {
            return $this->_helper->json->sendFailure();
        }
        $alpha = new Zend_Filter_Alnum();
        
        if (($alpha->filter($data['filename']) != $data['filename']) 
            || ($data['filename'] == '')) {

            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    "Incorect filename %s", $data['filename']
                )
            );
            return $this->_helper->json->sendFailure();
        }
        
        $filename = Ecart::config()->system->path . '/'
                  . $alpha->filter($data['filename']) . '.xml';
        
        $rows = $this->_table->find($data['id']);

        if ($rows->count()) {
            $config = $rows->current();
            /* remove old file */
            $oldFilename = Ecart::config()->system->path . '/' 
                         . $config->filename . '.xml';

            if (is_file($oldFilename)) {
                unlink($oldFilename);
            }
        }
        /* check for file exists */
        if (is_file($filename)) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'File already exist'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        if ($rows->count()) {// if edit, then update
            $this->_table->update(array(
                'filename' =>  $alpha->filter($data['filename']),
                'generated_at' => Ecart_Date::now()->toZendDbExpr(),
                //'usage_at' => Ecart_Date::now()->toZendDbExpr(),
                'site_id' => $data['site_ids']
            ), $this->db->quoteInto('id = ?', $data['id']));
        } else { // else insert new
            $data['id'] = $this->_table->insert(array(
                'filename' => $alpha->filter($data['filename']),
                'generated_at' => Ecart_Date::now()->toZendDbExpr(),
                'usage_at' => '0000-00-00',//Ecart_Date::now()->toZendDbExpr(),
                'site_id' => $data['site_ids'],
                'status' => 0
            ));
        }
        //seaech engines saving  
        if (!isset($data['engine_ids']))
            $data['engine_ids'] = array(1, 2, 3, 4);
            
        if (isset($data['engine_ids'])) {
            Ecart::single('sitemap/file_engine')
                ->set($data['engine_ids'], $data['id']);
        }
        // generate xml sitemap 
        $config = $this->_table->find($data['id'])->current()->toArray();
        $config['lang_id'] = $this->_langId;
        $this->_helper->json->sendJson(array(
            'success' => $this->_generateXmlSitemap($config)
        ));
        
    }
    
    public function listAction()
    {
        $this->layout->disableLayout();
        $field = new Ecart_Filter_DbField();
        if ($this->_hasParam('siteMapId')) {
            $this->view->siteMapId = $this->_getParam('siteMapId');
        }
        $params = array(
            'start' => (int) $this->_getParam('start', 0),
            'limit' => (int) $this->_getParam('limit', 20),
            'sort' => $field->filter($this->_getParam('sort', 'id')),
            'dir' => $field->filter($this->_getParam('dir', 'DESC')),
            'filters' => $this->_getParam('filter', array())
        );
        $dataset = $this->_table->getList($params);
        $data = array();
        $engineNames = Ecart::single('sitemap/file_engine')
            ->getEnginesNamesAssigns();
            
        foreach ($dataset as $item) {
            $item['engines'] =    isset($engineNames[$item['id']]) ? 
                implode(',', array_keys($engineNames[$item['id']])) : '';
            $data[] = $item;
        }
        $this->_helper->json->sendJson(array(
            'sitemap' => $data,
            'count' => $this->_table->getCount($params)
        ));
    }
    
    public function removeAction()
    {
        $this->_helper->layout->disableLayout();
        $ids = Zend_Json_Decoder::decode($this->_getParam('data'));
        if (!$ids) {
            return $this->_helper->json->sendFailure();
        }
        foreach ($ids as $id) {
            $row = $this->_table->find($id)->current();
            $filename = Ecart::config()->system->path . '/'
                      . $row->filename . '.xml';
            if (file_exists($filename))
                @unlink($filename);
        }
            
        $this->_table->delete($this->db->quoteInto('id IN(?)', $ids));
        return $this->_helper->json->sendSuccess();
    }
    
    public function pingAction()
    {
        $this->_helper->layout->disableLayout();
        $ids = Zend_Json_Decoder::decode($this->_getParam('ids'));
        
        if (!$ids) {
            return $this->_helper->json->sendFailure();
        }
        $modelSite = Ecart::single('core/site');

        $engineNames = Ecart::single('sitemap/file_engine')
            ->getEnginesNamesAssigns();
        $info = '';
        foreach ($ids as $id) {
            $status = 0;
            $config = $this->_table->find($id)->current()->toArray();

            $url = $modelSite->find($config['site_id'] )->current()->base
                 . $config['filename'] . '.xml';

            $engines = Ecart::single('sitemap/file_engine')->getEngines();
            foreach ($engines as  $engine) {
                if (isset($engine['url'])
                    && isset($engineNames[$id][$engine['id']])) {

                    $info[$id][$engine['name']] =
                        $this->_pingEngine($engine['url'], $url);

                    if (false !== $info[$id][$engine['name']]) {
                        $status++;
                    }
                }
            }
            // change status  
            if (count($engineNames[$id]) == $status) {
                $this->_table->update(
                    array('status' => 0, 'usage_at' => '0000-00-00'),
                    $this->db->quoteInto('site_id = ?', $config['site_id'])
                );
                $this->_table->update(array('status' =>  1, 
                    'usage_at' => Ecart_Date::now()->toZendDbExpr()), 
                    $this->db->quoteInto('id = ?', $id)
                );
            }
        }
        $this->_helper->json->sendSuccess(array(
            'info' => $info
        ));
    }
            
    public function quickSaveAction()
    {
        $this->_helper->layout->disableLayout();
        $data = Zend_Json_Decoder::decode($this->_getParam('data'));
        if (!$data)  {
            return;
        }
        $tableSitemapToEngine = Ecart::single('sitemap/file_engine');
        foreach ($data as $id => $item) {
            $this->_table->update(array(
                    'generated_at' => Ecart_Date::now()->toZendDbExpr(),
                    'site_id' => $item['site'],
                    'status' => 0
                ), 
                $this->db->quoteInto('id = ?', $id)
            );
            $config = $this->_table->find($id)->current()->toArray();
            $config['lang_id'] = $this->_langId;
            $this->_generateXmlSitemap($config);
            $tableSitemapToEngine->set(explode(',', $item['engines']), $id);
        }
        $this->_helper->json->sendSuccess();
    }
}