<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Tax_RateController extends Ecart_Admin_Controller_Back
{
    /**
     *
     * @var Ecart_Tag_Model_Rate
     */
    protected $_table;

    public function init()
    {
        parent::init();
        $this->_table = Ecart::single('tax/rate');
    }

    public function indexAction()
    {
        $this->view->zones = Ecart::single('location/geozone')
            ->fetchAll()->toArray();
        $this->view->taxClasses = Ecart::single('tax/class')
            ->fetchAll()->toArray();
        $this->view->customerGroups = Ecart::single('account/customer_group')
            ->fetchAll()->toArray();
        $this->view->pageTitle = Ecart::translate('tax')->__('Tax Rates');
        $this->render();
    }

    public function listAction()
    {
        $this->_helper->layout->disableLayout();

        $count  = $this->_table->count();
        $dbField = new Ecart_Filter_DbField();
        
        $order = $dbField->filter($this->_getParam('sort', 'id')) . ' '
               . $dbField->filter($this->_getParam('dir', 'ASC'));
        $data = $this->_table->fetchAll(
            null,
            $order,
            intval($this->_getParam('limit', 20)),
            intval($this->_getParam('start', 0))
        )->toArray();
        
        $this->_helper->json->sendSuccess(array(
            'data' => array_values($data),
            'count' => $count
        ));
    }

    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        $data = Zend_Json::decode($this->_getParam('data'));
        
        if (!sizeof($data)) {
            return;
        }
        // Saving exists values
        $this->_helper->json->sendJson(array(
            'success' => $this->_table->save($data)
        ));
    }
    
    public function deleteAction()
    {
        $this->_helper->layout->disableLayout();
        $ids = Zend_Json_Decoder::decode($this->_getParam('data'));
        if (!count($ids)) {
            return;
        }
        $this->_table->delete($this->db->quoteInto('id IN(?)', $ids));
    }
}