<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Template_BoxController extends Ecart_Admin_Controller_Back
{
    private $_templateId;
    
    /**
     * Box model
     *
     * @var Ecart_Core_Model_Template_Box
     */
    protected $_table;
    
    public function init()
    {
        parent::init();
        $this->_table = Ecart::single('core/template_box');
        $this->_helper->layout->disableLayout();
        $this->_templateId = (int) $this->_getParam('tId', 0);
    }
    
    private function _getBoxes()
    {
        $boxes = array();
        foreach (Ecart::single('core/template_box')->getList() as $box) {
            $type = 'dynamic';
            if (strstr($box, 'Ecart_Cms_Block')) {
                $type = 'static'; 
            }
            $boxes[$type][$box] = $box;
        }
        return $boxes;
    }
    
    private function _initForm()
    {
        $this->view->templateBoxes = $this->_getBoxes();
        $pageTable = Ecart::single('core/page');
        $pages = array();
        $orderBy = array('module_name', 'controller_name', 'action_name');
        foreach ($pageTable->fetchAll(null, $orderBy) as $row) {
            $pages[$row->module_name][$row->controller_name][$row->id] = $row->action_name;
        }
        $this->view->pages = $pages;
    }
    
    public function indexAction()
    {
        $where = NULL;
        if ($this->_templateId) {
            $where = 'template_id = ' . $this->_templateId;
        }
        $boxes = $this->_table->fetchAll($where)->toArray();
        Zend_Debug::dump($boxes);
    }
    
    public function listAction()
    {
        $boxes = $this->_table->fetchAll('template_id = ' . $this->_templateId, 'id DESC')->toArray();
        foreach ($boxes as &$box) {
            $show = Ecart::single('core/template_box_page')
                ->fetchAll(array('box_id = ' . $box['id']));
                
            $pages = array();
            foreach ($show as $item)
                $pages[] = $item->page_id;
            $box['show'] = implode(',', $pages);
        }
        return $this->_helper->json->sendSuccess(array(
            'data' => $boxes
        ));
    }
    
    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        
        $params = $this->_getAllParams();
        $boxId = $this->_getParam('boxId', 0);
        $params['id'] = $boxId;
        
        $data[$boxId] = $params;

        $success = Ecart::single('core/template_box')->save($this->_templateId, $data);
        
        return $this->_helper->json->sendSuccess();
    }
    
    public function batchSaveAction()
    {
        $this->_helper->layout->disableLayout();
        
        $data = Zend_Json::decode($this->_getParam('data'));
        
        return $this->_helper->json->sendJson(array('success' =>
            Ecart::single('core/template_box')->save(
                $this->_templateId, $data, 'batch'
        )));
    }
    
    public function editAction()
    {
        $boxId = $this->_getParam('boxId');
        $row = $this->_table->find($boxId)->current();
        $this->view->box = $row->toArray();
        $this->view->boxId = $boxId;
        $this->view->templateId = $this->_templateId;
        
        /* assignments */
        $boxToPage = Ecart::single('core/template_box_page');
        $assignments = array();
        foreach ($boxToPage->fetchAll('box_id = ' . intval($boxId)) as $item) {
            $assignments[$item->page_id] = $item->toArray();
        }
        $this->view->assignments = $assignments;
        
        $this->_initForm();
        $this->render('form');
    }
    
    public function deleteAction()
    {
        $ids = Zend_Json_Decoder::decode($this->_getParam('data'));
        
        if (!count($ids)) {
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    'No data to delete'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        
        $this->_table->delete($this->db->quoteInto('id IN (?)', $ids));

        Ecart::message()->addSuccess(
            Ecart::translate('admin')->__(
                'Data was deleted successfully'
            )
        );
        return $this->_helper->json->sendSuccess();
    }
}