<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_Template_MailController extends Ecart_Admin_Controller_Back
{
    protected $_table;

    public function init()
    {
        parent::init();
        $this->getHelper('layout')->disableLayout();
        $this->_table = Ecart::single('core/template_mail');
    }

    public function indexAction()
    {
        $this->_helper->layout->enableLayout();
        $this->view->pageTitle = Ecart::translate('admin')->__(
            'Email Templates'
        );
        $this->view->sitesList = Ecart::single('core/site')->fetchAll()->toArray();
        $this->render();
    }

    public function listAction()
    {
        $data = $this->_table->fetchAll()->toArray();
        $this->_helper->json->sendSuccess(array(
            'data' => array_values($data),
            'count'   => count($data)
        ));
    }

    public function listEventAction()
    {
        $events = Ecart_Collect_MailEvent::collect();
        
        $result = array();
        $i = 0;
        foreach ($events as $eventKey => $event) {
            $result[$i] = array('name' => $event, 'id' => $eventKey);
            $i++;
        }
        
        $this->_helper->json->sendSuccess(array(
            'data' => $result
        ));
    }
    
    public function listTemplateAction()
    {
        $templates = Ecart_Collect_MailTemplate::collect();
        
        $result = array();
        $i = 0;
        foreach ($templates as $templateKey => $template) {
            $result[$i] = array('name' => $template, 'id' => $templateKey);
            $i++;
        }
        
        $this->_helper->json->sendSuccess(array(
            'data' => $result
        ));
    }
    
    public function listMailAction()
    {
        $templates = Ecart_Collect_MailBoxes::collect();
        
        $result = array();
        $i = 0;
        foreach ($templates as $templateKey => $template) {
            $result[$i] = array('name' => $template, 'id' => $templateKey);
            $i++;
        }
        
        $this->_helper->json->sendSuccess(array(
            'data' => $result
        ));
    }

    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        
        if ($this->_getParam('data', false)) {
            $data = Zend_Json::decode($this->_getParam('data'));
        } else {
            $id = $this->_getParam('id', 0);
            $data[$id] = $this->_getAllParams();
            unset($data[$id]['module']);
            unset($data[$id]['controller']);
            unset($data[$id]['action']);
        }
        
        return $this->_helper->json->sendJson(array(
            'success' => Ecart::single('core/template_mail')->save($data)
        ));
    }
    
    public function deleteAction()
    {
        $ids = Zend_Json::decode($this->_getParam('data'));
        
        if (!count($ids)) {
            Ecart::message()->addError(
                Ecart::translate('admin')->__(
                    'No data to delete'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        $this->_table->delete($this->db->quoteInto('id IN(?)', $ids));

        Ecart::message()->addSuccess(
            Ecart::translate('admin')->__(
                'Data was deleted successfully'
            )
        );
        return $this->_helper->json->sendSuccess();
    }
    
    public function getInfoAction() 
    {
        $this->_helper->layout->disableLayout();
        
        $templateId = $this->_getParam('templateId');
        
        $info = Ecart::single('core/template_mail')->getInfo($templateId);
        
        return $this->_helper->json->sendSuccess(array(
            'data' => $info
        ));          
    }
}