<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Admin_UsersController extends Ecart_Admin_Controller_Back
{   
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('admin')->__('Users');
        $this->view->roles = Ecart::single('admin/acl_role')
            ->fetchPairs('id', 'role_name');
        $this->view->jsonRoles = Zend_Json::encode($this->view->roles);
        $this->render();
    }
    
    public function getListAction()
    {
        $alpha = new Zend_Filter_Alpha();
        
        $start = (int) $this->_getParam('start', 0);
        $limit = (int) $this->_getParam('limit', 25);
        $sort  = $alpha->filter($this->_getParam('sort', 'id'));
        $dir   = $alpha->filter($this->_getParam('dir', 'ASC'));
        
        $count = Ecart::single('admin/user')->count();
        
        $dataset = Ecart::single('admin/user')->fetchAll(
            null, $sort . ' ' . $dir, $limit, $start
        )->toArray();
        
        foreach ($dataset as &$row) {
            $row['password'] = '';
        }
        return $this->_helper->json->sendSuccess(array(
            'data' => $dataset,
            'count' => $count
        ));
    }
    
    public function saveAction()
    {
        $this->_helper->layout->disableLayout();
        $data = Zend_Json::decode($this->_getParam('data'));
        if (!sizeof($data)) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'No data to save'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        $exists = Ecart::single('admin/user')->getExist(array_keys($data));
        
        foreach ($data as $userId => $values) {
            $row = array(
                'role_id' => $values['role_id'],
                'firstname' => $values['firstname'],
                'lastname' => $values['lastname'],
                'email' => $values['email'],
                'username' => $values['username'],
                'is_active' => $values['is_active'] ? $values['is_active'] : 0
            );
            if (!empty($values['password'])) {
                $row['password'] = md5($values['password']);
            }
            
            if (in_array($userId, $exists)) {
                Ecart::single('admin/user')->update(
                    $row, $this->db->quoteInto('id = ?', $userId)
                );
            } else {
                Ecart::single('admin/user')->insert($row);
            }
        }

        Ecart::message()->addSuccess(
            Ecart::translate('admin')->__(
                'User was saved successfully'
            )
        );
        $this->_helper->json->sendSuccess();
    }
    
    public function deleteAction()
    {
        $this->layout->disableLayout();
        $data = Zend_Json::decode($this->_getParam('data'));
        if (!sizeof($data))
            return;
        Ecart::single('admin/user')->delete(
            $this->db->quoteInto('id IN(?)', $data)
        );
        Ecart::message()->addSuccess(
            Ecart::translate('admin')->__(
                'User was deleted successfully'
            )
        );
        $this->_helper->json->sendSuccess();
    }
    
    public function getRolesAction()
    {
        $roles = array_values(
            Ecart::single('admin/acl_role')->fetchAll()->toArray()
        );
        return $this->_helper->json->sendSuccess(array(
            'data' => $roles
        ));
    }
}