<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Box
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Box_Filters extends Ecart_Core_Box_Abstract
{
    protected $_title = 'Filters';
    protected $_class = 'box-filter';
    public $hurl;
    
    public function init()
    {
        $this->hurl = Ecart_HumanUri::getInstance();
        
        $result = array();
        
        /* Filters */
        $filters = $this->_getFilters();
        
        /* Get category filters */
        /*$catId = $this->hurl->hasParam('cat') ? 
            $this->hurl->getParamValue('cat') : Ecart::single('catalog/category')->getRoot()->id;
        $result['category'] = Ecart::single('catalog/category')->getChildItems($catId, false, true);*/
        
        /* Get price filters */
        $result['price'] = Ecart::single('catalog/product')->cache()
            ->getFiltersByPrice($filters);
        
        /* Get manufacturer filters */
        if (!$this->hurl->hasParam('manufacturer')) {
            $result['manufacturer'] = Ecart::single('catalog/product')->cache()
                ->getFiltersByManufacturer($filters);
        }
        
        /* Attribute filters */
        $result['attributes'] = Ecart::single('catalog/product')->cache()
            ->getFiltersByAttribute($filters);

        if (count($filters) > 1) { // categoryIds is always in filters and we don't need them
            $this->filters = $result;
            return;
        }
        
        foreach($result as $filter){
            if(count($filter)){
                $this->filters = $result;
                return;
            }
        }
        
        return false;
    }

    /**
     * @return array
     */
    private function _getFilters()
    {
        $filters = array(); 
        if ($this->hurl->hasParam('cat')) {
            $catId = $this->hurl->getParamValue('cat');
            $filters['categoryIds'] = array($catId);
        }
        // Filter by manufacturer
        if ($this->hurl->hasParam('manufacturer')) {
            $manufacturerId = $this->hurl->getParamValue('manufacturer');
            $filters['manufacturerId'] = $manufacturerId;
        }
        // by price
        if ($this->hurl->hasParam('price')) {
            list($from, $to) = explode(',', $this->hurl->getParam('price'));
            $filters['price'] = array(
                'from' => $from,
                'to' => $to
            );
        }
        // by attributes
        if ($this->hurl->hasParam('attributes')) {
            // TODO filter joins
            $filters['attributes'] = $this->hurl->getAttributeIds();
        }
        
        return $filters;
    }
    
    public function hasContent()
    {
        return $this->hasData('filters');
    }
}