<?php
/**
 * Ecart
 * 
 * @copyright Copyright 2008-2009 E-Cart LLC
 * @license GNU Public License V3.0
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Ecart_Catalog_Observer
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Model_Observer
{
    /**
     * Send in_stock notificatations
     * 
     * @param array $data
     *  Ecart_Catalog_Model_Product_Row|Ecart_Catalog_Model_Product_Variation_Row $product
     *  Ecart_Catalog_Model_Product_Stock_Row $stock
     * @return void
     */
    public function notifyStockUpdate($data)
    {
        if (($data['old_in_stock'] == 0 && $data['stock']->in_stock == 1)
            || ($data['old_backorder'] == 0 && $data['stock']->backorder == 1)
                && $data['product']->quantity > $data['stock']->min_qty) {
            
            if ($data['stock']->in_stock) {
                Ecart::dispatch('catalog_product_in_stock', $data);
            } else {
                Ecart::dispatch('catalog_product_backorder', $data);
            }
        }
    }
    
    /**
     * Send low_stock, in_stock, out_of_stock notificatations
     * 
     * @param array $data
     *  Ecart_Catalog_Model_Product_Row|Ecart_Catalog_Model_Product_Variation_Row $product
     *  Ecart_Catalog_Model_Product_Stock_Row $stock
     * @return void
     */
    public function notifyQuantityUpdate($data)
    {
        if (($data['old_quantity'] <= $data['stock']->min_qty 
            && $data['product']->quantity > $data['stock']->min_qty)
                && ($data['stock']->in_stock || $data['stock']->backorder)) {
            
            if ($data['stock']->in_stock) {
                Ecart::dispatch('catalog_product_in_stock', $data);
            } else {
                Ecart::dispatch('catalog_product_backorder', $data);
            }
        } elseif ($data['old_quantity'] > $data['stock']->notify_qty 
            && $data['product']->quantity <= $data['stock']->notify_qty) {
            
            Ecart::dispatch('catalog_product_low_stock', $data);
        } elseif ($data['old_quantity'] > $data['stock']->min_qty 
            && $data['product']->quantity <= $data['stock']->min_qty) {
            
            Ecart::dispatch('catalog_product_out_of_stock', $data);
        }
    }
}