<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Model_Product_Featured extends Ecart_Db_Table 
{
    protected $_name = 'catalog_product_featured';
    protected $_primary = array('product_id', 'site_id');
    protected $_referenceMap = array(
        'Product' => array(
            'columns'           => 'product_id',
            'refTableClass'     => 'Ecart_Catalog_Model_Product',
            'refColumns'        => 'id'
        )
    );
    
    /**
     * 
     * @param  params = array('siteId' => int , 
     *                        'currentDate' =>  Ecart_Date::now()->toPhpFormat()->toString("Y-m-d "),
     *                        'categoryId' => int 
     *                        'limit' => int
     * )
     * @return array
     */
    public function getIds(array $params = array())
    {
        $select = $this->getAdapter()->select();
        $select->distinct()->from(array('pf' => $this->_name), array('product_id'))
               ->where("pf.site_id = " . intval($params['siteId']))
               ->where("date_exp > '" . $params['currentDate'] . "' or date_exp is null")
               ->order(new Zend_Db_Expr("RAND()"))
               ->limit($params['limit']);
        
        $select->join(array('p' => self::$_db_prefix . 'catalog_product'),
            'pf.product_id = p.id',
            array())
            ->where('p.date_available IS NULL OR p.date_available <= ?', $params['currentDate'])
            ->where('p.is_active = 1');
        
        $select
            ->join(array('ptc' => self::$_db_prefix . 'catalog_product_category'),
                'pf.product_id = ptc.product_id',
                array())
            ->join(array('c' => self::$_db_prefix . 'catalog_category'),
                'ptc.category_id = c.id',
                array())
            ->where('c.site_id = ' . intval($params['siteId']));
            
        if ($disabledCategories = Ecart::single('catalog/category')->getDisabledIds()) {
            $select->where('c.id NOT IN (?)', $disabledCategories);
        }
        
        if (null !== $params['categoryId']) {
            $select->where("ptc.category_id = " . intval($params['categoryId']));
        }
        return $this->getAdapter()->fetchCol($select->__toString());
    }
}