<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Model_Product_Option extends Ecart_Db_Table 
{
    const TYPE_SELECT   = 0;
    const TYPE_STRING   = 1;
    const TYPE_RADIO    = 2;
    const TYPE_CHECKBOX = 3;
    const TYPE_TEXTAREA = 4;
    const TYPE_FILE     = 5;
    
    protected $_name = 'catalog_product_option';
    protected $_dependentTables = array(
        'Ecart_Catalog_Model_Product_Option_Text', 
        'Ecart_Catalog_Model_Product_Option_Value',
        'Ecart_Catalog_Model_Product_Attribute'
    );
    protected $_referenceMap = array(
        'ValueSet' => array(
            'columns'       => 'valueset_id',
            'refTableClass' => 'Ecart_Catalog_Model_Product_Option_ValueSet',
            'refColumns'    => 'id'
        )
    );
    protected $_rowClass = 'Ecart_Catalog_Model_Product_Option_Row';
    
    /**
     * Get options array by language
     * 
     * @param array $params
     * @return array
     */
    public function getList($params)
    {
        $select = $this->getAdapter()->select();
        
        $optionTextJoin = 'pot.product_option_id = po.id';
        
        if (!empty($params['languageId'])) {
            $optionTextJoin .= " AND language_id = {$params['languageId']}";
        }
        
        $select->from(array('po' => $this->_name))
            ->joinLeft(array('pot' => self::$_db_prefix . 'catalog_product_option_text'),
                $optionTextJoin,
                array('name', 'instruct')
            );
        if (!empty($params['sort']) && !empty($params['dir'])) {
            $select->order($params['sort'] . ' ' . $params['dir']);
        }
        if (!empty($params['limit']) && !empty($params['start'])) {
            $select->limit($params['limit'], $params['start']);
        }
        return $this->getAdapter()->fetchAll($select->__toString());
    }

    /**
     *
     * @static
     * @return <type>
     */
    public static function getTypes()
    {
        return array('select', 'string', 'radio', 'checkbox', 'textarea', 'file');
    }
    
    /**
     *  @param  array ('key' => 'value', 'key1' => 'value1')
     *  @param  int 
     *  @return array 
     */
    public function getAttributesByKeyword(array $keywords, $languageId = null)
    {
        if (null === $languageId) {
            $languageId = Ecart::getLanguageId();
        }
        $where = $this->getAdapter()->quoteInto('?', array_keys($keywords));
        $where2 = $this->getAdapter()->quoteInto('?', array_unique(array_values($keywords)));
        
        $query = "SELECT po.id as option_id, pot.name as option_name, pov.id as value_id, povt.name as value_name
            FROM {$this->_name} po
            INNER JOIN " . self::$_db_prefix . "catalog_product_option_text pot ON pot.product_option_id = po.id
            INNER JOIN " . self::$_db_prefix . "catalog_product_option_value pov ON pov.valueset_id = po.valueset_id
            INNER JOIN " . self::$_db_prefix . "catalog_product_option_value_text povt ON povt.product_option_value_id = pov.id
            WHERE pot.language_id = :langId AND pot.name IN($where) AND
                  povt.language_id = :langId AND povt.name IN($where2)";
        
        
        $res = $this->getAdapter()->query($query, array('langId' => $languageId));
        $attributes = array();
        while ($row = $res->fetch()) {
            if (!isset($keywords[$row['option_name']]) || $keywords[$row['option_name']] != $row['value_name'])
                continue;
            $attributes[$row['option_id']] = array(
                'value'       => $row['value_id'],
                'value_id'    => $row['value_id'],
                'seo'         => $row['option_name'] . '=' . $row['value_name'],
                'option_name' => $row['option_name'],
                'option_id'   => $row['option_id'], 
                'value_name'  => $row['value_name']
            );
        }

        return $attributes;
    }
    
    /**
     * Update or insert product otions
     * 
     * @param array $data
     * @return bool
     */
    public function save(array $data)
    {
        if (!$data['id'] || !$row = $this->find($data['id'])->current()) {
            $row = $this->createRow();
        }
        unset($data['id']);
        $row->setFromArray($data);
        $row->save();
        
        /* saving option text */
        foreach (array_keys(Ecart_Collect_Language::collect()) as $langId) {
            $textData = array(
                'name' => empty($data['names'][$langId]) ? '' : $data['names'][$langId],
                'instruct' => empty($data['instructs'][$langId]) ? '' : $data['instructs'][$langId]
            );
            if (!$text_row = Ecart::single('catalog/product_option_text')->find($row->id, $langId)->current()) {
                $text_row = Ecart::single('catalog/product_option_text')->createRow(array(
                    'product_option_id' => $row->id,
                    'language_id' => $langId
                ));
            }
            $text_row->setFromArray($textData);
            $text_row->save();
        }
        Ecart::message()->addSuccess(
            Ecart::translate('catalog')->__(
                'Option was saved successfully'
            )
        );
        return true;
    }


    protected function _getRelations($languageId = null)
    {
        if (null === $languageId) {
            $languageId = Ecart::getLanguageId();
        }
        $select = $this->getAdapter()->select()
            ->from(array('cpo' => $this->_name), array('valueset_id' => 'id'))
            ->join(array('cpot' => self::$_db_prefix . 'catalog_product_option_text'),
                $this->getAdapter()->quoteInto(
                    'cpot.product_option_id = cpo.id AND cpot.language_id = ?', $languageId
            ), array('valueset_name' => 'name'))
            ->join(array('cpov' => self::$_db_prefix . 'catalog_product_option_value'),
                'cpo.id = cpov.valueset_id', array()
            )
            ->join(array('cpovt' => self::$_db_prefix . 'catalog_product_option_value_text'),
                $this->getAdapter()->quoteInto(
                    'cpovt.product_option_value_id = cpov.id AND cpovt.language_id = ?', $languageId
            ), array('option_name' => 'name', 'option_id' => 'product_option_value_id'))
        ;
        
        return $this->getAdapter()->fetchAll($select->__toString());
    }
    
    public function getValueSets($languageId = null)
    {
        $valusets = array();

        foreach ($this->_getRelations($languageId) as $item) {
            $valusets[$item['valueset_id']]['name'] = $item['valueset_name'];
            $valusets[$item['valueset_id']]['option'][$item['option_id']] = $item['option_name'];

        }
        return $valusets;
    }

    public function getOptions($languageId = null)
    {
        $options = array();
        foreach ($this->_getRelations($languageId) as $item) {
            $options[$item['option_id']] = array(
                'name' => $item['option_name'],
                'valueset' => array(
                    'id' => $item['valueset_id'],
                    'name' => $item['valueset_name']
                )
            );
        }
        return $options;

    }
}