<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */

class Ecart_Catalog_IndexController extends Ecart_Core_Controller_Front
{
    public function init()
    {
        parent::init();
        $this->hurl = Ecart_HumanUri::getInstance();
        $this->view->crumbs()->add(
            Ecart::translate('catalog')->__('Catalog'),
            '/' . $this->view->catalogUrl
        );
        $this->view->tags()->disableAction();
    }
    
    /**
     *  View category action (listing product)
     *
     * @return 
     */
    public function viewAction()
    {
        $paging = array();
        
        /* If isset "product" param then display product info */
        if ($this->hurl->hasParam('product') 
            || $this->getRequest()->getParam('product')) {
            
            $this->_request->setActionName('product');
            $this->productAction();
            return;
        }
        
        /* else display product listing */
        if (!isset(Ecart::session()->catalog)) {
            Ecart::session()->catalog = new Zend_Session_Namespace('catalog'); 
        }
        $filters = array('siteId' => $this->_siteId); 
       
        /* Filter by category */
        if ($this->hurl->hasParam('cat')) {
            $categoryId = $this->hurl->getParamValue('cat');
            Zend_Registry::set('catalog/current_category', $categoryId);
            
            $pathItems = Ecart::single('catalog/category')
                ->cache()
                ->getParentItems($categoryId, $this->_langId);
            foreach ($pathItems as $item) {
                $this->view->crumbs()->add(
                    $item['name'],
                    $this->view->hurl(array(
                        'cat' => array('value' => $item['id'], 'seo' => $item['key_word'])
                    ))
                );
            }
            
            $filters['categoryIds'] = $categoryId;
        }
        
        $titleSuffix = '';
        /* Filter by manufacturer */
        if ($this->hurl->hasParam('manufacturer')) {
            $manufacturerId = $this->hurl->getParamValue('manufacturer');
            $filters['manufacturerId'] = $manufacturerId;
            $titleSuffix = ': ' . $this->hurl->getParam('manufacturer/title');
        }
        
        /* by price */
        if ($this->hurl->hasParam('price')) {
            list($from, $to) = explode(',', $this->hurl->getParam('price'));
            $filters['price'] = array(
                'from' => $from,
                'to' => $to
            );
        }
        
        if ($this->hurl->hasParam('attributes')) {
            $filters['attributes'] = $this->hurl->getAttributeIds();
        }
        
        /* LIMIT paging filter */
        if ($this->hurl->hasParam('limit')) {
            $limit = $this->hurl->getParam('limit');
        } elseif (isset(Ecart::session()->catalog->limit)) {
            $limit = Ecart::session()->catalog->limit;
        } else {
            $limit = Ecart::config()->catalog->main->perPageDefault;
        }
        
        $paging['limit'] = $limit;// = (int) $this->hurl->getParam('limit', Ecart::config()->catalog->main->perPageDefault);
        $paging['page']  = $page  = (int) $this->hurl->getParam('page', 1);
        
        if ($limit != 'all') {
            $filters['page'] = $page;
            $filters['limit'] = $limit;
        }
        
        /* Sort by $order */
        if ($this->hurl->hasParam('order')) {
            $paging['order'] = $this->hurl->getParam('order');           
        } elseif (isset(Ecart::session()->catalog->order)) {
            $paging['order'] = Ecart::session()->catalog->order;
        } else {
            $paging['order'] = 'name';
        }
        Ecart::session()->catalog->order = $paging['order'];
        $filters['order'] = $paging['order'];
        
        /* Sort by $dir */
        if ($this->hurl->hasParam('dir')) {
            $paging['dir'] = $this->hurl->getParam('dir');           
        } elseif (isset(Ecart::session()->catalog->dir)) {
            $paging['dir'] = Ecart::session()->catalog->dir;
        } else {
            $paging['dir'] = 'asc';
        }
        Ecart::session()->catalog->dir = $paging['dir'];
        $filters['dir'] = $paging['dir'];
        //wget -c -p -r -l5 -np -N -k -nv http://demo.ecartcommerce.com/ecart/
        //Get products by filters
        $products = Ecart::single('catalog/product')
            ->cache(/*Ecart::getCustomerId()*/)
            ->getProductsByFilter($filters);
        $paging['count'] = $products['count'];//"SELECT FOUND_ROWS()";
        unset($products['count']);
        $this->view->products = $products;
        
        $paging['perPage'] = array();
        foreach (explode(',', Ecart::config()->catalog->main->perPage) as $perPage) {
            $paging['perPage'][$this->view->hurl(array('limit' => $perPage, 'page' => null))] = $perPage;
        }
        
        $paging['sortBy'] = array();
        foreach (explode(',', Ecart::config()->catalog->main->sortBy) as $order) {
            $paging['sortBy'][$this->view->hurl(array('order' => strtolower($order), 'page' => null))] = $order;
        }
        
        /* Change dir for rendering */
        if (strtolower($paging['dir']) == 'asc') {
            $paging['dir'] = 'desc';
        } else {
            $paging['dir'] = 'asc';
        }
        
        Ecart::session()->catalog->limit = $limit;
        if ($limit == 'all') {
            $paging['limit'] = $paging['count'];
        }

        $title = Ecart::translate('catalog')->__('Catalog') . $titleSuffix;
        $this->view->pageTitle = $title;
        $this->view->description = '';
        $this->view->meta()->setTitle($title);
        if (isset($categoryId)) {
            $currentCategory = Ecart::single('catalog/category_description')
                ->cache($categoryId)
                ->find($categoryId, Ecart::getLanguageId())->current();
            if (!$currentCategory instanceof Ecart_Db_Table_Row) {
                $currentCategory = Ecart::single('catalog/category_description')
                    ->cache($categoryId, Ecart::config()->main->store->language)
                    ->find($categoryId, Ecart::config()->main->store->language)
                    ->current();
            }

            if ($currentCategory instanceof Ecart_Db_Table_Row) {
                $this->view->pageTitle = $currentCategory->name . $titleSuffix;
                $this->view->description = $currentCategory->description;
                $metaDescription = trim($currentCategory->meta_description) == '' ?
                        $currentCategory->description : $currentCategory->meta_description;
                $metaTitle = (trim($currentCategory->meta_title) == '' ?
                        $currentCategory->name : $currentCategory->meta_title) . $titleSuffix;
                $this->view->meta()
                    ->setDescription($metaDescription)
                    ->setTitle($metaTitle, 'category', $categoryId)
                    ->setKeywords($currentCategory->meta_keyword)
//                    ->set(array(
////                    'title' => (trim($currentCategory->meta_title) == '' ?
////                        $currentCategory->name : $currentCategory->meta_title) . $titleSuffix,
//                    'keywords' => $currentCategory->meta_keyword
//                ), 'category', $categoryId)
                ;
            }
        }
        
        $this->view->paging = $paging;
        if ($this->hurl->hasParam('mode')) {
            $mode = $this->hurl->getParam('mode');
        } elseif (isset(Ecart::session()->catalog->mode) 
            && (Ecart::session()->catalog->mode === 'grid' || Ecart::session()->catalog->mode === 'list')) 
        {
            $mode = Ecart::session()->catalog->mode;
        } else {
            $mode = Ecart::config()->catalog->main->listingType;
        }
        $this->view->mode = $mode;
        Ecart::session()->catalog->mode = $mode;
        
        Ecart::dispatch('catalog_product_list_view', array(
            'filters' => $filters
        ));
        
        $this->render('listing');
    }
    
    public function productAction()
    {
        $productId = $this->hurl->getParamValue('product');
        if (!$productId) {
            $productId = $this->_getParam('product', 0);
        }
        /**
         * @var $product Ecart_Catalog_Model_Product_Row
         */
        $product = Ecart::single('catalog/product')
            ->cache($productId /*Ecart::getCustomerId()*/)
            ->find($productId)
            ->current();

        $product->incViewed();

        $categoryId = null;
        if (isset($_SERVER['HTTP_REFERER'])) {
            preg_match(
                '/' . $this->view->catalogUrl . '\/(.[^\/]+)\//', 
                $_SERVER['HTTP_REFERER'], 
                $categoryUrl
            );
            if (isset($categoryUrl[1])) {
                $categoryId = Ecart::single('catalog/category')
                    ->cache($categoryUrl/*, Ecart::getCustomerId()*/)
                    ->getIdByUrl($categoryUrl[1], Ecart::getSiteId());
            }
        }
        
        if (is_numeric($categoryId)) {
            $pathItems = $product->getParentItems($categoryId);
        } else { 
            $pathItems = $product->getParentItems();
        }
        
        Zend_Registry::set('catalog/current_product', $productId);
        
        foreach ($pathItems as $item) {
            $this->view->crumbs()->add(
                $item['name'],
                $this->view->hurl(array(
                    'cat' => array('value' => $item['id'], 'seo' => $item['key_word'])
                ), false, true)
            );
            Zend_Registry::set('catalog/current_category', $item['id']);
        }
        
        $data = $product->toArray();
        $data['gallery'] = Ecart::single('catalog/product_gallery')
            ->cache($productId)
            ->getList($productId);
        $data['human_url'] = urlencode($this->hurl->getParamSeo('product'));
        $data['description'] = $product->cache($productId)->getDescription();
        $data['properties'] = $product->cache($productId)->getProperties();
        $data['modifiers'] = $product->cache($productId)->getModifiersFull();
        foreach ($product->cache()->getVariationAttributesData() as $key => $value) {
            $data[$key] = $value;
        }
        
        $data['stock'] = array();
        $data['stock']['quantity'] = $data['quantity'];
        $data['stock']['is_saleable'] = (int)$product->cache($productId)->isSaleable();
        $stock = Ecart::single('catalog/product_stock')
            ->cache($productId)
            ->find($productId)
            ->current()
            ->toArray();
        $data['stock'] = array_merge($stock, $data['stock']);
        
        unset($data['quantity']);
        
        $data['price_discount'] = $product->getPrice();

        $data['price'] = $product->getPriceRules();

        $data['discount_rules'] = Ecart::single('discount/discount')
            ->cache($productId)
            ->getRulesByProductId($productId);
        
        $data['price']['format'] = Ecart::single('locale/currency')
            ->getFormat();
        
        $this->view->product = $data;
        
        $this->view->crumbs()->add($data['description']['name']);
        $this->view->pageTitle = $data['description']['name'];

        $metaTitle = trim($data['description']['meta_title']) == '' ?
            $data['description']['name'] : $data['description']['meta_title'];

        $metaDescription = trim($data['description']['meta_description']) == '' ?
            strip_tags($data['description']['description']) :
                strip_tags($data['description']['meta_description']);
        
        $this->view->meta()
            ->setTitle($metaTitle, 'product', $productId)
            ->setDescription($metaDescription)
            ->setKeywords($data['description']['meta_keyword'])
        ;
        
        Ecart::dispatch('catalog_product_view', array(
            'product' => $product
        ));
        
        $this->render('product');
    }
    
    public function __call($method, $args)
    {
        $this->_request->setActionName('view');
        $this->viewAction(); 
    }
}