<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_ProductCompareController extends Ecart_Core_Controller_Front
{
    public function indexAction()
    {
        $this->view->pageTitle = Ecart::translate('catalog')->__(
            'Product Compare'
        );
        if (!Ecart::single('catalog/product_compare')->hasItems()) {
            $this->render('empty');
            return;
        }
        
        $products = Ecart::single('catalog/product')->getFullDescriptionByProductIds(
            Ecart::single('catalog/product_compare')->getItems()
        );
        $options = array();
        foreach ($products as &$product) {
            foreach ($product['options'] as $optionId => $option) {
                $options[$optionId] = $option['name'];
            }
        }
        $options = array_unique($options);
        $this->view->products = $products;
        $this->view->options = $options;
        $this->render();
    }
    
    public function addAction()
    {
        $productId = $this->_getParam('product', 0);
        if (!$productId) {
            return;
        }
        $productTable = Ecart::single('catalog/product');
        if (!count($productTable->find($productId))) {
            return;
        }
        Ecart::single('catalog/product_compare')->add($productId);
        $this->_redirect($_SERVER['HTTP_REFERER']);
    }
    
    public function removeAction()
    {
        $productId = $this->_getParam('product', 0);
        if (!$productId) {
            return;
        }
        Ecart::single('catalog/product_compare')->remove($productId);
        $this->_redirect($_SERVER['HTTP_REFERER']);
    }
}