<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * @var Ecart_Install_Model_Installer
 */
$installer = Ecart::single('install/installer');

$installer->startSetup();

/**
 * create: 
 *  catalog_category,
 *  catalog_category_description,
 *  catalog_hurl,
 *  catalog_product,
 *  catalog_product_attribute,
 *  catalog_product_attribute_value,
 *  catalog_product_category,
 *  catalog_product_description,
 *  catalog_product_featured,
 *  catalog_product_gallery,
 *  catalog_product_manufacturer,
 *  catalog_product_option,
 *  catalog_product_option_text,
 *  catalog_product_option_value,
 *  catalog_product_option_value_text,
 *  catalog_product_option_valueset,
 *  catalog_product_stock,
 *  catalog_product_variation
 * reference:
 *  core_site,
 *  locale_language,
 *  tax_rate
 */
$installer->run("

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_category')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_category')}` (
  `id` mediumint(8) unsigned NOT NULL auto_increment,
  `site_id` smallint(5) unsigned NOT NULL,
  `lft` smallint(5) unsigned NOT NULL,
  `rgt` smallint(5) unsigned NOT NULL,
  `lvl` tinyint(3) unsigned NOT NULL,
  `created_on` datetime NOT NULL,
  `modified_on` datetime default NULL,
  `cat_status` enum('enabled','disabled') default 'enabled',
  `image` varchar(64) default NULL,
  PRIMARY KEY  (`id`),
  KEY `i_site_id` (`site_id`),
  KEY `i_lft` (`lft`),
  KEY `i_rgt` (`rgt`),
  KEY `i_lvl` (`lvl`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;

INSERT INTO `{$installer->getTable('catalog_category')}` (`id`, `site_id`, `lft`, `rgt`, `lvl`, `created_on`, `modified_on`, `cat_status`, `image`) VALUES 
(1, 1, 1, 2, 0, '0000-00-00 00:00:00', '0000-00-00 00:00:00', 'enabled', '');

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_category_description')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_category_description')}` (
  `category_id` mediumint(8) unsigned NOT NULL,
  `language_id` smallint(5) unsigned NOT NULL,
  `name` varchar(128) NOT NULL,
  `description` text NOT NULL,
  `meta_title` varchar(128) NOT NULL,
  `meta_description` text NOT NULL,
  `meta_keyword` varchar(255) NOT NULL,
  PRIMARY KEY  (`category_id`,`language_id`),
  KEY `i_language_id` (`language_id`),
  CONSTRAINT `fk_category_id` FOREIGN KEY (`category_id`) REFERENCES `{$installer->getTable('catalog_category')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `fk_language_id` FOREIGN KEY (`language_id`) REFERENCES `{$installer->getTable('locale_language')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;

INSERT INTO `{$installer->getTable('catalog_category_description')}` (`category_id`, `language_id`, `name`, `description`) VALUES (1, 1, 'Main Store', 'Root Category');

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_hurl')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_hurl')}` (
  `key_word` varchar(128) NOT NULL,
  `site_id` smallint(5) unsigned NOT NULL,
  `key_type` enum('c','m','p') NOT NULL,
  `key_id` mediumint(8) unsigned NOT NULL,
  PRIMARY KEY  USING BTREE (`key_word`,`site_id`),
  KEY `i_elastic_url_site_id` (`site_id`),
  KEY `i_elastic_url_key_id` (`key_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product')}` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `manufacturer_id` smallint(5) unsigned default NULL,
  `quantity` decimal(15,4) unsigned NOT NULL default '0.0000',
  `sku` varchar(255) default NULL,
  `image_base` varchar(128) default NULL,
  `image_listing` varchar(128) default NULL,
  `image_thumbnail` varchar(128) default NULL,
  `price` decimal(15,4) unsigned NOT NULL default '0.0000',
  `date_available` date default NULL,
  `weight` decimal(10,3) default '0.000',
  `is_active` tinyint(1) unsigned NOT NULL default '0',
  `ordered` mediumint(8) unsigned NOT NULL default '0',
  `created_on` datetime NOT NULL,
  `modified_on` datetime default NULL,
  `tax_class_id` mediumint(8) unsigned default NULL,
  `viewed` mediumint(8) unsigned NOT NULL default '0',
  `new_from` date default NULL,
  `new_to` date default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `product_sku` (`sku`),
  KEY `i_manufacture_id` USING BTREE (`manufacturer_id`),
  CONSTRAINT `FK_catalog_product_manufacturer` FOREIGN KEY (`manufacturer_id`) REFERENCES `{$installer->getTable('catalog_product_manufacturer')}` (`id`) ON DELETE SET NULL,
  CONSTRAINT `FK_tax_class` FOREIGN KEY (`tax_class_id`) REFERENCES `{$installer->getTable('tax_class')}` (`id`) ON DELETE SET NULL ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_attribute')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_attribute')}` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `product_variation_id` int(10) unsigned default '0',
  `product_id` int(10) unsigned NOT NULL,
  `product_option_id` mediumint(8) unsigned NOT NULL,
  `product_option_value_id` int(10) unsigned default NULL,
  `price` decimal(15,4) default NULL,
  `price_type` enum('to','by','percent') default NULL,
  `weight` decimal(10,2) default NULL,
  `weight_type` enum('to','by','percent') default NULL,
  `modifier` tinyint(1) unsigned NOT NULL default '0',
  PRIMARY KEY  (`id`),
  KEY `FK_product_attribute2_pi` (`product_id`),
  KEY `FK_product_attribute2_pvi` (`product_variation_id`),
  KEY `FK_product_attribute2_poi` (`product_option_id`),
  KEY `FK_product_attribute2_povi` (`product_option_value_id`),
  CONSTRAINT `fk_product_attribute_option_id` FOREIGN KEY (`product_option_id`) REFERENCES `{$installer->getTable('catalog_product_option')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `fk_product_attribute_product_id` FOREIGN KEY (`product_id`) REFERENCES `{$installer->getTable('catalog_product')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `fk_product_attribute_value_id` FOREIGN KEY (`product_option_value_id`) REFERENCES `{$installer->getTable('catalog_product_option_value')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `fk_product_attribute_variation_id` FOREIGN KEY (`product_variation_id`) REFERENCES `{$installer->getTable('catalog_product_variation')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 ROW_FORMAT=FIXED;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_attribute_value')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_attribute_value')}` (
  `product_attribute_id` int(10) unsigned NOT NULL,
  `language_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_value` text,
  PRIMARY KEY  (`product_attribute_id`,`language_id`),
  KEY `FK_product_attribute_id` (`product_attribute_id`),
  KEY `FK_product_attribute_language_id` (`language_id`),
  CONSTRAINT `FK_product_attribute_id` FOREIGN KEY (`product_attribute_id`) REFERENCES `{$installer->getTable('catalog_product_attribute')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `FK_product_attribute_language_id` FOREIGN KEY (`language_id`) REFERENCES `{$installer->getTable('locale_language')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_category')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_category')}` (
  `category_id` mediumint(8) unsigned NOT NULL,
  `product_id` int(10) unsigned NOT NULL,
  PRIMARY KEY  (`category_id`,`product_id`),
  KEY `fk_product_id` (`product_id`),
  CONSTRAINT `fk_product_category_id` FOREIGN KEY (`category_id`) REFERENCES `{$installer->getTable('catalog_category')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `fk_product_id` FOREIGN KEY (`product_id`) REFERENCES `{$installer->getTable('catalog_product')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_description')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_description')}` (
  `product_id` int(10) unsigned NOT NULL,
  `language_id` smallint(5) unsigned NOT NULL,
  `name` varchar(255) NOT NULL,
  `description` text NOT NULL,
  `viewed` mediumint(8) unsigned NOT NULL default '0',
  `image_seo_name` varchar(128) NOT NULL,
  `meta_title` varchar(128) NOT NULL,
  `meta_description` text NOT NULL,
  `meta_keyword` text NOT NULL,
  `short_description` text NOT NULL,
  PRIMARY KEY  (`product_id`,`language_id`),
  KEY `fk_product_description_language_id` (`language_id`),
  CONSTRAINT `fk_product_description_product_id` FOREIGN KEY (`product_id`) REFERENCES `{$installer->getTable('catalog_product')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `fk_product_description_language_id` FOREIGN KEY (`language_id`) REFERENCES `{$installer->getTable('locale_language')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_featured')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_featured')}` (
  `product_id` int(10) unsigned NOT NULL,
  `site_id` smallint(5) unsigned NOT NULL,
  `date_exp` date default NULL,
  PRIMARY KEY  (`product_id`,`site_id`),
  KEY `FK_featured_product_site_id` (`site_id`),
  CONSTRAINT `FK_featured_product_id` FOREIGN KEY (`product_id`) REFERENCES `{$installer->getTable('catalog_product')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `FK_featured_product_site_id` FOREIGN KEY (`site_id`) REFERENCES `{$installer->getTable('core_site')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_gallery')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_gallery')}` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned NOT NULL,
  `image` varchar(128) NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `FK_product_gallery_id` (`product_id`),
  CONSTRAINT `FK_product_gallery_id` FOREIGN KEY (`product_id`) REFERENCES `{$installer->getTable('catalog_product')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_manufacturer')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_manufacturer')}` (
  `id` smallint(5) unsigned NOT NULL auto_increment,
  `name` varchar(128) NOT NULL,
  `image` varchar(255) default NULL,
  PRIMARY KEY  (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_manufacturer_title')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_manufacturer_title')}` (
  `manufacturer_id` smallint(5) unsigned NOT NULL,
  `language_id` smallint(5) unsigned NOT NULL,
  `title` varchar(255) NOT NULL,
  PRIMARY KEY  USING BTREE (`manufacturer_id`,`language_id`),
  KEY `fk_product_manufacturer_title_language_id` (`language_id`),
  CONSTRAINT `fk_product_manufacturer_title_manufacturer_id` FOREIGN KEY (`manufacturer_id`) REFERENCES `{$installer->getTable('catalog_product_manufacturer')}` (`id`) ON DELETE CASCADE,
  CONSTRAINT `fk_product_manufacturer_title_language_id` FOREIGN KEY (`language_id`) REFERENCES `{$installer->getTable('locale_language')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_option')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_option')}` (
  `id` mediumint(8) unsigned NOT NULL auto_increment,
  `code` varchar(32) NOT NULL,
  `option_type` tinyint(3) unsigned NOT NULL default '0',
  `sort_order` smallint(5) unsigned NOT NULL default '0',
  `searchable` tinyint(1) unsigned NOT NULL default '0',
  `comparable` tinyint(1) unsigned NOT NULL default '1',
  `languagable` tinyint(1) unsigned NOT NULL default '0',
  `filterable` tinyint(1) unsigned NOT NULL default '1',
  `valueset_id` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `product_option_code` (`code`),
  KEY `product_option_sort_order` USING BTREE (`sort_order`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_option_text')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_option_text')}` (
  `product_option_id` mediumint(8) unsigned NOT NULL,
  `language_id` smallint(5) unsigned NOT NULL,
  `name` varchar(128) NOT NULL,
  `instruct` varchar(128) NOT NULL,
  PRIMARY KEY  (`product_option_id`,`language_id`),
  CONSTRAINT `FK_product_option_id` FOREIGN KEY (`product_option_id`) REFERENCES `{$installer->getTable('catalog_product_option')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=FIXED;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_option_value')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('catalog_product_option_value')}` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `sort_order` tinyint(3) unsigned NOT NULL default '0',
  `valueset_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  USING BTREE (`id`),
  KEY `fk_valueset_id` (`valueset_id`),
  CONSTRAINT `FK_valueset_id` FOREIGN KEY (`valueset_id`) REFERENCES `{$installer->getTable('catalog_product_option_valueset')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 ROW_FORMAT=FIXED;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_option_value_text')}`;
CREATE TABLE  `{$installer->getTable('catalog_product_option_value_text')}` (
  `product_option_value_id` int(10) unsigned NOT NULL,
  `language_id` smallint(5) unsigned NOT NULL,
  `name` varchar(128) default NULL,
  PRIMARY KEY  USING BTREE (`product_option_value_id`,`language_id`),
  CONSTRAINT `FK_product_option_value_id` FOREIGN KEY (`product_option_value_id`) REFERENCES `{$installer->getTable('catalog_product_option_value')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=FIXED;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_option_valueset')}`;
CREATE TABLE  `{$installer->getTable('catalog_product_option_valueset')}` (
  `id` smallint(5) unsigned NOT NULL auto_increment,
  `name` varchar(128) NOT NULL,
  PRIMARY KEY  (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_stock')}`;
CREATE TABLE  `{$installer->getTable('catalog_product_stock')}` (
  `product_id` int(10) unsigned NOT NULL,
  `in_stock` int(1) NOT NULL default '1',
  `manage` int(1) NOT NULL default '1',
  `min_qty` int(8) NOT NULL default '0',
  `min_qty_allowed` int(8) NOT NULL default '1',
  `max_qty_allowed` int(8) NOT NULL default '0',
  `decimal` int(1) NOT NULL default '0',
  `notify_qty` int(8) NOT NULL default '0',
  `backorder` int(8) NOT NULL default '0',
  PRIMARY KEY  (`product_id`),
  CONSTRAINT `FK_product_stock_id` FOREIGN KEY (`product_id`) REFERENCES `{$installer->getTable('catalog_product')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('catalog_product_variation')}`;
CREATE TABLE  `{$installer->getTable('catalog_product_variation')}` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned default NULL,
  `price` decimal(15,4) NOT NULL,
  `sku` varchar(255) NOT NULL,
  `quantity` decimal(15,4) unsigned NOT NULL default '0.0000',
  `price_type` enum('to','by','percent') NOT NULL default 'by',
  `weight` decimal(10,2) NOT NULL default '0.00',
  `weight_type` enum('to','by','percent') NOT NULL default 'by',
  PRIMARY KEY  (`id`),
  KEY `FK_product_variation_product_id` (`product_id`),
  CONSTRAINT `FK_product_variation_product_id` FOREIGN KEY (`product_id`) REFERENCES `{$installer->getTable('catalog_product')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 ROW_FORMAT=FIXED;

INSERT INTO `{$installer->getTable('catalog_product_variation')}` (`id`, `product_id`, `price`, `sku`, `quantity`, `price_type`, `weight`, `weight_type`) VALUES (0, NULL, 0.0000, '', 0.0000, 'by', 0.00, 'by');

");

Ecart::single('core/config_field')
    ->add('catalog/main/catalogRoute', 'Catalog/Main/Catalog router', 'store', 'string', 'Catalog url (example.com/<b>catalogRoute</b>/product)')
    ->add('catalog/main/listingType', 'Listing type', 'grid', 'select', 'Default listing type', array('config_options' => 'grid,list'))
    ->add('catalog/main/perPage', 'Show per page', '6,9,18,32,64,all')
    ->add('catalog/main/sortBy', 'Sort By', 'Name,Rate,Price')
    ->add('catalog/main/perPageDefault', 'Default product count per page', 9)
    ->add('catalog/product/hurldelimiter', 'Catalog/Product description/Hurl world delimiter', '-')
    ->add('catalog/product/seodelimiter', 'Hurl world delimiter', '_')
    ->add('catalog/product/seodesclength', 'SEO Description Length (45 -150 chars)', 100)
    
    ->add('catalog/lightzoom/zoomStageWidth', 'Catalog/Lightzoom/Zoomer width', 250)
    ->add('catalog/lightzoom/zoomStageHeight', 'Zoomer height', 250)
    ->add('catalog/lightzoom/zoomStagePosition', 'Zoomer position', 'right', 'select', '', array('config_options' => 'left,right'))
    ->add('catalog/lightzoom/zoomStageOffsetX', 'Zoomer offset-x', 10)
    ->add('catalog/lightzoom/zoomStageOffsetY', 'Zoomer offset-y', 0)
    ->add('catalog/lightzoom/zoomLensOpacity', 'Lens opacity', 0.7)
    ->add('catalog/lightzoom/zoomCursor', 'Lens cursor', 'crosshair', 'select', '', array('config_options' => 'none,default,crosshair,pointer'))
    ->add('catalog/lightzoom/zoomOnTrigger', 'Zoom on trigger', 'mouseenter', 'select', 'Select none, if you wish to disable this event', array('config_options' => 'none,click,mouseenter'))
    ->add('catalog/lightzoom/zoomOffTrigger', 'Zoom off trigger', 'mouseleave', 'select', 'Select none, if you wish to disable this event', array('config_options' => 'none,click,mouseleave'))
    ->add('catalog/lightzoom/lightboxTrigger', 'Lightbox trigger', 'click', 'select', 'Select none, if you wish to disable this event', array('config_options' => 'none,click,dblclick'))
    ->add('catalog/lightzoom/lightboxResizeSpeed', 'Lightbox resize speed', 800, 'string', 'Animation speed, ms')
    ->add('catalog/lightzoom/lightboxFadeSpeed', 'Lightbox fade speed', 300, 'string', 'Animation speed, ms')
    ->add('catalog/lightzoom/lightboxMaskOpacity', 'Mask opacity', 0.8)
    ->add('catalog/lightzoom/switchImageTrigger', 'Switch image trigger', 'click', 'select', 'Select none, if you wish to disable this event', array('config_options' => 'none,click,mouseenter,dblclick'))
    
    ->add('image/main/cachePath', 'Images/Main/Cache path', '/media/cache', 'string', 'Image cache path, relative to ECART_ROOT')
    ->add('image/product/cache', 'Images/Product Images/Cache', 1, 'bool', 'Enable image cache')
    ->add('image/product/widthLarge', 'Large width', 0)
    ->add('image/product/heightLarge', 'Large height', 0)
    ->add('image/product/widthMedium', 'Product Info width', 250)
    ->add('image/product/heightMedium', 'Product Info height', 250)
    ->add('image/product/widthSmall', 'Small width', 150)
    ->add('image/product/heightSmall', 'Small height', 150)
    ->add('image/product/widthThumbnail', 'Thumbnail image width', 40)
    ->add('image/product/heightThumbnail', 'Thumbnail image height', 40)
    ->add('image/watermark/enabled', 'Images/Watermark/Enabled', 0, 'bool')
    ->add('image/watermark/image', 'Image path', 'catalog/watermark.png', 'string', 'Path relative to the skin images folder: catalog/watermark.png')
    ->add('image/watermark/position', 'Watermark Position', 'bottom_right', 'select', array('config_options' => 'stretch,top_left,top_center,top_right,middle_left,middle_center,middle_right,bottom_left,bottom_center,bottom_right'))
    ->add('image/watermark/opacity', 'Opacity', 50, 'string', 'Values [0 - 100]')
    ->add('image/watermark/repeat', 'Repeat', 0, 'bool');

Ecart::single('admin/acl_resource')
    ->add('admin/catalog', 'Catalog')
    
    ->add('admin/catalog_index', 'Products/Categories')
    ->add("admin/catalog_index/batch-save-product")
    ->add("admin/catalog_index/category-tree")
    ->add("admin/catalog_index/get-options")
    ->add("admin/catalog_index/get-product-info")
    ->add("admin/catalog_index/index")
    ->add("admin/catalog_index/list-bestseller")
    ->add("admin/catalog_index/list-products")
    ->add("admin/catalog_index/list-viewed")
    ->add("admin/catalog_index/move-products")
    ->add("admin/catalog_index/remove-product")
    ->add("admin/catalog_index/remove-product-from-category")
    ->add("admin/catalog_index/remove-product-from-site")
    ->add("admin/catalog_index/save-image")
    ->add("admin/catalog_index/save-product")
    ->add("admin/catalog_index/update-search-index")

    ->add('admin/catalog_product', 'Product Attributes')
    
    ->add('admin/catalog_product-attributes', 'Attributes')
    ->add("admin/catalog_product-attributes/delete")
    ->add("admin/catalog_product-attributes/edit")
    ->add("admin/catalog_product-attributes/index")
    ->add("admin/catalog_product-attributes/list")
    ->add("admin/catalog_product-attributes/save")

    ->add('admin/catalog_product-option-valueset', 'Product Attributes ValueSet')
    ->add("admin/catalog_product-option-valueset/delete-sets")
    ->add("admin/catalog_product-option-valueset/delete-values")
    ->add("admin/catalog_product-option-valueset/index")
    ->add("admin/catalog_product-option-valueset/list-sets")
    ->add("admin/catalog_product-option-valueset/list-values")
    ->add("admin/catalog_product-option-valueset/save-set")
    ->add("admin/catalog_product-option-valueset/save-values")

    ->add('admin/catalog_manufacturer', 'Manufacturer')
    ->add("admin/catalog_manufacturer/delete")
    ->add("admin/catalog_manufacturer/index")
    ->add("admin/catalog_manufacturer/list")
    ->add("admin/catalog_manufacturer/save")
    ->add("admin/catalog_manufacturer/save-image")

    ->add("admin/catalog_image", 'Catalog Images')
    ->add("admin/catalog_image/save-image")
    ->add("admin/catalog_image/tree-panel")
    
    ->add("admin/catalog_category", "Catalog categories")
    ->add("admin/catalog_category/delete")
    ->add("admin/catalog_category/form")
    ->add("admin/catalog_category/get-data")
    ->add("admin/catalog_category/get-items")
    ->add("admin/catalog_category/move")
    ->add("admin/catalog_category/save");

Ecart::single('admin/menu')
    ->add('Catalog', '#', 20)
    ->add('Catalog->Products catalog', 'catalog_index', 10)
    ->add('Catalog->Product attributes', '#', 20)
    ->add('Catalog->Product attributes->Attributes', 'catalog_product-attributes', 10)
    ->add('Catalog->Product attributes->Value sets', 'catalog_product-option-valueset', 20)
    ->add('Catalog->Product brands', 'catalog_manufacturer', 30)
    ;
    
Ecart::single('core/page')
    ->add('catalog/*/*')
    ->add('catalog/index/*')
    ->add('catalog/index/product')
    ->add('catalog/index/view')
    ->add('catalog/product-compare/*')
    ->add('catalog/product-compare/index');

$installer->registerModule('Ecart_Catalog');

$installer->endSetup();