<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
     * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_Controller_Checkout extends Ecart_Core_Controller_Front_Secure
{
    /**
     * @return Ecart_Checkout_Model_Checkout 
     */
    protected function _getCheckout()
    {
        return Ecart::single('checkout/checkout');
    }

    /**
     *
     * @return Ecart_Account_Model_Form_AddressBook
     */
    protected function _getAddressForm()
    {
        /**
         * @var Zend_Form $form
         */
        $form = Ecart::model('account/Form_AddressBook');
        $form->removeElement('primary_billing');
        $form->removeElement('primary_shipping');
        $form->setAction($this->view->href('/account/address-book/save', true));
        return $form;
    }
    
    /* TODO move to address model */
    protected function _checkAddress($addressId)
    {
        if (!$addressId) {
            return false;
        }
        $address = Ecart::single('account/customer_address')->find($addressId)->current();
        if (!$address || $address->customer_id != Ecart::getCustomerId()) {
            return false;
        }
        return true;
    }
    
    protected function _validateCart()
    {
        if (!$this->_getCheckout()->getCart()->getCount() || 
            !$this->_getCheckout()->getCart()->validateContent()) {
            
            $this->_redirect('/checkout/cart');
            return;
        }
    }
    
    public function addressListAction()
    {
        $type = $this->_getParam('type');
        $storage = $this->_getCheckout()->getStorage();
        
        $customerId = Ecart::getCustomerId();

        $addressModel = Ecart::single('account/customer_address');

        switch ($type) {
            case 'billing-address':
            	$this->view->pageTitle = Ecart::translate('checkout')->__(
                    'Select billing address'
                );
                if (isset($storage->billing['id'])) {
                    $selectedAddressId = $storage->billing['id'];
                } else {
                    $selectedAddressId = $addressModel->getDefaultBillingId($customerId);
                }
                break;
            case 'delivery-address':
            	$this->view->pageTitle = Ecart::translate('checkout')->__(
                    'Select delivery address'
                );
                if (isset($storage->delivery['id'])) {
                    $selectedAddressId = $storage->delivery['id'];
                } else {
                    $selectedAddressId = $addressModel->getDefaultDeliveryId($customerId);
                }
                break;
            default:
                $selectedAddressId = 0;
        }

        $this->view->assign(array(
            'selectedAddressId' => $selectedAddressId,
            'addressType'       => $type,
            'addressList'       => $addressModel->fetchAll("customer_id = " . intval($customerId))->toArray()
        ));

        $this->render();
    }
    
    public function shippingMethodAction()
    {
        $this->view->shippingMethods = Ecart_Shipping::getAllowedMethods();
        $this->render('shipping');
    }
    
    public function paymentMethodAction()
    {
        $this->view->paymentMethods = Ecart_Payment::getAllowedMethods();
        $this->render('payment');
    }
    
    public function confirmationAction()
    {
        if (!$this->_getCheckout()->check()) {
            $this->view->checkoutErrors = $this->_getCheckout()->getErrors();
            $this->render('checkout-errors');
            return;
        }
        
        $data = array();
        // addresses
        $data['delivery'] = $this->_getCheckout()->getStorage()->delivery;
        $data['billing']  = $this->_getCheckout()->getStorage()->billing;
        
        // read payment data from session storage
        $data['payment'] = $this->_getCheckout()->payment()->getTitle();
        $data['payment_code'] = $this->_getCheckout()->payment()->getCode();
        
        // read shipping data from session storage
        $data['shipping'] = $this->_getCheckout()->shipping()->getTitle();
        $data['shipping_code'] = $this->_getCheckout()->shipping()->getCode();
        
        $data['products'] = $this->_getCheckout()->getCart()->getProducts();
        
        $data['totals'] = $this->_getCheckout()->getTotal()->getCollects();
        $data['total']  = $this->_getCheckout()->getTotal()->getTotal();
        
        $this->view->checkout = $data;
        $this->render();
    }
    
    public function processAction()
    {
        $this->_forward('process', 'index', 'Ecart_Checkout');
    }
}