<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_Model_Cart_Product extends Ecart_Db_Table
{
    protected $_name = 'checkout_cart_product';
    protected $_rowClass = 'Ecart_Checkout_Model_Cart_Product_Row';
    
    public function findByCart($cartId)
    {
        $where = $this->getAdapter()->quoteInto('shopping_cart_id = ?', $cartId);
        return $this->fetchAll($where);
    }
    
    /**
     * 
     * @return object 
     */
    public function getAttributes($shoppingCartId, $productId)
    {
        return $this->getAdapter()->query(
            "SELECT scp.id, scp.quantity, scpa.product_attribute_id as attrId, scpa.product_attribute_value 
            FROM {$this->_name} scp
            LEFT JOIN " . self::$_db_prefix . "checkout_cart_product_attribute scpa ON scpa.shopping_cart_product_id = scp.id
            WHERE scp.shopping_cart_id = :scId AND scp.product_id = :prodId",
            array('scId' => $shoppingCartId, 'prodId' => $productId)
        );
    }
    
    /**
     * Retrieve the list of products including product hurl, options and attributes
     * 
     * @param id $shoppingCartId
     * @return array
     */
    public function getProducts($shoppingCartId)
    {
        $select = $this->getAdapter()->select()->from(array('scp' => $this->_name))
            ->where('scp.shopping_cart_id = ' . $shoppingCartId);
        
        if (!$this->fetchRow("shopping_cart_id = {$shoppingCartId}")) {
            return $this->getAdapter()->query($select);
        }
        
        $languageId = Ecart::getLanguageId();
        $select->joinLeft(array('p' => self::$_db_prefix . 'catalog_product'), 'p.id = scp.product_id', array('sku', 'price', 'image_thumbnail'))
            ->joinLeft(array('pd' => self::$_db_prefix . 'catalog_product_description'), 
                'pd.product_id = scp.product_id AND pd.language_id = ' . $languageId,
                array('name', 'description', 'image_seo_name'))
            ->joinLeft(array('hu' => self::$_db_prefix . 'catalog_hurl'),
                'hu.key_id = p.id AND hu.key_type = "p"',
                'key_word')
            ->joinLeft(array('ps' => self::$_db_prefix . 'catalog_product_stock'),
                'ps.product_id = p.id', 
                'decimal')
            ->joinLeft(array('scpa' => self::$_db_prefix . 'checkout_cart_product_attribute'),
                'scpa.shopping_cart_product_id = scp.id',
                array('scpaId' => 'id', 'product_attribute_value'))
            ->joinLeft(array('pa' => self::$_db_prefix . 'catalog_product_attribute'),
                'pa.id = scpa.product_attribute_id',
                array())
            ->joinLeft(array('po' => self::$_db_prefix . 'catalog_product_option'),
                'po.id = pa.product_option_id',
                'option_type')
            ->joinLeft(array('pot' => self::$_db_prefix . 'catalog_product_option_text'),
                'pot.product_option_id = pa.product_option_id AND pot.language_id = ' . $languageId,
                array('option_name' => 'name'))
            ->joinLeft(array('povt' => self::$_db_prefix . 'catalog_product_option_value_text'),
                'povt.product_option_value_id = pa.product_option_value_id AND povt.language_id = ' . $languageId,
                array('value_name' => 'name'))
            ->order(array('scp.product_id', 'po.id'));
            
        return $this->getAdapter()->query($select);
    }
    
    /**
     * Retrieve the list of products with minimal using of joins
     * Returns cart product rows with decimal flag. Used to calculate products count
     * 
     * @param id $shoppingCartId
     * @return array
     */
    public function getProductsSimple($shoppingCartId)
    {
        $select = $this->getAdapter()->select();
        $select->from(array('scp' => $this->_name))
            ->joinLeft(array('ps' => self::$_db_prefix . 'catalog_product_stock'),
                'ps.product_id = scp.product_id', 
                'decimal')
            ->where('scp.shopping_cart_id = ' . $shoppingCartId);
        
        return $select->query()->fetchAll();
    }
    
}