<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_Model_Total_ShippingTax extends Ecart_Checkout_Model_Total_Abstract
{
    protected $_code = 'Checkout_ShippingTax';
    protected $_title = 'Shipping Tax';

    public function collect(Ecart_Checkout_Model_Total $total)
    {
        $checkout = Ecart::single('checkout/checkout');
        
        if (null === $checkout->shipping()) {
            return false;
        }
        
        if (!$taxClassId = $checkout->shipping()->_config->taxClass) {
            if (!$taxClassId = Ecart::config()->tax->shipping->taxClass) {
                return false;
            }
        }
        
        if (!$taxBasis = $checkout->shipping()->_config->taxBasis) {
            if (!$taxBasis = Ecart::config()->tax->shipping->taxBasis) {
                return false;
            }
        }
        
        $address = $checkout->getStorage()->{$taxBasis};
        $countryId = isset($address['country_id']) ?
            $address['country_id'] : $address['country']['id'];
        $zoneId = isset($address['zone_id']) ?
            $address['zone_id'] :
                (isset($address['zone']['id']) ? $address['zone']['id'] : null);
        $geozoneIds = Ecart::single('location/geozone')
            ->getIds($countryId, $zoneId);
        if (!count($geozoneIds)) {
            return false;
        }
        
        $customerId = Ecart::getCustomerId();
        $customerGroupId = Ecart::single('account/customer')->getGroupId($customerId);
        if (!$customerGroupId) {
            return false;
        }
        $type = $checkout->shipping()->getType(
            $checkout->getShippingRequest(), $checkout->getStorage()->shipping
        );
        $tax = Ecart::single('tax/rate')->calculateByPrice(
            $type['price'], $taxClassId, $geozoneIds, $customerGroupId 
        );       
        
        $total->addCollect(array(
            'code'  => $this->getCode(),
            'title' => $this->getTitle(),
            'total' => $tax,
            'sortOrder' => $this->_config->sortOrder
        ));
    }
}