<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_IndexController extends Ecart_Checkout_Controller_Checkout
{
    /**
     * Return Checkout Model
     *
     * @return Ecart_Checkout_Model_Checkout
     */
    public function init()
    {
        parent::init();
        $this->view->crumbs()->add(
            Ecart::translate('checkout')->__(
                'Checkout'
            ),
            '/checkout/cart'
        );
    }
    
    public function indexAction()
    {
        $this->_redirect('/checkout/onepage');
    }
    
    private function _sendMails($order) 
    {
        $customer = Ecart::single('account/customer')
            ->find($order->customer_id)->current();
        $orderInfo = $order->toArray();
        $orderInfo['products'] = $order->getProducts();
        $orderInfo['totals'] = $order->getTotals();
        $orderInfo['addresses'] = $order->getAddresses();
        $mail = new Ecart_Mail();
        $mail->setConfig(array(
            'event'   => 'order_new-customer',
            'subject' => Ecart::translate('checkout')->__(
                'Your create new order'
            ),
            'data'    => array(
                'firstname' => $customer->firstname,
                'lastname'  => $customer->lastname,
                'orderId'   => $order->id,
                'order'     => $orderInfo
            ),
            'to'      =>  $order->customer_email
        ));
        $customerSendRet = $mail->send();
        
        $mail = new Ecart_Mail();
        $mail->setConfig(array(
            'event'   => 'order_new-owner',
            'subject' => Ecart::translate('checkout')->__(
                'Order created'
            ),
            'data'    => array(
                'order'     => $orderInfo
            ),
            'to'      =>  Ecart_Collect_MailBoxes::getName(
                Ecart::config()->orders->main->email
            ),
            'type'    => 'html'   
        ));
        return $customerSendRet && $mail->send();
    }
    
    public function processAction()
    {
        $this->_helper->layout->disableLayout();
        $checkout = $this->_getCheckout();
        
        if (!$checkout->check()) {
            foreach ($checkout->getErrors() as $error) {
               Ecart::message()->addError($error['text']);
            }
            $this->_redirect('/checkout/onepage');
        }

        $checkoutStorage = $checkout->getStorage();
        if ($checkoutStorage->registerGuest) {
            $form = Ecart::single('account/form_signup');
            $request = $checkoutStorage->billing;
            if (!isset($request['password'])
                && isset($request['register_password'])) {

                $request['password'] = $request['register_password'];
                $request['password_confirm'] = $request['register_password'];
                unset($request['register_password']);
            }
            if ($form->isValid($request)) {
                Ecart::single('account/customer')
                    ->add($request['email'], $request['password'], $request)
                    ->login($request['email'], $request['password']);
            }
        }

        /* Receive payment */
        try {
            $checkout->payment()->preProcess();
            /* create order */
            $order = Ecart::single('sales/order')->createFromCheckout();
            
            Ecart::dispatch('sales_order_create_success', array('order' => $order));
            
            $checkout->setOrderId($order->id);

            $checkout->payment()->postProcess($order);
            
            //$this->_sendMails($order);

            $this->render();
            $checkout->payment()->clear();
            $this->_redirect('/checkout/success');
            
        } catch (Exception $e) {
            Ecart::dispatch('sales_order_create_failed', array('exception' => $e));
            $message = $e->getMessage();
            if (!empty($message)) {
                Ecart::message()->addError($message);
                error_log($message);
            }
            $this->_redirect('/checkout/cart');
            return;
        }
    }
    
    public function successAction()
    {
        $this->view->pageTitle = Ecart::translate('checkout')->__(
            'Checkout Success'
        );
         /* analytic ZA4OT*/
        Ecart::config()->analytics->main->checkoutSuccess = true; 
        
        if (!$orderId = $this->_getCheckout()->getOrderId()) {
            $this->_redirect('/checkout/onepage');
        }
        $this->view->orderId = $orderId;
        $this->_getCheckout()->clean();
        
        $this->render();
    }
       
    public function cancelAction()
    {
        $this->_getCheckout()->clean();
        $this->_redirect('/checkout/cart');
    }   
     
}