<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_OnepageController extends Ecart_Checkout_Controller_Checkout
{
    public function init()
    {
        parent::init();
        $this->view->pageTitle = Ecart::translate('checkout')->__('Checkout Process');
        $this->view->crumbs()->add(
            Ecart::translate('checkout')->__(
                'Checkout Onepage'
            ),
            '/checkout/cart'
        );
        $this->view->meta()->setTitle(
            Ecart::translate('checkout')->__(
                'Checkout Process'
        ));
    }
    
    public function indexAction()
    {
        parent::_validateCart();
        
        $formSignup = Ecart::model('account/form_signup');
        $formSignup->setAttrib('onsubmit', 'register(); return false;');

        $this->view->formAddress = $this->_getAddressForm();
        $this->view->formSignup = $formSignup;
        $this->render('one-page');
    }

    public function asGuestAction()
    {
        $this->_helper->layout->disableLayout();
        /**
         *  @var Zend_Session_Namespace $storage
         */
        $storage = $this->_getCheckout()->getStorage();
        $storage->asGuest = true;

        $register = (bool)$this->_getParam('register_guest', false);
        $storage->registerGuest = $register;

        return $this->_helper->json->sendSuccess();
    }

    public function addressListAction()
    {
        $this->_helper->layout->disableLayout();
        $storage = $this->_getCheckout()->getStorage();
        if (!$storage->asGuest) {
            parent::addressListAction();
            return;
        }
        $formAddress = $this->_getAddressForm();
        $formAddress->getActionBar()->removeElement('submit');
        if ('billing-address' === ($type = $this->_getParam('type'))) {
            $formAddress->setAttrib('id', 'guest-billing-form-new-address');
            if ($storage->registerGuest) {
                $formAddress->addRegisterField();
            } else {
                $formAddress->addGuestField();
            }
        } else {
            $formAddress->setAttrib('id', 'guest-delivery-form-new-address');
            $formAddress->populate($storage->billing);
        }

        $formAddress->setAction(
            $this->view->href('/checkout/onepage/set-' . $type)
        );
        echo $formAddress->render();
    }
    
    public function setBillingAddressAction()
    {
        $this->_helper->layout->disableLayout();
        $checkout = $this->_getCheckout();
        $storage = $checkout->getStorage();

        if (!$storage->asGuest) {
            $addressId = $this->_getParam('billing-address-id');
            if (!$this->_checkAddress($addressId)) {
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        'Invalid address'
                    )
                );
                return $this->_helper->json->sendFailure();
            }
            if ($this->_getParam('use_as_delivery', false)) {
                $result = $checkout->setBillingAddress($addressId) &&
                          $checkout->setDeliveryAddress($addressId);
                return $this->_helper->json->sendJson(array('success' => $result));
            }
            return $this->_helper->json->sendJson(array(
                'success' => $checkout->setBillingAddress($addressId)
            ));
        }
        //GUEST
        $formAddress = $this->_getAddressForm();
        if ($storage->registerGuest) {
            $formAddress->addRegisterField();
        } else {
            $formAddress->addGuestField();
        }
        //form action must be "post"
        $request = $this->_request->getPost();
        
        if (!$formAddress->isValid($request)) {
            return $this->_helper->json->sendJson(
                $formAddress->getMessages(), false, false
            );
        }
        $address = $request;
        if ($this->_getParam('use_as_delivery', false)) {
            $result = $checkout->setBillingAddress($address) &&
                      $checkout->setDeliveryAddress($address);
            return $this->_helper->json->sendJson(array('success' => $result));
        }
        return $this->_helper->json->sendJson(array(
            'success' => $checkout->setBillingAddress($address)
        ));
    }
    
    public function setDeliveryAddressAction()
    {
        $this->_helper->layout->disableLayout();
        $checkout = $this->_getCheckout();
        $storage = $checkout->getStorage();

        if (!$storage->asGuest) {
            $addressId = $this->_getParam('delivery-address-id');
            if (!$this->_checkAddress($addressId)) {
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        'Invalid address'
                    )
                );
                return $this->_helper->json->sendFailure();
            }
            return $this->_helper->json->sendJson(array(
                'success' => $checkout->setDeliveryAddress($addressId)
            ));
        }
        
        $formAddress = $this->_getAddressForm();
        //form action must be "post"
        $request = $this->_request->getPost();

        if (!$formAddress->isValid($request)) {
            return $this->_helper->json->sendJson(
                $formAddress->getMessages(), false, false
            );
        }
        $address = $request;

        return $this->_helper->json->sendJson(array(
            'success' => $checkout->setDeliveryAddress($address)
        ));
    }

    public function shippingMethodAction()
    {
        $this->_helper->layout->disableLayout();
        parent::shippingMethodAction();
    }
    
    public function paymentMethodAction()
    {
        $this->_helper->layout->disableLayout();
        parent::paymentMethodAction();
    }
    
    public function setPaymentMethodAction()
    {
        $this->_helper->layout->disableLayout();
        $methodCode = $this->_getParam('method');
        
        if (!in_array($methodCode, Ecart_Payment::getMethodNames())) {
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    "'%s' method not found among installed modules", $methodCode
                )
            );
            return $this->_helper->json->sendFailure();
        }
        
        $method = Ecart_Payment::getMethod($methodCode);
        if (!$method instanceof Ecart_Method_Payment_Model_Abstract ||
            !$method->isEnabled() ||
            !$method->isAllowed($this->_getCheckout()->getPaymentRequest())) 
        {
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    'Selected payment method is not allowed'
                )
            );
            return $this->_helper->json->sendFailure();
        }
        
        $this->_getCheckout()->setPaymentMethod($methodCode);
        
        return $this->_helper->json->sendJson(array(
            'success' => $this->_getCheckout()->payment()->saveData(
                $this->_getAllParams()
            )
        ));
    }

    public function setShippingMethodAction()
    {
        $this->_helper->layout->disableLayout();
        $methodCode = $this->_getParam('method');
        
        // methodCode can include method type also - Pickup_Standart|Ups_Standart_WXS
        list($moduleName, $methodName) = explode('_', $methodCode);

        if (!in_array($moduleName . '_' . $methodName, Ecart_Shipping::getMethodNames())) {
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    "'%s' method not found among installed modules", $methodCode
                )
            );
            return $this->_helper->json->sendFailure();
        }
        
        $method = Ecart_Shipping::getMethod($methodCode);
        if (!$method instanceof Ecart_Method_Shipping_Model_Abstract
            || !$method->isEnabled()
            || !$method->isAllowed($this->_getCheckout()->getShippingRequest()))
            {
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        'Selected shipping method in not allowed'
                    )
                );
            return $this->_helper->json->sendFailure();
        }
        
        $this->_getCheckout()->setShippingMethod($methodCode);
        return $this->_helper->json->sendSuccess();
    }
    
    public function confirmationAction()
    {
        $this->_helper->layout->disableLayout();
        parent::confirmationAction();
    }
}