<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_WizardController extends Ecart_Checkout_Controller_Checkout
{
    public function init()
    {
        parent::init();
        $this->view->pageTitle = Ecart::translate('checkout')->__('Checkout');
        $this->view->crumbs()->disable();
        $this->view->crumbs()->add(
            Ecart::translate('checkout')->__(
                'Checkout Wizard'
            ),
            '/checkout/wizard'
        );
        $this->view->meta()->setTitle(
            Ecart::translate('checkout')->__(
            'Checkout'
        ));
        if (!Ecart::getCustomerId()
            && !$this->_getCheckout()->getStorage()->asGuest
            && !in_array(
                $this->_request->getActionName(),
                array('index', 'login', 'as-guest')
            )) {

            $this->_forward('login');
        }
    }
    public function indexAction()
    {
        parent::_validateCart();
        $this->_forward('login');
    }

    public function loginAction() 
    {
        $this->view->pageTitle = Ecart::translate('checkout')->__('Login');
        
        if (Ecart::getCustomerId()) {
            $this->_redirect('/checkout/wizard/billing-address');
        }
        
        $formSignup = Ecart::single('account/form_signup');
        $formSignup->setAttrib('onsubmit', 'register(); return false;');
        
        $this->view->formAddress = $this->_getAddressForm();
        $this->view->formSignup = $formSignup;
        $this->view->nextPage = '/checkout/wizard/billing-address';
        $this->render();
    }

    public function asGuestAction()
    {
        /**
         *  @var Zend_Session_Namespace $storage
         */
        $storage = $this->_getCheckout()->getStorage();
        $storage->asGuest = true;

        $storage->registerGuest = (bool)$this->_getParam('register_guest', false);
        $this->_redirect('/checkout/wizard/billing-address');
    }
    
    public function billingAddressAction()
    {
        $storage = $this->_getCheckout()->getStorage();
        if (!$storage->asGuest) {
            $this->_forward(
                'address-list', null, null, array('type' => 'billing-address')
            );
            return;
        }
        $formAddress = $this->_getAddressForm();
        $formAddress->getActionBar()
            ->getElement('submit')
            ->setLabel(Ecart::translate('checkout')->__('Continue'));
            
        if ($storage->registerGuest) {
            $formAddress->addRegisterField();
        } else {
            $formAddress->addGuestField();
        }

        $formAddress->setAction(
            $this->view->href('/checkout/wizard/set-billing-address')
        );
        if ($storage->billing && $formAddress->isValid($storage->billing)) {
            $formAddress->populate($storage->billing);
        }
        echo $formAddress->render();
    }
    
    public function setBillingAddressAction()
    {
        $checkout = $this->_getCheckout();
        $storage = $checkout->getStorage();
        if (!$storage->asGuest) {
            $addressId = $this->_getParam('billing-address-id');
            if (!$this->_checkAddress($addressId)) {
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        'Incorrect address recieved'
                    )
                );
                $this->_redirect('/checkout/wizard/billing-address');
                die;
            }
            if ($this->_getParam('use_as_delivery', false)) {
                $result = $checkout->setBillingAddress($addressId) &&
                          $checkout->setDeliveryAddress($addressId);
                
                $this->_redirect('/checkout/wizard/shipping-method');
                return ;
            }
            $checkout->setBillingAddress($addressId);
            $this->_redirect('/checkout/wizard/delivery-address');
        }
        $formAddress = $this->_getAddressForm();
        if ($storage->registerGuest) {
            $formAddress->addRegisterField();
        } else {
            $formAddress->addGuestField();
        }
        $request = $this->_request->getPost();

        $checkout->setBillingAddress($request);
        if (!$formAddress->isValid($request)) {

            $this->_redirect('/checkout/wizard/billing-address');
            return;
        }
        if ((bool)$this->_getParam('use_as_delivery', false)) {
            $checkout->setDeliveryAddress($request);
            $this->_redirect('/checkout/wizard/shipping-method');
            return ;
        }

        $this->_redirect('/checkout/wizard/delivery-address');
    }

    public function deliveryAddressAction()
    {
        $storage = $this->_getCheckout()->getStorage();
        if (!$storage->asGuest) {
            $this->_forward('address-list', null, null, array('type' => 'delivery-address'));
            return;
        }
        $formAddress = $this->_getAddressForm();
        $formAddress->getActionBar()
            ->getElement('submit')
            ->setLabel(Ecart::translate('checkout')->__('Continue'));
        
        $formAddress->populate($storage->billing);
        $formAddress->setAction(
            $this->view->href('/checkout/wizard/set-delivery-address')
        );
        if ($storage->delivery && $formAddress->isValid($storage->delivery)) {
            $formAddress->populate($storage->delivery);
        }
        echo $formAddress->render();
    }
    
    public function setDeliveryAddressAction()
    {
        $checkout = $this->_getCheckout();
        $storage = $checkout->getStorage();
        if (!$storage->asGuest) {
            $addressId = $this->_getParam('delivery-address-id');
            if (!$this->_checkAddress($addressId)) {
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        'Incorrect address recieved'
                    )
                );
                $this->_redirect('/checkout/wizard/delivery-address');
                exit();
            }
            $checkout->setDeliveryAddress($addressId);
            $this->_redirect('/checkout/wizard/shipping-method');
            return ;
        }
        $formAddress = $this->_getAddressForm();
        $request = $this->_request->getPost();
        $checkout->setDeliveryAddress($request);
        $this->_redirect('/checkout/wizard/shipping-method');
        return ;
    }
    
    public function shippingMethodAction() 
    {
        $this->view->pageTitle = Ecart::translate('checkout')->__(
            'Shipping Method'
        );
        parent::shippingMethodAction();
    }
    
    public function paymentMethodAction()
    {
        $this->view->pageTitle = Ecart::translate('checkout')->__(
            'Payment Method'
        );
        parent::paymentMethodAction();
    }
    
    public function setPaymentMethodAction()
    {   
        $methodCode = $this->_getParam('method');
        
        if (!in_array($methodCode, Ecart_Payment::getMethodNames())) {
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    "'%s' method not found among installed modules", $methodCode
                )
            );
            $this->_redirect('/checkout/wizard/payment-method');
            exit();
        }
        
        $method = Ecart_Payment::getMethod($methodCode);
        if (!$method instanceof Ecart_Method_Payment_Model_Abstract ||
            !$method->isEnabled() ||
            !$method->isAllowed($this->_getCheckout()->getPaymentRequest())) 
        {
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    'Selected payment method in not allowed'
                )
            );
            $this->_redirect('/checkout/wizard/payment-method');
            exit();
        }
        
        $this->_getCheckout()->setPaymentMethod($methodCode);
        $this->_redirect('/checkout/wizard/confirmation');    
    }
    
    public function setShippingMethodAction()
    {
        $methodCode = $this->_getParam('method');
        
        // methodCode can include method type also - Pickup_Standart|Ups_Standart_WXS
        list($moduleName, $methodName) = explode('_', $methodCode);
        
        if (!in_array($moduleName . '_' . $methodName, Ecart_Shipping::getMethodNames())) {
            Ecart::message()->addError(Ecart::translate('checkout')->__(
                "'%s' method not found among installed modules", $methodCode
            ));
            $this->_redirect('/checkout/wizard/shipping-method');
            exit();
        }
        
        $method = Ecart_Shipping::getMethod($methodCode);
        if (!$method instanceof Ecart_Method_Shipping_Model_Abstract ||
            !$method->isEnabled() || 
            !$method->isAllowed($this->_getCheckout()->getShippingRequest())) 
        {
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    'Selected shipping method in not allowed'
                )
            );
            $this->_redirect('/checkout/wizard/shipping-method');
            exit();
        }
        
        $this->_getCheckout()->setShippingMethod($methodCode);
        $this->_redirect('/checkout/wizard/payment-method');
    }
    
    public function confirmationAction()
    {
        $this->view->pageTitle = Ecart::translate('checkout')->__(
            'Confirm your order'
        );
        parent::confirmationAction();
    }
}