    <?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Cms
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Cms
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Cms_Model_Category extends Ecart_Db_Table 
{
    protected $_name = 'cms_category';
    protected $_rowClass = 'Ecart_Cms_Model_Category_Row';
    
    public function getCatId($link)
    {
        $query = "SELECT ccc.cms_category_id
                  FROM " . self::$_db_prefix . "cms_category_content as ccc
                  WHERE  ccc.link = '$link'";
                  
        return $this->getAdapter()->fetchCol($query);
    }
    
    public function getActiveCategory()
    {
        $siteId = Ecart::getSiteId();
        $langId = Ecart::getLanguageId();
        $select = $this->getAdapter()->select();
        $select->from(array('cc' => self::$_db_prefix . 'cms_category'), array('id', 'parent_id'))
            ->join(array('ccc' => self::$_db_prefix . 'cms_category_content'),
                  'ccc.cms_category_id = cc.id',
                  array('link', 'title'))
            ->where('cc.is_active = 1')
            ->where('cc.site_id = ?', $siteId)
            ->where('ccc.language_id = ?', $langId)
            ->where('ccc.link is not null');
        return $this->getAdapter()->fetchAll($select->__toString());
        
    }
    
    public function getIdsByPage($pageId)
    {
        return $this->getAdapter()->fetchAssoc(
            'SELECT cms_category_id FROM ' . self::$_db_prefix . 'cms_page_category c WHERE cms_page_id = ? ', 
            $pageId
        );
        
    }
    
    private function _recurse(&$arr, $id, &$res = array()) 
    {
        if (isset($arr[$id]) && (null !== $arr[$id])) {
            $res[$id] = $arr[$id];
        } else { 
            return $res;
        }       
        if (null !== $arr[$id]['parent_id'] ) {
            $this->_recurse($arr, $arr[$id]['parent_id'], $res);
        } 
        return $res;
    }
    
    public function getParentsCategory($id, $isPage = false)
    {
        $siteId = Ecart::getSiteId();
        $langId = Ecart::getLanguageId();
        $select = $this->getAdapter()->select();
        $select->from(array('cc' => self::$_db_prefix . 'cms_category'), array('id', 'parent_id'))
            ->join(array('ccc' => self::$_db_prefix . 'cms_category_content'),
                'ccc.cms_category_id = cc.id',
                array('link', 'title', 'meta_title', 'meta_description', 'meta_keyword'))
            ->where('cc.is_active = 1')
            ->where('cc.site_id = ?', $siteId)
            ->where('ccc.language_id = ?', $langId)
            ->where('ccc.link is not null');
        $all = $this->getAdapter()->fetchAssoc($select->__toString());
        
        if ($isPage) {
            $id = current(array_keys($this->getIdsByPage($id)));
        }
        $res = array();
        $this->_recurse($all, $id, $res);
        return array_reverse($res);
    }
    
    /**
     *  return parent category for some site and lang
     * 
     */
    public function getRootCategories()
    {
        $select = $this->getAdapter()->select();
        
        $siteId = Ecart::getSiteId();
        $langId = Ecart::getLanguageId();
        
        $select->from(array('cc' => self::$_db_prefix . 'cms_category'), array('id'))
               ->join(array('ccc' => self::$_db_prefix .        'cms_category_content'),
                    'ccc.cms_category_id = cc.id',
                    array('link', 'title', 'description'))
               ->where('cc.is_active = ?', 1)
               ->where('cc.site_id = ?', $siteId)
               ->where('ccc.language_id = ?', $langId)
               ->where('cc.parent_id is ?', new Zend_Db_Expr('NULL'))
               ->where('ccc.link is not ?', new Zend_Db_Expr('NULL'));
        return $this->getAdapter()->fetchAll($select->__toString());
    } 
}