<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Cms
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Cms
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Cms_ViewController extends Ecart_Core_Controller_Front 
{
    public function viewPageAction()
    {
        $pageId = Ecart::single('cms/page')
            ->cache()
            ->getPageId($this->_getParam('page'));

        $currentPage = Ecart::single('cms/page')
            ->cache()
            ->find($pageId)
            ->current();
        $content = $currentPage->cache()->getContent(true);
        
        $this->view->page = array();
        $this->view->pageTitle = Ecart::translate('cms')->__($content['title']);
        $this->view->crumbs()->add(
            Ecart::translate('cms')->__('Pages'),
            '/pages'
        );
        
        $categories = Ecart::single('cms/category')
            ->cache()
            ->getParentsCategory(current($pageId), true);

        foreach ($categories as $category) {
           $this->view->crumbs()->add(
               empty($category['title']) ? $category['link'] : $category['title'],
               $this->view->href('/cat/' . $category['link'])
           ); 
        }
        $this->view->page['id'] = $currentPage->id;
        $this->view->page['content'] = $content['content'];
        $this->view->page['is_commented'] = $currentPage->comment;
        
        if ($currentPage->comment) {
            // get all comments
            $comments = $currentPage->cache()->getComments();
            $i = 0;
            foreach ($comments as $comment) {
                $this->view->page['comment'][$i]['author'] = $comment['author'];
                $this->view->page['comment'][$i]['content'] = $comment['content'];
                $this->view->page['comment'][$i]['created_on'] = $comment['created_on'];
                $this->view->page['comment'][$i]['modified_on'] = $comment['modified_on'];
                $this->view->page['comment'][$i]['status'] = $comment['status'];
                $i++;
            }
            
            // create comment form
            $this->view->formComment =
                Ecart::model('cms/form_comment', array('pageId' => $currentPage->id));
        }

        $metaTitle = $content['meta_title'] == '' ?
                $content['title'] : $content['meta_title'];
        $this->view->meta()
            ->setTitle($metaTitle, 'cms_page', current($pageId))
            ->setDescription($content['meta_description'])
            ->setKeywords($content['meta_keyword'])
        ;
        
        $layout = substr($currentPage->layout, strpos($currentPage->layout, '_'));
        $this->_helper->layout->setLayout('layout' . $layout);
        $this->render();
    }
    
    public function viewCategoryAction()
    {
        $tableCategory = Ecart::single('cms/category')->cache();
        $categoryId = $tableCategory->getCatId($this->_getParam('cat'));
        $currentCategory = $tableCategory->find($categoryId)->current();
        if (!$currentCategory->is_active) {
            $this->render(); //@todo 404 page should be here
            return;
        }
        $content = $currentCategory->getContent();
        
        $this->view->category = array();
        $title = empty($content['content']['title']) ?
            $this->_getParam('cat') : $content['content']['title'];
        $this->view->pageTitle = Ecart::translate('cms')->__($title);
        $this->view->crumbs()->add(
            Ecart::translate('cms')->__('Pages'),
            '/pages'
        );
        $categories = $tableCategory->getParentsCategory(current($categoryId));
        array_pop($categories);
        foreach ($categories as $category) {
           $this->view->crumbs()->add(
               empty($category['title']) ? $category['link'] : $category['title'],
               $this->view->href('/cat/' . $category['link'])
           ); 
        }
        $metaTitle = $content['content']['meta_title'] == '' ?
            $content['content']['title'] : $content['content']['meta_title'];
        $metaDescription = $content['content']['meta_description'] == '' ?
            $content['content']['description'] : $content['content']['meta_description'];
        $this->view->meta()
            ->setTitle($metaTitle, 'cms_category', current($categoryId))
            ->setDescription($metaDescription)
            ->setKeywords($content['content']['meta_keyword']);
        
        $this->view->category['description'] = $content['content']['description'];
        $this->view->category['childs'] = $content['childs'];
        $this->view->category['pages'] = $content['pages'];
        
        $this->render();
    }
}