<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Community
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Community
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Community_Model_Review_Mark extends Ecart_Db_Table
{
    protected $_name = 'community_review_mark';
    protected $_primary = array('review_id', 'rating_id');
    
    /**
     * Checks is customer has been voted already for product
     * 
     * @param int $customerId
     * @param int $productId
     * @param int $reviewId review that will be excluded from query.
     *  This makes possible to change already posted review marks.
     * @return bool
     */
    public function isCustomerVoted($customerId, $productId, $reviewId = null)
    {
        if (!is_numeric($customerId)) { // guests do not have permission to vote
            return true;
        }
        
        $select = $this->select()
            ->from(array('crm' => $this->_name))
            ->join(array('cr' => self::$_db_prefix . 'community_review'),
                'crm.review_id = cr.id',
                array())
            ->where('cr.product_id = ?', $productId)
            ->where('cr.customer_id = ?', $customerId);
        
        if (null !== $reviewId) {
            $select->where('cr.id <> ?', $reviewId);
        }
            
        return count($this->fetchAll($select)) ? true : false;
    }
    
}