<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Core
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Core
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Core_Model_Site extends Ecart_Db_Table 
{
    protected $_name = 'core_site';
	
    /**
     *
     * @return array 
     */
    public function getList()
    {
        $langId = Ecart::getLanguageId();
        
        $query = "SELECT s.*,
                         c.id AS root_category,
                         cd.name AS category_name
                  FROM $this->_name AS s
                  LEFT JOIN " . self::$_db_prefix . "catalog_category AS c ON s.id = c.site_id AND c.lvl = 0
                  LEFT JOIN " . self::$_db_prefix . "catalog_category_description AS cd ON c.id = cd.category_id AND cd.language_id = $langId";
                  
        return $this->getAdapter()->fetchAll($query);
    }

    /**
     *
     * @param int $siteId
     * @return Zend_Db_Table_Row_Abstract|null The row results per the
     *     Zend_Db_Adapter fetch mode, or null if no row found.
     */
    public function getInfo($siteId)
    {
        $query = "SELECT * FROM {$this->_name} AS s WHERE s.id = {$siteId}";
        return $this->getAdapter()->fetchRow($query);
    }

    /**
     *
     * @param string $url
     * @return mixed
     */
    public function getIdByUrl($url)
    {
        $sites = $this->fetchAll()->toArray();

        $scheme = 'https://';
        $base = 'secure';
        if (substr($url, 0, strlen($scheme)) != $scheme) {
            $base = 'base';
            $scheme = 'http://';
        }

        foreach ($sites as $site) {
            $www = $site[$base];
            if (substr($url, 0, strlen($www)) === $www) {
               return $site['id'];
            }

            if (substr($www, 0, strlen($scheme . 'www.')) === $scheme . 'www.') {
                $www = str_replace($scheme . 'www.', $scheme , $www);
            }

            if (substr($url, 0, strlen($scheme . 'www.')) === $scheme . 'www.') {
                $url = str_replace($scheme . 'www.', $scheme, $url);
            }

            if (substr($url, 0, strlen($www)) === $www) {
               return $site['id'];
            }
        }
        return false;
    }

    /**
     *
     * @param string $url
     * @return string
     */
    public function getBaseUrlByUrl($url)
    {
        $sites = $this->fetchAll()->toArray();

        $scheme = 'https://';
        $base = 'secure';
        if (substr($url, 0, strlen($scheme)) != $scheme) {
            $base = 'base';
            $scheme = 'http://';
        }

        foreach ($sites as $site) {
            $www = $site[$base];
            if (substr($url, 0, strlen($www)) === $www) {
               return $site[$base];
            }

            if (substr($www, 0, strlen($scheme . 'www.')) === $scheme . 'www.') {
                $www = str_replace($scheme . 'www.', $scheme , $www);

            }

            if (substr($url, 0, strlen($scheme . 'www.')) === $scheme . 'www.') {
                $url = str_replace($scheme . 'www.', $scheme, $url);
            }

            if (substr($url, 0, strlen($www)) === $www) {
               return $site[$base];
            }
        }

        return false;
    }

    /**
     *
     * @param mixed (int|bool) $siteId[optional]
     * @return array
     */
    public function getCompanyInfo($siteId = false)
    {
        if (!$siteId) {
            $siteId = Ecart::getSiteId();
        }
        Ecart::config()->setSiteId($siteId);
        $company  = Ecart::config()->main->company->toArray();
        return array(
            'email' =>  Ecart_Collect_MailBoxes::getName($company['administratorEmail']),
            'city' =>  $company['city'],
            'country' =>  array(
                'name' => Ecart_Collect_Country::getName($company['country']),
                'id' => $company['country']
            ),
            'customerRelationEmail' =>
                Ecart_Collect_MailBoxes::getName($company['customerRelationEmail']),
            'fax' =>  $company['fax'],
            'name' =>  $company['name'],
            'phone' =>  $company['phone'],
            'postal_code' =>  $company['zip'],
            'sales_email' =>   Ecart_Collect_MailBoxes::getName($company['salesDepartmentEmail']),
            'site' =>  $company['site'],
            'street_address' =>  $company['street'],
            'support_email' =>  Ecart_Collect_MailBoxes::getName($company['supportEmail']),
            'zone' =>  array(
                'name' => Ecart_Collect_Zone::getName($company['zone']),
                'id' => $company['zone']
            )

        );
    }

    /**
     *
     * @param array $data
     * @return void
     */
    public function save($data)
    {
        $languages = Ecart_Collect_Language::collect();

        foreach ($data as $id => $values) {
            if ($values['id'] == 'new') {
                $siteId = $this->insert(array(
                    'name' => $values['name'],
                    'base' => $values['base'],
                    'secure' => $values['secure']
                ));
                $categoryId = Ecart::single('catalog/category')->insert(array(
                    'site_id' => $siteId,
                    'lft' => 1,
                    'rgt' => 2,
                    'lvl' => 0,
                    'created_on' => Ecart_Date::now()->toZendDbExpr(),
                    'cat_status' => 'active'
                ));
            
                foreach ($languages as $langId => $name) {

                    Ecart::single('catalog/category_description')->insert(array(
                        'category_id' => $categoryId,
                        'language_id' => $langId,
                        'name' => $values['name'],
                        'description' => 'Root Category'
                    ));
                }
            } else {
                $this->update(array(
                    'name'   => $values['name'],
                    'base' => $values['base'],
                    'secure' => $values['secure']
                ), "id = $values[id]");
            }
        }
    }
}