<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Core
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Core
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Core_Model_Template_Mail extends Ecart_Db_Table 
{
    protected $_name = 'core_template_mail';
    protected $_rowClass = 'Ecart_Db_Table_Row';
    
    /**
     * Updates or add record
     * 
     * @param array $data
     * @return bool
     */
    public function save($data)
    {
        if (!sizeof($data)) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'No data to save'
            ));
            return false;
        }

        foreach ($data as $id => $rowset) {
        
            /* saving content into file */ 
            if (!empty($data[$id]['content'])) {    
                $file = Ecart::config()->system->path . '/app/design/mail/' 
                          . $data[$id]['template'] . '_'
                          . $data[$id]['type'] . '.phtml';
                if (!is_writable($file)) {
                    Ecart::message()->addError(
                        Ecart::translate('core')->__(
                            "Can't open file with write permissions : %s", $file
                    ));
                } else {
                    if (!@file_put_contents($file, $data[$id]['content'])) {
                        Ecart::message()->addError(
                            Ecart::translate('core')->__(
                                "Can't write content to file :%s", $file
                        ));
                    }
                }
            }
        
            if (!isset($rowset['id']) 
                || !$row = $this->find($rowset['id'])->current()) {
                
                $row = $this->createRow();
            }
            $row->setFromArray($rowset);
            $row->save();
        }
        
        Ecart::message()->addSuccess(
            Ecart::translate('core')->__(
                'Data was saved successfully'
        ));
        return true;
    }
    
    /**
     * Retrieve template info by id
     * 
     * @param int $id
     * @return array
     */
    public function getInfo($id)
    {
        if (!is_numeric($id) || !$info = $this->find($id)->current()) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'Template not found'
            ));
            return array();
        }
        
        $info = $info->toArray();
        $templates = Ecart_Collect_MailTemplate::collect();
        
        $file = Ecart::config()->system->path . '/app/design/mail/' 
              . $templates[$info['template']] . '_'
              . $info['type'] . '.phtml';

        $content = '';
        if (is_readable($file)) {
            $content = @file_get_contents($file);
        }
        
        $info['content'] = $content;
        
        return $info;
    }
}