<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Locale
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Locale
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Locale_Model_Currency extends Ecart_Db_Table
{
    protected $_name = 'locale_currency';
    
    /** 
     * Currencies data (code, rate, etc.)
     *
     * @var array
     */
    private $_data = array();
    
    /**
     * @var string - 'USD'
     */
    private $_currentCurrencyCode = null;
    
    /**
     * Array of Ecart_Currency objects
     *
     * @var array
     */
    private $_currency = array();

    /**
     *
     * @return array
     */
    public function getFormat() 
    {
        $row = $this->getData();
        $currency = $this->getCurrencyObject();

        $positions = self::getPositionOptions();
        $position = $positions[$row['position']];
        if ($position === "Standart") {
           $position = $currency->toCurrency(1);
           $position = strpos($position, $currency->getSymbol());
           if ($position) {
               $position = 'Right';
           } else {
               $position = 'Left';
           } 
        }
        //$displays = self::getDisplayOptions();
        $symbols =  Zend_Locale_Data::getList(
            Ecart_Locale::getLocale()->toString(), 'symbols'
        );
        return array(
            'precision' => $row['currency_precision'],
            'requiredPrecision' => 2,
            'integerRequired' => 1, 
            'decimalSymbol' => $symbols['decimal'],
            'groupSymbol' => $symbols['group'],
            'groupLength' => 3,
            'position' => $position,
            'symbol' => null === $currency->getSymbol() ?
                $currency->getShortName() : $currency->getSymbol(),
            'shortName' => $currency->getShortName(),
            'name' => $currency->getName(),
            'display' => $row['display']//$displays[$row['display']]
        );
    }

    /**
     *
     * @param string $code
     * @return bool
     */
    public function isExists($code)
    {
        if (!empty($code) && $this->getData($code)) {
            return true;
        } 
        return false;
    }
    
    /**
     * Return Ecart_Currency object
     *
     * @param string $code
     * @return Ecart_Currency
     */
    public function getCurrencyObject($code = '')
    {
        if (empty($code)) {
            $code = $this->getCode();
        }
        if (!isset($this->_instanses[$code])) {
            $options = $this->_getCurrencyOptions($code);
            $currency = new Ecart_Currency(
                $options['currency'], 
                Ecart_Locale::getLocale()
            );
            $currency->setFormat($options);
            $this->_currency[$code] = $currency;
        }
        return $this->_currency[$code];
    }

    /**
     * @static
     * @return const array
     */
    public static function getPositionOptions()
    {
        return array(
            '8'     => Ecart::translate('locale')->__('Standard'),
            '16'    => Ecart::translate('locale')->__('Right'),
            '32'    => Ecart::translate('locale')->__('Left')
        );
    }

    /**
     * @static
     * @return const array
     */
    public static function getDisplayOptions()
    {
        return array(
            '1' => Ecart::translate('locale')->__('No Symbol'),
            '2' => Ecart::translate('locale')->__('Use Symbol'),
            '3' => Ecart::translate('locale')->__('Use Shortname'), 
            '4' => Ecart::translate('locale')->__('Use Name') 
        );
    }

    /**
     *
     * @return string
     */
    public function getCode()
    {
        if (null === $this->_currentCurrencyCode) {
            if (isset(Ecart::session()->currency)
                && $this->isExists(Ecart::session()->currency)) {
                
                $this->_currentCurrencyCode = Ecart::session()->currency;
                
            } elseif (isset(Ecart::config()->main->store->currency)
                && $this->isExists(Ecart::config()->main->store->currency)) {
                
                $this->_currentCurrencyCode = Ecart::config()
                    ->main->store->currency;
                
            } elseif ($this->isExists(Ecart_Locale::DEFAULT_CURRENCY)) {
                $this->_currentCurrencyCode = Ecart_Locale::DEFAULT_CURRENCY;
            } else {
                $this->_currentCurrencyCode = $this->fetchOne('code');
                if (!$this->_currentCurrencyCode) {
                    throw new Ecart_Exception(
                        Ecart::translate('locale')->__(
                            "No currencies found"
                    ));
                }
            }
        }
        return $this->_currentCurrencyCode;
    }

    /**
     *
     * @param string $code
     * @return array
     */
    private function _getCurrencyOptions($code)
    {
        $row = $this->getData($code);
        
        $options = array(
            'currency' => $row['code']
        );
        if ($row['position'])
            $options['position'] = (int) $row['position'];
        
        if ($row['display'])
            $options['display'] = (int) $row['display'];
        
        if ($row['format'])
            $options['format'] = Ecart_Locale::getLocale()->toString();
        
        if ($row['currency_precision'])
            $options['precision'] = (int) $row['currency_precision'];
        
        return $options;
    }
    
    //@todo swap arguments
    public function getData($code = '', $key = '')
    {
        if (empty($code)) {
            $code = $this->getCode();
        }
        
        if (!isset($this->_data[$code])) {
            $row = $this->getAdapter()->fetchRow(
                "SELECT * FROM {$this->_name} WHERE code = ?", $code
            );
            $this->_data[$code] = $row;
        }
        
        if (!empty($key)) {
            return $this->_data[$code][$key];
        }
        
        return $this->_data[$code];
    }

    /**
     *
     * @param float $price
     * @param bool $useRate [optional]
     * @param string $code [optional]
     * @param bool $format [optional]
     * @return <type>
     */
    public function toCurrency(
        $price, $useRate = true, $code = '', $format = true)
    {
        if ($useRate) {
            $price *= $this->getData($code, 'rate');
        } 
        if (!$format) {
            return $price;
        }
        return $this->getCurrencyObject($code)->toCurrency($price);
    }

    /**
     * From some currency to abstract currency
     *
     * @param float $price
     * @param string $code
     * @return float
     */
    public function from($price, $code = '')
    {
        return $price / $this->getData($code, 'rate');
    }

    /**
     * From abstract currency to code currency
     * 
     * @param float $price
     * @param string $code
     * @return float
     */
    public function to($price, $code = '')
    {
        return $price * $this->getData($code, 'rate');
    }

    /**
     * Convert currency
     *
     * @param float $price
     * @param string $from
     * @param string $to
     * @return float
     */
    public function convert($price, $from, $to)
    {
        return ($price * $this->getData($to, 'rate')) /
            $this->getData($from, 'rate');
    }

    public function getList()
    {
        $select = $this->getAdapter()->select();
        
        $select->from($this->_name);
        
        return $this->getAdapter()->fetchAll($select->__toString());
    }

    /**
     *
     * @param array $data
     * @return bool
     */
    public function save($data)
    {
        $db = $this->getAdapter();
        
        $ids = array_keys($data);
        $where = $db->quoteInto('id IN(?)', $ids);
        $exists = $db->fetchCol("SELECT id FROM {$this->_name} WHERE $where");

        $result = true;
        foreach ($data as $id => $values) {
            $row = array(
                'code' => $values['code'],
                'title' => $values['title'],
                'position' => $values['position'],
                'display' => $values['display'],
                'format' => $values['format'],
                'currency_precision' => $values['currency_precision'],
                'rate' => $values['rate']
            );
            
            if (in_array($id, $exists)) {
                $result = $result &&
                    $this->update($row, $db->quoteInto('id = ?', $id));
            } else {
                $result = $result && $this->insert($row);
            }
        }
        return $result;
    }
    
    public function getCurrentCurrency()
    {
        return $this->getCode();
    }
}