<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Location
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Location
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Location_Model_Country extends Ecart_Db_Table
{
	/**
	 * The default table name 
	 */
	protected $_name = 'location_country';

    /**
     *
     * @param array $data
     */
	public function save(array $data)
	{
	    $db = $this->getAdapter();
        
        $where = $db->quoteInto('id IN (?)', array_keys($data));
        $exists = $db->fetchCol("SELECT id FROM {$this->_name} WHERE $where");

        foreach ($data as $id => $values) {
            if (empty($values['iso_code_2'])
                || empty($values['iso_code_3'])) {

                Ecart::message()->addError(
                    Ecart::translate('core')->__(
                        'Required fields are missing'
                    )
                );
                continue;
            }
            $row = array(
                'name' => $values['name'],
                'iso_code_2' => $values['iso_code_2'],
                'iso_code_3' => $values['iso_code_3'],
                'address_format_id' => $values['address_format_id']
            );

            if (in_array($id, $exists)) {
                $this->update($row, $db->quoteInto('id = ?', $id));
            } else {
                $this->insert($row);
            }

            Ecart::message()->addSuccess(
                Ecart::translate('location')->__(
                    'Country "%s" has been saved succesfully', $values['name']
                )
            );
        }
	}
	
	/**
	 * Fetch country row by country_name
	 * 
	 * @param string Country name
	 * @return Zend_Db_Table_Row_Abstract
	 */
    public function fetchByName($name)
    {
        $where = $this->getAdapter()->quoteInto('name = ?', $name);
        return $this->fetchRow($where)->toArray();
    }
}
