<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Location
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Location
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Location_Model_Geozone extends Ecart_Db_Table
{
	protected $_name = 'location_geozone';
	
    public function update(array $data, $where)
    {
    	if (empty($data['modified_on'])) {
    		$data['modified_on'] = Ecart_Date::now()->toZendDbExpr();
    	}
    	return parent::update($data, $where);
    }
    
    public function insert(array $data)
    {
        if (empty($data['created_on'])) {
            $data['created_on'] = Ecart_Date::now()->toZendDbExpr();
        }
    	return parent::insert($data);
    }
    
    /**
     * Return Geozone row
     *
     * @param integer $addressId
     * @return Zend_Db_Table_Row_Abstract
     */
    public function findByAddress($addressId)
    {
        $address = Ecart::single('account/customer_address')->find($addressId);
        if (!$address) {
            return false;
        }
        $address = $address->current();
        
        $geozoneId = $this->getId($address->country_id, $address->zone_id);
        
        if (!$geozoneId) {
            return false;
        }
        return $this->find($geozoneId)->current();
    }
    
    /**
     *  Saving rowset (batch save)
     * @return bool
     */
    public function save($rows) 
    {
    // Saving exists values
        foreach ($rows as $row) {
            $rowset = array(
                'name' => $row['name'],
                'description' => $row['description'],
                'priority' => $row['priority']
            ); 
            if (isset($row['id'])) {// update
                $this->update($rowset, 'id = ' . intval($row['id']));
            } else { // insert
                $this->insert($rowset);
            }
        }
        return true;
    }
    
    /**
     * Return Geozone id by country & zone
     *
     * @param integer $countyrId
     * @param integer $zoneId
     * @return integer
     */
    public function getId($countryId, $zoneId = 0)
    {
        return current($this->getIds($countryId, $zoneId = 0));
    }
    
    /**
     * Return Geozone id by country & zone
     *
     * @param integer $countyrId
     * @param integer $zoneId
     * @return integer
     */
    public function getIds($countryId, $zoneId = 0)
    {
        // IF $countryId == 0 => ALL COUNTRY (in country table row with id 241)
        // IF $zoneId == 0 => ALL country zones 
        if (null !== $zoneId) {
            $zoneId .= ',';
        }
        
        if (null !== $countryId) {
            $countryId .= ',';
        }
        $query = "SELECT lg.id FROM {$this->_name} lg
            LEFT JOIN " . self::$_db_prefix . "location_geozone_zone lgz ON  lg.id = lgz.geozone_id
            WHERE lgz.country_id IN ({$countryId} 0, 241) AND lgz.zone_id IN ({$zoneId} 0, 200)
            ORDER BY lg.priority DESC";
//        Zend_Debug::dump($query);
//        die;
        $rawIds = $this->getAdapter()->fetchAll($query);
        $geozoneIds = array();
        foreach ($rawIds as $row) {
            $geozoneIds[] = $row['id'];
        }
        return $geozoneIds;
    }
}