<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_PaymentAuthorizenetAim
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * @var Ecart_Install_Model_Installer
 */
$installer = Ecart::single('install/installer');

$installer->startSetup();

$installer->run("

-- DROP TABLE IF EXISTS `{$installer->getTable('payment_authorizenetaim_standart_order')}`;
CREATE TABLE  `{$installer->getTable('payment_authorizenetaim_standart_order')}` (
  `order_id` int(10) unsigned NOT NULL,
  `trans_id` varchar(128) NOT NULL,
  `cc_type` varchar(32) NOT NULL,
  `cc_owner` varchar(64) NOT NULL,
  `cc_number` varchar(64) NOT NULL,
  `cc_expires` varchar(16) NOT NULL,
  `cc_issue` varchar(16) NOT NULL,
  `x_type` varchar(16) NOT NULL,
  `x_method` varchar(8) NOT NULL,
  PRIMARY KEY  USING BTREE (`order_id`),
  CONSTRAINT `FK_paymnet_authorizenetaimstandart_order_1` FOREIGN KEY (`order_id`) REFERENCES `{$installer->getTable('sales_order')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

");

Ecart::single('core/config_field')
    ->add('payment/AuthorizenetAim_Standart/title', 'Payment Methods/Authorize.Net Aim/Title', 'Authorize.Net', 'string', 'Title')
    ->add('payment/AuthorizenetAim_Standart/enabled', 'Enabled', 0, 'bool', 'Do you want to accept Authorize.net payments via the AIM Method?')
    ->add('payment/AuthorizenetAim_Standart/title', 'Title', 'AuthorizenetAim', 'string', 'Title')
    ->add('payment/AuthorizenetAim_Standart/shippings', 'Allowed Shippings', '0', 'multiple', 'If a shippings method is selected, only enable this shipping method for that shippings.', array('model' => 'Shipping'))
    ->add('payment/AuthorizenetAim_Standart/creditCard', 'Accepted credit cards', 'Visa', 'multiple', 'Types credit cards used in this payment', array('model' => 'CreditCard'))
    ->add('payment/AuthorizenetAim_Standart/geozone', 'Allowed Payment Zone', 1, 'select', 'If a zone is selected, only enable this payment method for that zone.', array('model' => 'Geozone'))
    ->add('payment/AuthorizenetAim_Standart/sortOrder', 'Sort Order', '1', 'string', 'Sort Order Lowest is displayed first.')
    ->add('payment/AuthorizenetAim_Standart/orderStatusId', 'Order Status on payment complete', 2, 'select', 'Set the status of orders made with this payment module to this value', /*array('model' => 'OrderStatus')*/array('config_options' => '{"2":"processing"}'))
    ->add('payment/AuthorizenetAim_Standart/authorizationType','Authorization Type', '1', 'select', 'Do you want submitted credit card transactions to be authorized only, or authorized and captured?', array('config_options' => '{"0":"Authorize","1"Authorize Capture"}'))
    ->add('payment/AuthorizenetAim_Standart/xLogin', 'Login Id', '', 'handler', 'The API Login ID used for the Authorize.net service', array('model' => 'Crypt'))
    ->add('payment/AuthorizenetAim_Standart/xTransactionKey','Transaction Key', '', 'handler', 'Transaction Key used for encrypting TP data (See your Authorizenet Account->Security Settings->API Login ID and Transaction Key for details.)', array('model' => 'Crypt'))
    ->add('payment/AuthorizenetAim_Standart/gateway', 'Gateway URL', 'https://test.authorize.net/gateway/transact.dll')
    ->add('payment/AuthorizenetAim_Standart/emailCustomer', 'Customer Notification', 0, 'bool', 'Should Authorize.Net email a receipt to the customer?')
    ->add('payment/AuthorizenetAim_Standart/emailMerchant', 'Merchant Email', '')
    ->add('payment/AuthorizenetAim_Standart/test', 'Test Mode', 0, 'bool')


    ;

$installer->registerModule('Ecart_PaymentAuthorizenetAim');

$installer->endSetup();