<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_PaymentPaypal
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_PaymentPaypal
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_PaymentPaypal_Model_Direct extends Ecart_PaymentPaypal_Model_Abstract
{
    protected $_code = 'Paypal_Direct';
    protected $_title = 'PayPal Direct Payment';
    
    protected $_buttonSourceDP = 'Ecart-DP_us';
    
    public function init()
    {
        parent::init();
        
        if ($this->_config->mode == 'Payflow-UK') {
            $this->_buttonSourceDP = 'Ecart-DP_uk';
        }
        if ($this->_config->mode == 'Payflow-US') {
            $this->_buttonSourceDP = 'Ecart-GW_us';
        }
    }
    
    public function pending(Ecart_Sales_Model_Order_Row $order)
    {
        $cc = $this->getCreditCard();
        Ecart::single('sales/order_creditcard')->save($cc, $order);
        $options = $this->getLineItemDetails();
        $billing = $order->getBilling();
        
        $optionsAll = array_merge(
            $options, 
            array(
                'STREET' => $billing->getStreet(),
                'ZIP'    => $billing->getZip(),
                'BUTTONSOURCE' => $this->_buttonSourceDP,
                'CURRENCY'     => $this->getBaseCurrencyCode(),
                'IPADDRESS'    => $_SERVER['REMOTE_ADDR']
            )
        );
        if ($cc->getCcIssueMonth() && $cc->getCcIssueYear()) {
            $optionsAll['CARDSTART'] = $cc->getCcIssueMonth() . substr($cc->getCcIssueYear(), -2);
        }
        
        $optionsNVP = array(
            'CITY'        => $billing->getCity(),
            'STATE'       => $billing->getStateCode() ?
                $billing->getStateCode() : $billing->getCity(),
            'COUNTRYCODE' => $billing->getCountryCode(),
            'EXPDATE'     => $cc->getCcExpiresMonth() . $cc->getCcExpiresYear(),
            'PAYMENTACTION' => ($this->_config->paymentAction == 'Authorization') ? 'Authorization' : 'Sale'
        );
        
        $delivery = $order->getDelivery();
        $optionsShip = array(
            'SHIPTONAME'   => $delivery->getFirstname() . ' ' . $delivery->getLastname(),
            'SHIPTOSTREET' => $delivery->getStreet(),
            'SHIPTOSTREET2'=> $delivery->getStreet2(),
            'SHIPTOCITY'   => $delivery->getCity(),
            'SHIPTOZIP'    => $delivery->getZip(),
            'SHIPTOSTATE'  => $delivery->getStateCode() ?
                $delivery->getStateCode() : $delivery->getCity(),
            'SHIPTOCOUNTRYCODE'=> $delivery->getCountryCode()
        );
        // if these optional parameters are blank, remove them from transaction
        if (isset($optionsShip['SHIPTOSTREET2']) && empty($optionsShip['SHIPTOSTREET2'])) {
            unset($optionsShip['SHIPTOSTREET2']);
        }
        
        $response = $this->getApi()->DoDirectPayment(
            sprintf('%.2f', $this->getAmountInBaseCurrency($order->getSubTotal())),
            $cc->getCcNumber(),
            $cc->getCcCvv(),
            $cc->getCcExpiresMonth() . $cc->getCcExpiresYear(),
            $billing->getFirstname(),
            $billing->getLastname(),
            $cc->getCcType(),
            $optionsAll, 
            array_merge($optionsNVP, $optionsShip)
        );

        if ($response['ACK'] != 'Success') {
            $this->log(
                "Response : " . Zend_Debug::dump($response, null, false)
            );
            
            foreach ($this->getMessages($response) as $severity => $messages) {
                Ecart::message()->batchAdd($messages, $severity);
            }
            
            throw new Ecart_Exception('DoDirectPayment Failure');
        }
       
        return true;
    }
}