<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_PaymentPaypal
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_PaymentPaypal
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_PaymentPaypal_ExpressCheckoutController extends Ecart_Checkout_Controller_Checkout
{

    /**
     *
     * @var Ecart_PaymentPaypal_Model_Express
     */
    private $_payment;

    protected function _getAddressForm()
    {
        $form = Ecart::model('account/Form_AddressBook');
        $form->removeElement('primary_billing');
        $form->removeElement('primary_shipping');
        $form->removeElement('address_submit');
        return $form;
    }
    
    protected function _checkAddress($addressId)
    {
        if (!$addressId) {
            return false;
        }
        /*
         * Check if address assigned to this customer
         */
        $address = Ecart::single('account/customer_address')->find($addressId)->current();
        if (!$address || $address->customer_id != Ecart::getCustomerId()) {
            return false;
        }
        return true;
    }
    
    public function init()
    {
        parent::init();
        
        $this->_payment = Ecart::single('paymentPaypal/express');
        if (!$this->_payment->isEnabled()) {
            $this->_redirect('/checkout/cart');
        }
    }
    
    public function indexAction()
    {
        //SetExpressCheckout get token
        if (!$this->_payment->runSetExpressCheckout()) {
            $this->_redirect('/checkout/cart');
        }
        
        $this->_getCheckout()->setPaymentMethod(
            $this->_payment->getCode()
        );

        $useraction = '&useraction=continue';
        if (true == $this->_payment->getStorage()->markflow) {
            $useraction = '&useraction=commit';
        }

        $this->_redirect(
            $this->_payment->getPayPalLoginServer()
                . "?cmd=_express-checkout&token="
                . $this->_payment->getStorage()->token
                . $useraction,
            array(),
            false
        );
    }

    public function detailsAction()
    {
        if (empty($this->_payment->getStorage()->token)) {
            $this->_redirect('/paymentpaypal/express-checkout/index');
        }

        $response = $this->_payment->runGetExpressCheckoutDetails();

        if (empty($this->_payment->getStorage()->payer['payer_id'])) {

            $this->_payment->getStorage()->token = null;
            $this->_redirect('/paymentpaypal/express-checkout/index');
        }

        if ($response) {
             $this->_getCheckout()->setDeliveryAddress(array(
                'firstname' => urldecode($response['FIRSTNAME']),
                'lastname' => urldecode($response['LASTNAME']),
                'street_address' =>  urldecode($response['SHIPTOSTREET']),
                'suburb' => urldecode($response['SHIPTOSTREET2']),
                'city' =>  urldecode($response['SHIPTOCITY']),
                'postcode' => urldecode($response['SHIPTOZIP']),
                'zone' => Ecart::single('location/zone')
                    ->getByCode(urldecode($response['SHIPTOSTATE'])),
                'country' => Ecart::single('location/country')
                    ->fetchByName(urldecode($response['SHIPTOCOUNTRYNAME'])),
                'company' => !empty($response['BUSINESS']) ?
                    urldecode($response['BUSINESS']) : ''
            ));
        }

        if (empty($this->_getCheckout()->getStorage()->delivery)) {
            $this->_redirect('/paymentpaypal/express-checkout/delivery');
        }

        if (empty($this->_getCheckout()->getStorage()->shipping)) {
            $this->_redirect('/paymentpaypal/express-checkout/shipping-method');
        }
        if (true == $this->_payment->getStorage()->markflow) {
            $this->_redirect('/paymentpaypal/express-checkout/process');
        }

        $this->_forward('confirmation');
    }

    public function confirmationAction()
    {
        $checkout = $this->_getCheckout();
        $this->view->co = array(
            'delivery' => $checkout->getStorage()->delivery,
            'email' => urldecode(
                $this->_payment->getStorage()->payer['payer_email']
            ),
            'products' => $checkout->getCart()->getProducts(),
            'totals' => $checkout->getTotal()->collectTotals()->getCollects(),
            'total' => $checkout->getTotal()->getTotal(),
            'shipping' => $checkout->shipping()->getTitle()
        );
        $this->render();
    }

    public function deliveryAction()
    {
        $selectedAddressId = null;

        $storage = $this->_getCheckout()->getStorage();

        if (isset($storage->delivery['id'])) {
            $selectedAddressId = $storage->delivery['id'];
        }
        $this->view->assign(array(
            'selectedAddressId' => $selectedAddressId,
            'addressType'       => 'delivery',
            'addressList'       => Ecart::single('account/customer_address')
                ->fetchAll(
                    "customer_id = " . intval(Ecart::getCustomerId())
                )->toArray(),
            'formAddress'       => $this->_getAddressForm()
        ));
        $this->render('delivery');
    }

    public function setShippingMethodAction()
    {
        $methodCode = $this->_getParam('method');
        // methodCode can include method type also - Pickup_Standart|Ups_Standart_WXS
        list($moduleName, $methodName) = explode('_', $methodCode);

        if (!in_array(
                $moduleName . '_' . $methodName,
                Ecart_Shipping::getMethodNames()
            )) {

            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    "'%s' method not found among installed modules", $methodCode
                )
            );
            $this->_redirect('/paymentpaypal/express-checkout/shipping-method');
        }

        $method = Ecart_Shipping::getMethod($methodCode);
        if (!$method instanceof Ecart_Method_Shipping_Model_Abstract ||
            !$method->isEnabled() ||
            !$method->isAllowed($this->_getCheckout()->getShippingRequest()))
        {
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    'Selected shipping method in not allowed'
                )
            );
            $this->_redirect('/paymentpaypal/express-checkout/shipping-method');
        }

        $this->_getCheckout()->setShippingMethod($methodCode);
        $this->_redirect('/paymentpaypal/express-checkout/confirmation');
    }
	
    public function setDeliveryAction()
    {   
        $addressId = $this->_getParam('delivery-id');
        if ($this->_checkAddress($addressId)) {
            $this->_getCheckout()->setDeliveryAddress($addressId);
        } else {
            $data = $this->_getAllParams();
            $form = Ecart::single('account/Form_AddressBook');

            if (!$form->isValid($data)) {
                $this->_helper->json->sendJson(
                    $form->getMessages(), false, false
                );
            }
            $address = array(
                 'firstname'      => $data['firstname'],
                 'lastname'       => $data['lastname'],
                 'street_address' => $data['street_address'],
                 'entry_suburb'   => '',
                 'city'           => $data['city'],
                 'postcode'       => $data['postcode'],
                 'zone'           => Ecart::single('location/zone')
                    ->find($data['zone_id'])->current()->toArray(),
                 'country'        => Ecart::single('location/country')
                    ->find($data['country_id'])->current()->toArray()
            );
            $this->_getCheckout()->setDeliveryAddress($address);
        }
        if (empty($this->_getCheckout()->getStorage()->shipping)) {
            $this->_redirect('/paymentpaypal/express-checkout/shipping-method');
        }
        $this->_redirect('/paymentpaypal/express-checkout/confirmation');
    }
	
    public function processAction()
    {
        $total = $this->_getCheckout()->getTotal()->getTotal();

        $response = $this->_payment->runDoExpressCheckoutPayment();

        if (!$response) {
            $this->_redirect('/paymentpaypal/express-checkout/cancel');
        }

        if (empty($this->_payment->getStorage()->payer['payer_email'])) {

            $delivery = $this->_getCheckout()->getStorage()->delivery;

            $this->_payment->getStorage()->payer['payer_email'] =
                $delivery['firstname'] . ' ' . $delivery['lastname'];
        }
        
        $this->_getCheckout()->setBillingAddress(array(
            'firstname' => 'Paypal Account: ',
            'lastname' => $this->_payment->getStorage()->payer['payer_email'],
            'phone' => '',
            'fax' => '',
            'company' => '',
            'street_address' => '',
            'suburb' => '',
            'city' => '',
            'postcode' => '',
            'state' => '',
            'country' => '',
            'address_format_id' => 0,
        ));

        $order = Ecart::single('sales/order')->createFromCheckout();

        $this->_getCheckout()->setOrderId($order->id);

        $this->_payment->clear();
        //$this->_getCheckout()->getCart()->clear();
        $this->_redirect('checkout/index/success');
    }
    
    /**
     * customer cancel payment from paypal
     */
    public function cancelAction()
    {
        $this->_payment->clear();
        $this->_redirect('/checkout/cancel');
    } 

    public function editAction()
    {
    	$this->_redirect(
    	    $this->_payment->getPayPalLoginServer()
    	    . '?cmd=_express-checkout&useraction=continue&token='
    	    . $this->_payment->getStorage()->token,
            array(),
    	    false
        );
    }
}