<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Poll
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Poll
 * @subpackage  Box
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Poll_Box_Poll extends Ecart_Account_Box_Abstract
{
    protected $_title = 'Poll';
    protected $_class = 'box-poll';
    protected $_url = 'poll';
    
    private $_questionId;
    private $_lastQuestionId;
    private $_langId;
    private $_status = false;
    
    public function initData()
    {
        if ($this->questionId) {
            $this->_questionId = $this->questionId;
            $this->_status = true;
        } else {
            $this->_questionId = Ecart::single('poll/question')->getOneActiveQuestion(Ecart::getSiteId(), ip2long($_SERVER['REMOTE_ADDR']));
        }
        if (!$this->_questionId) {
            $this->_questionId = Ecart::single('poll/question')->getOneQuestion(Ecart::getSiteId());
        }
        $this->_langId = Ecart::getLanguageId();
        $this->setQuestionId(null);
        
        if (!$this->_questionId || (!$this->_status && Ecart::single('poll/vote')->isVoted($this->_questionId))) {
            return false;
        }
        
        if ($this->_questionId == $this->_lastQuestionId && $this->hasData(totalCount)) {
            return true;
        }
        
        $this->_lastQuestionId = $this->_questionId;
        
        $questions = current(Ecart::single('poll/question')->getQuestionById($this->_questionId, $this->_langId));
        $answers = Ecart::single('poll/answer')->getAnswers(array(
            'languageId' => $this->_langId,
            'questionId' => $questions['id'],
        ));
        $results = array();
        $totalCount = 0;
        if ((isset($_COOKIE['polls'])
                && in_array($this->_questionId, explode(',', $_COOKIE['polls'])))
            || (Ecart::single('poll/vote')
                    ->isVoted($this->_questionId, ip2long($_SERVER['REMOTE_ADDR'])))) {
            
            $results = Ecart::single('poll/question')->find($this->_questionId)->current()->getResults();
            $totalCount = Ecart::single('poll/question')->find($this->_questionId)->current()->getTotalCount();
            $this->_status = true;
        }
        $this->updateData(array(
            'question' => $questions,
            'answers' => $answers,
            'results' => $results,
            'status' => $this->_status,
            'totalCount' => $totalCount
        ));
    }
    
    public function hasContent()
    {
        return (bool)$this->_questionId;
    }
}