<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Poll
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Poll
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Poll_Model_Question extends Ecart_Db_Table
{
    protected $_name = 'poll_question';
    protected $_rowClass = 'Ecart_Poll_Model_Question_Row';
    
    /**
     * Enter description here...
     *
     * @param unknown_type $ip
     * @param unknown_type $siteId
     * @return array
     */
    public function getOneActiveQuestion($siteId, $ip)
    { 
         $query = "SELECT q.id FROM "  . self::$_db_prefix . "poll_question AS q " .
                  "INNER JOIN " . self::$_db_prefix . "poll_answer AS a ON a.question_id = q.id " .
                  "INNER JOIN " . self::$_db_prefix . "poll_vote AS v ON v.answer_id = a.id " .
                  "WHERE v.ip = ? " ;
         $questionIds = implode(',', array_keys($this->getAdapter()->fetchAssoc($query, $ip)));
         return $this->getAdapter()->fetchOne(
             "SELECT q.id FROM {$this->_name} AS `q` " . 
             "INNER JOIN " . self::$_db_prefix . "poll_question_site as s on s.question_id = q.id " .
             "WHERE lock_status = 1 AND " .
             "s.site_id = ? AND " .
             "q.id NOT IN (?)" .
             "ORDER BY RAND() LIMIT 1 ", array($siteId, $questionIds)
         );
    }
    
    public function getAllActiveQuestion($siteId, $ip = false )
    {    if ($ip) {
             $query = "SELECT q.id FROM {$this->_name} AS q " .
                      "INNER JOIN " . self::$_db_prefix . "poll_answer AS a ON a.question_id = q.id " .
                      "INNER JOIN " . self::$_db_prefix . "poll_vote AS v ON v.answer_id = a.id " .
                      "WHERE v.ip = ? " ;
             $questionIds = ' AND q.id NOT IN (' . implode(',', array_keys($this->getAdapter()->fetchAssoc($query, $ip))) . ') ';
         } else $questionIds = ''; 
         return $this->getAdapter()->fetchAll(
             "SELECT q.id FROM {$this->_name} AS `q` " . 
             "INNER JOIN " . self::$_db_prefix . "poll_question_site as s on s.question_id = q.id " .
             "WHERE lock_status = 1 AND " .
             "s.site_id = {$siteId}" .
             $questionIds
         );
    }
    
    public function getOneQuestion($siteId)
    { 
        return $this->getAdapter()->fetchOne(
            "SELECT q.id FROM {$this->_name} AS `q` " . 
            "INNER JOIN " . self::$_db_prefix . "poll_question_site as s on s.question_id = q.id " .
            "WHERE lock_status = 1 AND s.site_id = ? " .
            "ORDER BY RAND() LIMIT 1 ", $siteId
        );
    }
    
    public function getQuestionById($questionId, $langId = 0 )
    { 
        $query = 'SELECT `qd`.`language_id` as languageId, q.id,`qd`.`question`, `q`.`type` ' .
            "FROM {$this->_name} AS `q` " .
            'LEFT JOIN ' .  self::$_db_prefix . 'poll_question_description AS `qd` ON qd.question_id = q.id ' . 
            'WHERE q.lock_status = 1 AND q.id = ?';
        if ($langId) 
            $query .= " AND qd.language_id = {$langId}"; 
        return $this->getAdapter()->fetchAssoc($query, $questionId  );
    }
    
    public function getQuestionByIds($questionIds, $langId = 0 )
    {
        $questionIds = $this->getAdapter()->quoteInto('q.id IN(?)', $questionIds);
        $query = 'SELECT q.id, `qd`.`language_id` as languageId,`qd`.`question`, `q`.`type` ' .
            "FROM {$this->_name} AS `q` " .
            'LEFT JOIN ' . self::$_db_prefix . 'poll_question_description AS `qd` ON qd.question_id = q.id ' . 
            'WHERE q.lock_status = 1';
        if ($langId) 
            $query .= " AND qd.language_id = {$langId}"; 
       $query .= " AND " . $questionIds; 
        return $this->getAdapter()->fetchAssoc($query);
    }
    
    public function getStatusById($questionId )
    { 
        $query = "SELECT  q.lock_status FROM {$this->_name} AS `q` WHERE q.id = ? ";
        return $this->getAdapter()->fetchOne($query, $questionId);
    }
    
    public function getQuestions()
    {
        $query = 'SELECT  q.*, qd.question ' . 
            "FROM {$this->_name} AS q " . 
            'LEFT JOIN ' . self::$_db_prefix . 'poll_question_description AS qd ' . 
                'ON qd.question_id = q.id AND qd.language_id = ?';
        $questions = $this->getAdapter()->fetchAssoc($query, Ecart_Locale::getLanguageId());
        
        /* select assigned sites names*/
        $sitesNames = Ecart::single('poll/question_site')->getSitesNamesAssigns();
        
        foreach (array_keys($questions) as $qId ) {
            $questions[$qId]['sites'] = isset($sitesNames[$qId]) ? 
                implode(',', array_keys($sitesNames[$qId])) : '';
        }
        
        $cnt = Ecart::single('poll/vote')->getVoteCount();
        foreach ($cnt as $qId => $count) {
            $questions[$qId]['cnt'] = $count;
        }
        return array_values($questions);
    }
    
    public function getIds()
    {
        return array_keys($this->getAdapter()->fetchPairs("SELECT id FROM {$this->_name} "));
    }
    
    
}