<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Poll
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */
 
/**
 * @var Ecart_Install_Model_Installer
 */
$installer = Ecart::single('install/installer');

$installer->startSetup();


/**
 * created tables:
 *  poll_answer 
 *  poll_answer_description
 *  poll_question
 *  poll_question_description
 *  poll_question_site
 *  poll_vote
 * use tables (FOREIGN KEY):
 *  locale_language
 *  core_site
 */

$installer->run("

-- DROP TABLE IF EXISTS `{$installer->getTable('poll_answer')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('poll_answer')}` (
    `id` mediumint(8) unsigned NOT NULL auto_increment,
    `question_id` mediumint(8) unsigned default NULL,
    PRIMARY KEY  (`id`),
    KEY `FK_poll_answer_1` (`question_id`),
    CONSTRAINT `FK_poll_answer_1` FOREIGN KEY (`question_id`) REFERENCES `{$installer->getTable('poll_question')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- DROP TABLE IF EXISTS `{$installer->getTable('poll_answer_description')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('poll_answer_description')}` (
    `answer_id` mediumint(8) unsigned NOT NULL auto_increment,
    `language_id` smallint(5) unsigned NOT NULL,
    `answer` varchar(255) default NULL,
    PRIMARY KEY  (`answer_id`,`language_id`),
    KEY `FK_poll_answer_description_2` (`language_id`),
    CONSTRAINT `FK_poll_answer_description_1` FOREIGN KEY (`answer_id`) REFERENCES `{$installer->getTable('poll_answer')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT `FK_poll_answer_description_2` FOREIGN KEY (`language_id`) REFERENCES `{$installer->getTable('locale_language')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- DROP TABLE IF EXISTS `{$installer->getTable('poll_question')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('poll_question')}` (
    `id` mediumint(8) unsigned NOT NULL auto_increment,
    `created_at` datetime default NULL,
    `changed_at` datetime default NULL,
    `lock_status` tinyint(3) unsigned default '1',
    `type` tinyint(3) unsigned NOT NULL,
    PRIMARY KEY  USING BTREE (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;

-- DROP TABLE IF EXISTS `{$installer->getTable('poll_question_description')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('poll_question_description')}` (
    `question_id` mediumint(8) unsigned NOT NULL,
    `language_id` smallint(5) unsigned NOT NULL,
    `question` varchar(255) NOT NULL,
    PRIMARY KEY  (`language_id`,`question_id`),
    KEY `FK_poll_question_description_1` (`question_id`),
    CONSTRAINT `FK_poll_question_description_1` FOREIGN KEY (`question_id`) REFERENCES `{$installer->getTable('poll_question')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('poll_question_site')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('poll_question_site')}` (
    `question_id` mediumint(8) unsigned NOT NULL auto_increment,
    `site_id` smallint(5) unsigned NOT NULL,
    PRIMARY KEY  USING BTREE (`question_id`,`site_id`),
    KEY `FK_poll_to_sites_2` (`site_id`),
    CONSTRAINT `FK_poll_to_sites_1` FOREIGN KEY (`question_id`) REFERENCES `{$installer->getTable('poll_question')}` (`id`) ON DELETE CASCADE,
    CONSTRAINT `FK_poll_to_sites_2` FOREIGN KEY (`site_id`) REFERENCES `{$installer->getTable('core_site')}` (`id`) ON DELETE NO ACTION
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- DROP TABLE IF EXISTS `{$installer->getTable('poll_vote')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('poll_vote')}` (
  `id` int(11) NOT NULL auto_increment,
  `ip` int(11) default NULL,
  `answer_id` mediumint(8) unsigned default NULL,
  `customer_id` int(10) unsigned default NULL,
  `visitor_id` int(10) unsigned default NULL,
  `created_at` datetime NOT NULL,
  `points` tinyint(3) unsigned NOT NULL default '1',
  PRIMARY KEY  (`id`),
  KEY `FK_poll_vote_1` (`answer_id`),
  CONSTRAINT `FK_poll_vote_1` FOREIGN KEY (`answer_id`) REFERENCES `{$installer->getTable('poll_answer')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

");

Ecart::single('admin/menu')
    ->add('CMS', '#', 90)
    ->add('CMS->Polls', 'poll_index', 40);

Ecart::single('admin/acl_resource')
    ->add('admin/poll', 'Polls All')
    ->add('admin/poll_index', 'Polls')
    ->add("admin/poll_index/clear")
    ->add("admin/poll_index/delete")
    ->add("admin/poll_index/get-question")
    ->add("admin/poll_index/index")
    ->add("admin/poll_index/list")
    ->add("admin/poll_index/quick-save")
    ->add("admin/poll_index/save");

Ecart::single('core/page')
    ->add('poll/*/*');

$installer->registerModule('Ecart_Poll');

$installer->endSetup();