<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Sales
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */
 
/**
 * @var Ecart_Install_Model_Installer
 */
$installer = Ecart::single('install/installer');

$installer->startSetup();


/**
 * created tables:
 *  sales_order,
 *  sales_order_creditcard,
 *  sales_order_product,
 *  sales_order_product_attribute,
 *  sales_order_status,
 *  sales_order_status_history,
 *  sales_order_total,
 *  sales_order_status_text
 *  
 * use tables (FOREIGN KEY):
 *  locale_language
 */

$installer->run("

-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('sales_order')}` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `customer_id` int(10) unsigned default '0',
    `customer_email` varchar(96) default NULL,
    `delivery_firstname` varchar(128) NOT NULL,
    `delivery_lastname` varchar(128) NOT NULL,
    `delivery_phone` VARCHAR(64) NOT NULL,
    `delivery_fax` VARCHAR(64) DEFAULT NULL,
    `delivery_company` varchar(128) default NULL,
    `delivery_street_address` varchar(128) default NULL,
    `delivery_suburb` varchar(128) default NULL,
    `delivery_city` varchar(64) default NULL,
    `delivery_postcode` varchar(20) default NULL,
    `delivery_state` varchar(64) default NULL,
    `delivery_country` varchar(64) default NULL,
    `delivery_address_format_id` int(5) default '0',
    `billing_firstname` varchar(128) NOT NULL,
    `billing_lastname` varchar(128) NOT NULL,
    `billing_phone` VARCHAR(64) NOT NULL,
    `billing_fax` VARCHAR(64) DEFAULT NULL,
    `billing_company` varchar(128) default NULL,
    `billing_street_address` varchar(64) default NULL,
    `billing_suburb` varchar(128) default NULL,
    `billing_city` varchar(64) default NULL,
    `billing_postcode` varchar(20) default NULL,
    `billing_state` varchar(64) default NULL,
    `billing_country` varchar(64) default NULL,
    `billing_address_format_id` int(5) default '0',
    `payment_method` varchar(128) NOT NULL default '',
    `payment_method_code` varchar(32) default NULL,
    `shipping_method` varchar(128) default NULL,
    `shipping_method_code` varchar(32) default NULL,
    `coupon_code` varchar(32) default NULL,
    `date_modified_on` datetime default NULL,
    `date_purchased_on` datetime NOT NULL,
    `date_finished_on` datetime default NULL,
    `order_status_id` mediumint(8) unsigned default NULL,
    `currency` char(3) NOT NULL,
    `currency_rate` decimal(7,4) UNSIGNED NOT NULL DEFAULT 1,
    `order_total` decimal(14,2) NOT NULL,
    `txn_id` int(11) NOT NULL default '0',
    `ip_address` varchar(96) default NULL,
    `site_id` smallint(5) unsigned NOT NULL,
    PRIMARY KEY  (`id`),
    KEY `idx_status_orders_cust` (`order_status_id`,`id`,`customer_id`),
    KEY `idx_date_purchased` (`date_purchased_on`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;

-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order_creditcard')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('sales_order_creditcard')}` (
  `order_id` int(10) unsigned NOT NULL,
  `cc_type` varchar(64) NOT NULL,
  `cc_owner` varchar(64) DEFAULT NULL,
  `cc_number` varchar(64) NOT NULL,
  `cc_expires_year` varchar(20) NOT NULL,
  `cc_expires_month` varchar(20) NOT NULL,
  `cc_issue_year` varchar(20) DEFAULT NULL,
  `cc_issue_month` varchar(20) DEFAULT NULL,
  `cc_cvv` varchar(20) DEFAULT NULL,
  PRIMARY KEY  USING BTREE (`order_id`),
  CONSTRAINT `FK_sales_order_creditcard_order_id` FOREIGN KEY (`order_id`) REFERENCES `{$installer->getTable('sales_order')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order_product')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('sales_order_product')}` (
    `id` int(11) NOT NULL auto_increment,
    `order_id` int(10) unsigned NOT NULL,
    `product_id` int(10) unsigned default NULL,
    `product_variation_id` int(11) default NULL,
    `product_sku` varchar(128) NOT NULL,
    `product_name` varchar(128) NOT NULL,
    `product_price` decimal(15,4) NOT NULL,
    `final_price` decimal(15,4) NOT NULL,
    `product_tax` decimal(7,4) NOT NULL default '0.0000',
    `product_quantity` decimal(15,4) NOT NULL,
    `backorder` int(1) NOT NULL default '0',
    PRIMARY KEY  USING BTREE (`id`),
    KEY `fk_order_id_constraint` (`order_id`),
    CONSTRAINT `fk_op_order_id` FOREIGN KEY (`order_id`) REFERENCES `{$installer->getTable('sales_order')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC AUTO_INCREMENT=1 ;

-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order_product_attribute')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('sales_order_product_attribute')}` (
    `id` int(11) NOT NULL auto_increment,
    `order_product_id` int(11) NOT NULL,
    `product_option` varchar(128) NOT NULL,
    `product_option_value` varchar(128) NOT NULL,
    PRIMARY KEY  USING BTREE (`id`),
    KEY `fk_order_product_id` (`order_product_id`),
    CONSTRAINT `fk_order_product_id` FOREIGN KEY (`order_product_id`) REFERENCES `{$installer->getTable('sales_order_product')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;

-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order_status')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('sales_order_status')}` (
    `id` smallint(5) NOT NULL auto_increment,
    `name` char(32) NOT NULL,
    `system` tinyint(1) NOT NULL,
    PRIMARY KEY  (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=10;

INSERT INTO `{$installer->getTable('sales_order_status')}` (`id`, `name`, `system`) VALUES
(0, 'new', 1),
(1, 'pending', 1),
(2, 'processing', 1),
(3, 'ship', 1),
(4, 'delivered', 1),
(5, 'complete', 1),
(6, 'hold', 1),
(7, 'cancel', 1),
(8, 'refund', 1),
(9, 'failed', 1);

-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order_status_relation')}`;
CREATE TABLE  `{$installer->getTable('sales_order_status_relation')}` (
  `from_status` smallint(5) NOT NULL,
  `to_status` smallint(5) NOT NULL,
  PRIMARY KEY  (`from_status`,`to_status`),
  KEY `sales_order_status_relation_from` (`from_status`),
  KEY `sales_order_status_relation_to` (`to_status`),
  CONSTRAINT `sales_order_status_relation_to` FOREIGN KEY (`to_status`) REFERENCES `{$installer->getTable('sales_order_status')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `sales_order_status_relation_from` FOREIGN KEY (`from_status`) REFERENCES `{$installer->getTable('sales_order_status')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO `{$installer->getTable('sales_order_status_relation')}` (`from_status`, `to_status`) VALUES
(0, 1), (0, 9), (1, 2), 
(1, 6), (1, 7), (1, 9), 
(2, 3), (2, 6), (2, 7), 
(2, 8), (2, 9), (3, 4), 
(3, 8), (3, 9), (4, 5), 
(4, 8), (6, 1), (6, 2), 
(6, 7)
;


-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order_status_history')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('sales_order_status_history')}` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `order_id` int(10) unsigned NOT NULL,
    `order_status_id` smallint(5) unsigned NOT NULL,
    `created_on` datetime NOT NULL,
    `notified` tinyint(1) unsigned NOT NULL,
    `comments` text,
    PRIMARY KEY  (`id`),
    KEY `i_order_id` (`order_id`,`order_status_id`),
    CONSTRAINT `fk_osh_order_id` FOREIGN KEY (`order_id`) REFERENCES `{$installer->getTable('sales_order')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;

-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order_total')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('sales_order_total')}` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `order_id` int(10) unsigned NOT NULL,
    `title` varchar(128) NOT NULL,
    `value` decimal(15,4) NOT NULL,
    PRIMARY KEY  (`id`),
    KEY `fk_ot_order_id` (`order_id`),
    CONSTRAINT `fk_ot_order_id` FOREIGN KEY (`order_id`) REFERENCES `{$installer->getTable('sales_order')}` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;

-- DROP TABLE IF EXISTS `{$installer->getTable('sales_order_status_text')}`;
CREATE TABLE IF NOT EXISTS `{$installer->getTable('sales_order_status_text')}` (
    `status_id` smallint(5) NOT NULL,
    `language_id` smallint(5) unsigned NOT NULL,
    `status_name` varchar(128) NOT NULL,
    PRIMARY KEY  (`status_id`,`language_id`),
    KEY `new_fk_constraint2` (`language_id`),
    CONSTRAINT `new_fk_constraint` FOREIGN KEY (`status_id`) REFERENCES `{$installer->getTable('sales_order_status')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT `new_fk_constraint2` FOREIGN KEY (`language_id`) REFERENCES `{$installer->getTable('locale_language')}` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;

INSERT INTO `{$installer->getTable('sales_order_status_text')}` (`status_id`, `language_id`, `status_name`) VALUES
(0, 1, 'New'),
(1, 1, 'Pending payment'),
(2, 1, 'Proccesing'),
(3, 1, 'Shipping'),
(4, 1, 'Delivered'),
(5, 1, 'Complete'),
(6, 1, 'Hold'),
(7, 1, 'Canceled'),
(8, 1, 'Refund'),
(9, 1, 'Failed');

");

Ecart::single('core/config_field')
    ->add('orders/main/defaultStatusId', 'Orders/Main/Default Order Status', 1, 'select', 'Default Order Status', /*array('model' => 'OrderStatus')*/array('config_options' => '{"1":"pending", "2":"processing"}')) //pending
    ->add('orders/main/email', 'Send custom order info here', 'email1', 'select', 'Send custom order info here', array('model' => 'MailBoxes'));

Ecart::single('admin/menu')
    ->add('Localization', '#', 80)
    ->add('Localization->Order status', 'sales_order-status', 30)
    ->add('Sales', '#', 30)
    ->add('Sales->Orders', 'sales_order', 10);

Ecart::single('admin/acl_resource')
    ->add('admin/sales', 'Sales')
    ->add('admin/sales_order', 'Orders')
    ->add("admin/sales_order/delete")
    ->add("admin/sales_order/get-order-info")
    ->add("admin/sales_order/get-status-history")
    ->add("admin/sales_order/index")
    ->add("admin/sales_order/list")
    ->add("admin/sales_order/print")
    ->add("admin/sales_order/set-status")
    ->add('admin/sales_order-status', 'Orders Statusses')
    ->add("admin/sales_order-status/batch-save")
    ->add("admin/sales_order-status/delete")
    ->add("admin/sales_order-status/get-childs")
    ->add("admin/sales_order-status/get-info")
    ->add("admin/sales_order-status/index")
    ->add("admin/sales_order-status/list")
    ->add("admin/sales_order-status/save");

$installer->registerModule('Ecart_Sales');

$installer->endSetup();