<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Search
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Search
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Search_Model_Lucene
{
    
    protected $_index;
    protected $_encoding = 'utf-8';
    
    /**
     * Construct, create index
     *
     * @param string $indexPath[optional]
     * @param string $encoding[optional]
     * @throws Ecart_Exception
     */
    public function __construct(
        array $params/* $indexPath = null, $encoding = 'utf-8'*/)
    {
        $encoding = 'utf-8';
        $indexPath = array_shift($params);
        if (count($params)) {
            $encoding = array_shift($params);
        }
        if (null === $indexPath) {
            $site = Ecart::single('core/site')
                ->find(Ecart::getSiteId())->current();
            $site = str_replace(
                array('http://', 'https://'), '',
                urlencode($site->name . '_'. $site->id)
            );
            $locale = Ecart::single('locale/language')
                ->find(Ecart::getLanguageId())->current()->locale;
        
            $indexPath = Ecart::config()->system->path
                       . '/var/index/'
                       . $site . '/'
                       . $locale;
        }
        
        if (!is_readable($indexPath)) {
            throw new Ecart_Exception(
                Ecart::translate('search')->__(
                    'Please, update search indexes, to enable search functionality'
            ));
        }
        
        /*
        $mySimilarity = new Ecart_Similarity();
        Zend_Search_Lucene_Search_Similarity::setDefault($mySimilarity);
        */
        
        Zend_Search_Lucene_Search_QueryParser::setDefaultEncoding($encoding);
        
        // add filter by words 
        $stopWords = array('a', 'an', 'at', 'the', 'and', 'or', 'is', 'am');
        $stopWordsFilter = new Zend_Search_Lucene_Analysis_TokenFilter_StopWords($stopWords);
        $analyzer = new Zend_Search_Lucene_Analysis_Analyzer_Common_Utf8_CaseInsensitive();
        $analyzer->addFilter($stopWordsFilter);
        
        Zend_Search_Lucene_Analysis_Analyzer::setDefault($analyzer);
        
        $this->_index = Zend_Search_Lucene::open($indexPath);
        $this->_encoding = $encoding;
    }
    
    /**
     * @return Zend_Search_Lucene
     */
    public function index()
    {
        return $this->_index;
    }
    
    /**
     * 
     * @return Zend_Search_Lucene_Search_Query $query
     * @param string $queryStr
     */
    public function createFuzzyQuery($queryStr)
    {
        $userQuery = Zend_Search_Lucene_Search_QueryParser::parse(
            $queryStr, $this->_encoding
        );
            
        $term = new Zend_Search_Lucene_Index_Term($queryStr, 'name');
        $fuzzy = new Zend_Search_Lucene_Search_Query_Fuzzy($term, 0.4);
        
        $query = new Zend_Search_Lucene_Search_Query_Boolean();
        $query->addSubquery($userQuery);
        $query->addSubquery($fuzzy);
        
        return $query;
    }
    
    /**
     * 
     * @return array  
     * @param string | Zend_Search_Lucene_Search_Query_Boolean $fuzzyQuery
     */
    public function findFuzzy($fuzzyQuery) 
    {
        if (!is_string($fuzzyQuery)) {
            $fuzzyQuery = $fuzzyQuery->__toString();
        }
        $result = array();
        foreach ($this->_index->find($fuzzyQuery) as $hit) {
            $result[] = $hit->id;
        }
        return $result;
    }

    /**
     *
     * @param int $id
     * @param <type> $query
     * @return array
     */
    public function getDocumentData($id, $query = null) 
    {
        $highlighter = Ecart::single('search/highlighter_default');
        
        $doc = $this->_index->getDocument($id);

        return array(
            'image' => $doc->image,
            //@todo remove "@" http://framework.zend.com/issues/browse/ZF-6574
            'nameHighlight' =>  null === $query ? $doc->name :
                @$query->htmlFragmentHighlightMatches($doc->name, 'utf-8', $highlighter),
            'name' =>  $doc->name,
            'contents' => null === $query ? $doc->contents :
                @$query->htmlFragmentHighlightMatches($doc->contents, 'utf-8', $highlighter),
            'urlHighlight'  => null === $query ? $doc->url :
                @$query->htmlFragmentHighlightMatches($doc->url, 'utf-8', $highlighter),
            'url'  => $doc->url
        );    
    }
}