<?php 
/**
 * Ecart 
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @category    Ecart
 * @package     Ecart_Admin
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

require_once('OFC/open_flash_chart_object.php');
require_once('OFC/open-flash-chart.php');

/**
 *
 * @category    Ecart
 * @package     Ecart_Admin
 * @subpackage  Helper
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_View_Helper_Chart
{
    protected $_data;
    protected $_title;
    protected $_yMax = 20;
    protected $_yMin = 0;
    protected $_xLabel = array(
        'Jan','Feb','Mar','Apr','May','Jun',
        'Jul','Aug','Sep', 'Oct', 'Nov', 'Des'
    );
    protected $_xTooltip = '';
    protected $_yTooltip = '';
    
    public function chart($url = null, $width = 650, $height = 350/*, $use_swfobject=true, $base=''*/)
    {
        if (null === $url)
            return $this;
        return open_flash_chart_object(
            $width, $height, 
            $url, 
            false, 
            Zend_Controller_Front::getInstance()->getBaseUrl()
        );
    }
    
    public function setData(
            array $data = array(), 
            array $xLabel = array(),
            $title = '', 
            $yMax = 20,
            $yMin = 0,
            $xTooltip = '',
            $yTooltip = ''
        )
    {
        if (!empty($data)) {
            $this->_data = $data;
        }
        if (!empty($xLabel))
            $this->_xLabel = $xLabel;
        $this->_yMax = /*$y_max <= 1 ? ceil($y_max) :*/ ceil($yMax/10)*10;
        $this->_yMin = $yMin <= 1 ? 0 :  floor($yMin/10)*10;
        $this->_xTooltip = $xTooltip;
        $this->_yTooltip = $yTooltip;
        $this->_title = $title; 
        return true;
    }
    
    public function getData()
    {
        
        $data2 = array();
        if (is_array($this->_data) && !empty($this->_data))
            $data2 = $this->_data;
        
        $g = new graph();
        $g->title($this->_title, '{font-size: 20px; color: #000 }' );
            
        /*$g->set_data($data_1);
        $g->bar( 50, '#A5B5C2', 'Bar', 10 );*/
        
        $dataNull = true;
        foreach ($this->_data as $value) {
            if ($value != 'null') {
                $dataNull = false;
                break;
            }    
        }
        if (!$dataNull) {// bug fix 
           // $g->line_hollow( 3, 5, '#006FBE', '', 10 );    // <-- 3px thick + dots
        //} else {
            $g->set_data($data2);
            $g->area_hollow( 2, 3, 25, '#006FBE', '', 12, '#73BAEC' );
        }
        
        $g->x_axis_colour( '#909090', '#D2D2FB' );
        $g->y_axis_colour( '#909090', '#D2D2FB' );
        
        $g->set_bg_colour('#FFFFFF');
        $g->set_x_labels($this->_xLabel);
        $g->set_y_max($this->_yMax);
        $g->set_y_min($this->_yMin);
        $g->set_tool_tip(
            "{$this->_xTooltip} #x_label#\n{$this->_yTooltip} #val#"
        );
        //$g->y_label_steps( 1 );
        //$g->set_y_legend( '', 12, '#D7E7F4' );
        return $g->render();
    }
}