/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ext.onReady(function(){

    var Option = {
        activeId: 0,
        
        create: function (){
            Option.window.show();
            Option.window.load({
                url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_product-attributes/edit/optId/0'
            });
        },
        
        getSelectedId: function() {
            var selectedItems = gridOption.getSelectionModel().selections.items;
            if (!selectedItems.length) {
                return false;
            }
            return selectedItems[0].id;
        },
        
        edit: function() {
            var id = Option.getSelectedId()
            if (!id)
                return;
            Option.window.show();
            Option.window.load({
                url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_product-attributes/edit/optId/' + id
            });
        },
        
        remove: function() {
            var selectedItems = gridOption.getSelectionModel().selections.items;
            
            if (!selectedItems.length)
                return;
            
            if (!confirm('Are you sure?'))
                return;
                
            var data = {};
            
            for (var i = 0; i < selectedItems.length; i++) {
                data[i] = selectedItems[i].id;
            }
                
            Ext.Ajax.request({
                url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_product-attributes/delete',
                params: {data: Ext.encode(data)},
                callback: function() {
                    dsOption.reload();
                }
            });
        },
        
        renderType: function(id) {
            if (types[id])
                return types[id];
            return '';
        },
        
        window: new Ext.Window({
            layout: 'fit',
            autoScroll: true,
            width: 550,
            height: 320,
            closeAction: 'hide',
            plain: false,
            title: 'Option'.l(),
            bodyStyle: 'background: white',
            buttons: [{
                text: 'Save'.l(),
                handler: function() {
                    Ext.Ajax.request({
                        url: jQuery('#form-option').attr('action'),
                        form: 'form-option',
                        callback: function(response, options) {
                            Option.window.hide();
                            dsOption.reload();
                        }
                    });
                }
            },{
                text: 'Cancel'.l(),
                handler: function(){
                    Option.window.hide();
                }
            }]
        }),
        grid: {}
    };
    
    var dsOption = new Ext.data.Store({
        proxy: new Ext.data.HttpProxy({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_product-attributes/list'
        }),
        reader: new Ext.data.JsonReader({
            root: 'data',
            totalProperty: 'count',
            id: 'id'
        }, [
            {name: 'id'},
            {name: 'code'},
            {name: 'option_type'},
            {name: 'sort_order'},
            {name: 'name'},
            {name: 'instruct'}
        ]),
        sortInfo: {field: 'id', direction: 'ASC'},
        remoteSort: true
    });
    
    var sm = new Ext.grid.CheckboxSelectionModel();
    
    var cmOption = new Ext.grid.ColumnModel([
    sm, {
        header: "Id".l(),
        dataIndex: 'id',
    }, {
        header: "Code".l(),
        dataIndex: 'code',
        width: 200
    }, {
        header: "Name".l(),
        dataIndex: 'name',
        width: 200
    }, {
        header: "Instruct".l(),
        dataIndex: 'instruct'
    }, {
        header: 'Type'.l(),
        dataIndex: 'option_type',
        renderer: Option.renderType
    }, {
        header: 'Sort Order'.l(),
        dataIndex: 'sort_order',
        width: 80
    }]);
    
    var gridOption = new Ext.grid.GridPanel({
        store: dsOption,
        cm: cmOption,
        sm: sm,
        height: 550,
        viewConfig: {
            forceFit: true,
            emptyText: 'No records found'.l()
        },
        renderTo: 'options',
        tbar: [{
            text: 'Add'.l(),
            icon: Ecart.skinUrl + '/images/icons/add.png',
            cls: 'x-btn-text-icon',
            handler: Option.create
        }, {
            text: 'Edit'.l(),
            icon: Ecart.skinUrl + '/images/icons/page_edit.png',
            cls: 'x-btn-text-icon',
            handler: Option.edit
        }, {
            text: 'Delete'.l(),
            icon: Ecart.skinUrl + '/images/icons/delete.png',
            cls: 'x-btn-text-icon',
            handler: Option.remove
        }, '->', {
            icon: Ecart.skinUrl + '/images/icons/refresh.png',
            cls: 'x-btn-icon',
            handler: function() {
                gridOption.getStore().reload();
            }
        }],
        bbar: new Ext.PagingToolbar({
            store: dsOption,
            pageSize: 20,
            displayInfo: true,
            displayMsg: 'Displaying records {0} - {1} of {2}'.l(),
            emptyMsg: 'No records found'.l()
        }),
        listeners: {
            'beforerender': function(cmp) {
                cmp.width = $('#ext-cmp-layout').width() - $('#ext-cmp-layout .sidebox').width();
            }
        }
    });
    
    gridOption.on('rowdblclick', Option.edit);
    
    dsOption.load({params:{start:0, limit:20}});
});