/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ext.namespace('Ecart', 'Ecart.Product', 'Ecart.Category');

var grid, tree, treeRoot, ds, productWin, categoryWin, optionTree, buffer = new Array(), productActionMenu, opts;
/* Category functions */
var Category = {
    
    id: 0,
    rootId: 0,
    
    add: function () {
        if (!tree.getSelectionModel().getSelectedNode() || Category.id == Category.rootId) {
            alert('Select parent category'.l());
            return false;
        }
        
        Ext.getCmp('categoryWindow').show();
        Ext.getCmp('categoryForm').getForm().reset();
        Ext.getCmp('categoryForm').getForm().setValues({
            'catId': 0,
            'pId': Category.id
        })
    },
    
    edit: function () {
        if (Category.id == Category.rootId || 
            Category.id == tree.getNodeById(Category.id).getPath().replace(/^\/+|\/+$/g,"").split('/')[1]) 
        {
            return false;
        }
        
        Ext.getCmp('categoryForm').getForm().setValues({
            'catId': Category.id,
            'pId': 0
        })
        
        Ext.getCmp('categoryWindow').show();
        
        Ext.getCmp('categoryForm').getForm().load({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_category/get-data/' + 'catId/' + Category.id,
            method: 'get',
            success: function(response, options) {
                var data = Ext.decode(response.responseText);
            }
        })
    },
    
    save: function () {
        Ext.getCmp('categoryForm').getForm().submit({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_category/save',
            params : {
                siteId: ds.baseParams.siteId
            },
            method : 'POST',
            success : function(form, response) {
                var obj = Ext.decode(response.response.responseText);
                var nodeId = Ext.getCmp('categoryForm').getForm().findField('catId').getValue();
                var parentId = Ext.getCmp('categoryForm').getForm().findField('pId').getValue();
                
                if (nodeId != '0') {
                    var node = tree.getNodeById(nodeId);
                } else {
                    var node = tree.getNodeById(parentId);
                }
                
                var path = node.getPath().replace(/^\/+|\/+$/g,"").split('/');
                var offset = path.length > 2 ? 2 : 1
                var level = path.length - offset;
                
                Category.reloadTree(path, level);
                Ext.getCmp('categoryForm').getForm().reset();
                Ext.getCmp('categoryWindow').hide();
            },
            failure: function(form, response) {
                if (response.failureType == 'client')
                    return;
            }
        })
    },
    
    remove: function () {
        if (Category.id == Category.rootId) {
            return false;
        }
        
        if (!confirm('Are you sure?'.l())) {
            return false;
        }
        
        Ext.Ajax.request({
            url : Ecart.baseUrl + Ecart.adminUrl + '/catalog_category/delete/',
            method : 'POST',
            params: {catId:Category.id},
            callback : function() {
                tree.getNodeById(Category.id).parentNode.reload();
                Category.id = Category.rootId;
            }
        });
    },
    
    reloadTree: function(path, level) {
        if (level == path.length) {
            var path_str = '';
            for (var i = 0; i < path.length; i++) {
                path_str += '/' + path[i];
            }
            tree.expandPath(path_str);
            if (Category.id != Category.rootId) {
                tree.getNodeById(Category.id).select();
            }
            return;
        }
        tree.getNodeById(path[level]).reload(function(){
            Category.reloadTree(path, ++level)
        });
    }
    
};

var Product = {
    Variations: Ecart.Variations,
    Modifiers:  Ecart.Modifiers,
    Properties: Ecart.Properties,
    
    clearForm : function() {
        this.Variations.clear();
        this.Modifiers.clear();
        this.Properties.clear();
        this.Images.clear();
        Ext.getDom('featured-date-exp').style.display = 'none';
        Ext.getDom('form-product').reset();
    },
    
    add: function() {
        Product.clearForm();
        $('#div-form-product').css('display', 'block');
        $('#prodId').val('0');
        
        Product.Images.init();
        productWin.setTitle('New product'.l());
        
        productWin.show();
        initTree('tab-categories-block', [Category.id]);
        
        productWin.doLayout();
    },
    
    edit: function(gridT, rowIndex, e) {
        Product.clearForm();
        
        if (isNaN(rowIndex)) {
            var prodId = grid.getSelectionModel().getSelected().id; //get first selected row in grid
        } else {
            var prodId = ds.data.items[rowIndex].id;
        }
        
        Product.load(prodId);
    },
    
    load: function(productId) {
        Product.id = productId;
        $(".x-form-invalid").removeClass('x-form-invalid');
        
        Ext.getDom('prodId').value = Product.id;
        
        Ext.Ajax.request({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/get-product-info/prodId/' + Product.id + '/',
            method: 'get',
            success: function(result, options) {
                var data = Ext.decode(result.responseText);
                if (!data.success){
                    return false;
                }
                var product = data.product;
                var desc = data.description;
                var featured = data.featured;
                var categories = data.categories;
                var special = data.special;
                var stock = data.stock;
                
                var fields = ['manufacturer_id', 'tax_class_id', 'quantity', 'sku', 
                              'date_available', 'price', 'weight'];
                for (i in fields) {
                    $('#product-' + fields[i]).val(product[fields[i]]);
                }
                Ext.getDom('product-is_active').checked = (product['is_active']=='1') ? true : false;
                $('#hurl').val(product['hurl'] ? product['hurl'] : '');
                
                Product.Images.init(product);
                
                if (featured) {
                    Ext.getDom('is_featured').checked = true;
                    Ext.getDom('featured-date-exp').value = featured['date_exp'] ? featured['date_exp'] : '';
                    Ext.getDom('featured-date-exp').style.display = '';
                } else {
                    Ext.getDom('featured-date-exp').style.display = 'none';
                    Ext.getDom('featured-date-exp').value = '';             
                }
                if (special) {
                    $('#special-price').val(special.price);
                    $('#special-from_date_exp').val(special.from_date_exp);
                    $('#special-to_date_exp').val(special.to_date_exp);
                }
                $('#new-from_date').val(data.product.new_from);
                $('#new-to_date').val(data.product.new_to);
                
                /* fill description fields */
                var descFields = ['name', 'description', 'image_seo_name', 
                                  'meta_title', 'meta_description', 'meta_keyword', 'short_description'];
                for (var i in aLangId) {
                    var langId = aLangId[i];
                    if (typeof(desc) == 'undefined' || desc[langId] == null) 
                        continue;
                    for (j = 0; j < descFields.length; j++) {
                        var field = descFields[j];
                        $('#desc-' + langId + '-' + field).val(desc[langId][field]);
                    }
                }
                
                /* initialize attributes */
                for (var i in data.variation) {
                    var variation = new Ecart.Variation(data.variation[i]);
                    variation.status = 'exists';
                    Product.Variations.add(variation);
                }
                
                if (data.attributes)
                for (var i in data.attributes) {
                    var attr = data.attributes[i];
                    attr.status = 'exists';
                    if (attr.product_variation_id != '0') {/* variation */
                        var option = new Ecart.Option(attr);
                        Product.Variations.get(attr.product_variation_id).addOption(option);
                    } else if (attr.modifier != '0') {// modifier
                        var modifier = new Ecart.Modifier(attr);
                        Product.Modifiers.add(modifier);
                    } else {// property
                        var property = new Ecart.Property(attr);
                        Product.Properties.add(property);
                    }
                }
                if (Ext.getDom('desc-' + currentLangId + '-name').value)
                    productWin.setTitle('Editing product'.l() + ': '
                        +  Ext.getDom('desc-' + currentLangId + '-name').value);
                
                // start stock section
                var fields = ['min_qty', 'min_qty_allowed', 'max_qty_allowed', 
                              'notify_qty', 'in_stock', 'backorder'];
                for (var i = 0; i < fields.length; i++) {
                    if ($('#stock-' + fields[i]).get(0).type != 'checkbox') {
                        $('#stock-' + fields[i]).val(stock[fields[i]]);
                    } else {
                        $('#stock-manage').attr('checked', stock[fields[i]]);
                    }
                }
                Ext.getDom('stock-manage').checked = (stock.manage == '1') ? true : false;
                Ext.getDom('stock-decimal').checked = (stock.decimal == '1') ? true : false;
                /* end stock*/
                        
                productWin.show();
                initTree('tab-categories-block', categories, Product.id);
                productWin.doLayout();
                
            }/*,
            failure: function (response, request)
            {
                var data = Ext.decode(response.responseText);
                if (!data.success){
                    alert(data.error);
                    return;
                }
            }*/ 
        });
    },
    
    validateForm: function() {
        var t = true;
        $(".x-form-invalid").removeClass('x-form-invalid');
        if (isNaN($('#product-quantity').val()) || !($('#product-quantity').val())) { 
            $("#product-quantity").addClass('x-form-invalid');
            prodTabs.activate(2);
            $("#product-quantity").focus();
            t = false;
        }
        if (!$('#product-sku').val()) {
            $("#product-sku").addClass('x-form-invalid');
            prodTabs.activate(0);
            $("#product-sku").focus();
            t = false;
        }
        if (!$('#product-price').val() || isNaN($('#product-price').val())) {
            $("#product-price").addClass('x-form-invalid');
            prodTabs.activate(0)
            $("#product-price").focus();
            t = false;
        }
        for (var i = 0; i < aLangId.length; i++) {
            if (!$('#desc-' + aLangId[i] + '-name').val()) {
                $('#desc-' + aLangId[i] + '-name').addClass('x-form-invalid');
                prodTabs.activate(1);
                $('#desc-' + aLangId[i] + '-name').focus();
                t = false;
            }
        }
        
        return t;
    },
    
    autoFill: function() {
        if (!Product.validateForm()) {
            return;
        }
        if (!$('#hurl').val()) {
            $('#hurl').val($('#desc-' + currentLangId + '-name').val().toLowerCase().split(' ').join(hurldelimiter));
        }
            
        for (var i = 0; i < aLangId.length; i++) {
            var val = $('#desc-' + aLangId[i] + '-name').val();
            if (!$('#desc-' + aLangId[i] + '-image_seo_name').val()) {
                $('#desc-' + aLangId[i] + '-image_seo_name').val(val.split(' ').join(seodelimiter));
            }
            if (!$('#desc-' + aLangId[i] + '-meta_title').val()) {
                $('#desc-' + aLangId[i] + '-meta_title').val(val);
            }
            if (!$('#desc-' + aLangId[i] + '-meta_keyword').val()) {
                $('#desc-' + aLangId[i] + '-meta_keyword').val(val);
                if ($('#product-manufacturer_id').get(0).selectedIndex) {
                    $('#desc-' + aLangId[i] + '-meta_keyword').val($('#desc-' + aLangId[i] + '-meta_keyword').val() +
                    ' ' +
                    $('#product-manufacturer_id').get(0).options[$('#product-manufacturer_id').get(0).selectedIndex].text);
                }
            }
            if (!$('#desc-' + aLangId[i] + '-meta_description').val()) {
                $('#desc-' + aLangId[i] + '-meta_description').val($('#desc-' + aLangId[i] + '-description').val().substring(0, seodesclength));
            }
        }
    },
    
    save: function() {
        if (!Product.validateForm())
           return;
        Ext.Ajax.request({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/save-product/',
            method: 'post',
            callback: function(request, flag, response) {
                  var obj = Ext.decode(response.responseText);
                  if (obj.success == false) {
                       return;
                  }
                productWin.hide();
                Product.clearForm();
                ds.reload();
            },
            params: {
                variations: Ext.encode(Product.Variations.toSaveArray()),
                modifiers: Ext.encode(Product.Modifiers.toSaveArray()),
                properties: Ext.encode(Product.Properties.toSaveArray()),
                siteId: ds.baseParams.siteId
            },
            form: 'form-product'
        });
    },
    
    remove: function() {
        if (!confirm('Are you sure?'.l()))
            return;
        var data = {};
        var selectedItems = grid.getSelectionModel().selections.items;
        for (var i = 0; i < selectedItems.length; i++) {
            data[i] = selectedItems[i].id;
        }
                
        Ext.Ajax.request({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/remove-product/',
            params: {data: Ext.encode(data)},
            callback: function() {
                ds.reload();
            }
        });
    },
    
    removeFromCategory: function() {
        if (!confirm('Are you sure?'.l()))
            return;
        var data = {};
        var selectedItems = grid.getSelectionModel().selections.items;
        for (var i = 0; i < selectedItems.length; i++) {
            data[i] = selectedItems[i].id;
        }
        if (Category.id == Category.rootId) {
            alert('Select some category');
            return false;
        }    
        Ext.Ajax.request({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/remove-product-from-category/',
            params: {prodIds: Ext.encode(data), catId:Category.id},
            callback: function() {
                ds.reload();
            }
        });
    },
    
    removeFromSite: function() {
        if (!confirm('Are you sure?'.l()))
            return;
        var data = {};
        var selectedItems = grid.getSelectionModel().selections.items;
        for (var i = 0; i < selectedItems.length; i++) {
            data[i] = selectedItems[i].id;
        }
                
        Ext.Ajax.request({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/remove-product-from-site/',
            params: {prodIds: Ext.encode(data), siteId: ds.baseParams.siteId},
            callback: function() {
                ds.reload();
            }
        });
    },
    
    cut: function() {
        var data = {};
        if (typeof(buffer.data) != 'undefined') {
           for (var i in buffer.data) {
                data[buffer.data[i].product_id] = buffer.data[i];
            }
        }
        var selectedItems = grid.getSelectionModel().selections.items;
        var actions = {};
        for (var i = 0; i < selectedItems.length; i++) {
            if (typeof(data[selectedItems[i].id]) != 'undefined')
                if (typeof(data[selectedItems[i].id].actions) != 'undefined')
                   actions =  data[selectedItems[i].id].actions;
            actions[Category.id] = 'cut';//{action:'cut', category_id: Category.id}; 
            data[selectedItems[i].id] = {product_id: selectedItems[i].id, actions:actions};
            
        }
        buffer = {data : data};
        productActionMenu.items.get('paste-action-menu-item').enable();

    },
    
    copy: function() {
        var data = {};
        if (typeof(buffer.data) != 'undefined') {
           for (var i in buffer.data) {
                data[buffer.data[i].product_id] = buffer.data[i];
            }
        }
        var selectedItems = grid.getSelectionModel().selections.items;
        var actions = {};
        for (var i = 0; i < selectedItems.length; i++) {
            if (typeof(data[selectedItems[i].id]) != 'undefined')
                if (typeof(data[selectedItems[i].id].actions) != 'undefined')
                   actions =  data[selectedItems[i].id].actions;
            actions[Category.id] = 'copy';//{action:'copy', category_id: Category.id}; 
            data[selectedItems[i].id] = {product_id: selectedItems[i].id, actions:actions };
        }
        buffer = {data : data};
        productActionMenu.items.get('paste-action-menu-item').enable();
    },
    
    paste: function() {
        if (!buffer) {
            alert('Buffer is empty'.l());
            return;
        }
        Ext.Ajax.request({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/move-products/',
            params: {data: Ext.encode(buffer.data), currentCategoryId: Category.id },
            callback: function() {
                ds.reload();
            }
        });
    },
    
    Images: {
        _newImageId :-1,
        _thumbnailImageId : -1,
        _listingImageId : -1,
        _baseImageId : -1,
        _gallery:  new Array(),
        _names: ['image_base', 'image_listing', 'image_thumbnail'],
        uploadCount: 0,
        init: function(product) {
            if (typeof(product) != 'undefined') {
                for (var i = 0, n = this._names.length; i < n; i++) {
                    var name = this._names[i];
                    Product.Images.setData(name, product[name]);
                }
                this._thumbnailImageId = product.image_thumbnail_id;
                this._listingImageId = product.image_listing_id;
                this._baseImageId = product.image_base_id;
                this._gallery = product.gallery;
            } else  
               this._gallery = [];
            var dragZone = new ImageDragZone(
                imagesView, 
                {
                    containerScroll:true,
                    ddGroup: 'organizerDD'
                }
            );
            Product.Images.reload();
            var dropZoneList = new Ext.dd.DropTarget('preview_image_thumbnail', {
               containerScroll:true,
               ddGroup: 'organizerDD',
               notifyDrop: function(dd,e,data) {
                   var d = imagesView.getSelectedRecords()
                   Product.Images._thumbnailImageId = d[0].id;
                   $('#preview_image_thumbnail img').attr('src', Ecart.baseUrl + '/media/product' + d[0].data.image);
                   $('#images-image_thumbnail').val(d[0].data.image);
               }
                   
            });
            var dropZoneMedium = new Ext.dd.DropTarget('preview_image_listing', {
                containerScroll:true,
                ddGroup: 'organizerDD',
                notifyDrop: function(dd,e,data) {
                    var d = imagesView.getSelectedRecords()
                    Product.Images._listingImageId = d[0].id;
                    $('#preview_image_listing img').attr('src', Ecart.baseUrl + '/media/product' + d[0].data.image);
                    $('#images-image_listing').val(d[0].data.image);
               }
            });
            var dropZoneLarge = new Ext.dd.DropTarget('preview_image_base', {
                containerScroll:true,
                ddGroup: 'organizerDD',
                notifyDrop: function(dd,e,data) {
                    var d = imagesView.getSelectedRecords()
                    Product.Images._baseImageId = d[0].id;
                    $('#preview_image_base img').attr('src', Ecart.baseUrl + '/media/product' + d[0].data.image);
                    $('#images-image_base').val(d[0].data.image);
                }
                   
            });
            Product.Images.setEvent();
            //imagesView.render();
        },
        setEvent:function (){
          //  for (var i = 0, n = this._names.length; i < n; i++) {
          //      var name = this._names[i];
                
                $('#preview_image_base').click(function() {
                    $('.x-view-selected').removeClass("x-view-selected");
                    imagesView.select('wrap-' + Product.Images._baseImageId , true);
                });
                $('#preview_image_listing').click(function() {
                    $('.x-view-selected').removeClass("x-view-selected");
                    imagesView.select('wrap-' + Product.Images._listingImageId , true);
                });
                $('#preview_image_thumbnail').click(function() {
                    $('.x-view-selected').removeClass("x-view-selected");
                    imagesView.select('wrap-' + Product.Images._thumbnailImageId , true);
                });
         //  }
        },
        reload: function() {
            imagesStore.loadData(this._gallery);
        },
        
        getGallery: function () {
            return this._gallery;
        },
        
        unset: function (id){
            $('#preview_' + id + ' img').attr('src', Ecart.skinUrl + '/images/no_picture.gif');
            $('#images-' + id).val('');
        },
        
        setAll: function(id) {
            var val = $('#images-' + id).val();
            for (var i = 0, n = this._names.length; i < n; i++) {
                    var name = this._names[i];
                    $('#preview_' + name + ' img').attr('src', Ecart.baseUrl + '/media/product' + val);
                    $('#images-' + name).val(val);
                }
            
        },
        
        remove: function (){
            var selected = imagesView.getSelectedIndexes();
            var records = imagesView.getSelectedRecords();
            for (var i = 0; i < selected.length; i++) {
                this._gallery.splice(selected[i], 1);
                if (records[i].id == Product.Images._baseImageId) {
                    $('#preview_image_base img').attr('src', Ecart.skinUrl + '/images/no_picture.gif');
                    $('#images-image_base').val('');
                }
                if (records[i].id == Product.Images._listingImageId) {
                    $('#preview_image_listing img').attr('src', Ecart.skinUrl + '/images/no_picture.gif');
                    $('#images-image_listing').val('');
                }
                if (records[i].id == Product.Images._thumbnailImageId) {
                    $('#preview_image_thumbnail img').attr('src', Ecart.skinUrl + '/images/no_picture.gif');
                    $('#images-image_thumbnail').val('');
                }
            }
            Product.Images.reload();
        },
        
        clear: function() {
           // $('.image_preview').empty();
            $('.image_preview img').attr('src', Ecart.skinUrl + '/images/no_picture.gif')
            this._gallery = {};
            Product.Images.reload();
        },
        
        add: function() {
            Spinner.show();
            $.ajaxFileUpload({
                url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/save-image/',
                secureuri: false,
                fileElementId: 'image-add',
                dataType: 'json',
                success: function (data, status) {
                    Spinner.hide();
                    Product.Images.addToStore(data['image-add']);
                },
                error: function (data, status, e) {
                    Spinner.hide();
                    alert(e);
                }
            });
        },
        
        addToStore: function(path)
        {
            Product.Images._newImageId--;
            Product.Images._gallery.push({
                'id': Product.Images._newImageId, 
                'image': path
            });
            Product.Images.reload();
        },
        
        /*ajaxFileUpload: function(id) {
            ++Product.Images.uploadCount;
            $.ajaxFileUpload({
                url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/save-image/',
                secureuri: false,
                fileElementId: id,
                dataType: 'json',
                success: function (data, status) {
                    for (id in data) {
                       
                        $('#product_' + id).attr({value: data[id]});
                        Product.Images.setData(id, data[id]);
                        --Product.Images.uploadCount;
                    }
                },
                error: function (data, status, e) {
                    alert(e);
                }
            });
            
            return false;
        },*/
        
        setData: function(id, path) {
           Product.Images.setPath(id, path);
           Product.Images.setPreview(id, path);
        },
        
        setPath: function(id, path) {
            $('#product_' + id).attr({value: path});
        },
        
        setPreview: function(id, val) {
            $('#preview_' + id ).empty();
            var src = val;
            if (src == '') {
                src = Ecart.skinUrl + '/images/no_picture.gif';
            } else {
                src = Ecart.baseUrl + '/media/product' + src;
            }
            $('#preview_' + id).append(
                '<img src="' + 
                src + '" height="75" />' +
                '<input type="hidden" id="images-' + id + '" value="' + 
                 val + '" name="product[' +
                id + ']"/>'
            );
        }
    }
}

Ext.onReady(function(){
    
    $(".date-picker").datepicker({ dateFormat: 'yy-mm-dd'});
    
    Ext.QuickTips.init();
    
    /*product menu*/
    productActionMenu = new Ext.menu.Menu({
        id: 'productActionMenu',
        items: [{
            text: 'Edit'.l(),
            icon: Ecart.skinUrl + '/images/icons/page_edit.png',
            handler: Product.edit
        }, {
            text: 'Cut'.l(),
            icon: Ecart.skinUrl + '/images/icons/page_edit.png',
            handler: Product.cut
        }, {
            text: 'Copy'.l(),
            icon: Ecart.skinUrl + '/images/icons/copy_multiple.png',
            handler: Product.copy
        }, {
            text: 'Paste'.l(),
            icon: Ecart.skinUrl + '/images/icons/move.png',
            id: 'paste-action-menu-item',
            disabled: true,
            handler: Product.paste
        }, {
            text: 'Clear Buffer'.l(),
            icon: Ecart.skinUrl + '/images/icons/page_white.png',
            handler: function() {
                buffer = {}; 
                productActionMenu.items.get('paste-action-menu-item').disable();
            }
        }, {
            text: 'Delete'.l(),
            icon: Ecart.skinUrl + '/images/icons/delete.png',
            menu: {
                items: [{
                    text: 'From category'.l(),
                    handler: Product.removeFromCategory
                }, {
                    text: 'From site'.l(), 
                    handler: Product.removeFromSite
                }, {
                    text: 'From base'.l(),
                    handler: Product.remove
                }]
            }
        }]
    });
    
    /* @do not delete this
    var addRootCategory = function(){
        Ext.Msg.show({
            prompt: true,
            modal: false,
            buttons: Ext.Msg.OKCANCEL,
            title: 'Create category',
            width: 250,
            fn: function(btn, name){
                if (btn != 'ok' || name == '')
                    return false;
                    
                Ext.Ajax.request({
                    url : Ecart.baseUrl + Ecart.adminUrl + '/catalog_category/save',
                    params : {
                        root: true,
                        catName: name,
                        catDesc: 'Root category',
                        catStatus: 'active'
                    },
                    method : 'POST',
                    callback : function() {
                        tree.getRootNode().reload();
                    }
                });
            }
        })        
    }
    */
        
    /*
     * Initialize Tree
     */
    var Tree = Ext.tree;
    tree = new Tree.TreePanel({
        el: 'tree-div',
        maskDisabled: true,
        animate:true, 
        loader: new Tree.TreeLoader({
            dataUrl: Ecart.baseUrl + Ecart.adminUrl + '/catalog_category/get-items/'
        }),
        height: 550,
        width: 200,
        enableDD:true,
        autoScroll: true,
        containerScroll: true,
        rootVisible: false,
        tbar: [/* @do not delete this
            new Ext.SplitButton({
                text: 'Add'.l(),
                tooltip: {text: 'Add SubCategory to selected category', title: 'Add Category'},
                cls: 'x-btn-text-icon',
                icon: Ecart.skinUrl + '/images/icons/add.png',
                menu: new Ext.menu.Menu({
                    items: [{
                        text: 'Add root category'.l(),
                        icon: Ecart.skinUrl + '/images/icons/add.png',
                        handler: addRootCategory
                    }]
            }),
            handler: Category.add
        }),*/ {
            text: 'Add'.l(),
            tooltip: {text: 'Add SubCategory to selected category', title: 'Add Category'},
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/add.png',
            handler: Category.add
        }, {
            text: 'Edit'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/page_edit.png',
            handler: Category.edit
        }, {
            text: 'Delete'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/delete.png',
            handler: Category.remove
        }, '->', {
            cls: 'x-btn-icon',
            icon: Ecart.skinUrl + '/images/icons/refresh.png',
            handler: function(){
                treeRoot.reload();
            }
        }]
    });
               
    // set the root node
    treeRoot = new Tree.AsyncTreeNode({
        text: 'Sites'.l(),
        draggable: false,
        id: '0'
    });
    tree.on('click', function(node, e) {
        Category.id = node.id;
        if (node.id == Category.rootId) {
            return true;
        }
        ds.baseParams = {catId : node.id, siteId: node.attributes.site_id};
        ds.load({params:{start: 0, limit: 25}});
    });
    tree.setRootNode(treeRoot);
    
    tree.getLoader().on('beforeload', function(loader, node) {
        if (node !== tree.getRootNode()) {
            return;
        }
        treeRoot.appendChild(
            new Ext.tree.TreeNode({
                text:'Uncategorized'.l(), 
                id: 'uncategorized',
                iconCls: 'icon-bin',
                allowDrag: false
            })
        );
    }, this);
    
    tree.render();
    treeRoot.expand();
    tree.on('beforemovenode', function(tree, node, oldParent, newParent, index) {
        if (node.attributes.site_id != newParent.attributes.site_id) {
            alert('You can\'t move categories through sites'.l())
            return false;
        }
        var moveType, id;
        if (newParent.childNodes.length == 0) {
            moveType = 'moveTo';
            id = newParent.id;
        } else if (newParent.childNodes.length > index) {
            moveType = 'moveBefore';
            id = newParent.childNodes[index].id;
        } else {
            moveType = 'moveTo';
            id = newParent.id;
        }
        
        Ext.Ajax.request({
            url : Ecart.baseUrl + Ecart.adminUrl + '/catalog_category/move/',
            params : {
                catId : node.id,
                newParentId: id,
                moveType : moveType
            }
        });
    }, this);
    
    Ext.QuickTips.init();

    ds = new Ext.data.Store({
        proxy: new Ext.data.HttpProxy({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/list-products/'
        }),
        reader: new Ext.data.JsonReader({
            root: 'data',
            totalProperty: 'count',
            id: 'id'
        }, [
            {name: 'id'},
            {name: 'is_active'}, 
            {name: 'name'}, 
            {name: 'date_available', type: 'date', dateFormat: 'Y-m-d'},
            {name: 'is_featured'}, 
            {name: 'date_exp', type: 'date', dateFormat: 'Y-m-d'},
            {name: 'price'}
        ]),
        remoteSort: true
    });

    var sm = new Ext.grid.CheckboxSelectionModel();
    
    var isActive = new Ext.grid.CheckColumn({
        header: "Is Active".l(),
        dataIndex: 'is_active',
        width: 100
    });
    var isFeatured = new Ext.grid.CheckColumn({
        header: "Featured".l(),
        dataIndex: 'is_featured',
        width: 60
    });

    var cm = new Ext.grid.ColumnModel([
        sm, {
            header: "Id".l(),
            dataIndex: 'id',
            width: 30,
            sortable: true
        }, {
            header: "Name".l(), 
            dataIndex: 'name', 
            width: 300,
            css: 'white-space:normal;',
            sortable: true
        },
        isActive, 
        {
            header: "Date Available".l(),
            dataIndex: 'date_available',
            width: 100,
            editor: new Ext.form.DateField({allowBlank: true, format: 'Y-m-d'}),
            renderer: function(v) {
                return Ext.util.Format.date(v);
            }
        },
        isFeatured,
        {
            header: "Date Expire".l(),
            dataIndex: 'date_exp',
            width: 120,
            editor: new Ext.form.DateField({allowBlank: true, format: 'Y-m-d'}),
            renderer: function(v) {
                return Ext.util.Format.date(v);
            }
        }, {
            header: "Price".l(),
            dataIndex: 'price',
            width: 80,
            editor: new Ext.form.TextField({allowBlank: false})
        }
    ]);
    cm.defaultSortable = true;
    
    grid = new Ext.grid.EditorGridPanel({
        renderTo: 'ajax-grid-prod',
        ds: ds,
        cm: cm,
        sm: sm,
        height: 550,
        viewConfig: {
            forceFit: true,
            emptyText: 'No records found'.l()
        },
        plugins: [isActive, isFeatured],
        clicksToEdit: 1,
        bbar: new Ext.PagingToolbar({
            pageSize: 25,
            store: ds,
            displayInfo: true,
            displayMsg: 'Displaying records {0} - {1} of {2}'.l(),
            emptyMsg: 'No records found'.l()
        }),
        tbar: [{
            text: 'Add Product'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/add.png',
            handler: Product.add
        }, {
            text: 'Actions'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/menu_action.png',
            menu: productActionMenu
        }, {
            text: 'Save'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/save_multiple.png',
            handler: function(){
                var modified = ds.getModifiedRecords();
                if (!modified.length)
                    return false;
                    
                var data = {};
                for (var i = 0, n = modified.length; i < n; i++) {
                    data[modified[i]['id']] = modified[i]['data'];
                }
                
                var jsonData = Ext.encode(data);
                Ext.Ajax.request({
                    url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/batch-save-product/',
                    params: {
                        data: jsonData,
                        siteId: ds.baseParams.siteId
                    },
                    callback: function() {
                        ds.commitChanges();
                        ds.reload();
                    }
                })
            }
        }, '->', {
            text: 'Reload'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/refresh.png',
            handler: function() {
                ds.reload();
            }
        }],
        listeners: {
            'beforerender': function(cmp) {
                cmp.width = $('#ext-cmp-layout').width() - $('#ext-cmp-layout .sidebox').width();
            }
        }
    });
    grid.on('rowdblclick', Product.edit);

    $('#div-form-product').css('display', 'none');
    
    optionTree = new Tree.TreePanel({
        title: 'Options'.l(),
        id: 'options-tree',
        animate:true, 
        border: false,
        autoScroll: true,
        loader: new Tree.TreeLoader({
            dataUrl: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/get-options/'
        }),
        containerScroll: true,
        selModel: new Ext.tree.MultiSelectionModel()
    });
    Ecart.Variations.tree = Ecart.Modifiers.tree = Ecart.Properties.tree = optionTree;
    
    var optionTreeRoot = new Tree.AsyncTreeNode({
        text: 'Options'.l(),
        draggable: false,
        id: '0'
    });
    optionTree.setRootNode(optionTreeRoot);
    
    optionTreeRoot.on('dblclick', function(){
        Ext.getCmp('options-tree').getRootNode().reload();
    })
    
    opts = new Ext.Panel({
        layout: 'accordion',
        title: 'Panel'.l(),
        id : 'form-product-rigth-panel',
        region: 'east',
        split: true,
        width: 200,
        frame: false,
        collapsible: true,
        items: [
            optionTree,
            imagesTreePanel
        ]
    });
    
    var tabsWraper = new Ext.Panel({
        region: 'center',
        collapsible: false,
        border: false,
        layout: 'fit',
        contentEl: 'form-product',
        autoScroll: true
    });
    
    /* Creating popup windows */
    productWin = new Ext.Window({
        items: [tabsWraper, opts], //tabPanel, created in form-product.phtml
        layout: 'border',
        constrainHeader: true, 
        width: 800,
        height: 530,
        closeAction: 'hide',
        maximizable: true,
        border: false,
        buttons: [{
            text: 'Save'.l(),
            handler: Product.save
        }, {
            text: 'AutoFill'.l(),
            handler: Product.autoFill
        }, {
            text: 'Cancel'.l(),
            handler: function(){
                productWin.hide();
            }
        }]
    });
    
}, this);

function initTree(root, categories, productId) {
    Ext.Ajax.request({
        url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_index/category-tree',
        method: 'post',
        params: {
            'productId': productId
        },
        callback: function(options, success, response) {
            $('#' + root).empty().html(response.responseText);
            
            var inputs = document.getElementById(root).getElementsByTagName('input');
            for (var i=0; i < inputs.length; i++) {
                if (inputs[i].checked) { 
                    inputs[i].parentNode.className = inputs[i].parentNode.className + " selected";
                }
            }
            
            $(".tree-cat").click(function(){
                if ($(this).is(".selected")) {
                    $(this).removeClass("selected");
                    $(this).children(":first").attr("checked", false);
                } else {
                    $(this).addClass("selected");
                    $(this).children(":first").attr("checked", true);
                }
            });
            
            $(".toggle-header").click(function(){
                $(this).next().toggle('fast');
            });
            
            $('.selected').removeClass("selected");
            if (categories && categories.length) {
                for (var i = 0; i < categories.length; i++) {
                    if (isNaN(Ext.getDom('category-' + categories[i]))) {
                       Ext.getDom('category-' + categories[i]).checked = true;
                       $('#category-' + categories[i]).parent().addClass('selected');
                    }
                }
            }
        }
    });
}

    
/*checkbox column*/
Ext.grid.CheckColumn = function(config){
    Ext.apply(this, config);
    if(!this.id){
        this.id = Ext.id();
    }
    this.renderer = this.renderer.createDelegate(this);
};

Ext.grid.CheckColumn.prototype = {
    init : function(grid){
        this.grid = grid;
        this.grid.on('render', function(){
            var view = this.grid.getView();
            view.mainBody.on('mousedown', this.onMouseDown, this);
        }, this);
    },

    onMouseDown : function(e, t){
        if(t.className && t.className.indexOf('x-grid3-cc-'+this.id) != -1){
            e.stopEvent();
            var index = this.grid.getView().findRowIndex(t);
            var record = this.grid.store.getAt(index);
            record.set(this.dataIndex, record.data[this.dataIndex] == 0 ? 1 : 0);
        }
    },

    renderer : function(v, p, record){
        v = parseInt(v);
        p.css += ' x-grid3-check-col-td'; 
        return '<div class="x-grid3-check-col'+( v!=0 ? '-on' : '')+' x-grid3-cc-'+this.id+'">&#160;</div>';
    }
};
/*end of checkbox column*/