/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ext.onReady(function (){
    var editWin;
    Manufacture = {
        add: function (){
            $('#mLogo').val('');
            $('#noPicture').val(1);
            $('#manufacturerId').val(-1);
            $('#manufacture-picture').attr('src', Ecart.skinUrl + '/images/no_picture.gif');
            $('#manufacture-image').val('');
            Manufacture.getWin();
            editWin.setTitle('Add new Manufacture');
            editWin.show();
        },
        edit: function(gridT, rowIndex, e) {
            if (isNaN(rowIndex)) {
                var manufacturerId = gridManufacture.getSelectionModel().getSelected().id; //get first selected row in grid
            } else {
                var manufacturerId = storeManufacture.data.items[rowIndex].id;
            }
            Manufacture.load(manufacturerId);
        },
        load: function (manufacturerId) {
            $('#mLogo').val('');
            $('#manufacture-image').val('');
            var record = storeManufacture.getById(manufacturerId).data;
            $('#manufactureName').val(record.name);
            $('#manufacturerId').val(record.id);
            $('#noPicture').val(1);
            var image = Ecart.skinUrl + '/images/no_picture.gif';
            if (record.image != '') {
               $('#noPicture').val(0);
               $('#mLogo').val(record.image);
               image = Ecart.baseUrl + '/media/manufacturer' + record.image;
            }
            
            $('#manufacture-picture').attr('src', image);
            Manufacture.getWin();
            editWin.setTitle('Edit Manufacture : ');
            editWin.show();
        },
        noPicture: function() {
            $('#manufacture-picture').attr('src', Ecart.skinUrl + '/images/no_picture.gif');
            $('#noPicture').val(1);
            return false;
        },
        remove: function() {
            if (!confirm('Delete manufacture(s)?'))
                return;
            var data = {};
            var selectedItems = gridManufacture.getSelectionModel().selections.items;
            for (var i = 0; i < selectedItems.length; i++) {
                data[i] = selectedItems[i].id;
            }
                    
            Ext.Ajax.request({
                url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_manufacturer/delete',
                params: {data: Ext.encode(data)},
                callback: function() {
                    storeManufacture.reload();
                }
            });
        },
        save: function () {
            Ext.Ajax.request({
                url : Ecart.baseUrl + Ecart.adminUrl + '/catalog_manufacturer/save',
                form: 'manufacture-form',
                secureuri: false,
                callback: function(options, success, response) {
                    var oRet = Ext.decode(response.responseText);
                    if (true == success ) {
                        editWin.hide();
                        storeManufacture.reload();
                    }
                }
            });
        },
        changeImage: function () {
            Spinner.show();
            $.ajaxFileUpload({
                url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_manufacturer/save-image',
                secureuri: false,
                fileElementId: 'manufacturer-image',
                dataType: 'json',
                success: function (data, status) {
                    Spinner.hide();
                    $('#noPicture').val(0);
                    $('#mLogo').val(data);
                    $('#manufacture-picture').attr('src', Ecart.baseUrl + '/media/manufacturer' + data);
                },
                error: function (data, status, e) {
                    Spinner.hide();
                    alert(e);
                }
            });
            return false;
        },
        getWin: function(){
            if (!editWin) {
                $('#manufacture').show();
                editWin = new Ext.Window({
                    contentEl: 'manufacture',
                    layout: 'fit',
                    width: 350,
                    /*height: 500,*/
                    closeAction: 'hide',
                    plain: true,
                    buttons: [{
                    text: 'Save'.l(),
                    handler: function() {
                        Manufacture.save();
                    }
                    },{
                        text: 'Cancel'.l(),
                        handler: function(){
                            editWin.hide();
                        }
                    }]
                   
                });
            }
            return editWin;
        }
    };
    
    var storeManufacture = new Ext.data.Store({
       url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_manufacturer/list',
       reader: new Ext.data.JsonReader({
               root : 'data',
               totalProperty: 'count',
               id: 'id'
           },
           [{name: 'id', type: 'int'}, 'name', 'image', 'created_on', 'modified_on']
       ),
       autoLoad: false
   });
   
   var selectionModel = new Ext.grid.CheckboxSelectionModel();
   
   var columnsManufacture = new Ext.grid.ColumnModel([
       selectionModel,{
           header: "Id".l(), 
           width: 30, 
           dataIndex: 'id'
       }, {
           header: "Name".l(), 
           width: 145, 
           dataIndex: 'name'
       }, {
           header: "Image".l(), 
           width: 170, 
           dataIndex: 'image',
           renderer: function(value) {
               if (!value) return 'None';
               else return value;
           }
       }, {
           header: "Date created".l(), 
           width: 145, 
           dataIndex: 'created_on'
       }, {
           header: "Date updated".l(), 
           width: 170,
           dataIndex: 'modified_on'
       }
    ]);
    columnsManufacture.defaultSortable = true;
    
    gridManufacture = new Ext.grid.GridPanel({
        store: storeManufacture,
        cm: columnsManufacture,
        sm: selectionModel,
        stripeRows: true,
        renderTo: 'manufacture-grid',
        height : 550,
        viewConfig: {
            forceFit: true,
            emptyText: 'No records found'.l()
        },
        tbar: [{ 
            text: 'Add'.l(),
            icon: Ecart.skinUrl + '/images/icons/add.png',
            cls: 'x-btn-text-icon',
            handler: Manufacture.add
        }, {
            text: 'Edit'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/page_edit.png',
            handler: Manufacture.edit
        }, {
            text: 'Delete'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/delete.png',
            handler: Manufacture.remove
        }, '->', {
            cls: 'x-btn-icon',
            icon: Ecart.skinUrl + '/images/icons/refresh.png',
            handler: function(){
                gridManufacture.getStore().reload();
            }
        }],
        plugins:[
            new Ext.ux.grid.Search({
                mode:'local',
                iconCls:false,
                dateFormat:'Y-m-d',
                width: 150,
                minLength:2
            })
        ],
        bbar: [],
        listeners: {
            'beforerender': function(cmp) {
                cmp.width = $('#ext-cmp-layout').width() - $('#ext-cmp-layout .sidebox').width();
            }
        }
    });
    
    function setManufacture(id) {
       var store = gridManufacture.getStore();
       store.lastOptions = {};
       gridManufacture.filters.filters.get('id').setValue({'eq': id});
    }
    
    gridManufacture.on('rowdblclick', Manufacture.edit);
    
    if (typeof(manufacturerId) !== "undefined")
         setManufacture(manufacturerId);
    else
         storeManufacture.load();
})