/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 */

Ext.onReady(function(){
    
    var record = [
        {name: 'id', type: 'int'},
        {name: 'name', type: 'string'},
        {name: 'url', type: 'string'},
        {name: 'image', type: 'string'}
    ];
    for (var langId in Ecart.languages) {
        record.push({'name': 'title_' + langId});
    }
    var manufacturer_object = Ext.data.Record.create(record);
    
    var ds = new Ext.data.Store({
        url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_manufacturer/list',
        method: 'get',
        reader: new Ext.data.JsonReader({
            root : 'data',
            totalProperty: 'count',
            id: 'id'
        }, manufacturer_object),
        remoteSort: false,
        pruneModifiedRecords: true,
        sortInfo: {
            field: 'id',
            direction: 'DESC'
        }
    })
    
    var columns = [];
    var sm = new Ext.grid.CheckboxSelectionModel();
    columns.push(sm);
    columns.push({
        header: 'Id'.l(),
        dataIndex: 'id',
        width: 40,
        menuDisabled: true
    }, {
        header: 'Name'.l(),
        dataIndex: 'name',
        menuDisabled: true,
        editor: new Ext.form.TextField({
            allowBlank: false,
            maxLength: 128
        })
    }, {
        header: 'Url'.l(),
        dataIndex: 'url',
        menuDisabled: true,
        editor: new Ext.form.TextField({
            allowBlank: false,
            maxLength: 128
        })
    }, {
        header: 'Image'.l(),
        dataIndex: 'image',
        menuDisabled: true,
        editor: new Ext.form.TextField({
            allowBlank: true,
            maxLength: 255
        })
    });
    for (var langId in Ecart.languages) {
        columns.push({
            header: 'Title ({language})'.l('core', Ecart.languages[langId]),
            dataIndex: 'title_' + langId,
            width: 150,
            menuDisabled: true,
            editor: new Ext.form.TextField({
               allowBlank: false,
               maxLength: 255
            })
        });
    }
    var cm = new Ext.grid.ColumnModel(columns);
    cm.defaultSortable = true;
    
    var grid = new Ext.grid.EditorGridPanel({
        cm: cm,
        sm: sm,
        id: 'grid',
        enableColumnMove: false,
        height: 550,
        renderTo: 'manufacturer-grid',
        store: ds,
        clicksToEdit: 1,
        viewConfig: {
            forceFit: true,
            deferEmptyText: true,
            emptyText: 'No records found'.l()
        },
        tbar: [{
            text: 'Add'.l(),
            handler: add,
            iconCls: 'btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/add.png'
        }, {
            text: 'Edit'.l(),
            handler: function(){
                edit();
            },
            iconCls: 'btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/page_edit.png'
        }, {
            text: 'Save'.l(),
            handler: save,
            iconCls: 'btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/save_multiple.png'
        }, {
            text: 'Delete'.l(),
            handler: deleteSelected,
            iconCls: 'btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/delete.png'
        },'->', {
            text: 'Reload'.l(),
            handler: reload,
            iconCls: 'btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/refresh.png'
        }],
        listeners: {
            'beforerender': function(cmp) {
                cmp.width = $('#ext-cmp-layout').width() - $('#ext-cmp-layout .sidebox').width();
            }
        }
    })
    
    Ext.getCmp('grid').store.load();
    Ext.getCmp('grid').on('rowdblclick', function(grid, index){
        edit(grid.getStore().getAt(index));
    });
    
    function reload(){
        Ext.getCmp('grid').store.reload();
    }
    
    function add(){
        Ext.getCmp('window').show();
        Ext.getCmp('window').setTitle('New Manufacturer'.l());
        Ext.getCmp('form').getForm().reset();
        $('#window #img-preview').attr('src', '');
    }
    
    function edit(row){
        selected = row || Ext.getCmp('grid').getSelectionModel().getSelected();
        
        if (!selected) {
            return;
        }
        
        Ext.getCmp('window').show();
        Ext.getCmp('window').setTitle('Edit Manufacturer'.l());
        Ext.getCmp('form').getForm().reset();
        fillForm(selected); //see index.phtml
    }
    
    function save(){
        var modified = Ext.getCmp('grid').store.getModifiedRecords();
        
        if (!modified.length) {
            return false;
        }
        
        var data = {};
        for (var i = 0, len = modified.length; i < len; i++) {
            data[modified[i]['id']] = modified[i]['data'];
        }
        Ext.Ajax.request({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_manufacturer/batch-save',
            method: 'post',
            params: {
                data: Ext.encode(data)
            },
            success: reload
        })
    }
    
    function deleteSelected(){
        var selections = Ext.getCmp('grid').getSelectionModel().getSelections();
        
        if (!selections.length || !confirm('Are you sure?'.l())) {
            return;
        }
         
        var data = {};   
        for (var i = 0, len = selections.length; i < len; i++) {
            data[i] = selections[i]['id'];
        }
        Ext.Ajax.request({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_manufacturer/delete',
            method: 'post',
            params: {
                data: Ext.encode(data)
            },
            success: reload
        })
    }
})

/*checkbox column*/
Ext.grid.CheckColumn = function(config){
    Ext.apply(this, config);
    if(!this.id){
        this.id = Ext.id();
    }
    this.renderer = this.renderer.createDelegate(this);
};

Ext.grid.CheckColumn.prototype ={
    init : function(grid){
        this.grid = grid;
        this.grid.on('render', function(){
            var view = this.grid.getView();
            view.mainBody.on('mousedown', this.onMouseDown, this);
        }, this);
    },

    onMouseDown : function(e, t){
        if(t.className && t.className.indexOf('x-grid3-cc-'+this.id) != -1){
            e.stopEvent();
            var index = this.grid.getView().findRowIndex(t);
            var record = this.grid.store.getAt(index);
            record.set(this.dataIndex, record.data[this.dataIndex] == 'enabled' ? 'disabled' : 'enabled');
        }
    },

    renderer : function(v, p, record){
        p.css += ' x-grid3-check-col-td';
        return '<div class="x-grid3-check-col'+(v=='enabled'?'-on':'')+' x-grid3-cc-'+this.id+'">&#160;</div>';
    }
};
/*end of checkbox column*/