/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ecart.Modifiers = {
    tree: {},
    _modifiers: {},
    
    toSaveArray: function() {
        var result = {};
        for (var i in this._modifiers) {
            result[i] = this._modifiers[i].toSaveArray();
        }
        return result;
    },
    
    add: function (modifier) {
         //todo sortOrder
        this._modifiers[modifier.getId()] = modifier;
        modifier.render();
    },
    
    addModifiers: function() {
        var modifiers = Ecart.Modifiers.tree.getSelectionModel().getSelectedNodes();
        for (var i = 0, n = modifiers.length; i < n; i++) {
            var node = modifiers[i];
            
            var config = {
                status: 'new',
                modifierId: this.id,
                renderTo: 'modifiers'
            };
            if (node.attributes.type == 'option') {
                this._modifiers[config.id]
                config.product_option_id = node.id;
                config.optionName = node.text;
            } else if(node.attributes.type == 'value') {
                config.product_option_value_id = node.attributes.value_id;
                config.valueName = node.text;
                config.product_option_id = node.parentNode.id;
                config.optionName = node.parentNode.text;
            } else continue;
            
            var modifier = new Ecart.Modifier(config);
            if (this._modifiers[modifier.getId()]) {
                if (!this._modifiers[modifier.getId()].isRemoved())
                    continue;
                modifier.status = 'update';
            }
            this.add(modifier);
        }
    },
    
    remove: function(modifierId) {
        var modifier = this._modifiers[modifierId];
        modifier.unrender();
        if (modifier.status == 'new') {
            delete this._modifiers[modifier.getId()];
        } else modifier.status = 'removed';
    },
    
    clear: function() {
        this._modifiers = {};
        $('#modifiers').empty();
    }
};

Ecart.Modifier = function (config) {
    for (var i in config) {
        this[i] = config[i];
    }
    this.rendered = false;
};

Ecart.Modifier.prototype = {
    toSaveArray: function() {
        return {
            id: this.id,
            product_option_id: this.product_option_id,
            product_option_value_id: this.product_option_value_id ? this.product_option_value_id : null,
            status: this.status
        };
    },
    
    render: function () {
        // check for wrapper exists
        var config = {
            id: this.getId(),
            name: this.valueName,
            sku: this.sku,
            price: this.price,
            weight: this.weight
        };
        config['price_' + this.price_type] = 'selected="1"';
        config['weight_' + this.weight_type] = 'selected="1"';
        if (this.product_option_value_id) {
            if (!$('#m-wraper-' + this.product_option_id).length)
                this.renderWraper();
            var tpl = Ext.Template.from('m-value-tpl');
            config['name'] = this.valueName;
            tpl.insertFirst('m-' + this.product_option_id + '-values', config);
        } else {
            var tpl = Ext.Template.from('m-option-tpl');
            config['name'] = this.optionName
            tpl.insertFirst('modifiers', config);
        }
        this.rendered = true;
    },
    
    renderWraper: function () {
        var tpl = Ext.Template.from('m-wraper-tpl');
        tpl.insertFirst('modifiers', {
            id: this.product_option_id,
            name: this.optionName
        });
    },
    
    unrender: function () {
        $('#modifier-' + this.getId()).remove();
        
        /* if this is last value of option then remove modifier wrapper */
        if (!$('#m-' + this.product_option_id + '-values > *').length)
            $('#m-wraper-' + this.product_option_id).remove();
    },
    
    isRemoved: function () {
        return this.status == 'removed';
    },
    
    getId: function () {
        if (this.product_option_value_id)
            return this.product_option_id + '_' + this.product_option_value_id
        return this.product_option_id;
    }
};