/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ecart.Properties = {
    tree: {},
    _properties: {},
    
    toSaveArray: function() {
        var result = {};
        for (var i in this._properties) {
            result[i] = this._properties[i].toSaveArray();
        }
        return result;
    },
    
    add: function (property) {
        this._properties[property.getId()] = property;
        property.render();
    },
    
    addProperties: function() {
        var properties = Ecart.Properties.tree.getSelectionModel().getSelectedNodes();
        for (var i = 0, n = properties.length; i < n; i++) {
            var node = properties[i];
            
            var config = {
                status: 'new'
            };
            if (node.attributes.type == 'option') {
                this._properties[config.id]
                config.product_option_id = node.id;
                config.optionName = node.text;
                config.inputType = node.attributes.inputType;
                config.languagable = node.attributes.languagable;
            } else if(node.attributes.type == 'value') {
                config.product_option_value_id = node.attributes.value_id;
                config.valueName = node.text;
                config.product_option_id = node.parentNode.id;
                config.optionName = node.parentNode.text;
            } else continue;
            
            var property = new Ecart.Property(config);
            if (this._properties[property.getId()]) {
                if (!this._properties[property.getId()].isRemoved())
                    continue;
                property.status = 'update';
            }
            if (property.product_option_value_id || property.inputable())
                this.add(property);
        }
    },
    
    remove: function(propertyId) {
        var property = this._properties[propertyId];
        property.unrender();
        if (property.status == 'new') {
            delete this._properties[property.getId()];
        } else property.status = 'removed';
    },
    
    clear: function() {
        this._properties = {};
        $('#properties').empty();
    }
};

Ecart.Property = function (config) {
    for (var i in config) {
        this[i] = config[i];
    }
};

Ecart.Property.prototype = {
    toSaveArray: function() {
        return {
            id: this.id,
            product_option_id: this.product_option_id,
            product_option_value_id: this.product_option_value_id ? this.product_option_value_id : null,
            status: this.status
        };
    },
    render: function () {
        // check for wrapper exists
        var tpl = Ext.Template.from('property-tpl');
        var value = '';
        if (this.inputable()) {
            var valueTpl = Ext.Template.from('property-value-' + this.inputType);
            if (this.languagable == '0') {
                for (var i in Ecart.languages) {
                    var currentlanguageId = i;
                    break;
                }
                value = valueTpl.applyTemplate({
                    'id': this.product_option_id,
                    'langId': currentlanguageId,
                    'value': function(){
                        var result = '';
                        for (i in this.values) {
                            result = this.values[i];
                            break;
                        }
                        return result;
                    }.apply(this)
                });
            } else {
                for (var i in Ecart.languages) {
                    lang = Ecart.languages[i];
                    value += valueTpl.applyTemplate({
                        'id': this.product_option_id,
                        'language': lang,
                        'langId': i,
                        'value': this.values && this.values[i] ? this.values[i] : ''
                    });
                }
            }
        } else {
            value = this.valueName;
        }
        tpl.insertFirst('properties', {
            'id': this.getId(),
            'name': this.optionName,
            'value': value
        });
        this.rendered = true;
    },
    
    unrender: function () {
        $('#property-' + this.getId()).remove();
    },
    
    isRemoved: function () {
        return this.status == 'removed';
    },
    
    getId: function () {
        return this.product_option_id;
    },
    inputable: function () {
        if (!this.inputType)
            return false;
        if (this.inputType == '1' || 
            this.inputType == '4' || 
            this.inputType == '5')
            return true;
        return false;
    }
};