/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ext.onReady(function() {

    Ext.QuickTips.init();

    Ext.form.Field.prototype.msgTarget = 'under';

    var sm = new Ext.grid.CheckboxSelectionModel();

    var status = new Ext.grid.CheckColumn({
        header: 'Status'.l(),
        width: 60,
        dataIndex: 'is_active'
    });

    var blockColumn = new Ext.grid.ColumnModel([
        sm, {
            header: 'Name'.l(),
            width: 250,
            dataIndex: 'name',
            editor: new Ext.form.Field ({
                allowBlank: false,
                maxLength: 45
            })
        },
        status
    ]);
    blockColumn.defaultSortable = true;

    var blockFields = new Ext.data.Record.create ([
        'id',
        'name',
        'is_active'
    ]);

    var blockStore = new Ext.data.Store({
        proxy: new Ext.data.HttpProxy({
            url: Ecart.baseUrl + Ecart.adminUrl + '/cms_block/get-blocks',
            method: 'get'
        }),
        reader: new Ext.data.JsonReader({root: 'blocks', id: 'id'}, blockFields)
    });

    blockGrid = new Ext.grid.EditorGridPanel ({
        renderTo: 'blockGrid',
        height: 550,
        sm: sm,
        cm: blockColumn,
        store: blockStore,
        viewConfig: {
            forceFit: true,
            emptyText: 'No records found'.l()
        },
        autoScroll: true,
        clicksToEdit: 1,
        plugins: status,
        stripeRows: true,
        tbar: [{
            text: 'Add'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/add.png',
            handler: createBlock
        }, {
            text: 'Edit'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/page_edit.png',
            handler: editBlock
        },{
            text: 'Save'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/save_multiple.png',
            handler: saveChanges
        }, {
            text: 'Delete'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/delete.png',
            handler: deleteBlock
        }, '->', {
            text: 'Reload'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/refresh.png',
            handler: reloadGrid
        }],
        listeners: {
            'beforerender': function(cmp) {
                cmp.width = $('#ext-cmp-layout').width() - $('#ext-cmp-layout .sidebox').width();
            }
        }
    });

    blockGrid.on('rowdblclick', function(grid, blockId, e) {
        editBlock();
    })

    blockGrid.getStore().load();
})

function reloadGrid() {
    blockGrid.getStore().reload();
}

function deleteBlock() {
    var selectedItems = blockGrid.getSelectionModel().selections.items;
    
    if (selectedItems.length < 1)
        return; 
        
    if (!confirm('Are you sure?'.l()))
        return;
        
    var data = {};
   
    for (var i = 0; i < selectedItems.length; i++) {
        data[i] = selectedItems[i].id;
    }
    var jsonData = Ext.encode(data);
    Ext.Ajax.request({
        url: Ecart.baseUrl + Ecart.adminUrl + '/cms_block/delete-block',
        params: {data: jsonData},
        callback: function() {
            blockGrid.getStore().reload();
        }
    });
}

function saveChanges() {
    var modified = blockGrid.getStore().getModifiedRecords();
    if (modified.length < 1) return alert('Nothing to save');
    var data = {};
    
    for (var i = 0; i < modified.length; i++) {
        data[modified[i]['id']] = modified[i]['data'];
    }
    
    var jsonData = Ext.encode(data);
    Ext.Ajax.request({
        url:  Ecart.baseUrl + Ecart.adminUrl + '/cms_block/quick-save-block',
        params: {data: jsonData},
        callback: function() {
            blockGrid.getStore().commitChanges();
            blockGrid.getStore().reload();
        }
    })
}

function editBlock() {
    block = blockGrid.getSelectionModel().getSelected();
    if (!block) return;
    currentBlock = block.id;

    blockForm.getForm().reset();
    blockWindow.show();
    blockForm.getForm().load({
        url:  Ecart.baseUrl + Ecart.adminUrl + '/cms_block/get-block-data',
        params: {blockId: currentBlock}
    });
}

function createBlock() {
    currentBlock = 'new';
    blockForm.getForm().reset();
    blockWindow.show();
}

/*checkbox column*/
Ext.grid.CheckColumn = function(config){
    Ext.apply(this, config);
    if(!this.id){
        this.id = Ext.id();
    }
    this.renderer = this.renderer.createDelegate(this);
};

Ext.grid.CheckColumn.prototype ={
    init : function(grid){
        this.grid = grid;
        this.grid.on('render', function(){
            var view = this.grid.getView();
            view.mainBody.on('mousedown', this.onMouseDown, this);
        }, this);
    },

    onMouseDown : function(e, t){
        if(t.className && t.className.indexOf('x-grid3-cc-'+this.id) != -1){
            e.stopEvent();
            var index = this.grid.getView().findRowIndex(t);
            var record = this.grid.store.getAt(index);
            record.set(this.dataIndex, !record.data[this.dataIndex]);
        }
    },

    renderer : function(v, p, record){
        p.css += ' x-grid3-check-col-td'; 
        return '<div class="x-grid3-check-col'+(v=='1'?'-on':'')+' x-grid3-cc-'+this.id+'">&#160;</div>';
    }
};
/*end of checkbox column*/
