/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ext.onReady(function(){
    Ext.form.Field.prototype.msgTarget = 'under';
    var cId;
    var Item = {        
        getId: function() {
            var selectedItems = grid.getSelectionModel().selections.items;
            if (selectedItems.length) {
                return selectedItems[0].id;
            }
            return 0;
        },
        create: function() {
            Item.window.show();
            cId = '';
            formDetails.getForm().reset();
        },
        edit: function(grid, rowIndex, e) {
            cId = ds.data.items[rowIndex].id; 
            document.location.href = Ecart.baseUrl + Ecart.adminUrl + '/customer_index/edit/customerId/' + cId;
        }, 
        quickedit: function(grid, rowIndex, e) {
            cId = Item.getId();
            if (!cId) return alert('Select customer');
            formDetails.getForm().reset();
            Item.window.show();
            formDetails.getForm().load({    
                url: Ecart.baseUrl + Ecart.adminUrl + '/customer_index/load-fields-data/cId/' + cId
            });
        }, 
        save: function() {
            formDetails.getForm().submit({
                url: Ecart.baseUrl + Ecart.adminUrl + '/customer_index/save/',
                params: {id: cId},
                clientValidation: true,
                success: function(){
                    Item.window.hide();
                    ds.reload();
                }
            })
        }, 
        quickSave: function() {
            var data={};
            var modified = ds.getModifiedRecords();
            var length = modified.length;
            
            if (length < 1) return;
            
            for (var i = 0; i < length; i++) {
                data[modified[i]['id']] = modified[i]['data'];
            }
            Ext.Ajax.request({
                url: Ecart.baseUrl + Ecart.adminUrl + '/customer_index/quick-save/',
                params: {data: Ext.encode(data)},
                callback: function() {
                    ds.commitChanges();
                    ds.reload();
                }
            })
        },   
        sendmail: function() {
            formMail.getForm().submit({
                url: Ecart.baseUrl + Ecart.adminUrl + '/customer_mail/send/',
                params: {email: formMail.getForm().findField('email').el.dom.value},
                success: function() {
                    Item.windowMail.hide();
                }
            });      
        }, 
        mail: function() {
            cId = Item.getId();
            if (!cId) {
                return false;
            }
            formMail.getForm().reset();
            Item.windowMail.show();
            var mail = grid.getSelectionModel().getSelected().data['email'];
            var inputMail = formMail.getForm().findField('email').el.dom;
            inputMail.value = mail;
        },
        remove: function() {
            if (!confirm('Delete item(s)?'))
                return;
            var data = {};
            var selectedItems = grid.getSelectionModel().selections.items;
            for (var i = 0; i < selectedItems.length; i++) {
                data[i] = selectedItems[i].id;
            }
            var jsonData = Ext.encode(data);
            Ext.Ajax.request({
                url: Ecart.baseUrl + Ecart.adminUrl + '/customer_index/delete',
                params: {data: jsonData},
                callback: function() {
                    ds.reload();
                }
            });
        }
    };
    
    formDetails = new Ext.form.FormPanel({
        labelWidth: 100,
        autoScroll: true,
        bodyStyle:'padding:5px',
        border: false,
        reader: new Ext.data.JsonReader({
            root: 'fields',
            id: 'id'
        }, formReader),
        items: formItems
    });
    
    formMail = new Ext.form.FormPanel({
        labelWidth: 80,
        autoScroll: true,
        bodyStyle: 'padding: 5px',
        border: false,
        defaults: {
            width: 300,
            anchor: '98%'
        },
        items: [{
            fieldLabel: 'Email'.l(),
            name: 'email',
            readOnly: true,
            xtype: 'textfield',
            vtype: 'email',
            allowBlank:false
        }, {
            fieldLabel: 'Subject'.l(),
            name: 'subject',
            xtype: 'textfield',
            allowBlank: true
        }, {
            fieldLabel: 'Message'.l(),
            name: 'message',
            xtype: 'textarea',
            height: 130,
            allowBlank: false
        }]
    });
    
    Item.window = new Ext.Window({
        layout: 'fit',
        width: 510,
        height: 450,
        closeAction: 'hide',
        bodyStyle: 'padding: 5px; background: white;',
        title: 'Customer'.l(),
        items: formDetails,
        buttons: [{
            text: 'Save'.l(),
            handler: Item.save
        }, {
            text: 'Cancel'.l(),
            handler: function(){
                Item.window.hide();
            }
        }]
    });
    
    Item.windowMail = new Ext.Window({
        layout: 'fit',
        width: 450,
        height: 300,
        resizable: true,
        closeAction: 'hide',
        bodyStyle:'padding:5px; background: white;',
        title: 'Send email'.l(),
        items: formMail,
        buttons: [{
            text: 'Send'.l(),
            handler: Item.sendmail
        },{
            text: 'Cancel'.l(),
            handler: function(){
                Item.windowMail.hide();
            }
        }]
    });

    Ext.QuickTips.init();
    
    var sm = new Ext.grid.CheckboxSelectionModel();
    
    var record = new Ext.data.Record.create([
        {name: 'id'},
        {name: 'email'},
        {name: 'site_id'},
        {name: 'site_name'},
        {name: 'group_id'},
        {name: 'is_active'}
    ]);
    var Ed = Ext.grid.GridEditor, fm = Ext.form;
    
    var ds = new Ext.data.Store({
        proxy: new Ext.data.HttpProxy({
            url: Ecart.baseUrl + Ecart.adminUrl + '/customer_index/list'
        }),
        reader: new Ext.data.JsonReader({
               root: 'data',
               totalProperty: 'count',
               id: 'id'
            }, 
            record
        ),
        sortInfo:{field:'id', direction:'DESC'},
        remoteSort: true
    });
    
    siteStore = new Ext.data.JsonStore({
        url: Ecart.baseUrl + Ecart.adminUrl + '/site/get-list',
        root: 'data',
        id: 'id',
        fields: ['id', 'name'],
        autoLoad: true
    });
    
    groupStore = new Ext.data.JsonStore({
        url: Ecart.baseUrl + Ecart.adminUrl + '/customer_group/get-groups',
        root: 'data',
        id: 'id',
        fields: ['id', 'name'],
        autoLoad: true
    });
    
    var cm = new Ext.grid.ColumnModel([
    sm,{
        header: "Id".l(), 
        dataIndex: 'id', 
        width: 35
    }, {
        header: 'Email'.l(), 
        dataIndex: 'email',  
        width: 200,
        renderer:function (value, meta, record) {
            meta.attr = 'ext:qtip="Open in new window ' + value + '"';
            return String.format(
                    '<a href="{1}" class="grid-link-icon user" target="_blank" >{0}</a>',
                    value, Ecart.baseUrl + Ecart.adminUrl + '/customer_index/edit/customerId/' + record.data.id);
    } 
    }, {
        header: 'Site'.l(), 
        dataIndex: 'site_id',
        editor: new Ext.form.ComboBox({
            triggerAction: 'all',
            displayField: 'name',
            typeAhead: true,
            mode: 'local',
            forceSelection: false,
            store: siteStore,
            lazyRender: true,
            valueField: 'id'
        }),
        renderer: function(value) {
            if (!value || value == '0') {
                return "None";
            } else {
               return siteStore.getById(value).get('name');
            }
        },
        width: 100
    }, {
        header: "Status".l(), 
        dataIndex: 'is_active',  
        width: 80,
        editor: new Ext.form.ComboBox({
            typeAhead: true,
            triggerAction: 'all',
            lazyRender: true,
            forceSelection: false,
            store: new Ext.data.SimpleStore({
                fields: ['id', 'value'],
                data: [
                    ['0', 'Disabled'], 
                    ['1', 'Enabled']
                ]
            }),
            displayField: 'value',
            valueField: 'id',
            mode: 'local'
        }),
        renderer: function(value) {
            return value == '1' ? 'Enabled' : 'Disabled';
        }
    },{
        header: "Group".l(),
        dataIndex: 'group_id',
        width: 130,
        sortable: true,
        editor: new Ext.form.ComboBox({
            triggerAction: 'all',
            displayField: 'name',
            typeAhead: true,
            mode: 'local',
            forceSelection: false,
            store: groupStore,
            lazyRender: true,
            valueField: 'id'
        }),
        renderer: function(value) {
            if (!value || typeof(groupStore.getById(value).get('name')) == 'undefined') {
                return "None";
            } else {
               return groupStore.getById(value).get('name');
            }
        }
    }]);
    
    cm.defaultSortable = true;  
    
    var grid = new Ext.grid.EditorGridPanel({
        renderTo: 'grid-customers',
        ds: ds,
        cm: cm,
        sm: sm,
        height: 550,
        viewConfig: {
            forceFit:true,
            emptyText: 'No records found'.l()
        },
        clicksToEdit: 1,
        bbar: new Ext.PagingToolbar({
            pageSize: 20,
            store: ds,
            displayInfo: true,
            displayMsg: 'Displaying records {0} - {1} of {2}'.l(),
            emptyMsg: 'No records found'.l()
        }),
        tbar: [{
            text: 'Add'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/add.png',
            handler: Item.create
        }, {
            text: 'Send email'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/email.png',
            handler: Item.mail
        }, {
            text: 'Delete'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/delete.png',
            handler: Item.remove
        }, {
            text: 'Save'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/save_multiple.png',
            handler: Item.quickSave
        },{
            text: 'Custom fields'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/page_edit.png',
            handler: Item.quickedit
        }, '->' , {
            text: 'Reload'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/refresh.png',
            handler: function() {
                grid.getStore().reload();
            }
        }],
        listeners: {
            'beforerender': function(cmp) {
                cmp.width = $('#ext-cmp-layout').width() - $('#ext-cmp-layout .sidebox').width();
            }
        }
    });
    
    siteStore.on('load', function(){
        tryLoadGrid();
    })
    groupStore.on('load', function(){
        tryLoadGrid();
    })
    siteStore.on('loadexception', function(){
        siteStore.load();
    })
    groupStore.on('loadexception', function(){
        groupStore.load();
    })
    
    var loadedStore = 0;
    
    function tryLoadGrid() {
        if (++loadedStore == 2) {
            ds.load();
        }
    }
    
    grid.on('rowdblclick', Item.edit);
    
});


