/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ext.onReady(function (){
     AddressBook = {
         activeAddressId: 0,
         _idNew : 0,
         
         getNewId: function()
         {
             return --AddressBook._idNew;
         },
         
         address_form_template : new Ext.Template.from('address-form-tpl'),
         address_template      : new Ext.Template.from('address-tpl'),
         drawForm: function(record)
         {
             AddressBook.address_form_template.insertFirst('tab-addresses-block-form-list', record);
             $('#edit_elements_countries_' + record.id).change(AddressBook.setZones);  
             AddressBook.selectByValue('edit_elements_countries_' + record.id,record.country_id);
             $('#edit_elements_countries_' + record.id).change();
             AddressBook.selectByValue('edit_elements_zones_' + record.id, record.zone_id);
             if (record.id == defaultBillingAddressId) {
                 $('#set-us-primary-billing-address-' + record.id).get(0).checked = true;
             }
             if (record.id == defaultShippingAddressId) {
                 $('#set-us-primary-shipping-address-' + record.id).get(0).checked = true;
             }
             $('#set-us-primary-billing-address-' + record.id).click(AddressBook.setDefaultBillingAddress);
             $('#set-us-primary-shipping-address-' + record.id).click(AddressBook.setDefaultShippingAddress) ;
         },
         
         drawList: function (id)
         {
             if ($('#edit_elements_' + id).length == 0 ) 
                 return;
             var recordTemp = {
                 id:         id
             };
             var inputs = $('#edit_elements_' + id + ' input, #edit_elements_' + id+ ' select');
             
             for (var i = 0, n = inputs.length; i < n; i++) {
                 if (inputs[i].name.split('][').length != 1 ) {
                     var fieldname = inputs[i].name.split('][')[1].replace(/]/, '');
                     if (inputs[i].type == 'select-one' && inputs[i].options.length > 0)
                         recordTemp[fieldname] = inputs[i].options[inputs[i].selectedIndex].text ;
                     else
                         recordTemp[fieldname] =  inputs[i].value ;
                 }
             }
             
             if ($('#address_' + id).length) {
                AddressBook.apply_address_template = AddressBook.address_template.applyTemplate(recordTemp);
                $('#address_' + id).replaceWith(AddressBook.apply_address_template);
             } else
                 AddressBook.address_template.append('tab-addresses-block-info-list',recordTemp);
             $('#address_' + id).click(AddressBook.activeMe);
             $('#address_' + id).mouseover(AddressBook.overMe);
               $('#address_' + id).mouseout(AddressBook.outMe);
             $('#delete-address-' + id).click(AddressBook.removeMe);
             $('#set-us-primary-billing-address-' + id).click(AddressBook.setDefaultBillingAddress);
             $('#set-us-primary-shipping-address-' + id).click(AddressBook.setDefaultShippingAddress);

             AddressBook.showDefaultAddress();
         },
         
         newAddress: function() 
         {
             var id = AddressBook.getNewId();
             AddressBook.drawForm({id: id});
             AddressBook.drawList(id);
             AddressBook.activeId(id);
         },
         
         setZones: function()
         {
             var country_id = this.value;
             var zoneOptions = $('#' + this.getAttribute('assignedZones')).get(0).options;
             while (zoneOptions.length) {
                 zoneOptions[zoneOptions.length - 1] = null;     
             }
             if (Zones[country_id])
                  for (var zoneId in Zones[country_id]) {
                      zoneOptions[zoneOptions.length] = new Option(Zones[country_id][zoneId], zoneId);
                  }
         },
         
         selectByValue: function (id, value)
         {
             var options = $('#'+id).get(0).options;
             for (var j in options) {
                 if (options[j].value == value) {
                     options[j].selected = true;
                     return true;
                 }
              }
              return false;
         },
         
         showMe: function(id) 
         {
             $(".address-form").css('display', 'none');
             $('#edit_elements_'+ id).show();
         },
         
         activeId: function(id) 
         {
             if (activeAddressId != 0 && id != activeAddressId  )
                AddressBook.drawList(activeAddressId);
             AddressBook.showMe(id);
             $('.activeAddress').removeClass('activeAddress');
             $('#address_'+ id).addClass('activeAddress');
             activeAddressId = id;
         },
     
         activeMe: function() 
         {
             id = this.id.replace(/address_/,'');
             AddressBook.activeId(id);
         },
     
         removeMe: function()
         {
             if (!confirm('Delete Address?'))
                 return;
             var id = this.id.replace(/delete-address-/, '');
             if (id >0 ) 
                deleteAddressesId[deleteAddressesId.length] = id;
             $('#address_'+id).remove();
             $('#edit_elements_'+id).remove();
         },
         
         overMe: function() 
         {
             id = this.id.replace(/address_/,'');
             $('.over').removeClass('over');
             $('#address_'+ id).addClass('over');
         },
     
         outMe: function() 
         {
             $(".over").removeClass('over');
         },
         
         showDefaultAddress: function()
         {
             $(".billing_address").css('display', 'none');
             $(".shipping_address").css('display', 'none');
             $('#primary-billing-address-' + defaultBillingAddressId).show();
             $('#primary-shipping-address-' + defaultShippingAddressId).show();
         },
         
         setDefaultBillingAddress: function()
         {
             id = this.id.replace(/set-us-primary-billing-address-/,'');
             defaultBillingAddressId = id;
             AddressBook.showDefaultAddress();
         },
         
         setDefaultShippingAddress: function()
         {
             id = this.id.replace(/set-us-primary-shipping-address-/,'');
             defaultShippingAddressId = id;
             AddressBook.showDefaultAddress();
         }
     };
});