/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ext.onReady(function(){
    
    var contextMenu = new Ext.menu.Menu({
        id: 'context-menu',
        items: [{
            text: 'Export category'.l(),
            id: 'menu-category',
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/move.png',
            handler: function(){
                exportItems(0)
            }
        }, {
            text: 'Export branch'.l(),
            id: 'menu-branch',
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/tree_expand.png',
            handler: function(){
                exportItems(1)
            }
        }]
    })
    
    var rootNode = new Ext.tree.AsyncTreeNode({
        text: 'Sites'.l(),
        draggable: false,
        id: '0'
    })
    
    var categoryTree = new Ext.tree.TreePanel({
        renderTo: 'categoryTree',
        id: 'extCategoryTree',
        animate: false,
        enableDD: false,
        useArrows: true,
        width: 200,
        height: 500,
        rootVisible: true,
        containerScroll: true,
        root: rootNode,
        loader: new Ext.tree.TreeLoader({
            url: Ecart.baseUrl + Ecart.adminUrl + '/catalog_category/get-items/'
        })
    })
    
    var progressBar = new Ext.ProgressBar({
        text:'Initializing...'.l(),
        id: 'extProgressBar',
        renderTo: 'lightbox-info'
    })
    
    categoryTree.on('click', function(node, e) {
        if (Ext.getCmp('extTabs').getActiveTab().id == 'gbaseGrid') {
            gbaseGrid.getStore().baseParams = {
                site_id : node.attributes.site_id,
                product_type: node.attributes.name
            };
            gbaseGrid.getStore().load({params:{
                start: 0, 
                limit: 25
            }});
        } else if (node.id == 0) {
            return false;
        } else {
            siteId = node.attributes.site_id;
            
            localGrid.getStore().baseParams = {
                catId: node.id
            };
            localGrid.getStore().load({params:{
                start: 0, 
                limit: 25
            }});
        }
    });
    
    categoryTree.on('contextmenu', function(node, e) {
        e.preventDefault();
        if (node.id == 0) 
            return false;
        
        if (node.attributes.site_node) {
            Ext.getCmp('menu-category').hide();
            Ext.getCmp('menu-branch').show();
        }
        else if (!node.hasChildNodes()) {
            Ext.getCmp('menu-branch').hide();
            Ext.getCmp('menu-category').show();
        }
        else {
            Ext.getCmp('menu-branch').show();
            Ext.getCmp('menu-category').show();
        }
            
        contextMenu.showAt(e.getXY());
        node.select();
    })
    
    categoryTree.getRootNode().expand();
})

function exportItems(recursive){
    var node = Ext.getCmp('extCategoryTree').getSelectionModel().getSelectedNode();
    
    if (!node || node.id == 0)
        return false;
    
    var params = {};
        
    params['catId'] = node.id;
    params['siteId'] = node.attributes.site_id;
    params['language'] = Ext.getDom('language').value;
    params['country']  = Ext.getDom('gCountry').value;
    params['currency'] = Ext.getDom('currency').value;
    params['recursive'] = recursive;
    
    if (isNaN(params['siteId'])) {
        alert('Site is undefined');
        return false;
    }
    
    ajaxExportBranch(params, 1);
    
}

function ajaxExportBranch(params, clearSession){
    
    if (clearSession) {
        Ext.getCmp('extProgressBar').reset();
        Ext.getCmp('extProgressBar').updateText('Initializing...');
        Ext.get('lightbox-info').show();
    }
    
    Ext.Ajax.request({
        url: Ecart.baseUrl + Ecart.adminUrl + '/gbase_index/export-branch',
        method: 'post',
        params: {
            language: params['language'],
            country: params['country'],
            currency: params['currency'],
            site: params['siteId'],
            catId: params['catId'],
            recursive: params['recursive'],
            clearSession: clearSession
        },
        callback: function(options, success, response){
            if (success) {
                var obj = Ext.util.JSON.decode(response.responseText);
                Ext.getCmp('extProgressBar').updateProgress(obj.processed/obj.count, 'Exported ' + obj.processed + ' of ' + obj.count);
                if (!obj.finalize) {
                    ajaxExportBranch(params, 0);
                } else {
                    Ext.get('lightbox-info').hide();
                }
            } else {
                Ext.getCmp('extProgressBar').updateText('An error has been occured. Connecting...');
                ajaxExportBranch(params, 0);
            }
        }
    })
}