/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

Ext.onReady(function() {
    var Item = {
        
        activeId: 0,
        
        record: Ext.data.Record.create([
            {name: 'id'},
            {name: 'code'},
            {name: 'name'},
            {name: 'country_id'}
        ]),
        
        create: function () {
            grid.stopEditing();
            var record = new Item.record({
                name: '',
                code: '',
                country_id: ''
            });
            ds.insert(0, record);
            grid.startEditing(0, 0);
        },
        
        save: function() {
            var modified = ds.getModifiedRecords();
            
            if (!modified.length)
                return;
                
            var data = {};
            
            for (var i = 0; i < modified.length; i++) {
                data[modified[i]['id']] = modified[i]['data'];
            }
            
            Ext.Ajax.request({
                url: Ecart.baseUrl + Ecart.adminUrl + '/location_zone/save',
                params: {
                    data: Ext.encode(data)
                },
                callback: function() {
                    ds.commitChanges();
                    ds.reload();
                }
            });
        },
        
        remove: function() {
            var selectedItems = grid.getSelectionModel().selections.items;
            
            if (!selectedItems.length)
                return;
            
            if (!confirm('Are you sure?'))
                return;
                
            var data = {};
            
            for (var i = 0; i < selectedItems.length; i++) {
                if (!selectedItems[i]['data']['id']) continue;
                data[i] = selectedItems[i]['data']['id'];
            }
            
            Ext.Ajax.request({
                url: Ecart.baseUrl + Ecart.adminUrl + '/location_zone/delete',
                params: {data: Ext.encode(data)},
                callback: function() {
                    ds.reload();
                }
            })
        }
    }
    
    var dsCountry = new Ext.data.JsonStore({
        id: 'id',
        fields: ['id', 'name'],
        data: countries
    });
    
    var ds = new Ext.data.Store({
        proxy: new Ext.data.HttpProxy({
            url: Ecart.baseUrl + Ecart.adminUrl + '/location_zone/list'
        }),
        reader: new Ext.data.JsonReader({
            root: 'data',
            totalProperty: 'count',
            id: 'id'
        }, Item.record),
        remoteSort: true,
        pruneModifiedRecords: true
    });
    
    var selectModel = new Ext.grid.CheckboxSelectionModel();
    
    var cm = new Ext.grid.ColumnModel([
        selectModel, {
            header: "Name".l(),
            dataIndex: 'name',
            width: 350,
            sortable: true,
            editor: new Ext.form.TextField({
               allowBlank: false
            })
        }, {
            header: "Code".l(),
            dataIndex: 'code',
            width: 80,
            sortable: true,
            editor: new Ext.form.TextField({
               allowBlank: false
            })
        }, {
            header: "Country".l(),
            dataIndex: 'country_id',
            width: 300,
            sortable: true,
            mode: 'local',
            editor: new Ext.form.ComboBox({
               typeAhead: true,
               triggerAction: 'all',
               lazyRender: true,
               store: dsCountry,
               displayField: 'name',
               valueField: 'id',
               mode: 'local'
            }),
            renderer: function(value) {
                if (value == '0' || value == '') {
                    return "None";
                } else {
                    return dsCountry.getById(value).get('name');
                }
            }
        }
    ]);
    
    var grid = new Ext.grid.EditorGridPanel({
        ds: ds,
        cm: cm,
        sm: selectModel,
        height: 550,
        renderTo: 'grid',
        loadMask: false,
        clicksToEdit:1,
        viewConfig: {
            forceFit: true,
            emptyText: 'No records found'.l()
        },
        bbar: new Ext.PagingToolbar({
            pageSize: 21,
            store: ds,
            displayInfo: true,
            displayMsg: 'Displaying records {0} - {1} of {2}'.l(),
            emptyMsg: "No records found".l()
        }),
        tbar: [{
            text: 'Add'.l(),
            icon: Ecart.skinUrl + '/images/icons/add.png',
            cls: 'x-btn-text-icon',
            handler : Item.create
        }, {
            text: 'Save'.l(),
            icon: Ecart.skinUrl + '/images/icons/save_multiple.png',
            cls: 'x-btn-text-icon',
            handler : Item.save
        }, {
            text: 'Delete'.l(),
            icon: Ecart.skinUrl + '/images/icons/delete.png',
            cls: 'x-btn-text-icon',
            handler : Item.remove
        }, '->', {
            text: 'Reload'.l(),
            icon: Ecart.skinUrl + '/images/icons/refresh.png',
            cls: 'x-btn-text-icon',
            handler : function() {
                grid.getStore().reload();
            }
        }],
        listeners: {
            'beforerender': function(cmp) {
                cmp.width = $('#ext-cmp-layout').width() - $('#ext-cmp-layout .sidebox').width();
            }
        }
    });
    
    grid.render();
    
    ds.load({params:{start:0, limit:21}});
}, this);