/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

var dataStore, editWin, resultWin;
Ext.onReady(function (){
    Ext.QuickTips.init();  
    dataStore = new Ext.data.Store({
        proxy: new Ext.data.HttpProxy({
            url: Ecart.baseUrl + Ecart.adminUrl + '/poll_index/list'
        }),
        reader: new Ext.data.JsonReader({
            root: 'data',
            id: 'id'
        }, [{name: 'id'}, 
            {name: 'changed_at', type: 'date', dateFormat: 'Y-m-d H:i:s'}, 
            {name: 'created_at', type: 'date', dateFormat: 'Y-m-d H:i:s'},
            {name: 'question'},
            {name: 'lock_status'},
            {name: 'sites'},
            {name: 'type'},
            {name: 'cnt'}]
        ),
        remoteSort: true,
        pruneModifiedRecords: true
    });
    
    var actions = new Ext.ux.grid.RowActions({
        header:'Actions'.l(),
        actions:[{
            iconCls:'icon-details'
            ,tooltip:'Results'.l()
        },{
            iconCls:'icon-edit'
            ,tooltip:'Edit'.l()
        },{
            iconCls:'icon-delete'
            ,tooltip:'Delete'.l()
        }]
        ,callbacks: {
            'icon-details':function(grid, record, action, row, col) {
                Ext.Ajax.request({
                    params : {questionId : record.json.id,getResult:true},
                    url: Ecart.baseUrl + Ecart.adminUrl +  '/poll_index/get-question',
                    callback: function(options, success, response) {
                        $("#question-result").empty();
                        $("#answers-results").empty();
                         var oResponse = Ext.decode(response.responseText);
                         var answer_result_template = new Ext.Template.from('answer-result-template');
                         for (answer in oResponse.answers) {
                             oResponse.answers[answer]['style'] = 'style="width: ' + oResponse.answers[answer]['percent']  + '%"'; 
                             if (!$('#answer-result-' + oResponse.answers[answer].id).length &&
                                (oResponse.answers[answer].languageId == currentLanguageId)) {
                                 
                                 answer_result_template.insertFirst('answers-results', oResponse.answers[answer]);
                             }
                         }
                         $("#answer-result-").remove();
                         // $('#answers-results').show();
                         var question_result_template = new Ext.Template.from('question-result-template');
                         var question = oResponse.question;
                         question_result_template.insertFirst('question-result', question[currentLanguageId]);
                         getResultsWindow();
                         resultWin.setTitle('Results Poll  [ ' + (question[currentLanguageId] ? question[currentLanguageId].question : '') + ' ]');
                         resultWin.show();
                    }
                }) 
            },
            'icon-edit':function(grid, record, action, row, col) {
                Ext.Ajax.request({
                    url: Ecart.baseUrl + Ecart.adminUrl +  '/poll_index/get-question',
                    params : {questionId : record.json.id},
                    callback: function(options, success, response) {
                        removeQuestion();
                        var oResponse = Ext.decode(response.responseText);
                        hideQuestion();
                        removeAnswers();
                        var answer_template = new Ext.Template.from('answer-template');
                        for (answer in oResponse.answers) {
                            if ($('#questions-template-answers-' + oResponse.answers[answer].languageId).length){
                                answer_template.insertFirst(
                                    'questions-template-answers-' + oResponse.answers[answer].languageId,
                                    oResponse.answers[answer]
                                );
                                for (var i = 0; i < languagesIds.length; i++)
                                    if (oResponse.answers[answer].languageId == languagesIds[i])
                                        $('#answer-value-' +  oResponse.answers[answer].id + '-' + 
                                            languagesIds[i]).val(oResponse.answers[answer].answer
                                        );
                            }
                        }
                        $("#answer--").remove(); // delete null answer @todo fix this bugs
                        var question_template = new Ext.Template.from('questions-template');
                        var question = oResponse.question;
                        for (var i = 0; i < languagesIds.length; i++) {
                             if (typeof(question[languagesIds[i]]) == 'undefined' ){
                                question[languagesIds[i]] = question[languagesIds[i-1]];
                                question[languagesIds[i]].languageId = languagesIds[i];
                                question[languagesIds[i]].question = ''; 
                             }
                             
                             if (question[languagesIds[i]].languageId == languagesIds[i]) {  
                                question_template.insertFirst('tab' + languagesIds[i], question[languagesIds[i]]);
                            }
                        }
                        $("#lock_status").selectOptions(question.lock_status);
                        $("#site_ids").selectOptions(question.sites[0], true);   
                        for (var k in question.sites)  
                            $("#site_ids").selectOptions(question.sites[k]);   
                        showQuestion();
                        getWindow();
                        editWin.setTitle('Edit Question');
                        editWin.show();
                    }
                });
                questionId = record.json.id;
            },
            'icon-delete':function(grid, record, action, row, col) {
                if (!confirm('Delete question: [ '+ record.json.question + ' ]'))
                    return;
                deleteQuestionsIds[0] = record.json.id; 
                Ext.Ajax.request({
                    params : {data :  deleteQuestionsIds},
                    url: Ecart.baseUrl + Ecart.adminUrl +  '/poll_index/delete',
                    callback: function(response, options) {
                        dataStore.reload();
                    }
                }) 
            }
        }
    });
    
    var status = new Ext.grid.CheckColumn({
        header: "Status".l(),
        dataIndex: 'lock_status',
        width: 60
    });
    var selectionModel = new Ext.grid.CheckboxSelectionModel();
    var storeSites = new Ext.data.JsonStore({
        id: 'id',
        fields: ['id', 'name'],
        data: sites
    });
    var columnsModel = new Ext.grid.ColumnModel([
        selectionModel,{
            header: 'Id'.l(),
            dataIndex: 'id',
            width: 40
        }, {
            header: "Question".l(), 
            sortable: true,
            dataIndex: 'question'
        },
        status,
        {
            header: "Sites".l(), 
            sortable: true,
            dataIndex: 'sites',
            editor:  new Ext.ux.Andrie.Select(Ext.applyIf({
                fieldLabel:  'Field',
                multiSelect: true,
                minLength: 1
            }, {
                store:storeSites,
                valueField:'id',
                displayField:'name',
                triggerAction:'all',
                mode:'local'
            })),
            renderer: function(value, meta) {
                var ret = new Array();
                value = value.split(',');
                for (var i = 0, n = value.length; i < n; i++) {
                    if (value[i] != '') {
                        ret.push(storeSites.getById(value[i]).data.name);
                    }
                }
                ret = ret.join(', ');
                return ret ;
            }
        }, {
            header: "Type".l(), 
            sortable: true,
            dataIndex: 'type',
            editor: new Ext.form.ComboBox({
                triggerAction: 'all',
                displayField: 'value',
                typeAhead: true,
                mode: 'local',
                valueField: 'id',
                store: new Ext.data.SimpleStore({
                    fields: ['id', 'value'],
                    data: [['0', 'Singleselect'.l()], ['1', 'Multiselect'.l()]]
                })
            }),
            renderer: function(value, meta) {
                return value == '1' ? 'Multiselect'.l() : 'Singleselect'.l();
            }
        }, {
            header: "Created".l(), 
            sortable: true,
            dataIndex: 'created_at',
            renderer: function(v) {
                return Ext.util.Format.date(v);
            }
        }, {
            header: "Modified".l(), 
            dataIndex: 'changed_at',
            renderer: function(v) {
                return Ext.util.Format.date(v);
            }
        }, {
            header: "Votes".l(), 
            dataIndex: 'cnt'
        }
        ,actions
    ]); 
    
    gridQuestion = new Ext.grid.EditorGridPanel({
        store: dataStore,
        cm: columnsModel,
        clicksToEdit:1,
        renderTo: 'questions-grid',
        viewConfig:{
            forceFit:true,
            emptyText: 'No records found'.l()
        },
        sm:selectionModel, 
        stripeRows: true,
        height : 550,
        plugins:[status,actions],
        tbar: [{
            text: 'Add'.l(),
            icon: Ecart.skinUrl + '/images/icons/add.png',
            cls: 'x-btn-text-icon',
            handler : function(){
                removeQuestion();
                removeAnswers();
                var answer_template = new Ext.Template.from('answer-template');
                var someId =  getNewAnswerId(); 
                var someId2 =  getNewAnswerId();
                for (var i = 0; i < languagesIds.length; i++){
                    answer_template.insertFirst('questions-template-answers-' + languagesIds[i],
                            {"id":someId,"count":"0","percent":"0", "answer":"", languageId : languagesIds[i]});
                    answer_template.insertFirst('questions-template-answers-'+languagesIds[i],
                            {"id":someId2, "count":"0","percent":"0", "answer":"", languageId : languagesIds[i]});
                }
                
                var question_template = new Ext.Template.from('questions-template');
                for (var i = 0; i < languagesIds.length; i++){
                    var question = {"id":"0","lock_status":"0","question":"","languageId" : languagesIds[i]};
                    question_template.insertFirst('tab' + languagesIds[i], question);
                }
                showQuestion();
                getWindow();
                editWin.setTitle('Add new Question');
                editWin.show();
                questionId = 0;
            }
        },{
            text: 'Save'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/save_multiple.png',
            handler: function(){
                var modified = gridQuestion.getStore().getModifiedRecords();
                if (modified.length < 1) return alert('Nothing to save');
                var data = {};
                for (var i = 0; i < modified.length; i++) {
                    data[modified[i]['data'].id] = 
                        {'status': modified[i]['data'].lock_status,
                         'type': modified[i]['data'].type,
                         'sites': modified[i]['data'].sites
                        };
                }
                Ext.Ajax.request({
                    url: Ecart.baseUrl + Ecart.adminUrl +  '/poll_index/quick-save',
                    params: {data: Ext.encode(data)},
                    callback: function() {
                        gridQuestion.getStore().commitChanges();
                        gridQuestion.getStore().reload();
                    }
                })
            }
        },{
            text: 'Delete'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/delete.png',
            handler: function(){
                var selectedItems = gridQuestion.getSelectionModel().selections.items;
                if (selectedItems.length < 1)
                    return; 
                if (!confirm('Are you sure?'.l()))
                    return;
                var data = {};
                for (var i = 0; i < selectedItems.length; i++) {
                    data[i] = selectedItems[i].id;
                }
                Ext.Ajax.request({
                    url: Ecart.baseUrl + Ecart.adminUrl +  '/poll_index/delete',
                    params: {data: Ext.encode(data)},
                    callback: function() {
                        gridQuestion.getStore().reload();
                    }
                });
              
            }
        },{
            text: 'Clear Voted '.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/drive_delete.png',
            handler: function(){
                var selectedItems = gridQuestion.getSelectionModel().selections.items;
                if (selectedItems.length < 1)
                    return; 
                if (!confirm('Clear Voted?'))
                    return;
                var data = {};
                for (var i = 0; i < selectedItems.length; i++) {
                    data[i] = selectedItems[i].id;
                }
                Ext.Ajax.request({
                    url: Ecart.baseUrl + Ecart.adminUrl +  '/poll_index/clear',
                    params: {data: Ext.encode(data)},
                    callback: function() {
                        gridQuestion.getStore().reload();
                    }
                });
            }
        },'->',{
            text: 'Reload'.l(),
            cls: 'x-btn-text-icon',
            icon: Ecart.skinUrl + '/images/icons/refresh.png',
            handler: function(){
                gridQuestion.getStore().reload();
            }
        }],
        listeners: {
            'beforerender': function(cmp) {
                cmp.width = $('#ext-cmp-layout').width() - $('#ext-cmp-layout .sidebox').width();
            }
        }
    });
    
    dataStore.load();
   
    function getWindow() {
        if (!editWin) {
            editWin = new Ext.Window({
                contentEl: 'question',
                layout: 'fit',
                width: 310,
                closeAction: 'hide',
                plain: true,
                autoScroll:true,
                buttons: [{
                text: 'Save'.l(),
                handler: function() {
                    Ext.Ajax.request({
                       url : Ecart.baseUrl + Ecart.adminUrl +  '/poll_index/save',
                       form: 'question-form',
                       params: {questionId : questionId ,  deleteAnswersIds:  Ext.encode(deleteAnswersIds)},
                       callback: function(options, success, response) {
                         var oRet = Ext.decode(response.responseText);
                            if (success) {
                                editWin.hide();
                            }
                        }
                    });
                    dataStore.reload();
                }
                },{
                    text: 'Cancel'.l(),
                    handler: function(){
                        editWin.hide();
                    }
                }]
               
            });
        }
        return editWin;
    } 
    
    function getResultsWindow() {
        if (!resultWin) {
            resultWin = new Ext.Window({
                contentEl: 'question-result',
                layout: 'fit',
                width: 220,
                closeAction: 'hide',
                plain: true,
                autoScroll:true,
                buttons: [{
                    text: 'Close'.l(),
                    handler: function(){
                        resultWin.hide();
                    }
                }]
               
            });
        }
        return resultWin;
    }   
});

var newId = 0 ;

function getNewAnswerId()
{
    return  --newId;
}
 
function newAnswer()
{
    var id = getNewAnswerId();
    var record = {
        id: id,
        count: 0,
        percent: 0
    };
    var answer_template = new Ext.Template.from('answer-template');
    for (var i = 0; i < languagesIds.length; i++){
        record.languageId = languagesIds[i];
        answer_template.insertFirst('questions-template-answers-' + languagesIds[i], record);
    }
    $("#answer--").empty();
    editWin.hide();
    editWin.show();
} 
    
function deleteAnswer (id) {
    if (id > 0) 
        deleteAnswersIds[deleteAnswersIds.length] = id;
    for (var i = 0; i < languagesIds.length; i++)    
       $("#answer-" + id + "-" + languagesIds[i]).remove();
    editWin.hide();
    editWin.show();
} 

function showQuestion() {
    $('#question').show();
}    
    
function hideQuestion() {
    $("#question").hide();
}   
    
function removeQuestion(){
    $("#question .question").remove()
} 
    
function removeAnswers() {
    for (var i = 0; i < languagesIds.length; i++)    
        $("#questions-template-answers-" + languagesIds[i]).empty();
} 

/*checkbox column*/
Ext.grid.CheckColumn = function(config){
    Ext.apply(this, config);
    if(!this.id){
        this.id = Ext.id();
    }
    this.renderer = this.renderer.createDelegate(this);
};

Ext.grid.CheckColumn.prototype ={
    init : function(grid){
        this.grid = grid;
        this.grid.on('render', function(){
            var view = this.grid.getView();
            view.mainBody.on('mousedown', this.onMouseDown, this);
        }, this);
    },

    onMouseDown : function(e, t){
        if(t.className && t.className.indexOf('x-grid3-cc-'+this.id) != -1){
            e.stopEvent();
            var index = this.grid.getView().findRowIndex(t);
            var record = this.grid.store.getAt(index);
            record.set(this.dataIndex, parseInt(record.data[this.dataIndex]) == 1 ? 0 : 1);
        }
    },

    renderer : function(v, p, record){
        p.css += ' x-grid3-check-col-td'; 
        return '<div class="x-grid3-check-col'+(v=='1'?'-on':'')+' x-grid3-cc-'+this.id+'">&#160;</div>';
    }
};
/*end of checkbox column*/
//eof

