<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Locale
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Locale
 * @subpackage  Collect
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Collect_ZendLocale implements Ecart_Collect_Interface
{
    /**
     * @static
     * @return array
     */
    public static function collect()
    {
        $options    = array();
        $locale = Ecart_Locale::getLocale();
        
        $locales = $locale->getLocaleList();
        $languages  = $locale->getTranslationList('language', $locale);
        $countries  = $locale->getTranslationList('territory', $locale, 2);
        
        if (!$languages || !$countries) {
            $languages  = $locale->getTranslationList(
                'language', Ecart_Locale::DEFAULT_LOCALE
            );
            $countries  = $locale->getTranslationList(
                'territory', Ecart_Locale::DEFAULT_LOCALE, 2
            );
        }
        
        foreach ($locales as $code => $isActive) {
            if (strstr($code, '_')) {
                $data = explode('_', $code);
                if (!isset($languages[$data[0]]) 
                    || !isset($countries[$data[1]])) {

                    continue;
                }
                $options[$code] = ucfirst($languages[$data[0]]) . ' ('
                    . $countries[$data[1]] . ')';
            }
        }
        return $options;
    }

    /**
     *
     * @static
     * @param int $id
     * @return mixed string|void
     */
    public static function getName($id)
    {
        if (empty($id)) {
            return;
        }
        $data = explode('_', $id);
        $locale = Ecart_Locale::getLocale();
        $language  = $locale->getTranslation($data[0], 'language', $locale);
        $country  = $locale->getTranslation($data[1], 'country', $locale);
        
        return ucfirst($language) . ' (' . $country . ')';
    }
}