<?php
/**
 * Ecart
 *
 * This file is part of Ecart.
 *
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Core
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Core
 * @subpackage  Helper
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Controller_Action_Helper_Json extends Zend_Controller_Action_Helper_Json
{
    /**
     * Encode JSON response and immediately send
     *
     * @param  mixed   $data
     * @param  boolean|array $keepLayouts
     * NOTE:   if boolean, establish $keepLayouts to true|false
     *         if array, admit params for Zend_Json::encode as enableJsonExprFinder=>true|false
     *         if $keepLayouts and parmas for Zend_Json::encode are required
     *         then, the array can contains a 'keepLayout'=>true|false
     *         that will not be passed to Zend_Json::encode method but will be passed
     *         to Zend_View_Helper_Json
     * @return string|void
     */
    public function sendJson($data, $keepLayouts = false, $wrap = true)
    {
        if (isset($data['success']) && !is_bool($data['success'])) {
            $data['success'] = (bool) $data['success'];
        }

        if ($wrap) {

            $messages = Ecart::message()->getAll();
//            if (isset(false === $data['success']) && $data['success']) {
//                unset($messages['success']);
//            }

            $data = array_merge(array('messages' => $messages), $data);
        }

        $data = $this->encodeJson($data, $keepLayouts);
        $response = $this->getResponse();
        $response->setBody($data);

        if (!$this->suppressExit) {
            Zend_Wildfire_Channel_HttpHeaders::getInstance()->flush(); //Ecart_FirePhp
            $response->sendResponse();
            exit;
        }

        return $data;
    }

    /**
     * Encode JSON response and immediately send
     * with success => true
     * @param  mixed   $data
     * @param  boolean|array $keepLayouts
     * @return string|void
     */
    public function sendSuccess($data = array(), $keepLayouts = false)
    {
        $data = array_merge($data, array('success' => true));
        return $this->sendJson($data, $keepLayouts, true);
    }

    /**
     * Encode JSON response and immediately send
     * success => false
     * @param  mixed   $data
     * @param  boolean|array $keepLayouts
     * @return string|void
     */
    public function sendFailure($data = array(), $keepLayouts = false)
    {
        $data = array_merge($data, array('success' => false));
        return $this->sendJson($data, $keepLayouts, true);
    }

}
