<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Method
 * @author      Ecart Core Team <core@ecartcommerce.com>
 * @abstract
 */
abstract class Ecart_Method_Shipping_Model_Abstract extends Ecart_Method_Abstract
{
    protected $_section = 'shipping';
    protected $_file = __FILE__;

    /**
     * @var string
     */
    protected $_type = null;
    
    /**
     * @var array
     */
    protected $_types = null;

    /**
     * Construct shipping class method
     *
     * @param string $type 
     */
    public function __construct($type = null)
    {
        parent::__construct();
        
        //fix  single call 
        if (is_array($type) && !count($type)) {
            $type = null;
        }

        $this->_type = $type;
        try {
            $writer = new Zend_Log_Writer_Stream(
                Ecart::config()->system->path .
                Ecart::config()->log->main->shipping
            );
            $this->_logger = new Zend_Log($writer);
        } catch (Exception $e) {}
    }

    /**
     * Loging
     *
     * @param mixed $message
     */
    public function log($message)
    {
        //Ecart_FirePhp::log($this->_code . ' ' . $message);
        Ecart::message()->addError($this->_code . ': ' . $message);
        $this->_logger->info($this->_code . ' ' . $message);
    }
    
    /**
     * Сan not used other method this class on this method
     *
     * @param mixed array $request shipping request 
     * @return mixed array
     */
    abstract function getTypes($request);
    
    /**
     * Return Shipping Method
     *
     * @param $request Shipping request
     * @param $code shipping code
     * @return Ecart_Method_Shipping_Model_Abstract
     * @throws Ecart_Exception if none found
     */
    public function getType($request, $code)
    {
        if (!Zend_Registry::isRegistered('shippingType')) {
            $types = $this->getTypes($request);
            if (!count($types)) {
                throw new Ecart_Exception(
                    Ecart::translate('checkout')->__(
                        'Shipping Method does not exists for this shipping request'
                    )
                );
            }
            foreach ($types as $type) {
                if ($type['id'] == $code) {
                    Zend_Registry::set('shippingType', $type);
                    return $type;
                }
            }
            throw new Ecart_Exception(
                Ecart::translate('checkout')->__(
                    'Shipping Method not found'
                )
            );
        }
        return Zend_Registry::get('shippingType');
    }
    
    /**
     * 
     * @param $request shipping request
     * @return bool 
     */
    public function isAllowed($request)
    {
        if (!isset($this->_config->geozone) 
            || !intval($this->_config->geozone)) {

            return true;
        }
        $zoneId = null;
        if (isset($request['zone_id'])) {
            $zoneId = $request['zone_id'];
        } elseif (isset($request['zone']['id'])) {
            $zoneId = $request['zone']['id'];
        }
        return Ecart::single('location/geozone_zone')->inGeozone(
            $this->_config->geozone,
            $request['country']['id'],
            $zoneId
        );
    }
    
    /**
     * Return translated shipping method title
     * (Dont use on getTypes function current method)
     *
     * @see library/Ecart/Method/Ecart_Method_Abstract#getTitle()
     * @return string 
     */
    public function getTitle()
    {
        $title = isset($this->_config->title) ?
            $this->_config->title : $this->_title;

        if (null !== $this->_type &&
            $request = Ecart::single('checkout/checkout')->getShippingRequest()) {
            
            $type = $this->getType($request, $this->getCode());
            $title = $type['title'];
        }
        return $this->getTranslator()->__($title);
    }
    
    /**
     * Return shipping code
     * 
     * @see library/Ecart/Method/Ecart_Method_Abstract#getCode()
     * @return string 
     */
    public function getCode()
    {
//        if ($type = current(func_get_args())) {
//            $this->_type = $type;
//        }
        if (null !== $this->_type) {
            return $this->_code . '_' . $this->_type;
        }
        return $this->_code;
    }
    
    /**
     * Only Ecart shippings are supported
     * @todo return module with category name
     *  example: Ecart_ShippingFlat
     *  
     * @return Ecart_Translator
     */
    public function getTranslator()
    {
        return Ecart::translate('Shipping' . current(explode('_', $this->_code)));
    }
}